

C   ********************************************************************M
C   *                                                                  *M
C   * MAIN PROGRAM F418; FORTRAN IV VERSION OF T418.                   *M
C   * H.M.RIETVELD,REACTOR CENTRUM NEDERLAND,PETTEN(N.H.)              *M
C   * ORIGINAL VERSION: H.M.RIETVELD DECEMBER 1970                     *M
C   *                                                                  *M
C   ********************************************************************M
C   *                                                                  *M
C   * ANISOTROPIC TEMPERATURE FACTOR  ROUTINES ADDED BY:               *M
C   * A.W.HEWAT, MATERIALS PHYSICS DIVISION A.E.R.E HARWELL        1972*M
C   *                                                                  *M
C   ********************************************************************M
C   *                                                                  *M
C   * EXTENSIVELY REWRITTEN AND IMPROVED BY:                           *M
C   * P.J.CLARKE, CHEMICAL CRYSTALLOGRAPHY LABORATORY,OXFORD APRIL 1975*M
C   *                                                                  *M
C   ********************************************************************M
C   * THIS VERSION FOR SIMULTANEOUS REFINEMENT OF UP TO 3 PATTERNS     *M
C   * IN SAME SCAN                                                     *M
C   * P.J.BENDALL, INORGANIC CHEMISTRY LABORATORY,OXFORD           1978*M
C   * M.W.THOMAS,  MATERIALS PHYSICS DIVISION A.E.R.E. HARWELL         *M
C   ********************************************************************M
C   * FTN4 --> FTN5 + CHANGES                                           M
C   * H. SCHRADER, INST. F. KRIST. UNIV. MUENCHEN       JANUAR  1986    M
C   * REFINEMENT OF UP TO 5 PATTERNS                       MAY  1986
C   ********************************************************************M
C   *                                                                    *
C   * EXTENDED FOR ANHARMONIC THERMAL MOTION                             *
C   * GRAM-CHARLIER EXPANSION UP TO FOURTH ORDER                         *
C   * H. BOYSEN                                          APRIL 1988      *
C   **********************************************************************
C   *                                                                    *
C   * UPGRADED FOR X-RAYS:                                               *
C   * ANOMALOUS DISPERSION, TWO WAVELENGTHS                              *
C   * PROFILES: LORENTZIAN,MOD LOR + INTERM. LOR                         *
C   * BOYSEN                                             MARCH 1990      *
C   **********************************************************************
C    *                    VARIABLE DIMENSIONS                          *M
C    *      R=MAXIMUM NUMBER OF EQUIVALENT POSITIONS          (MNEQPO) *M
C    *      K=MAXIMUM NUMBER OF SCATTERING LENGTHS            (MNEUTS) *M
C    *      F=MAXIMUM NUMBER OF FORMFACTORS                   (MNXMAG) *M
C    *      N=MAXIMUM NUMBER OF ATOMS                         (MNATOM) *M
C    *      M=MAXIMUM NUMBER OF MAGNETIC ROTATION MATRICES    (MNMAGR) *M
C    *      P=MAXIMUM NUMBER OF LEAST SQUARES PARAMETERS      (MMATOR) *M
C    *      C=MAXIMUM NUMBER OF STRICT CONSTRAINT FUNCTIONS   (MNCONF) *M
C    *      T=MAXIMUM NUMBER OF TERMS IN CONSTRAINT FUNCTION  (MNCONT) *M
C    *      L=MAXIMUM NUMBER OF OVERLAPPING REFLECTIONS       (MNOVRF) *M
C    *      Q=MAXIMUM NUMBER OF POINTS IN A MAGNETIC                   *M
C    *                          SCATTERING CURVE              (MNMAGP) *M
C    *     NP=MAXIMUM NUMBER OF POWDER PATTERNS/SCAN          (MNPATS) *M
C    *      N4=N+3*NP+1                                                *M
C    *                              *****                              *M
C    *                 ARRAYS WITH VARIABLE DIMENSIONS                 *M
C    *     ICODE(L)            MEQ(N)              MTYP(N)             *M
C    *     NTYP(N)             ICON(C,T)           JCON(C,T)           *M
C    *     JCID(C,T)           JTF(N)              LCID(C,T)           *M
C    *     LCONI(C,T)          LCONJ(C,T)          LN(C)               *M
C    *     LP(N4,39)           AKEEP(L)            ATEXT(N4)           *M
C    *     B(K)                                                        *M
C    *     BKEEP(L)            CI(C,T)             CJ(C,T)             *M
C    *     CONA(C,T)           COSAR(N,R)          COSARI(R)           *M
C    *     DERIV(PC)           DERSTO(L,PC)        DF(C)               *M
C    *                         F(F,Q)                                  *M
C    *     FESD(L)             FF(F)               FLAG(N4,39)         *M
C    *     FMAG(L)             FNUC(L)             FOBS(L)             *M
C    *     H(R,3)              HH(3,L)             HW(L)               *M
C    *                                             RM(NP,R,M,9)        *M
C    *     RSA(N,3)            RSB(N,3)            S(F,Q)              *M
C    *     SA(N)               SB(N)               SINAR(N,R)          *M
C    *     SINARI(R)           SJJ(L)              SM(NP,R,3,3)        *M
C    *     SMM(P*(P+1)/2)      SNEX(N)                                 *M
C    *     SOMEGA(L)           T(NP,R,3)           TANN(L)             *M
C    *     TEMP(N)             THETA(L)            TR(R)               *M
C    *     V(P)                XL(N4,39)                               *M
C    *     AL(NP,3,3)          ICENTT(NP)          NHALP(NP)           *M
C    *     CELLN(NP,6)         IPREFF(NP)          NUNIAX(NP)          *M
C    *     RCELLN(NP,6)        KM(NP)              NCUBIC(NP)          *M
C    *     PRCP(NP,6)          MN(NP)              NGAUSS(NP)          *M
C    *     VOL(NP)             HN(NP,3)            NCAUCH(NP)          *M
C    *     NA(NP)              PH(NP,3)            NPAC(NP)            *M
C    *                         B1(NP)                                  *M
C    *     KXR(NP)             B2(NP)              PTEXT(NP)           *M
C    *     IRLL(NP)            KL(NP)              SCALE(NP,2)         *M
C    *******************************************************************M
C                                                                       M
C                            ******************                         M
C                            * PRESENT VALUES *                         M
C                            *      R =24     *                         M
C                            *      K = 8     *                         M
C                            *      F = 4     *                         M
C                            *      N =50     *                         M
C                            *      N4=66     *                         M
C                            *      M = 2     *                         M
C                            *      P =99     *                         M
C                            *      C = 4     *                         M
C                            *      L =500    *                         M
C                            *      T = 9     *                         M
C                            *      Q =21     *                         M
C                            *      NP= 5     *                         M
C                            ******************                         M
C                       ****************************                    M
C                       *   UNIT 3 = INPUT FILE    *                    M
C                       *   UNIT10 = SCRATCH FILE  *                    M
C                       *   UNIT 5 = CARD READER   *                    M
C                       *   UNIT 6 = LINEPRINTER   *                    M
C                       *   UNIT 7 = CARD PUNCH    *                    M
C                       *   UNIT 9 = SCRATCH FILE  *                    M
C                       *   UNIT17 = FOURIER FILE  *                    M
C                       *   UNIT18 = BAYESIAN FILE *                    M
C                       ****************************                    M
C                                                                       M
C                                                                       M
      EXTERNAL PPP
C                                                                       M
      LOGICAL FORCEL,FORCED,PUNCH,STAR,TYPE,VERT,YES,ANGCOU                    M
C                                                                       M
C                                                                       M
      DIMENSION TT(3,3)                                                 M
C                                                                       M
      DIMENSION IREDIN(50)                                              M
C                                                                       M
      PARAMETER (ML=500,MP=99,MX=103)
C                                                                       M
      COMMON /REFLN/ARG,DELTA,DLABDA(2),DUMPA(500),EPS,FLOG,FLOG2,      M
     2 OMEGA,PI,SKEW,SLABDA,SLOG,W,XN,YCALC,YOBS                        M
C                                                                       M
      COMMON /PREF/IALFA,IOMEG,IORD1,IORD2,IPREV,ISTAP,ISS,KSTART,LIMO, M
     2 LLL,NCD(50),NKEY,NO,NTEXT,NUM,NTYP(50),MEQ(50),MTYP(50)          M
C                                                                       M
      COMMON /SYMCEL/ALL(3,3),AL(5,3,3),ATEXT(60),B(8),CELL(6),CELLN(5, M
     2 6),F(4,21),FF(4),PH(5,3),RM(5,24,2,9),S(4,21),SM(5,24,3,3),T(5,  M
     3 24,3),TR(5,24),RCELL(6),RCELLN(5,6),VOLUME,VOL(5),PTEXT(5,4)     M
C                                                                       M
      COMMON /MINIM/COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),HL(3,3),  M
     2 RSA(50,3),RSB(50,3),SA(50),SB(50),SINAR(50,24),SINARI(24),       M
     3 SNEX(50),TEMP(50),TEXT(20),XL(66,39)                             M
C                                                                       M
      COMMON /CONTRL/NCYCLE,MAXS,KXR(5),KKZ,LIM,LOUT,IPONS,LMAT,LCORD,  M
     2 LCOORD,ICELL,ICENT,ICENTT(5),IRL,IRLL(5),KL(5),KLL,KM(5),KMN,    M
     3 MN(5),MNN,ICYCLE,MAXCON,IFOUR,IPLOT,KLOOP,LINES,IWID,IPREF(5),   M
     4 IPREFF,NHALP(5),NUNIAX(5),NCUBIC(5),NGAUSS(5),NCAUCH(5),NPAC(5)  M
     5 ,CLOR(5),ELOR(5)
C                                                                       M
      COMMON /MATR/MATSIZ,MATORD,SIGD,V(MP),SMM(MP*(MP+1)/2)                   M
C                                                                       M
      COMMON /SCONTS/ICON(4,9),JCON(4,9),LCONI(4,9),LCONJ(4,9),LN(4),   M
     2 LCID(4,9),JCID(4,9),CI(4,9),CJ(4,9),CONA(4,9),DF(4)              M
C                                                                       M
      COMMON /MAXDIM/MNCONF,MNCONT,MMATOR,MNEUTS,MNXMAG,MNATOM,MNMAGR,  M
     2 MNEQPO,MNOVRF,MNMAGP,MNPATS                                      M
C                                                                       M
      COMMON /BFAC/ITF,JTF(50),PRCP(5,6),ITF3,ITF4                      M
C                                                                       M
      COMMON /REFS/HN(5,3),HH(3,ML),ICODE(ML),SJJ(ML),SOMEGA(ML),       M
     2 FOBS(ML),FESD(ML),FNUC(ML),FMAG(ML),THETA(ML),TANN(ML),HW(ML),   M
     3 DERSTO(ML,MX),AKEEP(ML),BKEEP(ML),PKEEP(ML),ICODD(ML)            M
C                                                                       M
      COMMON /SUMS/SW,SUMAW,SUMW,SUMCAL,SUMOBS,SUMDIF,SUMWBS,SOM,       M
     2 SUMFOB,SUMDFF,FNUCDI,FNUCOB,FMAGDI,FMAGOB,SUWFOB,SUWDFF          M
C                                                                       M
      COMMON /LOGIC/FORCEL,FORCED,PUNCH,STAR,TYPE,VERT,YES,ANGCOU              M
C                                                                       M
      COMMON /INOUT/LCA,LPR,LPRE,LT4,LT8,LFOR,LPUN                      M
C                                                                       M
      COMMON /NUMB/N,N1,N2,N3,N4,N6,N7,NN,NN5,NA(5),NPAT,NREF,NSPAR
C                                                                       M
      COMMON /MOVE/AH(5),BH(5),CH(5),B1(5),B2(5),ZERO(5),SCALE(5,2)     M
C                                                                       M
      COMMON /BAYES/NBAYES,LBAY                                         M
C                                                                       M
      COMMON /KEYS/LP(66,39),FLAG(66,39)                                M
C                                                                       M
      COMMON /RELX/RELAXB,RELAXC,RELAXS,RELAXH                          M
C                                                                       M
      COMMON /GEOM/DEGRAD,RADDEG                                        M
C                                                                       M
      COMMON /INPUT/NREAD,NPOINT,IRED,MODE,LINO,IDISML,ICHARS(14),      M
     2 ICARD(80),READIN(50)                                             M
C                                                                       M
      COMMON /BACKD/DB(50)                                              M
C                                                                       M
      COMMON /OPTION/ISF,ISUB,IXTRA,ISW                                 M
C                                                                       M
C                                                                       M
      EQUIVALENCE (HH(2,22),TT(1,1))                                    M
C                                                                       M
      EQUIVALENCE (READIN(1),IREDIN(1))                                 M
C                                                                       M
C     WHEN ANGLES IN 1/100 TH DEGREES,INSERT NEXT STATEMENT             M
C     PI=36000./3.14159265359                                           M
      PI=36000./3.14159265359                                           M
C                                                                       M
      OPEN(3,FILE='tempo',FORM='UNFORMATTED',STATUS='OLD')
      OPEN(16,FILE='lppf')
      OPEN(18,FILE='yco')
      OPEN(22,FILE='marker')
      OPEN(2,FILE='iobs')
      OPEN(5,FILE='profein',STATUS='OLD')
      OPEN(12,FILE='shores')
      OPEN(17,FILE='fourier')
C     DEGREES TO RADIANS AND VICE VERSA                                 M
C                                                                       M
      DEGRAD=3.14159265359/180.0                                        M
      RADDEG=1.0/DEGRAD                                                 M
C     INITIALISE PERIPHERALS                                            M
C     CARD READER                                                       M
      LCA=5                                                             M
C     LINE PRINTER                                                      M
      LPR=16                                                             M
C     CARD PUNCH                                                        M
      LPUN=7                                                            M
C     INPUT FILE FROM PRELIMINARY PROGRAM                               M
      LPRE=3                                                            M
C     SCRATCH FILES                                                     M
      LT4=10                                                            M
      LT8=9                                                             M
C     FILE FOR FOURIER OUTPUT                                           M
      LFOR=17                                                           M
C     FILE FOR BAYESIAN CONSTRAINT INSTRUCTIONS                         M
      LBAY=18                                                           M
C                                                                       M
C                                                                       M
C     SET MAXIMUM VALUES OF ARRAY DIMENSIONS                            M
C     THESE HAVE TO BE RE-SET IF THE PROGRAM IS RE-DIMENSIONED          M
C                                                                       M
C     MAXIMUM NUMBER OF LINEAR AND QUADRATIC CONSTRAINT FUNCTIONS:      M
      MNCONF=4                                                          M
C                                                                       M
C     MAXIMUM NUMBER OF TERMS PER LINEAR OR QUADRATIC CONSTRAINT:       M
      MNCONT=9                                                          M
C                                                                       M
C     MAXIMUM ORDER FOR NORMAL MATRIX                                   M
      MMATOR=MP                                                         M
C                                                                       M
C     MAXIMUM NUMBER OF NEUTRON SCATTERING LENGTHS OR X-RAY DISPERSION TM
      MNEUTS=8                                                          M
C                                                                       M
C     MAXIMUM OF NEUTRON MAGNETIC OR X-RAY SCATTERING FACTORS:          M
      MNXMAG=4                                                          M
C                                                                       M
C     MAXIMUM NUMBER OF ATOMS:                                          M
      MNATOM=50                                                         M
C                                                                       M
C     MAXIMUM NUMBER OF MAGNETIC VECTOR ROTATION MATRICES PER           M
C     EQUIVALENT POSITION:                                              M
      MNMAGR=2                                                          M
C                                                                       M
C     MAXIMUM NUMBER OF EQUIVALENT POSITIONS INCLUDING IDENTITY SYMMETRYM
      MNEQPO=24                                                         M
C                                                                       M
C     MAXIMUM NUMBER OF OVERLAPPING REFLECTIONS AT ONE PROFILE POINT:   M
      MNOVRF=ML                                                         M
C                                                                       M
C     MAXIMUM NUMBER OF POINTS IN A NEUTRON MAGNETIC OR X-RAY           M
C     SCATTERING CURVE                                                  M
      MNMAGP=21                                                         M
C                                                                       M
C     MAXIMUM NUMBER OF LINES PER PAGE                                  M
      LINES=60                                                          M
C            0                                                          M
C                                                                       M
C     WIDTH OF PAGE IN USEFUL CHARACTERS (120 OR 130; SEE               M
C     SUBROUTINE PROFPL)                                                M
      IWID=130                                                          M
C                                                                       M
C                                                                       M
C     MAXIMUM NUMBER OF DIFFRACTION PATTERNS IN A SCAN;                 M
      MNPATS=5                                                          M
      MODE=0
      LINO=1
      CALL ZARRAY
      MATORD=0
10    ICYCLE=0                                                          M
      FORCEL=.FALSE.
      FORCED=.FALSE.                                                    M
      CALL DATAIN                                                       M
      IF(NBAYES)20,30,20                                                M
20    CALL INBAYS                                                       M
30    ICYCLE=ICYCLE+1                                                   M
      IF(ICYCLE.GE.NCYCLE) FORCEL=.TRUE.
      IF(NCYCLE)50,50,40                                                M
40    IF(ICYCLE-NCYCLE)50,50,140                                        M
C     CLEAR ARRAY STORAGE BEFORE EACH CYCLE                             M
50    CALL CLEAR                                                        M
      IF(NCYCLE)100,100,60                                              M
60    IF(MAXCON)80,80,70                                                M
70    CALL CONPRE                                                       M
80    IF(NBAYES)100,100,90                                              M
90    CALL BAYMAT                                                       M
100   CALL ITER                                                         M
      IF(NCYCLE)120,120,110                                             M
110   CALL MATRIX                                                       M
120   CALL OUTPUT                                                       M
130   IF(NCYCLE)180,180,30                                              M
C                                                                       M
C     WERE ANY STRUCTURE FACTOR OUTPUT OPTIONS REQUESTED?               M
C                                                                       M
140   L=MAX0(IFOUR,LOUT,IPONS,IPLOT)                                    M
C                                                                       M
C                                                                       M
C     IF SO GO BACK AND CALCULATE NEW STRUCTURE FACTORS                 M
      IF(NBAYES)150,160,150                                             M
  150 continue
c50   CALL TENS(CELL,TT)                                                M
      REWIND LBAY                                                       M
C----- NEXT LINE WAS SET TO COMMENT ----------------------
C     CALL CHECK                                                        M
160   IF(L)180,180,170                                                  M
170   NCYCLE=0                                                          M
      FORCEL=.TRUE.
      GOTO 50                                                           M
180   STOP                                                              M
      END                                                               M
      FUNCTION FLEES(AA)                                                F
      PARAMETER (ML=500,MP=99,MX=103)
C     **************************************************************    F
C     THIS FUNCTION CAUSES A TRANSFER OF 500 WORDS FROM TAPE INTO THE   F
C     ARRAY DUMPA EACH TIME AN ARRAY HAS BEEN EXHAUSTED BY              F
C     ASSIGNING CONSECUTIVE ELEMENT VALUES TO THE FUNCTION.             F
C     **************************************************************    F
C                                                                       F
      COMMON /REFLN/ARG,DELTA,DLABDA(2),DUMPA(500),EPS,FLOG,FLOG2,      F
     2 OMEGA,PI,SKEW,SLABDA,SLOG,W,XN,YCALC,YOBS                        F
C                                                                       F
      COMMON /PREF/IALFA,IOMEG,IORD1,IORD2,IPREV,ISTAP,ISS,KSTART,LIMO, F
     2 LLL,NCD(50),NKEY,NO,NTEXT,NUM,NTYP(50),MEQ(50),MTYP(50)          F
C                                                                       F
      COMMON /SYMCEL/ALL(3,3),AL(5,3,3),ATEXT(60),B(8),CELL(6),CELLN(5, F
     2 6),F(4,21),FF(4),PH(5,3),RM(5,24,2,9),S(4,21),SM(5,24,3,3),T(5,  F
     3 24,3),TR(5,24),RCELL(6),RCELLN(5,6),VOLUME,VOL(5),PTEXT(5,4)     F
C                                                                       F
      COMMON /MINIM/COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),HL(3,3),  F
     2 RSA(50,3),RSB(50,3),SA(50),SB(50),SINAR(50,24),SINARI(24),       F
     3 SNEX(50),TEMP(50),TEXT(20),XL(66,39)                             F
C                                                                       F
      COMMON /INOUT/LCA,LPR,LPRE,LT4,LT8,LT9,LPUN                       F
C                                                                       F
      LLL=LLL+1                                                         F
      FLEES=DUMPA(LLL)                                                  F
      IF(LLL.NE.500)GOTO 10                                             F
      READ(LPRE,END=20)DUMPA                                            F
      NKEY=NKEY+1                                                       F
      LLL=0                                                             F
10    RETURN                                                            F
20    CALL ERR(2,0)                                                     F
      END                                                               F
      SUBROUTINE PERM(I,J,K)                                            P
      PARAMETER (ML=500,MP=99,MX=103)
C     **************************************************************    P
C     PERMS USEFUL COMBINATIONS OF INTEGERS IN THE RANGE 1 TO 3         P
C     **************************************************************    P
      IF(I-2)10,20,30                                                   P
10    J=2                                                               P
      K=3                                                               P
      RETURN                                                            P
20    J=3                                                               P
      K=1                                                               P
      RETURN                                                            P
30    J=1                                                               P
      K=2                                                               P
      RETURN                                                            P
      END                                                               P
      SUBROUTINE TRCL(CELLN,RCELLN,V,NP)                                T
      PARAMETER (ML=500,MP=99,MX=103)
C     **************************************************************    T
C     TRANSFORMS REAL CELL TO RECIPROCAL OR VICE VERSA                  T
C     INPUT CELL IS IN ARRAY CELL AS LENGTHS AND COSINES                T
C     **************************************************************    T
C                                                                       T
      DIMENSION CELLN(5,6),RCELLN(5,6),SINA(5),V(5)                     T
C                                                                       T
      ABC=1.0                                                           T
      PROD=2.0                                                          T
      V(NP)=-2.0                                                        T
      DO 10 I=1,3                                                       T
      L=I+3                                                             T
      SINA(I)=1.0-CELLN(NP,L)**2                                        T
      V(NP)=V(NP)+SINA(I)                                               T
      SINA(I)=SQRT(SINA(I))                                             T
      PROD=PROD*CELLN(NP,L)                                             T
10    ABC=ABC*CELLN(NP,I)                                               T
      V(NP)=ABC*SQRT(V(NP)+PROD)                                        T
C      V IS CELL VOLUME                                                 T
C     PUT INVERTED CELL INTO RCELL                                      T
      DO 20 I=1,3                                                       T
      CALL PERM(I,J,K)                                                  T
      RCELLN(NP,I)=CELLN(NP,J)*CELLN(NP,K)*SINA(I)/V(NP)                T
      L=I+3                                                             T
20    RCELLN(NP,L)=(CELLN(NP,J+3)*CELLN(NP,K+3)-CELLN(NP,L))/(SINA(J)   T
     2 *SINA(K))                                                        T
      RETURN                                                            T
      END                                                               T
      SUBROUTINE BCONV(RCELLN,AL,NP)                                    B
      PARAMETER (ML=500,MP=99,MX=103)
C     ++++++++++++++++++++++++++++++++++++++++++++++++++++              B
C     CALCULATES COEFFS TO CONVERT BISO TO BIJ                          B
C     ++++++++++++++++++++++++++++++++++++++++++++++++++++              B
C                                                                       B
      DIMENSION RCELLN(5,6),AL(5,3,3)                                   B
C                                                                       B
      DO 30 I=1,3                                                       B
      CALL PERM(I,J,K)                                                  B
      RCELLN(NP,I)=AL(NP,I,I)*0.25                                      B
      IF(J-K)20,30,10                                                   B
10    M=K                                                               B
      K=J                                                               B
      J=M                                                               B
20    RCELLN(NP,7-I)=AL(NP,J,K)*0.125                                   B
30    CONTINUE                                                          B
C     RCELLN NOW CONTAINS THE COEFFICIENTS TO CONVERT BISO TO BIJ       B
      RETURN                                                            B
      END                                                               B
      SUBROUTINE REPCEL(RCELLN,AL,NP)                                   R
      PARAMETER (ML=500,MP=99,MX=103)
C     ******************************************                        R
C     OBTAINS RECIPROCAL CELL FROM METRIC TENSOR                        R
C     ******************************************                        R
C                                                                       R
C                                                                       R
      DIMENSION RCELLN(5,6),AL(5,3,3)                                   R
      DO 10 I=1,3                                                       R
10    RCELLN(NP,I)=SQRT(AL(NP,I,I))                                     R
      DO 40 I=1,3                                                       R
      CALL PERM(I,J,K)                                                  R
      IF(J-K)30,40,20                                                   R
20    M=K                                                               R
      K=J                                                               R
      J=M                                                               R
30    RCELLN(NP,I+3)=0.5*AL(NP,J,K)/(RCELLN(NP,J)*RCELLN(NP,K))         R
40    CONTINUE                                                          R
      RETURN                                                            R
      END                                                               R
      SUBROUTINE SHIFT                                                  S
      PARAMETER (ML=500,MP=99,MX=103)
C     ********************************************************          S
C     PUTS NEW VALUES FROM XL INTO VARIABLES USED BY SUBROUTINES        S
C     ********************************************************          S
C                                                                       S
      COMMON /REFLN/ARG,DELTA,DLABDA(2),DUMPA(500),EPS,FLOG,FLOG2,      S
     2 OMEGA,PI,SKEW,SLABDA,SLOG,W,XN,YCALC,YOBS                        S
C                                                                       S
      COMMON /PREF/IALFA,IOMEG,IORD1,IORD2,IPREV,ISTAP,ISS,KSTART,LIMO, S
     2 LLL,NCD(50),NKEY,NO,NTEXT,NUM,NTYP(50),MEQ(50),MTYP(50)          S
C                                                                       S
      COMMON /SYMCEL/ALL(3,3),AL(5,3,3),ATEXT(60),B(8),CELL(6),CELLN(5, S
     2 6),F(4,21),FF(4),PH(5,3),RM(5,24,2,9),S(4,21),SM(5,24,3,3),T(5,  S
     3 24,3),TR(5,24),RCELL(6),RCELLN(5,6),VOLUME,VOL(5),PTEXT(5,4)     S
C                                                                       S
      COMMON /MINIM/COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),HL(3,3),  S
     2 RSA(50,3),RSB(50,3),SA(50),SB(50),SINAR(50,24),SINARI(24),       S
     3 SNEX(50),TEMP(50),TEXT(20),XL(66,39)                             S
C                                                                       S
      COMMON /MOVE/AH(5),BH(5),CH(5),B1(5),B2(5),ZERO(5),SCALE(5,2)     S
C                                                                       S
      COMMON /NUMB/N,N1,N2,N3,N4,N6,N7,NN,NN5,NA(5),NPAT,NREF,NSPAR
C                                                                       S
      COMMON /CONTRL/NCYCLE,MAXS,KXR(5),KKZ,LIM,LOUT,IPONS,LMAT,LCORD,  S
     2 LCOORD,ICELL,ICENT,ICENTT(5),IRL,IRLL(5),KL(5),KLL,KM(5),KMN,    S
     3 MN(5),MNN,ICYCLE,MAXCON,IFOUR,IPLOT,KLOOP,LINES,IWID,IPREF(5),   S
     4 IPREFF,NHALP(5),NUNIAX(5),NCUBIC(5),NGAUSS(5),NCAUCH(5),NPAC(5)  S
     5 ,CLOR(5),ELOR(5)
C                                                                       S
      DO 40 NP=1,NPAT                                                   S
      ZERO(NP)=XL(N6,NP)                                                S
      N1=N+NP                                                           S
      N2=N7+NP                                                          S
      N3=N6+NP                                                          S
      AH(NP)=XL(N3,1)                                                   S
      BH(NP)=XL(N3,2)                                                   S
      CH(NP)=XL(N3,3)                                                   S
      B1(NP)=XL(N1,1)                                                   S
      B2(NP)=XL(N1,2)                                                   S
      SCALE(NP,1)=1.0/XL(N1,3)                                          S
      SCALE(NP,2)=1.0/XL(N1,4)                                          S
      DO 30 I=1,3                                                       S
      AL(NP,I,I)=XL(N2,I)                                               S
      CALL PERM(I,J,K)                                                  S
      IF(J-K)10,20,20                                                   S
10    AL(NP,J,K)=XL(N2,I+3)                                             S
      GOTO 30                                                           S
20    AL(NP,K,J)=XL(N2,I+3)                                             S
30    CONTINUE                                                          S
40    CONTINUE                                                          S
      RETURN                                                            S
      END                                                               S
      SUBROUTINE PARS(I,NPAR)                                           P
      PARAMETER (ML=500,MP=99,MX=103)
C     *******************************************************           P
C     DETERMINES THE NUMBER OF MEANINGFUL PARAMETERS IN A LINE          P
C     OF THE PARAMETER ARRAY XL.  XL(I,J),J=1,NPAR                      P
C     *******************************************************           P
C                                                                       P
      COMMON /BFAC/ITF,JTF(50),PRCP(5,6),ITF3,ITF4                      P
C                                                                       P
      COMMON /NUMB/N,N1,N2,N3,N4,N6,N7,NN,NN5,NA(5),NPAT,NREF,NSPAR
C                                                                       P
      IF(I-N)10,10,60                                                   P
10    IF(JTF(I))30,20,30                                                P
20    NPAR=8                                                            P
      RETURN                                                            P
30    NPAR=14                                                           P
      IF(JTF(I).EQ.3) NPAR=24
      IF(JTF(I).EQ.4) NPAR=39
      RETURN                                                            P
40    NPAR=5                                                            P
      RETURN                                                            P
50    NPAR=10                                                           P
      RETURN                                                            P
60    IF(I-N6)50,70,80                                                  P
70    NPAR=NPAT                                                         P
      RETURN                                                            P
80    IF(I-N7)90,90,100                                                 P
90    NPAR=3                                                            P
      RETURN                                                            P
100   NPAR=6                                                            P
      RETURN                                                            P
      END                                                               P
      SUBROUTINE XCOPY(KK,CHAR,DUM)                                     X
      PARAMETER (ML=500,MP=99,MX=103)
C     ***********************************************************       X
C     LOADS CHARACTER CHAR INTO LOCATION (KK+1) OF ARRAY DUM            X
C     ***********************************************************       X
C                                                                       X
      DIMENSION DUM(50)                                                 X
      KK=KK+1                                                           X
      DUM(KK)=CHAR                                                      X
      RETURN                                                            X
      END                                                               X
      SUBROUTINE LOC(I,J,NU,MATORD)                                     L
      PARAMETER (ML=500,MP=99,MX=103)
C     ********************************************************          L
C     OBTAINS THE LOCATION, NU, IN UPPER TRIANGLE OF NORMAL             L
C     MATRIX FOR ELEMENT (I,J) WHERE J.GE.I                             L
C     ********************************************************          L
C     TEST FOR J.GE.I AND SWAP IF NECESSARY                             L
      IF(J.GE.I)GOTO 10                                                 L
      K=J                                                               L
      J=I                                                               L
      I=K                                                               L
10    L=I-1                                                             L
      NU=MOD(L,2)                                                       L
      IF(NU)30,20,30                                                    L
20    NU=-(L*L+L)/2                                                     L
      GOTO 40                                                           L
30    NU=-L*(L/2+1)                                                     L
40    NU=NU+L*(MATORD+1)                                                L
      NU=NU+J-I+1                                                       L
      RETURN                                                            L
      END                                                               L
      SUBROUTINE CONVER(B,COMP,NP)                                      C
      PARAMETER (ML=500,MP=99,MX=103)
C     *********************************************************         C
C     CALCULATES ANISOTROPIC COMPONENTS FROM ISOTROPIC B-FACTOR         C
c     *********************************************************         C
C                                                                       C
      DIMENSION COMP(6)                                                 C
C                                                                       C
      COMMON /BFAC/ITF,JTF(50),PRCP(5,6),ITF3,ITF4                      C
      DO 10 I=1,6                                                       C
      COMP(I)=B*PRCP(NP,I)                                              C
10    CONTINUE                                                          C
      RETURN                                                            C
      END                                                               C
      SUBROUTINE ERR(NKEY,IMAX)
      PARAMETER (ML=500,MP=99,MX=103)
C     *****************************************************************
C     TIDY UP AFTER A MISTAKE IN INPUT DATA. THE ERROR CODE
C     NUMBERS ARE OBVIOUS FROM THE COMPUTED GO TO STATEMENT
C     *****************************************************************
      COMMON /INOUT/ LCA,LPR,LPRE,LT4,LT8,LFOR,LPUN
C
      PRINT*,' ERROR, SEE LPPF!!!!!!!!!!'
      WRITE(LPR,1000)
 1000 FORMAT(3(10(5X,5HERROR)/))
      GO TO (1,1,3,4,5,6,7,8,9,10,11,12,13,14,15,16,
     1  17,18,19,20,21,22,23,24,25,26),NKEY
    1 WRITE(LPR,910) LPRE
  910 FORMAT(21H0END OF FILE ON UNIT ,I2)
      WRITE(LPR,920)
  920 FORMAT(37H PROBABLE CAUSE INCORRECT FILE LOADED)
      STOP
    3 WRITE(LPR,30)
   30 FORMAT(38H0TOO MANY SYMMETRY OPERATORS SUPPLIED )
      GO TO 999
    4 WRITE(LPR,40)
   40 FORMAT(64H0TOO MANY NEUTRON MAGNETIC OR X-RAY SCATTERING FACTORS S
     1UPPLIED )
      GO TO 999
    5 WRITE(LPR,50)
   50 FORMAT(71H0TOO MANY NEUTRON SCATTERING LENGTHS OR X-RAY DISPERSION
     1 TERMS SUPPLIED)
       GO TO 999
    6 WRITE(LPR,60)
   60 FORMAT(24H0TOO MANY ATOMS SUPPLIED)
      GO TO 999
    7 WRITE(LPR,70)
   70 FORMAT(60H0TOO MANY LINEAR AND QUADRATIC CONSTRAINT FUNCTIONS SUPP
     1LIED)
      GO TO 999
    8 WRITE(LPR,80)
   80 FORMAT(37H0ALLOWED NORMAL MATRIX ORDER EXCEEDED)
      GO TO 999
    9 WRITE(LPR,90)
   90 FORMAT(61H0TOO MANY TERMS SUPPLIED FOR A LINEAR OR QUADRATIC CONST
     1RAINT)
      GO TO 999
   10 WRITE(LPR,100)
  100 FORMAT(51H0MAXIMUM NUMBER OF OVERLAPPING REFLECTIONS EXCEEDED)
      GO TO 999
   11 WRITE(LPR,110)
  110 FORMAT(52H0TOO MANY MAGNETIC VECTOR ROTATION MATRICES SUPPLIED)
      GO TO 999
   12 WRITE(LPR,120) IMAX
  120 FORMAT(47H0MORE THAN ONE FIXED TERM IN STRICT CONSTRAINT ,I3)
      GO TO 1001
   13 WRITE(LPR,130) IMAX
  130 FORMAT(29H0NO FIXED TERM IN CONSTRAINT ,I3)
      GO TO 1001
   14 WRITE(LPR,140)
  140 FORMAT(30H0SINGULAR LEAST SQUARES MATRIX)
      GO TO 1001
   15 WRITE(LPR,150) IMAX
  150 FORMAT(59H0MATRIX INVERSION ROUTINE FAILED WHEN DEALING WITH ELEME
     1NT ,I3)
      GO TO 14
   16 WRITE(LPR,160) IMAX
  160 FORMAT(42H0UNABLE TO SOLVE STRICT CONSTRAINT NUMBER ,I2,
     1  26H FOR NON-REFINED PARAMETER)
      GO TO 1001
   17 WRITE(LPR,170)
  170 FORMAT(58H0THE ABOVE BAYESIAN CONSTRAINT DIRECTIVE IS NOT RECOGNIS
     1ED)
      GO TO 1001
   18 WRITE(LPR,180)
  180 FORMAT(68H0END OF CARD INPUT STREAM DETECTED WHEN READING BAYESIAN
     1 CONSTRAINTS)
      GO TO 1001
   19 WRITE(LPR,190)
  190 FORMAT(70H0NO BAYESIAN CONSTRAINT DIRECTIVES ENCOUNTERED BEFORE ST
     1OP OR END CARD)
      GO TO 1001
   20 WRITE(LPR,200)
  200 FORMAT(50H0ERROR(S) IN INSTRUCTIONS FOR BAYESIAN CONSTRAINTS)
      GO TO 1001
   21 WRITE(LPR,2100)
 2100 FORMAT(26H0INPUT DATA UNINTELLIGIBLE)
      GO TO 1001
   22 WRITE(LPR,2200)
 2200 FORMAT(74H0TOO MANY POINTS SUPPLIED FOR A NEUTRON MAGNETIC OR X-RA
     1Y SCATTERING CURVE)
      GO TO 999
   23 WRITE(LPR,2300) IMAX
 2300 FORMAT(28H0THE MAXIMUM BUFFER SIZE IS ,I3)
      GO TO 1001
   24 WRITE(LPR,2400)
 2400 FORMAT(78H0THE NUMBER OF VARIABLES IS NOT CONSISTENT WITH THE NUMB
     1ER OF UNIQUE CODEWORDS)
      GO TO 1001
   25 WRITE(LPR,2500)
 2500 FORMAT(19H0TOO MANY PATTERNS!)
  999 WRITE(LPR,101) IMAX
  101 FORMAT(31H THE MAXIMUM NUMBER ALLOWED IS ,I5)
      GOTO 1001
26    WRITE(LPR,102) IMAX
102   FORMAT(' EOF TAPE9 IN ITER ,IOSTAT = ',I3)
 1001 STOP
      END
      FUNCTION CIGN(A,B)                                                C
C     +++++++++++++++++++++++++++++++++                                 C
C     TAKES SIGN OF ZERO AS POSITIVE                                    C
C     +++++++++++++++++++++++++++++++++                                 C
      IF(B)10,20,10                                                     C
10    C=B                                                               C
      GOTO 30                                                           C
20    C=1.0                                                             C
30    CIGN=SIGN(A,C)                                                    C
      RETURN                                                            C
      END                                                               C
      SUBROUTINE CLEAR                                                  C
      PARAMETER (ML=500,MP=99,MX=103)
C     +++++++++++++++++++++++++++++++++++++++++++++++++++               C
C     CLEARS MATRIX,VECTOR AND SUMMATION VARIABLES BEFORE               C
C     LEAST SQUARES CYCLE                                               C
C     +++++++++++++++++++++++++++++++++++++++++++++++++++               C
C                                                                       C
      COMMON /SUMS/SW,SUMAW,SUMW,SUMCAL,SUMOBS,SUMDIF,SUMWBS,SOM,       C
     2 SUMFOB,SUMDFF,FNUCDI,FNUCOB,FMAGDI,FMAGOB,SUWFOB,SUWDFF          C
C                                                                       C
      COMMON /MATR/MATSIZ,MATORD,SIGD,V(MP),SMM(MP*(MP+1)/2)                   C
C                                                                       C
      COMMON /CONTRL/NCYCLE,MAXS,KXR(5),KKZ,LIM,LOUT,IPONS,LMAT,LCORD,  C
     2 LCOORD,ICELL,ICENT,ICENTT(5),IRL,IRLL(5),KL(5),KLL,KM(5),KMN,    C
     3 MN(5),MNN,ICYCLE,MAXCON,IFOUR,IPLOT,KLOOP,LINES,IWID,IPREF(5),   C
     4 IPREFF,NHALP(5),NUNIAX(5),NCUBIC(5),NGAUSS(5),NCAUCH(5),NPAC(5)  C
     5 ,CLOR(5),ELOR(5)
C                                                                       C
      SW=0.0                                                            C
      SUMAW=0.0                                                         C
      SUMW=0.0                                                          C
      SUMCAL=0.0                                                        C
      SUMOBS=0.0                                                        C
      SUMDIF=0.0                                                        C
      SUMWBS=0.0                                                        C
      SOM=0.0                                                           C
      SUMFOB=0.0                                                        C
      SUMDFF=0.0                                                        C
      SUWFOB=0.0
      SUWDFF=0.0
      FNUCDI=0.0                                                        C
      FNUCOB=0.0                                                        C
      FMAGDI=0.0                                                        C
      FMAGOB=0.0                                                        C
      IF(NCYCLE.EQ.0)RETURN                                             C
C                                                                       C
      DO 10 I=1,MATORD                                                  C
10    V(I)=0.0                                                          C
      DO 20 I=1,MATSIZ                                                  C
20    SMM(I)=0.0                                                        C
      RETURN                                                            C
      END                                                               C
      SUBROUTINE FREEIN                                                 F
      PARAMETER (ML=500,MP=99,MX=103)
C     ***********************************************************       F
C      FREE FORMAT INPUT ROUTINE FOR INTEGER AND DECIMAL VALUES         F
C     ***********************************************************       F
C                                                                       F
      DIMENSION IREDIN(50)                                              F
C                                                                       F
C                                                                       F
      COMMON /INPUT/NREAD,NPOINT,IRED,MODE,LINO,IDISML,ICHARS(14),      F
     2 ICARD(80),READIN(50)                                             F
C                                                                       F
      COMMON /INOUT/LCA,LPR,LPRE,LT4,LT8,LFOR,LPUN                      F
C                                                                       F
C                                                                       F
      EQUIVALENCE (READIN(1),IREDIN(1))                                 F
C                                                                       F
C     NREAD IS  THE NUMBER OF VALUES TO BE PICKED UP FROM THE INPUT STREF
C                                                                       F
C     NPOINT IS THE POSITION OF THE POINTER IN ARRAY ICARD              F
C                                                                       F
C     MODE  = 0 ALL INPUT VALUES ON THE SAME LINE                       F
C           = 1 INPUT VALUES MAY BE SPREAD OVER MORE THAN ONE LINE      F
C               THE PROGRAM ASSUMES THAT VALUES NOT PICKED UP ARE ZERO WF
C               BUT REQUIRES ALL VALUES TO BE PUNCHED WHEN MODE= 1      F
C                                                                       F
C     LINO IS THE LINE NUMBER IN THE INPUT DATA CURRENTLY BEING DEALT WIF
C          (STARTS FROM ZERO)                                           F
C                                                                       F
C     IRED IS (THE NUMBER OF VALUES READ PREVIOUSLY +1)                 F
C          (I.E. THE SUBSCRIPT OF ARRAY READIN)                         F
C                                                                       F
C     IDISML= -1 READ DECIMAL VALUES                                    F
C              1 READ INTEGER VALUES                                    F
C                                                                       F
C                                                                       F
C     INPUT BUFFER DIMENSION                                            F
C                                                                       F
      NBUFF=50                                                          F
C                                                                       F
C     CONSTANT VALUES USED BY CHARACTER COMPARISON ROUTINE              F
C                                                                       F
      IBEG=1                                                            F
      IC=1                                                              F
      IE=1                                                              F
C                                                                       F
C     SET VALUES TO BE READ TO ZERO                                     F
C                                                                       F
      L=IRED+NREAD-1                                                    F
      IF(IDISML)10,30,30                                                F
10    DO 20 I=IRED,L                                                    F
      READIN(I)=0.0                                                     F
20    CONTINUE                                                          F
      GOTO 50                                                           F
30    DO 40 I=IRED,L                                                    F
      IREDIN(I)=0                                                       F
40    CONTINUE                                                          F
C                                                                       F
C     LOOP THROUGH INPUT VALUES                                         F
C                                                                       F
50    DO 370 I=1,NREAD                                                  F
C                                                                       F
C     KEY FOR BEGINNING OF VALUE                                        F
C                                                                       F
      ISTART=-1                                                         F
C                                                                       F
C     KEY FOR SIGN OF VALUE                                             F
C                                                                       F
60    ISYN=1                                                            F
C                                                                       F
C     KEYS FOR DECIMAL PART OF VALUE                                    F
C                                                                       F
      DPT=0.1                                                           F
      IDPT=-1                                                           F
C                                                                       F
      L=NPOINT                                                          F
      IF(NPOINT-80)70,70,250                                            F
C                                                                       F
C     CHECK THAT INPUT BUFFER IS NOT OVERFLOWED                         F
C                                                                       F
70    IF(IRED-NBUFF)80,80,420                                           F
C                                                                       F
C     LOOP ALONG CARD                                                   F
C                                                                       F
80    DO 240 J=L,80                                                     F
C                                                                       F
C     SPACE CHARACTER?                                                  F
C                                                                       F
      IA=1                                                              F
      IB=ICHARS(14)                                                     F
      ID=ICARD(J)                                                       F
c     CALL COMP(IA,IB,IC,ID,IE)                                         F
      call comp(ia,ichars(14),ic,icard(j),ie)
      NPOINT=NPOINT+1                                                   F
      IF(IA-1)100,90,100                                                F
C                                                                       F
C     IF SPACE JUMP TO 20; IF NOT TO 30                                 F
C                                                                       F
90    IF(ISTART)240,330,330                                             F
C                                                                       F
C     JUMP TO 270 IF END OF VALUE                                       F
C                                                                       F
C     SET KEY FOR A VALUE IN ACCUMULATION                               F
C                                                                       F
100   ISTART=ISTART+1                                                   F
      DO 110 K=1,13                                                     F
C                                                                       F
C     CHECK ICARD(J) AGAINST CHARACTER LIST                             F
C                                                                       F
      IA=1                                                              F
      IB=ICHARS(K)                                                      F
      CALL COMP(IA,IB,IC,ID,IE)                                         F
      IF(IA-1)110,120,110                                               F
C                                                                       F
C     JUMP TO 50 IF CHARACTER IDENTIFIED                                F
C                                                                       F
110   CONTINUE                                                          F
C                                                                       F
C     CHARACTER NOT IDENTIFIED                                          F
C                                                                       F
      GOTO 390                                                          F
C                                                                       F
C     IS CHARACTER AN INTEGER?                                          F
C                                                                       F
120   IF(K-11)190,130,160                                               F
C                                                                       F
C     JUMP TO 120 IF INTEGER; 60 IF DECIMAL POINT; 90 IF SIGN.          F
C                                                                       F
130   IF(IDISML)140,140,400                                             F
C                                                                       F
C     JUMP TO 900 IF SPURIOUS DECIMAL POINT IN INTEGER VALUE            F
C                                                                       F
140   IF(IDPT)150,400,400                                               F
C                                                                       F
C     SET KEY FOR DECIMAL POINT FOUND                                   F
C                                                                       F
150   IDPT=1                                                            F
      GOTO 240                                                          F
C                                                                       F
C     CHECK FOR SPURIOUS SIGN                                           F
C                                                                       F
160   IF(ISTART)170,170,400                                             F
C                                                                       F
C     JUMP TO 900 IF SPURIOUS                                           F
C                                                                       F
170   IF(K-12)180,180,240                                               F
C                                                                       F
C     NEGATIVE SIGN                                                     F
C                                                                       F
180   ISYN=-1                                                           F
      GOTO 240                                                          F
C                                                                       F
C     INTEGER OR DECIMAL INPUT?                                         F
C                                                                       F
190   IF(IDISML)200,230,230                                             F
C                                                                       F
C     JUMP TO 130 IF DECIMAL; 160 IF INTEGER                            F
C                                                                       F
C     DECIMAL POINT FOUND?                                              F
C                                                                       F
200   IF(IDPT)210,220,220                                               F
C                                                                       F
C     JUMP TO 140 IF NOT YET FOUND                                      F
C                                                                       F
210   READIN(IRED)=READIN(IRED)*10.0+FLOAT(K-1)                         F
      GOTO 240                                                          F
C                                                                       F
C     DECIMAL POINT PREVIOUSLY FOUND IN THIS VALUE                      F
C                                                                       F
220   READIN(IRED)=READIN(IRED)+FLOAT(K-1)*DPT                          F
      DPT=DPT/10.0                                                      F
      GOTO 240                                                          F
C                                                                       F
C     INTEGER VALUE                                                     F
C                                                                       F
230   IREDIN(IRED)=IREDIN(IRED)*10+K-1                                  F
240   CONTINUE                                                          F
C                                                                       F
C     END OF LINE REACHED. WAS A VALUE IN PROGRESS?                     F
C                                                                       F
250   IF(ISTART)300,260,260                                             F
260   IF(IDISML)270,280,280                                             F
C                                                                       F
C     JUMP TO 220 IF DECIMAL VALUE; 230 IF INTEGER                      F
C                                                                       F
270   READIN(IRED)=FLOAT(ISYN)*READIN(IRED)                             F
      GOTO 290                                                          F
280   IREDIN(IRED)=IREDIN(IRED)*ISYN                                    F
290   IRED=IRED+1                                                       F
300   IF(IRED-NREAD)310,310,370                                         F
C                                                                       F
C     JUMP TO 250 IF MORE VALUES TO BE READ                             F
C                                                                       F
310   IF(MODE)380,380,320                                               F
C                                                                       F
C     JUMP TO 999 IF VALUES NOT PICKED UP ARE ASSUMED ZERO              F
C                                                                       F
320   CALL INITT(IBEG)                                                  F
      IF(ISTART)60,370,370                                              F
C                                                                       F
C     JUMP TO 15 IF VALUE NOT TERMINATED BY END OF CARD; OTHERWISE 300  F
C                                                                       F
C     END OF VALUE SIGNALLED BY SPACE CHARACTER                         F
C                                                                       F
330   IF(IDISML)340,350,350                                             F
340   READIN(IRED)=READIN(IRED)*FLOAT(ISYN)                             F
      GOTO 360                                                          F
350   IREDIN(IRED)=IREDIN(IRED)*ISYN                                    F
360   IRED=IRED+1                                                       F
C                                                                       F
C     END OF LOOP ON INPUT VALUES                                       F
C                                                                       F
370   CONTINUE                                                          F
380   RETURN                                                            F
C                                                                       F
C     ERROR TRAPS                                                       F
C                                                                       F
C     UNIDENTIFIED CHARACTER                                            F
C                                                                       F
390   IRUB=ICARD(NPOINT-1)                                              F
      GOTO 410                                                          F
C                                                                       F
C     SPURIOUS SIGN OR DECIMAL POINT                                    F
C                                                                       F
400   IRUB=ICHARS(K)                                                    F
410   WRITE(LPR,440)IRUB,NPOINT,LINO                                    F
      WRITE(LPR,450)(ICARD(I),I=1,80)                                   F
      CALL ERR(21,0)                                                    F
C                                                                       F
C     INPUT BUFFER OVERFLOWED                                           F
C                                                                       F
420   WRITE(LPR,460)LINO                                                F
      CALL ERR(23,NBUFF)                                                F
C                                                                       F
430   FORMAT(80A1)                                                      F
440   FORMAT(11H0SPURIOUS ',A1,18H' AT ABOUT COLUMN ,I3,9H ON LINE ,I3, F
     2 14H OF INPUT FILE)                                               F
450   FORMAT(29H0THE LINE IN ERROR CONTAINS: ,80A1)                     F
460   FORMAT(43H0INPUT BUFFER OVERFLOWED WHEN READING LINE ,I3)         F
      END                                                               F
      SUBROUTINE INITT(IBEG)                                            I
      PARAMETER (ML=500,MP=99,MX=103)
C     *************************************************************     I
C     READS CARDS FROM THE INPUT STREAM AS CHARACTER STRINGS,           I
C     AND RESTS THE LINE NUMBER AND POINTER FOR SUBROUTINE              I
C     FREEIN TO OPERATE                                                 I
C     *************************************************************     I
C                                                                       I
      COMMON /INPUT/NREAD,NPOINT,IRED,MODE,LINO,IDISML,ICHARS(14),      I
     2 ICARD(80),READIN(50)                                             I
C                                                                       I
      COMMON /INOUT/LCA,LPR,LPRE,LT4,LT8,LFOR,LPUN                      I
C                                                                       I
      READ(LCA,10)(ICARD(I),I=IBEG,80)                                  I
      LINO=LINO+1                                                       I
      NPOINT=IBEG                                                       I
      RETURN                                                            I
C                                                                       I
10    FORMAT(BZ,80A1)                                                      I
      END                                                               I
      SUBROUTINE COMP(I,IA,J,IB,K)
      PARAMETER (ML=500,MP=99,MX=103)
c     DIMENSION IA(1),IB(1)
      I1=1
      IC=0
      K1=J
      K2=K
      DO 10 II=1,I1
        IF(IA.NE.IB) GOTO 20
      IC=IC+1
      K1=K1+1
      K2=K2+1
   10 CONTINUE
   20 I=IC
      RETURN
      END
      BLOCK DATA PPP
C
      COMMON /INPUT/ NREAD,NPOINT,IRED,MODE,LINO,IDISML,
     &  ICHARS(14),ICARD(80),READIN(50)
C
      DATA(ICHARS(I),I=1,8)/"0", "1", "2", "3", "4", "5", "6", "7"/
      DATA(ICHARS(I),I=9,14)/"8", "9", ".", "-", "+", " "/
      END
      SUBROUTINE ZARRAY
      PARAMETER (ML=500,MP=99,MX=103)
C     ZEROISES ARRAYS; NECESSARY FOR THE ICL2980 AT OXFORD
C
      LOGICAL FORCEL,FORCED,PUNCH,STAR,TYPE,VERT,YES,ANGCOU                    M
C                                                                       M
C                                                                       M
      DIMENSION TT(3,3)                                                 M
C                                                                       M
      DIMENSION IREDIN(50)                                              M
C                                                                       M
C                                                                       M
      COMMON /REFLN/ARG,DELTA,DLABDA(2),DUMPA(500),EPS,FLOG,FLOG2,      M
     2 OMEGA,PI,SKEW,SLABDA,SLOG,W,XN,YCALC,YOBS                        M
C                                                                       M
      COMMON /PREF/IALFA,IOMEG,IORD1,IORD2,IPREV,ISTAP,ISS,KSTART,LIMO, M
     2 LLL,NCD(50),NKEY,NO,NTEXT,NUM,NTYP(50),MEQ(50),MTYP(50)          M
C                                                                       M
      COMMON /SYMCEL/ALL(3,3),AL(5,3,3),ATEXT(60),B(8),CELL(6),CELLN(5, M
     2 6),F(4,21),FF(4),PH(5,3),RM(5,24,2,9),S(4,21),SM(5,24,3,3),T(5,  M
     3 24,3),TR(5,24),RCELL(6),RCELLN(5,6),VOLUME,VOL(5),PTEXT(5,4)     M
C                                                                       M
      COMMON /MINIM/COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),HL(3,3),  M
     2 RSA(50,3),RSB(50,3),SA(50),SB(50),SINAR(50,24),SINARI(24),       M
     3 SNEX(50),TEMP(50),TEXT(20),XL(66,39)                             M
C                                                                       M
      COMMON /CONTRL/NCYCLE,MAXS,KXR(5),KKZ,LIM,LOUT,IPONS,LMAT,LCORD,  M
     2 LCOORD,ICELL,ICENT,ICENTT(5),IRL,IRLL(5),KL(5),KLL,KM(5),KMN,    M
     3 MN(5),MNN,ICYCLE,MAXCON,IFOUR,IPLOT,KLOOP,LINES,IWID,IPREF(5),   M
     4 IPREFF,NHALP(5),NUNIAX(5),NCUBIC(5),NGAUSS(5),NCAUCH(5),NPAC(5)  M
     5 ,CLOR(5),ELOR(5)
C                                                                       M
      COMMON /MATR/MATSIZ,MATORD,SIGD,V(MP),SMM(MP*(MP+1)/2)                   M
C                                                                       M
      COMMON /SCONTS/ICON(4,9),JCON(4,9),LCONI(4,9),LCONJ(4,9),LN(4),   M
     2 LCID(4,9),JCID(4,9),CI(4,9),CJ(4,9),CONA(4,9),DF(4)              M
C                                                                       M
      COMMON /MAXDIM/MNCONF,MNCONT,MMATOR,MNEUTS,MNXMAG,MNATOM,MNMAGR,  M
     2 MNEQPO,MNOVRF,MNMAGP,MNPATS                                      M
C                                                                       M
      COMMON /BFAC/ITF,JTF(50),PRCP(5,6),ITF3,ITF4                      M
C                                                                       M
      COMMON /REFS/HN(5,3),HH(3,ML),ICODE(ML),SJJ(ML),SOMEGA(ML),       M
     2 FOBS(ML),FESD(ML),FNUC(ML),FMAG(ML),THETA(ML),TANN(ML),HW(ML),   M
     3 DERSTO(ML,MX),AKEEP(ML),BKEEP(ML),PKEEP(ML),ICODD(ML)            M
C                                                                       M
      COMMON /SUMS/SW,SUMAW,SUMW,SUMCAL,SUMOBS,SUMDIF,SUMWBS,SOM,       M
     2 SUMFOB,SUMDFF,FNUCDI,FNUCOB,FMAGDI,FMAGOB,SUWFOB,SUWDFF          C
C                                                                       M
      COMMON /LOGIC/FORCEL,FORCED,PUNCH,STAR,TYPE,VERT,YES,ANGCOU              M
C                                                                       M
      COMMON /INOUT/LCA,LPR,LPRE,LT4,LT8,LFOR,LPUN                      M
C                                                                       M
      COMMON /NUMB/N,N1,N2,N3,N4,N6,N7,NN,NN5,NA(5),NPAT,NREF,NSPAR
C                                                                       M
      COMMON /MOVE/AH(5),BH(5),CH(5),B1(5),B2(5),ZERO(5),SCALE(5,2)     M
C                                                                       M
      COMMON /BAYES/NBAYES,LBAY                                         M
C                                                                       M
      COMMON /KEYS/LP(66,39),FLAG(66,39)                                M
C                                                                       M
      COMMON /RELX/RELAXB,RELAXC,RELAXS,RELAXH                          M
C                                                                       M
      COMMON /GEOM/DEGRAD,RADDEG                                        M
C                                                                       M
      COMMON /INPUT/NREAD,NPOINT,IRED,MODE,LINO,IDISML,ICHARS(14),      M
     2 ICARD(80),READIN(50)                                             M
C                                                                       M
      COMMON /BACKD/DB(50)                                              M
C                                                                       M
      COMMON /OPTION/ISF,ISUB,IXTRA,ISW                                 M
C     CELLS
      DO 20 I=1,6
      RCELL(I)=0.0
      CELL(I)= 0.0
        DO 10 NP=1,MNPATS
        CELLN(NP,I)= 0.0
        RCELLN(NP,I)=0.0
   10 CONTINUE
   20 CONTINUE
C
C     ML REFLECTIONS PER PROFILE POINT
      DO 40 I=1,ML
      ICODE(I)=0
      ICODD(I)=0
      SJJ(I)=0.0
      SOMEGA(I)=0.0
      FOBS(I)=0.0
      FESD(I)=0.0
      FNUC(I)=0.0
      FMAG(I)=0.0
      THETA(I)=0.0
      TANN(I)=0.0
      HW(I)=0.0
      AKEEP(I)=0.0
      BKEEP(I)=0.0
      PKEEP(I)=0.0
        DO 30 J=1,MX
   30 DERSTO(I,J)=0.0
   40 CONTINUE
C
C     OPTIONS
      DO 50 NP=1,MNPATS
      KXR(NP)=0
      ICENTT(NP)=0
      IRLL(NP)=0
      KL(NP)=0
      KM(NP)=0
      MN(NP)=0
      IPREF(NP)=0
      NHALP(NP)=0
      NUNIAX(NP)=0
      NCUBIC(NP)=0
      NGAUSS(NP)=0
      NCAUCH(NP)=0
      NPAC(NP)  =0
   50 CONTINUE
C
C*********************** FUER MNPAT=5 IST N4=66
      N4=66
      DO 60 I=1,N4
      DO 60 J=1,39
      LP(I,J)=0
      FLAG(I,J)=0.0
      XL(I,J)  =0.0
   60 CONTINUE
C
      DO 70 I=1,50
      JTF(I)=0
      DB(I)=0.0
      NTYP(I)=0
      MEQ(I) =0
      MTYP(I)=0
      NCD(I) =1
   70 CONTINUE
C
      DATA V/MP*0./
      PUNCH=.FALSE.
      TYPE=.FALSE.
      VERT=.FALSE.
      STAR=.FALSE.
C
      IORD1=1
      IORD2=1
      IPREFF=0
      IPREV=0
      NBAYES=0
      NPAT=1
      ISS=0
      ICELL=0
      LINO=1
      RETURN
      END
      SUBROUTINE INBAYS
      PARAMETER (ML=500,MP=99,MX=103)
      RETURN
      END
      SUBROUTINE BAYMAT
      PARAMETER (ML=500,MP=99,MX=103)
      RETURN
      END
      SUBROUTINE MATRIX                                                 M
      PARAMETER (ML=500,MP=99,MX=103)
C    *******************************************************************M
C     THIS SUBROUTINE ORGANISES SOLUTION OF THE NORMAL EQUATIONS AND    M
C     CALCULATES STANDARD DEVIATIONS OF THE REFINED PARAMETERS          M
C    *******************************************************************M
C                                                                       M
      LOGICAL FORCEL,FORCED,PUNCH,STAR,TYPE,VERT,YES,ANGCOU                    M
C                                                                       M
C     REUSE SPACE IN ARRAY DERSTO                                       M
C     TAKE CARE WITH THE  EQUIVALENCE WHEN REDIMENSIONING  THE PROGRAM  M
C     AT PRESENT THE ARRAY STAND OCCUPIES DERSTO(1,1) TO DERSTO(2,10)   M
C     INCLUSIVE                                                         M
C                                                                       M
C                                                                       M
      DIMENSION STAND(50)                                               M
C                                                                       M
C                                                                       M
      COMMON /REFLN/ARG,DELTA,DLABDA(2),DUMPA(500),EPS,FLOG,FLOG2,      M
     2 OMEGA,PI,SKEW,SLABDA,SLOG,W,XN,YCALC,YOBS                        M
C                                                                       M
      COMMON /PREF/IALFA,IOMEG,IORD1,IORD2,IPREV,ISTAP,ISS,KSTART,LIMO, M
     2 LLL,NCD(50),NKEY,NO,NTEXT,NUM,NTYP(50),MEQ(50),MTYP(50)          M
C                                                                       M
      COMMON /SYMCEL/ALL(3,3),AL(5,3,3),ATEXT(60),B(8),CELL(6),CELLN(5, M
     2 6),F(4,21),FF(4),PH(5,3),RM(5,24,2,9),S(4,21),SM(5,24,3,3),T(5,  M
     3 24,3),TR(5,24),RCELL(6),RCELLN(5,6),VOLUME,VOL(5),PTEXT(5,4)     M
C                                                                       M
      COMMON /MINIM/COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),HL(3,3),  M
     2 RSA(50,3),RSB(50,3),SA(50),SB(50),SINAR(50,24),SINARI(24),       M
     3 SNEX(50),TEMP(50),TEXT(20),XL(66,39)                             M
C                                                                       M
      COMMON /MATR/MATSIZ,MATORD,SIGD,V(MP),SMM(MP*(MP+1)/2)                   M
C                                                                       M
      COMMON /REFS/HN(5,3),HH(3,ML),ICODE(ML),SJJ(ML),SOMEGA(ML),       M
     2 FOBS(ML),FESD(ML),FNUC(ML),FMAG(ML),THETA(ML),TANN(ML),HW(ML),   M
     3 DERSTO(ML,MX),AKEEP(ML),BKEEP(ML),PKEEP(ML),ICODD(ML)            M
C                                                                       M
      COMMON /SUMS/SW,SUMAW,SUMW,SUMCAL,SUMOBS,SUMDIF,SUMWBS,SOM,       M
     2 SUMFOB,SUMDFF,FNUCDI,FNUCOB,FMAGDI,FMAGOB,SUWFOB,SUWDFF          C
C                                                                       M
      COMMON /LOGIC/FORCEL,FORCED,PUNCH,STAR,TYPE,VERT,YES,ANGCOU              M
C                                                                       M
      COMMON /INOUT/LCA,LPR,LPRE,LT4,LT8,LT9,LPUN                       M
C                                                                       M
      COMMON /NUMB/N,N1,N2,N3,N4,N6,N7,NN,NN5,NA(5),NPAT,NREF,NSPAR
C                                                                       M
      COMMON /MOVE/AH(5),BH(5),CH(5),B1(5),B2(5),ZERO(5),SCALE(5,2)     M
C                                                                       M
      COMMON /CONTRL/NCYCLE,MAXS,KXR(5),KKZ,LIM,LOUT,IPONS,LMAT,LCORD,  M
     2 LCOORD,ICELL,ICENT,ICENTT(5),IRL,IRLL(5),KL(5),KLL,KM(5),KMN,    M
     3 MN(5),MNN,ICYCLE,MAXCON,IFOUR,IPLOT,KLOOP,LINES,IWID,IPREF(5),   M
     4 IPREFF,NHALP(5),NUNIAX(5),NCUBIC(5),NGAUSS(5),NCAUCH(5),NPAC(5)  M
     5 ,CLOR(5),ELOR(5)
C                                                                       M
C                                                                       M
      EQUIVALENCE (DERSTO(1,1),STAND(1))                                M
C                                                                       M
C     TEST FOR ZERO DIAGONAL MATRIX ELEMENT. THIS CANNOT BE HANDLED BY  M
C     THE CHOLESKI INVERSION ROUTINE.                                   M
C                                                                       M
      NFAIL=0                                                           M
      II=1                                                              M
      IID=MATORD                                                        M
      DO 30 I=1,MATORD                                                  M
c      IF(SMM(I))20,10,20                                                M
c     ersetzt durch folgenden Befehl, da nur die Diagonalelemente
c     geprueft werden sollen.   H.B. Okt. 1997
      l=1+(i-1)*matord-(i-1)*(i-2)/2
      if(smm(l))20,10,20
10    NFAIL=1                                                           M
      WRITE(LPR,160)I                                                   M
20    II=II+IID                                                         M
      IID=IID-1                                                         M
30    CONTINUE                                                          M
      IF(NFAIL)40,50,40                                                 M
40    CALL ERR(14,0)                                                    M
50    CONTINUE                                                          M
      JJ=MATORD*(MATORD+1)/2                                            M
C                                                                       M
C     INVERT THE NORMAL MATRIX BY THE CHOLESKI METHOD                   M
C                                                                       M
      CALL MATINV(NFAIL)                                                M
C                                                                       M
C     TEST FOR SINGULAR MATRIX                                          M
C                                                                       M
      IF(NFAIL)60,70,60                                                 M
C                                                                       M
C     SINGULARITY ENCOUNTERED                                           M
C                                                                       M
60    CALL ERR(15,NFAIL)                                                M
C                                                                       M
C     MULTIPLY MATRIX AND VECTOR AND STORE THE RESULT IN ARRAY DERIV    M
C                                                                       M
70    CONTINUE                                                          M
      DO 110 I=1,MATORD                                                 M
      DERIV(I)=0.0                                                      M
      II=I                                                              M
      IID=MATORD-1                                                      M
      DO 100 J=1,MATORD                                                 M
      DERIV(I)=DERIV(I)+SMM(II)*V(J)                                    M
      IF(J-I)80,90,90                                                   M
80    II=II+IID                                                         M
      IID=IID-1                                                         M
      GOTO 100                                                          M
90    II=II+1                                                           M
100   CONTINUE                                                          M
110   CONTINUE                                                          M
C                                                                       M
C     CALCULATE SQRT SUM(W*DELTAY**2)/(DEGREES OF FREEDOM)              M
C                                                                       M
      SIGD=SQRT(SUMAW/FLOAT(NUM-MATORD))                                M
C                                                                       M
C     CALCULATE STANDARD DEVIATIONS OF PARAMETERS, STORE IN ARRAY       M
C     STAND, AND TEST FOR CONVERGENCE                                   M
C                                                                       M
C     NOTE THAT STAND AND DERSTO ARE EQUIVALENCED                       M
C                                                                       M
      II=1                                                              M
      NFAIL=0                                                           M
      IID=MATORD                                                        M
      DO 130 I=1,MATORD                                                 M
      STAND(I)=SQRT(SMM(II))*SIGD                                       M
      II=II+IID                                                         M
      IID=IID-1                                                         M
C                                                                       M
C     CONVERGENCE TEST                                                  M
C                                                                       M
      X=STAND(I)*EPS                                                    M
      IF(ABS(DERIV(I))-ABS(X))120,120,130                                    M
120   NFAIL=NFAIL+1                                                     M
130   CONTINUE                                                          M
C                                                                       M
C     TEST WHETHER ALL PARAMETERS FULFILL CONVERGENCE CRITERION         M
C                                                                       M
      IF(NFAIL-MATORD)150,140,140                                       M
C                                                                       M
C     CONVERGENCE ATTAINED                                              M
C                                                                       M
140   NCYCLE=ICYCLE                                                     M
      FORCED=.TRUE.                                                     M
      FORCEL=.TRUE.
150   RETURN                                                            M
160   FORMAT(36H0ZERO MATRIX DIAGONAL FOR PARAMETER ,I3)                M
      END                                                               M
      SUBROUTINE MATINV(NFAIL)                                          M
      PARAMETER (ML=500,MP=99,MX=103)
C     ******************************************************************M
C     INVERTS A SYMMETRIC, POSITIVE DEFINITE MATRIX BY THE CHOLESKI     M
C     METHOD INPUT MATRIX IS UPPER TRIANGLE                             M
C     AND THIS IS REPLACED BY THE INVERSE                               M
C     ******************************************************************M
C                                                                       M
      DOUBLEPRECISION TERM,DENOM,SUMA                                   M
C                                                                       M
C     SEGMENT ONE OF CHOLESKI INVERSION                                 M
C      FACTOR MATRIX INTO LOWER TRIANGLE X TRANSPOSE                    M
C                                                                       M
C                                                                       M
      DIMENSION AM(MP*(MP+1)/2)                                                M
C                                                                       M
C                                                                       M
      COMMON /MATR/MATSIZ,MATORD,SIGD,V(MP),SMM(MP*(MP+1)/2)                   M
C                                                                       M
C                                                                       M
      EQUIVALENCE (MATORD,N),(SMM(1),AM(1))                             M
C                                                                       M
      K=1                                                               M
      IF(N-1)10,20,30                                                   M
10    NFAIL=1                                                           M
      GOTO 220                                                          M
20    AM(1)=1.0/AM(1)                                                   M
      GOTO 210                                                          M
C     LOOP M OF A(L,M)                                                  M
30    DO 120 M=1,N                                                      M
      IMAX=M-1                                                          M
C     LOOP L OF A(L,M)                                                  M
      DO 110 L=M,N                                                      M
      SUMA=0.0D0                                                        M
      KLI=L                                                             M
      KMI=M                                                             M
      IF(IMAX)60,60,40                                                  M
C     SUM OVER I=1,M-1 A(L,I)*A(M,I)                                    M
40    DO 50 I=1,IMAX                                                    M
      SUMA=SUMA+DBLE(AM(KLI)*AM(KMI))                                   M
      J=N-I                                                             M
      KLI=KLI+J                                                         M
50    KMI=KMI+J                                                         M
C     TERM C(L,I) -SUM                                                  M
60    TERM=DBLE(AM(K))-SUMA                                             M
      IF(L-M)70,70,100                                                  M
70    IF(TERM)90,90,80                                                  M
C     A(M,M)= SQRT(TERM)                                                M
80    DENOM=DSQRT(TERM)                                                 M
      AM(K)=SNGL(DENOM)                                                 M
      GOTO 110                                                          M
90    NFAIL=K                                                           M
      GOTO 220                                                          M
C     A(L,M)= TERM/A(M,M)                                               M
100   AM(K)=SNGL(TERM/DENOM)                                            M
110   K=K+1                                                             M
120   CONTINUE                                                          M
C                                                                       M
C     SEGMENT TWO OF CHOLESKI INVERSION                                 M
C      INVERSION OF TRIANGULAR MATRIX                                   M
130   AM(1)=1.0/AM(1)                                                   M
      KDM=1                                                             M
C     STEP L OF B(L,M)                                                  M
      DO 160 L=2,N                                                      M
      KDM=KDM+N-L+2                                                     M
C     RECIPROCAL OF DIAGONAL TERM                                       M
      TERM=DBLE(1.0/AM(KDM))                                            M
      AM(KDM)=SNGL(TERM)                                                M
      KMI=0                                                             M
      KLI=L                                                             M
      IMAX=L-1                                                          M
C     STEP M OF B(L,M)                                                  M
      DO 150 M=1,IMAX                                                   M
      K=KLI                                                             M
C     SUM TERMS                                                         M
      SUMA=0.0D0                                                        M
      DO 140 I=M,IMAX                                                   M
      II=KMI+I                                                          M
      SUMA=SUMA-DBLE(AM(KLI)*AM(II))                                    M
140   KLI=KLI+N-I                                                       M
C     MULT SUM * RECIP OF DIAGONAL                                      M
      AM(K)=SNGL(SUMA*TERM)                                             M
      J=N-M                                                             M
      KLI=K+J                                                           M
150   KMI=KMI+J                                                         M
160   CONTINUE                                                          M
C                                                                       M
C     SEGMENT THREE OF CHOLESKI INVERSION                               M
C      PREMULTIPLY LOWER TRIANGLE BY TRANSPOSE                          M
170   K=1                                                               M
      DO 200 M=1,N                                                      M
      KLI=K                                                             M
      DO 190 L=M,N                                                      M
      KMI=K                                                             M
      IMAX=N-L+1                                                        M
      SUMA=0.0D0                                                        M
      DO 180 I=1,IMAX                                                   M
      SUMA=SUMA+DBLE(AM(KLI)*AM(KMI))                                   M
      KLI=KLI+1                                                         M
180   KMI=KMI+1                                                         M
      AM(K)=SNGL(SUMA)                                                  M
190   K=K+1                                                             M
200   CONTINUE                                                          M
210   NFAIL=0                                                           M
220   RETURN                                                            M
      END                                                               M
      SUBROUTINE DATAIN
      PARAMETER (ML=500,MP=99,MX=103)
C     *****************************************************************
C     THIS SUBROUTINE READS AND PRINTS THE INPUT DATA,AND INITIALIZES
C     OTHER DATA. INPUT IS IN A PSEUDO-FREE-FORMAT MODE ALLOWED BY
C     SUBROUTINE FREEIN
C     *****************************************************************
C
      LOGICAL FORCEL,FORCED,PUNCH,STAR,TYPE,VERT,YES,ANGCOU
C
      DIMENSION IN(5,3),HH(5),IREDIN(50),IM(8),Z(5),TT(5)
C
      COMMON /REFLN/ ARG,DELTA,DLABDA(2),DUMPA(500),EPS,FLOG,FLOG2,
     1    OMEGA,PI,SKEW,SLABDA,SLOG,W,XN,YCALC,YOBS
C
      COMMON /PREF/  IALFA,IOMEG,IORD1,IORD2,IPREV,ISTAP,ISS,
     1    KSTART,LIMO,LLL,NCD(50),NKEY,NO,NTEXT,NUM,NTYP(50),
     2    MEQ(50),MTYP(50)
C
      COMMON /SYMCEL/ ALL(3,3),AL(5,3,3),ATEXT(60),B(8),CELL(6),
     1    CELLN(5,6),F(4,21),FF(4),PH(5,3),RM(5,24,2,9),S(4,21),
     2    SM(5,24,3,3),T(5,24,3),TR(5,24),RCELL(6),RCELLN(5,6),
     3    VOLUME,VOL(5),PTEXT(5,4)
C
      COMMON /MINIM/ COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),
     1    HL(3,3),RSA(50,3),RSB(50,3),SA(50),SB(50),
     2    SINAR(50,24),SINARI(24),SNEX(50),TEMP(50),TEXT(20),XL(66,39)
C
      COMMON /SCONTS/ ICON(4,9),JCON(4,9),LCONI(4,9),LCONJ(4,9),
     1  LN(4),LCID(4,9),JCID(4,9),CI(4,9),CJ(4,9),CONA(4,9),DF(4)
C
      COMMON /MAXDIM/ MNCONF,MNCONT,MMATOR,MNEUTS,MNXMAG,MNATOM,
     1  MNMAGR,MNEQPO,MNOVRF,MNMAGP,MNPATS
C
      COMMON /CONTRL/ NCYCLE,MAXS,KXR(5),KKZ,LIM,LOUT,IPONS,LMAT,LCORD,
     1  LCOORD,ICELL,ICENT,ICENTT(5),IRL,IRLL(5),KL(5),KLL,KM(5),KMN,
     2  MN(5),MNN,ICYCLE,MAXCON,IFOUR,IPLOT,KLOOP,LINES,IWID,IPREF(5),
     3  IPREFF,NHALP(5),NUNIAX(5),NCUBIC(5),NGAUSS(5),NCAUCH(5),NPAC(5)
     5 ,CLOR(5),ELOR(5)
C
      COMMON /MATR/ MATSIZ,MATORD,SIGD,V(MP),SMM(MP*(MP+1)/2)
C
      COMMON /INOUT/LCA,LPR,LPRE,LT4,LT8,LFOR,LPUN
C
      COMMON /NUMB/N,N1,N2,N3,N4,N6,N7,NN,NN5,NA(5),NPAT,NREF,NSPAR
C
      COMMON /BAYES/ NBAYES,LBAY
C
      COMMON /REFS/ HN(5,3),HP(3,ML),NCODE(ML),SJP(ML),SOMEGA(ML),
     1  FOBS(ML),FESD(ML),FNUC(ML),FMAG(ML),THETA(ML),TANN(ML),
     2  HW(ML),DERSTO(ML,MX),AKEEP(ML),BKEEP(ML),PKEEP(ML),NCODD(ML)
C
      COMMON /BFAC/ ITF,JTF(50),PRCP(5,6),ITF3,ITF4
C
      COMMON /LOGIC/FORCEL, FORCED,PUNCH,STAR,TYPE,VERT,YES,ANGCOU
C
      COMMON /KEYS/ LP(66,39),FLAG(66,39)
C
      COMMON /RELX/ RELAXB,RELAXC,RELAXS,RELAXH
C
      COMMON /INPUT/ NREAD,NPOINT,IRED,MODE,LINO,IDISML,
     1  ICHARS(14),ICARD(80),READIN(50)
C
      COMMON /BACKGR/ IBACK,BAKGRD,WAVE1,WAVE3,VOLW3,VOLW(5),
     1    CROSS(8),BMAG(8),BAKG(5)
C
      COMMON /OPTION/ ISF,ISUB,IXTRA,ISW
C
      DIMENSION LPS(66),LPPS(66)
C
      EQUIVALENCE (READIN(1),IREDIN(1))
C
      OPEN(27,FILE='neupar')
      REWIND 27
C.....READ PROBLEM IDENTIFICATION
C
      READ(LCA,1) TEXT
      WRITE(27,1) TEXT
C
C.....READ PROBLEM PARAMETERS
C
      NREAD=5
      IRED=1
      IBEG=1
      IDISML=1
      CALL INITT(IBEG)
      CALL FREEIN
      NCYCLE=IREDIN(1)
      IF(NCYCLE) 10,10,20
   10 IREDIN(3)=0
   20 NPAT=IREDIN(2)
      MAXS=IREDIN(3)
      LIM=IREDIN(4)
      IBACK=IREDIN(5)
        IF(NPAT.GT.MNPATS) CALL ERR(25,MNPATS)
      WRITE(27,*)NCYCLE,NPAT,MAXS,LIM,IBACK
C     TITLE FOR EACH PATTERN
      DO 25 NP=1,NPAT
      READ(LCA,6) (PTEXT(NP,J),J=1,4)
      WRITE(27,6) (PTEXT(NP,J),J=1,4)
   25 CONTINUE
C     TYPE OF DATA
C     READ(LCA,3) KXR(NP),IPREF(NP)
      KXRNP=0
        DO 30  NP=1,NPAT
        NREAD=2
        IRED=1
        CALL INITT(IBEG)
        CALL FREEIN
        KXR(NP)  =IREDIN(1)
        IPREF(NP)=IREDIN(2)
        KXRNP=KXRNP+KXR(NP)
      WRITE(27,*)KXR(NP),IPREF(NP)
   30 CONTINUE
C
C.....OUTPUT OPTIONS
C
C     READ(LCA,4) LIM,LCOORD,IPLOT,LOUT,IPONS,LMAT,LCORD,IFOUR
      NREAD=11
      IRED=1
      CALL INITT(IBEG)
      CALL FREEIN
      WRITE(27,*)(IREDIN(I),I=1,7)
      LCOORD=IREDIN(1)
      IPLOT =IREDIN(2)
      LOUT  =IREDIN(3)
      IPONS=IREDIN(4)
      LMAT=IREDIN(5)
      LCORD=IREDIN(6)
      IFOUR=IREDIN(7)
      ISW  =IREDIN(8)
      ISF  =IREDIN(9)
      ISUB =IREDIN(10)
      IXTRA=IREDIN(11)
C
C.....CONSTRAINT MARKERS
C
C     READ(LCA,96) MAXCON,NBAYES
      NREAD=2
      IRED=1
      CALL INITT(IBEG)
      CALL FREEIN
      MAXCON=IREDIN(1)
C     REMOVE NEXT LINE IF BAYESIAN CONSTANTS ALLOWED
      IREDIN(2)=0
      NBAYES=IREDIN(2)
      WRITE(27,*)MAXCON,NBAYES
C
C.....MARKERS FOR SYMMETRY, ATOMS, TYPE OF DATA
C
      KMN=0
      N1=0
      N2=0
      KLL=0
      N=0
      NA(3)=0
      NA(2)=0
      DO 40  NP=1,NPAT
C     READ(LCA,3)ICENTT(NP),IRLL(NP),NA(NP),KL(NP),KM(NP),MN(NP)
      NREAD=6
      IRED=1
      CALL INITT(IBEG)
      CALL FREEIN
      ICENTT(NP)=IREDIN(1)
      IRLL(NP)=IREDIN(2)
      NA(NP)=IREDIN(3)
      KL(NP)=IREDIN(4)
      KM(NP)=IREDIN(5)
      MN(NP)=IREDIN(6)
      WRITE(27,*)(IREDIN(I),I=1,6)
      IRLL(NP)=IRLL(NP)+1
C.....TOTALS FOR ALL PATTERNS
      N=N+NA(NP)
       KLL=KLL+KL(NP)
      KMN=KMN+KM(NP)
   40 CONTINUE
      N4=N+3*NPAT+1
      N6=N+NPAT+1
      N7=N6+NPAT
      IF(KMN) 50,50,60
   50 KLOOP=1
      GOTO 70
   60 KLOOP=KMN
C
C.....TERMINATION, RELAXATION FACTORS AND WAVELENGTH
C
   70 IDISML=-1
        IF(KXR(1)-4) 90,80,80
   80 NREAD=7
      I=4
      GOTO 100
   90 NREAD=6
      I=3
  100 IRED=1
      CALL INITT(IBEG)
      CALL FREEIN
      EPS=READIN(1)
      DLABDA(1)=READIN(2)
        IF(KXR(1)-4) 110,120,120
  110 DLABDA(2)=0.
      GOTO 130
  120 DLABDA(2)=READIN(3)
  130 RELAXC=READIN(I)
      RELAXB=READIN(I+1)
      RELAXS=READIN(I+2)
      RELAXH=READIN(I+3)
C
C.....CHECK RELAXATION FACTORS AND SET TO 0.8 IF INPUT IS ZERO
C
      IF(RELAXC.LE.0.0) RELAXC=0.8
      IF(RELAXB.LE.0.0) RELAXB=0.8
      IF(RELAXS.LE.0.0) RELAXS=0.8
      IF(RELAXH.LE.0.0) RELAXH=0.8
      WRITE(27,*)(READIN(I),I=1,NREAD)
C
      WAVE1=DLABDA(1)
      WAVE3=WAVE1*WAVE1*WAVE1
C
C.....PREFERRED ORIENTATION AXIS
C
      DO 180  NP=1,NPAT
      IPREFF=IPREF(NP)
          IF(IPREFF) 160,140,160
  140 DO 150 I=1,3
      PH(NP,I)=0.0
  150 CONTINUE
      GOTO 180
C 160 READ(LCA,64) (PH(NP,I),I=1,3)
  160 NREAD=3
      IRED=1
      CALL INITT(IBEG)
      CALL FREEIN
      WRITE(27,*)(READIN(I),I=1,3)
      DO 170 I=1,3
      PH(NP,I)=READIN(I)
  170 CONTINUE
  180 CONTINUE
C
C.....UNIQUE MAGNETIC AXES
      DO 182 NP=1,NPAT
        IF(KXR(NP).NE.2)GOTO 182
      NREAD=3
      IRED =1
      CALL INITT(IBEG)
      CALL FREEIN
      WRITE(27,*)(READIN(I),I=1,3)
        DO 181 J=1,3
        HL(NP,J)=READIN(J)
  181   CONTINUE
  182 CONTINUE
C
C.....PRINT PROGRAM HEADING
C
  190 WRITE(LPR,5)
      WRITE(LPR,161) TEXT
      WRITE(12,161) TEXT
      WRITE(LPR,41)NCYCLE,MAXS,NPAT
        IF(LIM.GT.0) WRITE(LPR,607)LIM
      WRITE(LPR,11) EPS
      WRITE(LPR,2051) RELAXC,RELAXB,RELAXS,RELAXH
      WRITE(LPR,2060)
      IF(LCOORD.EQ.1) WRITE(LPR,2061)
      IF(IPLOT.EQ.1)  WRITE(LPR,2062)
      IF(LOUT.EQ.1)   WRITE(LPR,2063)
      IF(IPONS.EQ.1)  WRITE(LPR,2064)
      IF(IPONS.EQ.2)  WRITE(LPR,2065)
      IF(LMAT.EQ.1)   WRITE(LPR,2066)
      IF(LCORD.LT.0)  WRITE(LPR,2067)
      IF(LCORD.GT.0)  WRITE(LPR,2068) LCORD
      IF(IFOUR.GT.0)  WRITE(LPR,2069) IFOUR
      IF(ISW.NE.0)    WRITE(LPR,2073)
      IF(ISUB.GT.0)   WRITE(LPR,2071) ISUB
      IF(IXTRA.GT.0)  WRITE(LPR,2072)
      WRITE(LPR,2070)LCOORD,IPLOT,LOUT,IPONS,LMAT,LCORD,IFOUR,ISW,ISF,
     +  ISUB,IXTRA
C
C.....PRINT WAVELENGTH(S)
        IF(KXR(1)-3)183,183,184
  183 WRITE(LPR,21) DLABDA(1)
      GOTO 185
  184 WRITE(LPR,1021) DLABDA(1),DLABDA(2)
  185   DO 186 I=1,2
  186   DLABDA(I)=0.25*DLABDA(I)*DLABDA(I)
C
C.....READ FIRST BLOCK OF INPUT DATA
C
  200 READ(LPRE,END=220)DUMPA
C     COUNTERS FOR INPUT BLOCK TRANSFERS;
C     LLL=MAX. NO. OF OVERLAPPING REFLECTIONS
C     NKEY=(NO. OF BLOCKS TRANSFERRED)+1
  210 LLL=2
      NKEY=1
      GOTO 230
  220 CALL ERR(1,0)
  230 LIMO=FLEES(0.)+0.5
        IF(LIMO.GT.MNOVRF) CALL ERR(10,MNOVRF)
C
C.....OPTIONS FOR SCATTERING TYPE
C
      DO 750 NP=1,NPAT
      WRITE(LPR,235) NP,(PTEXT(NP,J),J=1,4)
  235 FORMAT(/,1X,30(1H*),/,1X,1H*,7HPATTERN,I2,2X,4A4,1X,1H*,/,1X,
     A30(1H*))
C     SET OPTION PARAMS TO ZERO
      NHALP(NP)=0
      NUNIAX(NP)=0
      NGAUSS(NP)=0
      NCAUCH(NP)=0
      NCUBIC(NP)=0
C
      KKZ=KXR(NP)+1
      GOTO (310,240,250,280,290,300,291,292),KKZ
  240 NHALP(NP)=1
      WRITE(LPR,13)
      GOTO 320
  250 WRITE(LPR,16)
      NUNIAX(NP)=1
      WRITE(LPR,270) NP,(HL(NP,J),J=1,3)
  270 FORMAT(33H0UNIQUE MAGNETIC AXIS FOR PATTERN,I2,2H =,3F3.0)
      GOTO 320
280   NCUBIC(NP)=1
      WRITE(LPR,19)
      GOTO 320
  290 WRITE(LPR,604)
      NGAUSS(NP)=1
      GOTO 320
  300 WRITE(LPR,605)
      NCAUCH(NP)=1
      CLOR(NP)=4.
      ELOR(NP)=1.
      GOTO 320
  291 WRITE(LPR,614)
      CLOR(NP)=4.*(SQRT(2.)-1.)
      ELOR(NP)=2.
      GOTO 320
  292 WRITE(LPR,615)
      CLOR(NP)=4.*(2.**(2./3.)-1.)
      ELOR(NP)=1.5
      GOTO 320
614   FORMAT(' X-RAY REFINEMENT WITH MOD LORENTZIAN')
615   FORMAT(' X-RAY REFINEMENT WITH INT LORENTZIAN')
  310 WRITE(LPR,606)
  320 IPREFF=IPREF(NP)
        IF(IPREFF) 340,330,340
  330 WRITE(LPR,609)
      GOTO 350
  340 WRITE(LPR,22) (PH(NP,I),I=1,3)
  350 CONTINUE
C
C.....SET UP IDENTITY MATRIX FOR POSITION (X,Y,Z)
C
      DO 470 I=1,3
      T(NP,1,I)=0.
      DO 470 J=1,3
        IF(I-J) 410,400,410
  400 SM(NP,1,I,J)=1.
      GOTO 420
  410 SM(NP,1,I,J)=0.
  420   IF(MN(NP)) 470,470,430
  430  MNN=MN(NP)
        DO 460 K=1,MNN
        IJ=(I-1)*3+J
        IF(I-J) 450,440,450
  440   RM(NP,1,K,IJ)=1.
        GOTO 460
  450   RM(NP,1,K,IJ)=0.
  460   CONTINUE
  470 CONTINUE
C
C.....READ SYMMETRY MATRICES AND VECTORS
C
      IRL=IRLL(NP)
        IF(IRL-1) 550,550,480
  480 IF(IRL.GT.MNEQPO) CALL ERR(3,MNEQPO)
C     IRLL(NP) IS THE NUMBER OF SYMMETRY MATRICES INPUT
C     MN IS THE NUMBER OF MAGNETIC VECTOR ROTATION MATRICES PER
C     EQUIVALENT POSITION
      MNN=MN(NP)
        IF(MNN.GT.MNMAGR) CALL ERR(11,MNMAGR)
      NREAD=12
      DO 540 IR=2,IRL
C     READ(LCA,30)((SM(NP,IR,I,J),J=1,3),I=1,3),(T(NP,IR,I),I=1,3)
      IRED=1
      CALL INITT(IBEG)
      CALL FREEIN
      WRITE(27,*)(READIN(I),I=1,12)
      IRED=1
        DO 500 I=1,3
        T(NP,IR,I)=READIN(I+9)
          DO 490 J=1,3
          SM(NP,IR,I,J)=READIN(IRED)
          IRED=IRED+1
  490     CONTINUE
  500   CONTINUE
      IF(MNN)540,540,510
C
C.....READ MAGNETIC VECTOR ROTATION MATRICES
C
  510 NREAD=9
      DO 530 K=1,MNN
C     READ(LCA,30)(RM(IR,K,IJ),IJ=1,9)
      IRED=1
      CALL INITT(IBEG)
      CALL FREEIN
      WRITE(27,*)(READIN(I),I=1,9)
        DO 520 IJ=1,9
        RM(NP,IR,K,IJ)=READIN(IJ)
  520   CONTINUE
  530 CONTINUE
      NREAD=12
  540 CONTINUE
  550 CONTINUE
C
C.....PRINT OF SYMMETRY MATRICES
C
      IRL=IRLL(NP)
      MNN=MN(NP)
      ICENT=ICENTT(NP)
      IF(ICENT.EQ.2) GOTO 690
      WRITE(LPR,301)
      GOTO 700
  690 WRITE(LPR,303)
  700 IF(IRL-2) 750,710,710
  710 WRITE(LPR,49)
      WRITE(LPR,1611)
      DO 740 IR=1,IRL
      WRITE(LPR,152) IR,((SM(NP,IR,I,J),J=1,3),I=1,3),(T(NP,IR,I),I=1,3)
      IF(MNN) 740,740,720
  720 DO 730 K=1,MNN
  730 WRITE(LPR,55) K,(RM(NP,IR,K,IJ),IJ=1,9)
  740 CONTINUE
  750 CONTINUE
C
C.....READ SCATTERING LENGTHS(NEUTRONS) OR DELTA F" (X-RAYS)
C
C     IF(KLL.GT.MNEUTS) CALL ERR(5,MNEUTS)
C     READ(LCA,33) (B(2),I=1,KLL)
      NREAD=KLL
      IRED=1
      CALL INITT(IBEG)
      CALL FREEIN
      WRITE(27,*)(READIN(I),I=1,KLL)
        DO 560 I=1,KLL
        B(I)=READIN(I)
  560   CONTINUE
        IF(IBACK) 570,590,570
  570 IRED=1
      CALL INITT(IBEG)
      CALL FREEIN
      WRITE(27,*)(READIN(I),I=1,KLL)
        DO 580 I=1,KLL
        CROSS(I)=READIN(I)
  580   CONTINUE
  590 CONTINUE
      IF(KMN)670,670,600
  600 IF(KMN.GT.MNXMAG) CALL ERR(4,MNXMAG)
      MODE=1
      DO 640 I=1,KMN
      IRED=1
      CALL INITT(IBEG)
      IDISML=1
      NREAD=1
      CALL FREEIN
      J=IREDIN(1)
      WRITE(27,*) J
      NREAD=2*J
      IF(J.GT.MNMAGP) CALL ERR(22,MNMAGP)
      IDISML=-1
      IRED=1
      CALL FREEIN
      WRITE(27,2727) (READIN(L),L=1,NREAD)
2727  FORMAT(8F10.4)
      IRED=1
        DO 610 L=1,J
        S(I,L)=READIN(IRED)
        IRED=IRED+1
        F(I,L)=READIN(IRED)
        IRED=IRED+1
  610   CONTINUE
C     READ(LCA,37) J,(S(I,L),F(I,L),L=1,J)
      IM(I)=J
      IRED=J+1
      IF(IRED-MNMAGP) 620,620,640
  620   DO 630 L=IRED,MNMAGP
        S(I,L)=S(I,J)
        F(I,L)=F(I,J)
  630   CONTINUE
  640 CONTINUE
      IF(IBACK) 650,760,650
  650 NREAD=KMN
      IRED=1
      MODE=0
      CALL INITT(IBEG)
      CALL FREEIN
      WRITE(27,*)(READIN(I),I=1,KMN)
      DO 660 I=1,KMN
      BMAG(I)=READIN(I)
  660 CONTINUE
  670 CONTINUE
C
C.....PRINT SCATTERING FACTORS
C
  760 IF(KXR(1)-4) 770,780,780
  770 WRITE(LPR,51)
      GOTO 790
  780 WRITE(LPR,151)
  790 WRITE(LPR,56) (B(I),I,I=1,KLL)
      IF(IBACK) 800,810,800
  800 WRITE(LPR,2010)
      WRITE(LPR,2020) (CROSS(I),I,I=1,KLL)
C     WRITE(27,2020) (CROSS(I),I=1,KLL)
  810 IF(KMN) 930,930,820
  820 IF(KXR(1)-4) 830,840,840
  830 WRITE(LPR,59)
      GOTO 850
  840 WRITE(LPR,159)
  850 DO 910 I=1,KMN
C     WRITE(27,37) IM(I),(S(I,J),F(I,J),J=1,IM(I))
      L1=1
  860 L2=L1+9
      IF(L2-IM(I)) 880,880,870
  870 L2=IM(I)
  880 WRITE(LPR,62) (S(I,J),J=L1,L2)
      WRITE(LPR,617)(F(I,J),J=L1,L2)
      L1=L1+10
      IF(L1-IM(I)) 860,860,890
  890 IF(L2-IM(I)) 900,910,910
  900 L1=L2+1
      GOTO 870
  910 CONTINUE
      IF(IBACK) 920,930,920
  920 WRITE(LPR,2030)
      WRITE(LPR,2040) (BMAG(I),I,I=1,KMN)
C.....N IS THE TOTAL NUMBER OF ATOMS INPUT
  930 IF(N.GT.MNATOM) CALL ERR(6,MNATOM)
      WRITE(LPR,57)
      MODE=0
C
C.....READ AND PRINT FOR EACH ATOM
C
      ITF=0
      ITF3=0
      ITF4=0
      DO 1060 I=1,N
      READ(LCA,1623) ATEXT(I),(ICARD(J),J=5,80)
      MEQ(I)=0
        DO 940 J=6,8
        XL(I,J)=0.0
  940   CONTINUE
      MODE=0
      NPOINT=5
      LINO=LINO+1
      IRED=1
      NREAD=2
      IDISML=1
      CALL FREEIN
      NCD(I)=IREDIN(1)
      NP=NCD(I)
      NTYP(I)=IREDIN(2)
      IDISML=-1
      NREAD=5
      IRED=1
      CALL FREEIN
        DO 950 J=1,5
        XL(I,J)=READIN(J)
  950   CONTINUE
C.....ARE THERE ANY MAGNETIC ATOMS?
      IF(KXRNP.GT.1) GOTO 960
      IF(KMN.GT.0) GOTO 960
      MTYP(I)=0
      GOTO 990
  960 IDISML=1
      NREAD=1
      IRED=1
      CALL FREEIN
      MTYP(I)=IREDIN(1)
      IF(MTYP(I)) 990,990,970
  970   IF(KXR(NP).GT.3) GOTO 990
C       IF(KXRNP.EQ.3) GOTO 990
      MODE=1
      IRED=1
      CALL FREEIN
      MEQ(I)=IREDIN(1)
      IDISML=-1
      NREAD=3
      IRED=1
      CALL FREEIN
        DO 980 J=1,3
        XL(I,J+5)=READIN(J)
  980   CONTINUE
  990 IDISML=1
      NREAD=1
      IRED=1
      CALL FREEIN
      JTF(I)=IREDIN(1)
      IF(JTF(I)) 1010,1010,1000
 1000 ITF=1
      IF(JTF(I).GE.3) ITF3=1
      IF(JTF(I).GE.4) ITF4=1
 1010 CONTINUE
 1020   DO 1030 J=9,39
 1030   XL(I,J)=0.0
      IF(JTF(I).EQ.2.OR.JTF(I).EQ.0) GOTO 1060
C 621 READ(LCA,1622) (XL(I,J),J=9,14)
 1040 NREAD=6
      IDISML=-1
      IRED=1
      MODE=1
      CALL FREEIN
        DO 1050 J=1,6
        XL(I,J+8)=READIN(J)
 1050   CONTINUE
      IF(JTF(I).EQ.1) GOTO 1060
      READ(LCA,*) (XL(I,J),J=15,24)
      IF(JTF(I).EQ.3) GOTO 1060
      READ(LCA,*) (XL(I,J),J=25,34)
      READ(LCA,*) (XL(I,J),J=35,39)
 1060 CONTINUE
C
      WRITE(LPR,171) (ATEXT(I),NCD(I),NTYP(I),MTYP(I),MEQ(I),
     2(XL(I,J),J=1,8),I=1,N)
      IF (ITF)  1070,1120,1070
 1070 L=0
        DO 1110 I=1,N
        IF(JTF(I).EQ.2.OR.JTF(I).EQ.0) GOTO 1110
 1080   IF(L) 1090,1090,1100
 1090   WRITE(LPR,628)
        WRITE(LPR,626)
        L=1
 1100   WRITE(LPR,627) ATEXT(I),(XL(I,J),J=9,14)
 1110   CONTINUE
      IF(ITF3.EQ.0) GOTO 1120
      WRITE(LPR,638)
      WRITE(LPR,636)
      DO 1113 I=1,N
      IF(JTF(I).LT.3) GOTO 1113
      WRITE(LPR,637) ATEXT(I),(XL(I,J),J=15,24)
 1113 CONTINUE
      IF(ITF4.EQ.0) GOTO 1120
      WRITE(LPR,648)
      WRITE(LPR,646)
      DO 1114 I=1,N
      IF(JTF(I).LT.4) GOTO 1114
      WRITE(LPR,647) ATEXT(I),(XL(I,J),J=25,39)
 1114 CONTINUE
C
  638 FORMAT(1H0,45X,'ANHARMONIC ATOMS OF 3RD ORDER')
  648 FORMAT(1H0,45X,'ANHARMONIC ATOMS OF 4TH ORDER')
  636 FORMAT(5H ATOM,11X,4HC111,6X,4HC222,6X,4HC333,6X,4HC112,6X,
     14HC122,6X,4HC113,6X,4HC133,6X,4HC223,6X,4HC233,6X,4HC123)
  637 FORMAT(1H ,A4,5X,10E10.4)
  646 FORMAT(5H ATOM,10X,5HD1111,5X,5HD2222,5X,5HD3333,5X,5HD1112,5X,
     15HD1222,5X,5HD1113,5X,5HD1333,5X,5HD2223,5X,5HD2333,5X,
     25HD1122,/,15X,5HD1133,5X,5HD2233,5X,5HD1123,5X,5HD1223,5X,
     35HD1233)
  647 FORMAT(1H ,A4,5X,10E10.4,/,10X,5E10.4)
C
C.....READ SCALE AND OVERALL TEMPERATURE FACTORS
C
 1120 DO 1185 NP=1,NPAT
      N1=N+NP
      N2=N7+NP
      N3=N6+NP
      IDISML=-1
      MODE=0
      IF(KXR(1)-4) 1130,1140,1140
 1130 NREAD=2
      GOTO 1150
 1140 NREAD=3
 1150 IRED=1
      CALL INITT(IBEG)
      CALL FREEIN
      XL(N1,3)=READIN(1)
      XL(N1,5)=READIN(NREAD)
      IF(KXR(1)-4) 1160,1170,1170
 1160 XL(N1,4)=XL(N1,3)
      GOTO 1180
 1170 XL(N1,4)=READIN(2)
 1180 WRITE(LPR,66) (XL(N1,I),I=3,5)
 1185 CONTINUE
C
C.....READ ZEROPOINT
C
      IDISML=-1
      NREAD=NPAT
      IRED =1
      CALL INITT(IBEG)
      CALL FREEIN
        DO 1187 NP=1,NPAT
        XL(N6,NP)=READIN(NP)
      IF(NP.EQ.1)GOTO 1187
C     IF(XL(N6,NP).EQ.0.0) XL(N6,NP)=XL(N6,1)
 1187 WRITE(LPR,69) NP,XL(N6,NP)
C
C.....READ AND PRINT PROFILE PARAMETERS
C
      DO 1195 NP=1,NPAT
      N3=N6+NP
      IDISML=-1
      NREAD=3
      IRED=1
      CALL INITT(IBEG)
      CALL FREEIN
        DO 1190 I=1,3
        XL(N3,I)=READIN(I)
 1190   CONTINUE
      WRITE(LPR,68) (XL(N3,I),I=1,3)
 1195 CONTINUE
C
C.....INPUT CELL DIMENSIONS
C
      DO 1210 NP=1,NPAT
      N2=N7+NP
      CALL DCELL(NP)
        DO 1200 I=1,3
 1200   XL(N2,I)=AL(NP,I,I)
      XL(N2,4)  =AL(NP,2,3)
      XL(N2,5)  =AL(NP,1,3)
      XL(N2,6) =AL(NP,1,2)
 1210 WRITE(LPR,76) (XL(N2,I),I=1,6)
C
C.....READ PREFFERED ORIENT. & ASYM. PARAMS
      DO 1310 NP=1,NPAT
      N1=N+NP
      IPREFF=IPREF(NP)
      XL(N1,1)=0.0
      XL(N1,2)=0.0
      NREAD=0
      IF(IPREFF) 1220,1230,1220
 1220 NREAD=1
 1230 IF(LIM) 1250,1250,1240
 1240 NREAD=NREAD+1
 1250 IF(NREAD) 1310,1310,1260
 1260 IRED=1
      CALL INITT(IBEG)
      CALL FREEIN
      IF(IPREFF) 1270,1280,1270
 1270 XL(N1,1)=READIN(1)
 1280 IF(LIM) 1300,1300,1290
 1290 XL(N1,2)=READIN(NREAD)
 1300 WRITE(LPR,78) (XL(N1,I),I=1,2)
 1310 CONTINUE
C
      NN=FLEES(0.)+0.5
C
C.....CONVERT ISOTROPIC ATOMS TO ANISOTROPIC IF NECESSARY
C
      NN5=NN*5+4+NPAT*3
 1315 DO 1340 NP=1,NPAT
      IF(ITF.EQ.1) CALL BPREP
        DO 1320 I=1,N4
        DO 1320 J=1,39
        FLAG(I,J)=0.
 1320   LP(I,J)=0
        DO 1330 I=1,3
        HN(NP,I)=FLEES(0.)
 1330   IN(NP,I)=HN(NP,I)+0.5
      WRITE(LPR,994) (IN(NP,I),I=1,3)
 1340 CONTINUE
      IF(NCYCLE.LE.0)GOTO 1660
      WRITE(LPR,82)
C
C.....READ AND PRINT CODEWORDS FOR ATOMIC PARAMETERS
C
C
C     THE ELEMENTS OF THE ARRAYS XL, LP AND FLAG HAVE BEEN REORDERED THUS
C
C     (N)   ROWS 1 TO N ARE FOR ATOMIC PARAMETERS
C     (N1)  THE FOLLOWING 'NPAT' ROWS ARE FOR 5 OVERALL PARAMETERS FOR
C           EACH PATTERN.
C     (N6)  A ROW FOR THE ZEROPOINT(S)
C     (N3)  THE FOLLOWING 'NPAT' ROWS ARE FOR THE HALFWIDTH PARAMETERS FO
C           EACH PATTERN.
C     (N2)  THE FOLLOWING 'NPAT' ROWS FOR EITHER THE CELL CONSTANTS OR TH
C           METRIC TENSOR FOR EACH PATTERN.
C
C     WITHIN THE PROGRAM, THE ELEMENTS ARE LOCATED BY:-
C
C           N6 = N   +  NPAT + 1
C           N7 = N6  +  NPAT           )  ABSOLUTE ROW LOCATION
C           N4 = N   +  NPAT * 3 + 1   )
C
C           N1 = N   +  NP             )
C           N3 = N6  +  NP             )  RELATIVE ROW LOCATION
C           N2 = N7  +  NP             )
C
C     (NP IS THE SERIAL NUMBER OF THE PATTERN UNDER CONSIDERATION.)
C
      DO 1380 I=1,N
C     READ(LCA,86) (FLAG(I,J),J=1,8)
      NREAD=8
      IRED=1
      CALL INITT(IBEG)
      CALL FREEIN
        DO 1350 J=1,8
        FLAG(I,J)=READIN(J)
 1350   CONTINUE
        IF(JTF(I)) 1360,1380,1360
C1360 READ(LCA,86) (FLAG(I,J),J=9,14)
 1360 NREAD=6
      IRED=1
      CALL INITT(IBEG)
      CALL FREEIN
        DO 1370 J=1,6
        FLAG(I,J+8)=READIN(J)
 1370   CONTINUE
      IF(JTF(I).LT.3) GOTO 1380
      READ(LCA,*) (FLAG(I,J),J=15,24)
      IF(JTF(I).LT.4) GOTO 1380
      READ(LCA,*) (FLAG(I,J),J=25,34)
      READ(LCA,*) (FLAG(I,J),J=35,39)
 1380 CONTINUE
      WRITE(LPR,85) (ATEXT(I),(FLAG(I,J),J=1,8),I=1,N)
      IF(ITF) 1390,1420,1390
 1390 WRITE(LPR,628)
      WRITE(LPR,626)
      DO 1410 I=1,N
      IF(JTF(I)) 1400,1410,1400
 1400 WRITE(LPR,85) ATEXT(I),(FLAG(I,J),J=9,14)
 1410 CONTINUE
      IF(ITF3.EQ.0) GOTO 1420
      WRITE(LPR,638)
      WRITE(LPR,636)
      DO 1413 I=1,N
      IF(JTF(I).LT.3) GOTO 1413
      WRITE(LPR,83) ATEXT(I),(FLAG(I,J),J=15,24)
 1413 CONTINUE
      IF(ITF4.EQ.0) GOTO 1420
      WRITE(LPR,648)
      WRITE(LPR,646)
      DO 1414 I=1,N
      IF(JTF(I).LT.4) GOTO 1414
      WRITE(LPR,84) ATEXT(I),(FLAG(I,J),J=25,39)
 1414 CONTINUE
C
   83 FORMAT(1X,A4,5X,10F10.2)
   84 FORMAT(1X,A4,5X,10F10.2,/,10X,5F10.2)
C
C
C....READ AND PRINT CODEWORDS FOR SCALES,
C     OVERALL TEMP FACTORS,PREFERRED ORIENTATION.
C
 1420 DO 1560 NP=1,NPAT
      N1=N+NP
C     READ(LCA,86)(FLAG(N1,J),J=3,5)
C     READ(LCA,86)(FLAG(N1,J),J=1,4)
C     READ(LCA,86)(FLAG(N1,J),J=1,2)
      IF(KXR(1)-4) 1440,1430,1430
 1430 NREAD=3
      GOTO 1450
 1440 NREAD=2
 1450 IRED=1
      CALL INITT(IBEG)
      CALL FREEIN
      FLAG(N1,3)=READIN(1)
      FLAG(N1,5)=READIN(NREAD)
      IF(KXR(1)-4) 1465,1460,1460
 1460 FLAG(N1,4)=READIN(2)
 1465 WRITE(LPR,89) FLAG(N1,3),FLAG(N1,4)
      WRITE(LPR,88) FLAG(N1,5)
 1560 CONTINUE
C
C.....READ AND PRINT CODEWORDS FOR PROFILE PARAMETERS
C
C     ZEROPOINT
C     READ(LCA,86) FLAG(N6,1)
      NREAD=NPAT
      IRED=1
      CALL INITT(IBEG)
      CALL FREEIN
        DO 1563 NP=1,NPAT
         FLAG(N6,NP)=READIN(NP)
          IF(NP.EQ.1)GOTO 1563
C         IF(FLAG(N6,NP).EQ.0.0) FLAG(N6,NP)=FLAG(N6,1)
 1563 CONTINUE
      WRITE(LPR,92) (FLAG(N6,NP),NP=1,NPAT)
C
C     HALFWIDTH PARAMETERS
        DO 1570 NP=1,NPAT
        N3=N6+NP
C     READ(LCA,86)(FLAG(N3,J),J=1,3)
        NREAD=3
        IRED=1
        CALL INITT(IBEG)
        CALL FREEIN
          DO 1565 J=1,3
          FLAG(N3,J)=READIN(J)
 1565     CONTINUE
      WRITE(LPR,91)(FLAG(N3,J),J=1,3)
 1570 CONTINUE
C
C.....READ AND PRINT CODEWORDS FOR LATTICE PARAMETERS
      DO 1590 NP=1,NPAT
      N2=N7+NP
C     READ(LCA,86)(FLAG(N2,J),J=1,6)
      NREAD=6
      IRED=1
      CALL INITT(IBEG)
      CALL FREEIN
        DO 1580 J=1,6
        FLAG(N2,J)=READIN(J)
 1580   CONTINUE
      WRITE(LPR,93)(FLAG(N2,J),J=1,6)
 1590 CONTINUE
C
C..... READ & PRINT CODEWORDS FOR POP & ASP.
      DO 1599 NP=1,NPAT
      N1=N+NP
      NREAD=0
      IF(IPREF(NP)) 1591,1592,1591
 1591 NREAD=1
 1592  IF(LIM) 1594,1594,1593
 1593 NREAD=NREAD+1
 1594 IRED=1
      CALL INITT(IBEG)
      CALL FREEIN
        IF(IPREF(NP)) 1595,1596,1595
 1595 FLAG(N1,1)=READIN(1)
 1596   IF(LIM) 1598,1598,1597
 1597 FLAG(N1,2)=READIN(NREAD)
 1598 WRITE(LPR,95) FLAG(N1,1)
      WRITE(LPR,94) FLAG(N1,2)
 1599 CONTINUE
C
C.....UNSCRAMBLE CODEWORDS FOR ALL PARAMETERS
C
      ISPAR=0
      DO 1610 I=1,N4
      CALL PARS(I,NPAR)
        DO 1600 J=1,NPAR
        X=FLAG(I,J)
        Y=INT(ABS(X))/10
        LP(I,J)=Y+0.5
       IF(LP(I,J).NE.0.AND.I.LE.N) THEN
       ISPAR=ISPAR+1
       LPS(ISPAR)=LP(I,J)
       ENDIF
 1600   FLAG(I,J)=(ABS(X)-10.*Y)*CIGN(1.,X)
 1610 CONTINUE
C
C     DETERMINE NUMBER OF STRUCTURAL PARAMETERS
C
      NSPAR=1
      IF(ISPAR.EQ.0) GOTO 1613
      NSPAR=2
      IF(ISPAR.EQ.1) GOTO 1613
      LPPS(1)=LPS(1)
      II=1
      DO 1612 I=2,ISPAR
      LPPS(I)=0
      MLPS=0
      DO 1614 J=1,I-1
      IF(LPS(I).EQ.LPPS(J)) MLPS=1
 1614 CONTINUE
      IF(MLPS.EQ.0) THEN
      II=II+1
      LPPS(II)=LPS(I)
      NSPAR=NSPAR+1
      ENDIF
 1612 CONTINUE
 1613 CONTINUE
C
C.....CONSTRAINTS INPUT
C     STRICT CONSTRAINTS ARE INPUT FOLLOWED BY THOSE OF THE BAYESIAN TYPE
C
      IF(MAXCON.GT.MNCONF) CALL ERR(7,MNCONF)
      MATORD=MAXS
      MATSIZ=MATORD*(MATORD+1)
      MATSIZ=MATSIZ/2
C
C.....MATORD AND MATSIZ ARE RESPECTIVELY THE ORDER AND NUMBER
C     OF STORAGE LOCATIONS REQUIRED FOR THE NORMAL MATRIX
C
      IF(MATORD.GT.MMATOR) CALL ERR(8,MMATOR)
C.....CHECK THAT THE CODEWORDS INPUT ARE REASONABLE
C
      CALL CHEKUP
      IF(MAXCON.EQ.0)GOTO 1660
C
C.....READ STRICT CONSTRAINT FUNCTIONS
C
      DO 1650 I=1,MAXCON
      IL=0
 1620 IL=IL+1
C     IL COUNTS THE NUMBER OF TERMS PER CONSTRAINT FUNCTION
      IF(IL.GT.MNCONT+1) CALL ERR(9,MNCONT)
C     READ(LCA,98)CONA(I,IL),LCONI(I,IL),LCONJ(I,IL),ICON(I,IL),
C    X JCON(I,IL)
      NREAD= 1
      IRED= 1
      IDISML= -1
      CALL INITT(IBEG)
      CALL FREEIN
      IF(READIN(1)+100.) 1630,1640,1630
 1630 CONA(I,IL)= READIN(1)
      NREAD= 4
      IRED= 1
      IDISML= 1
      CALL FREEIN
      LCONI(I,IL)= IREDIN(1)
      LCONJ(I,IL)= IREDIN(2)
      ICON(I,IL)= IREDIN(3)
      JCON(I,IL)= IREDIN(4)
      GO TO 1620
 1640 LN(I)=IL-1
C     READ(LCA,98)DF(I)
      NREAD= 1
      IRED= 1
      CALL INITT(IBEG)
      CALL FREEIN
      DF(I)= READIN(1)
 1650 CONTINUE
      GO TO 1670
 1660 WRITE(LPR,202)
      GOTO 1680
 1670 WRITE(LPR,203)MAXCON
C     PRINT CONSTRAINT FUNCTIONS
      CALL CONPNT
C.....END OF RIETVELD INPUT
C
C.....TRANSFER DATA TO STORAGE USED BY SUBROUTINES
C
 1680 CALL SHIFT
      FLOG =4.*ALOG(2.)
      FLOG2=2.*FLOG
      SLOG =1./SQRT(FLOG)
C
C.....SET UP FOR PREFERRED ORIENTATION AND UNIAXIAL MAGNETIC CONTRIBUTI
C
      DO 1770 NP=1,NPAT
      IPREFF=IPREF(NP)
      IF(IPREFF) 1690,1700,1690
 1690 NPAC(NP)= 1
      GO TO 1710
 1700 NPAC(NP)= 0
 1710 CONTINUE
      IF(NPAC(NP).EQ.0)GOTO 1740
      DO 1720 I=1,3
 1720 PH(NP,I)=PH(NP,I)/HN(NP,I)
C     PH IS INDICES OF PREFERRED ORIENTATION DIRECTION
      TT(NP)=0.
      DO 1730 I=1,3
      DO 1730 J=I,3
 1730 TT(NP)=AL(NP,I,J)*PH(NP,I)*PH(NP,J)+TT(NP)
C     TT IS DSTAR**2 FOR DIRECTION PH
 1740 IF(NUNIAX(NP).EQ.0)GOTO 1770
      DO 1750 I=1,3
 1750 HL(NP,I)=HL(NP,I)/HN(NP,I)
C     HL IS INDICES OF MAGNETIC AXIS
      Z(NP)=0.
      DO 1760 I=1,3
      DO 1760 J=I,3
 1760 Z(NP)=AL(NP,I,J)*HL(NP,I)*HL(NP,J)+Z(NP)
C     Z IS DSTAR**2 FOR MAGNETIC AXIS
 1770 CONTINUE
C
C.....CYCLE THROUGH INPUT REFLECTIONS
C
 1780 DO 1960 I=1,NN
      ICODE   =FLEES(0.)+0.5
C
C.....BREAK UP COMPOSITE CODEWORD INTO PATTERN AND TYPE
C     ICODD IS SERIAL NUMBER OF PATTERN
      ICODD=ICODE/10
      ICODE=ICODE-ICODD*10
      NP  =ICODD
C
      DO1830J=1,3
 1830 HH(  J)=FLEES(0.)/HN(NP,J)
C     HH IS INDICES OF REFLECTION
      SJJ   =FLEES(0.)
C     SJJ IS MULTIPLICITY* STEP WIDTH/SQRT(PI)
      X=0.
      DO 1840 J=1,3
      DO 1840 K=J,3
 1840 X=AL(NP,J,K)*HH(  J)*HH(  K)+X
C     X IS DSTAR**2 FOR REFLECTION HH
      FAC=0.
      PAK=0.
      IC=ICODE
      IF(IC.GE.4)IC=1
      IF(NUNIAX(NP).EQ.0.OR.IC.EQ.1) GOTO 1860
      Y=0.
      DO 1850 J=1,3
      DO 1850 K=J,3
 1850 Y=AL(NP,J,K)*(HL(NP,J)*HH(K)+HL(NP,K)*HH(J))*0.5+Y
      FAC   =Y*Y/(Z(NP)*X)
C     FAC IS COS**2 FOR ANGLE BETWEEN HL AND HH
 1860 IF(NPAC(NP).EQ.0)GOTO 1910
      BB=0.
      DO 1870 J=1,3
      DO 1870 K=J,3
 1870 BB=AL(NP,J,K)*(PH(NP,J)*HH(K)+PH(NP,K)*HH(J))*0.5+BB
      BB=BB*BB/(TT(NP)*X)
C     BB IS COS**2 FOR ANGLE BETWEEN PH AND HH
      IF(BB)1890,1880,1890
 1880 BB=1.5707963268
      GOTO 1900
 1890 BB=ATAN(SQRT(ABS((1.-BB)/BB)))
C     BB IS NOW TAN OF ANGLE BETWEEN PH AND HH
 1900 PAK   =BB*BB
 1910 X=SQRT(X)*0.5
C     X IS DSTAR/2 FOR HH
      FF(1)=0.
      IF(KMN)1950,1950,1920
 1920 DO 1940 J=1,KMN
      DO 1930 K=1,MNMAGP
      K1=K-1
      IF(K1.EQ.0)K1=1
      IF(X.GT.S(J,K))GOTO 1930
C
C.....LINEAR INTERPOLATION OF SCATTERING FACTOR
C
      FF(J)=(S(J,K1)-X)/(S(J,K1)-S(J,K))*(F(J,K)-
     2F(J,K1))+F(J,K1)
      GOTO 1940
 1930 CONTINUE
      FF( J)=F(J,MNMAGP)
 1940 CONTINUE
 1950 WRITE(LT8,*)(HH(J),J=1,3),ICODE,ICODD,SJJ,FAC,PAK,(FF(J),J=1,
     1 KLOOP)
 1960 CONTINUE
      ENDFILE LT8
C.....
    1 FORMAT(20A4)
C   2 FORMAT(3I8)
C   3 FORMAT(6I8)
C   4 FORMAT(8I8)
    5 FORMAT(111H0NUCLEAR AND MAGNETIC STRUCTURE REFINEMENT PROGRAM FOR
     1NEUTRON POWDER PROFILES-FORTRAN IV BAYESIAN VERSION 1975 )
    6 FORMAT(4A4)
   11 FORMAT(49H FORCED TERMINATION WHEN ALL SHIFTS ARE LESS THAN ,F6.2
     X,9H * E.S.D.)
   13 FORMAT(68H MAGNETIC INTENSITIES CALCULATION ACCORDING TO HALPERN
     1AND JOHNSON.)
   16 FORMAT(41H UNIAXIAL  CONFIGURATIONAL SPIN SYMMETRY.)
   19 FORMAT(37H CUBIC CONFIGURATIONAL SPIN SYMMETRY.)
   22 FORMAT(33H PREFERRED ORIENTATION DIRECTION=,3F3.0)
   21 FORMAT(20H0NEUTRON WAVELENGTH ,F8.5)
C  30 FORMAT(3(4F6.2))
   32 FORMAT(9F8.2)
C  33 FORMAT(8F8.3)
   37 FORMAT(I5/6(8F10.4/))
C  40 FORMAT(7F8.4)
   41 FORMAT(1H ,I3,27H CYCLE(S) OF REFINEMENT OF ,I3,13H PARAMETER(S),
     12X,4H FOR,I2,9H PATTERNS)
   49 FORMAT(1H+,38X,33HAND THE EQUIVALENT POSITIONS ARE:)
   51 FORMAT(35H0NEUTRON NUCLEAR SCATTERING LENGTHS/)
   55 FORMAT(17H0MAGNETIC MATRIX ,I2,3X,9F6.1)
   56 FORMAT(1H0,8(F7.3,5H = B(,I1,2H);))
   57 FORMAT(/52X,18HINITIAL PARAMETERS   /23H0ATOM  NTYP  MTYP  MROT,6X
     1,1HX,9X,1HY,9X,1HZ,9X,1HB,9X,1HN,8X,2HKX,8X,2HKY,8X,2HKZ/)
   59 FORMAT(36H0NEUTRON MAGNETIC SCATTERING FACTORS)
   61 FORMAT(11H0ATOM TYPE ,I2)
   62 FORMAT(17H0SIN(THETA/LAMDA),10F10.4)
C  64 FORMAT(3F8.6)
   65 FORMAT(A4,I4,5F8.5,2I4,3F8.4)
   66 FORMAT(22H0OVERALL SCALE FACTORS,2F10.5/22H OVERALL TEMP. FACTOR ,
     1F10.5)
C  67 FORMAT(4F8.2)
   68 FORMAT(22H HALFWIDTH PARAMETERS ,3F10.2)
   69 FORMAT(11H ZEROPOINT(,I2,1H),11X,F10.3)
   75 FORMAT(6F10.8)
   76 FORMAT(16H CELL CONSTANTS ,6X,6F10.7)
C  77 FORMAT(2F8.4)
   78 FORMAT(33H PREFERRED ORIENTATION PARAMETER ,F9.3,/,
     1  21H ASYMMETRY PARAMETER ,11X,F10.6)
   82 FORMAT(1H ,50X,19HCODING OF VARIABLES/5H ATOM,24X,1HX,9X,1HY,9X,1H
     1Z,9X,1HB,9X,1HN,9X,2HKX,8X,2HKY,8X,2HKZ)
   85 FORMAT(1X,A4,16X,8F10.2)
C  86 FORMAT(8F8.2)
   89 FORMAT(22H0OVERALL SCALE FACTORS,2F10.2)
   88 FORMAT(22H OVERALL TEMP. FACTORS,F10.2)
   91 FORMAT(22H HALFWIDTH PARAMETERS ,3F10.2)
   92 FORMAT(11H ZEROPOINT ,11X,5F10.2)
   93 FORMAT(16H CELL CONSTANTS ,6X,6F10.2)
   94 FORMAT(21H ASYMMETRY PARAMETER ,1X,F10.2)
  994 FORMAT(32H DENOMINATORS OF MILLER INDICES ,3I6)
   95 FORMAT(33H PREFERRED ORIENTATION PARAMETERS,F9.2)
C  96 FORMAT(3I8)
C  98 FORMAT(F8.4,4I8)
  151 FORMAT(35H0IMAGINARY X-RAY SCATTERING FACTORS)
  159 FORMAT(30H0REAL X-RAY SCATTERING FACTORS)
  152 FORMAT(9H POSITION ,I3,10X,9F6.1,3F10.5)
  161 FORMAT(/1H ,20X,20A4/)
  171 FORMAT(1X,A4,I2,I3,2I6,8F10.5)
  202 FORMAT(32H NO STRICT CONSTRAINT FUNCTIONS.)
  203 FORMAT(1H0,I2,1X,30HSTRICT CONSTRAINT FUNCTION(S) )
  206 FORMAT(/)
  207 FORMAT(6(F6.2,3H*X(,I2,1H,,I2,4H) + ))
  208 FORMAT(3H = ,F6.2//)
  215 FORMAT(32H INCORRECT FILE LOADED ON UNIT 3)
  301 FORMAT(38H0THE STRUCTURE IS NON CENTROSYMMETRIC,)
  303 FORMAT(34H0THE STRUCTURE IS CENTROSYMMETRIC,)
  604 FORMAT(42H X-RAY REFINEMENT WITH GAUSSIAN PEAK SHAPE)
  605 FORMAT(40H X-RAY REFINEMENT WITH CAUCHY PEAK SHAPE)
  606 FORMAT(32H NEUTRON NUCLEAR SCATTERING ONLY)
  607 FORMAT(/1H ,'ASYMMETRY CORRECTION APPLIED BELOW',I6,'/100 DEGREES'
     & ,/)
  609 FORMAT(36H NO PREFERRED ORIENTATION CORRECTION)
  617 FORMAT(16X,1HF,10F10.4/)
  626 FORMAT(5H ATOM,31X,3HB11,7X,3HB22,7X,3HB33,7X,3HB12,7X,3HB13,7X,
     X3HB23)
  627 FORMAT(1H ,A4,35X,6F10.5)
  628 FORMAT(1H0,51X,17HANISOTROPIC ATOMS)
 1021 FORMAT(28H0X-RAY WAVELENGTHS: ALPHA 1 ,
     1  F8.5,10H; ALPHA 2 ,F8.5)
 1611 FORMAT(42X,15HROTATION MATRIX,25X,18HTRANSLATION VECTOR)
C1622 FORMAT(6F10.7)
 1623 FORMAT(A4,76A1)
 2010 FORMAT(37H0INCOHERENT SCATTERING CROSS SECTIONS)
 2020 FORMAT(1H0,8(F7.3,5H = C(,I1,2H);))
 2030 FORMAT(39H0ELECTRONIC SPIN FOR PARAMAGNETIC ATOMS)
 2040 FORMAT(1H0,8(F7.3,5H = S(,I1,2H);))
  221 RETURN
 2051 FORMAT(/,1X,23HRELAXATION FACTORS FOR:,/,
     A6X,17HATOMIC PARAMETERS,10X,F4.2,/,
     B6X,19HTEMPERATURE FACTORS,8X,F4.2,/,
     C6X,25HSCALE,OCCUPATION,MAGNETIC,2X,F4.2,/,
     D6X,25HZEROPOINT,HALFWIDTHS,CELL,2X,F4.2)
 2060 FORMAT(///,1X,'OPTIONS CALLED:')
 2061 FORMAT(1X,'FINAL PARAMETERS OUTPUT TO  FILE "NEUPAR"')
 2062 FORMAT(1X,'LINEPRINTER PLOT OF OBS. & CALC. INTENSITIES')
 2063 FORMAT(1X,'LINEPRINTER O/P  OF OBS. & CALC. INTENSITIES')
 2064 FORMAT(1X,'OUTPUT OF FINAL OBS. & CALC. INTENSITIES TO FILE')
 2065 FORMAT(1X,'INTENSITIES OUTPUT TO FILE FOR USE BY PLOTTING PROGRAM'
     +)
 2066 FORMAT(1X,'VARIANCE-COVARIANCE MATRIX WILL BE SENT TO OUTPUT FILE'
     +)
 2067 FORMAT(1X,'CORRELATION MATRIX WILL BE SENT TO LINEPRINTER')
 2068 FORMAT(1X,'CORRELATION COEFFS. GREATER THAN',1X,I3,1X,'PRINTED')
 2069 FORMAT(1X,'FILE FOR USE WITH FOURIER PROGRAM CREATED FOR PATTERN',
     +I3)
 2070 FORMAT(1X,'(LINE 5 CONTAINS:',11I3,')',///)
 2071 FORMAT(1X,'SUBSIDIARY PATTERNS WILL BE SUBTRACTED FROM PATTERN',I3
     +)
 2072 FORMAT(1X,'DATA FOLLOWED BY CODEWORDS FOR MORE CYCLES')
 2073 FORMAT(1X,'MINIMISATION OF PROPORTIONATE DISCREPANCIES')
      END
      SUBROUTINE BPREP                                                  B
      PARAMETER (ML=500,MP=99,MX=103)
C     ****************************************                          B
C     PREPARES B-FACTORS PRIOR TO REFINEMENT                            B
C     ****************************************                          B
C                                                                       B
      DIMENSION COMP(6)                                                 B
C                                                                       B
C                                                                       B
      COMMON /REFLN/ARG,DELTA,DLABDA(2),DUMPA(500),EPS,FLOG,FLOG2,      B
     2 OMEGA,PI,SKEW,SLABDA,SLOG,W,XN,YCALC,YOBS                        B
C                                                                       B
      COMMON /PREF/IALFA,IOMEG,IORD1,IORD2,IPREV,ISTAP,ISS,KSTART,LIMO, B
     2 LLL,NCD(50),NKEY,NO,NTEXT,NUM,NTYP(50),MEQ(50),MTYP(50)          B
C                                                                       B
      COMMON /SYMCEL/ALL(3,3),AL(5,3,3),ATEXT(60),B(8),CELL(6),CELLN(5, B
     2 6),F(4,21),FF(4),PH(5,3),RM(5,24,2,9),S(4,21),SM(5,24,3,3),T(5,  B
     3 24,3),TR(5,24),RCELL(6),RCELLN(5,6),VOLUME,VOL(5),PTEXT(5,4)     B
C                                                                       B
      COMMON /MINIM/COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),HL(3,3),  B
     2 RSA(50,3),RSB(50,3),SA(50),SB(50),SINAR(50,24),SINARI(24),       B
     3 SNEX(50),TEMP(50),TEXT(20),XL(66,39)                             B
C                                                                       B
      COMMON /BFAC/ITF,JTF(50),PRCP(5,6),ITF3,ITF4                      B
C                                                                       B
      COMMON /NUMB/N,N1,N2,N3,N4,N6,N7,NN,NN5,NA(5),NPAT,NREF,NSPAR
C                                                                       B
      DO 40 I=1,N                                                       B
      NP=NCD(I)                                                         B
      IF(JTF(I)-1)40,10,10                                              B
C     REDISTRIBUTE ISOTROPIC COMPONENT FOR ANISOTROPIC ATOM             B
C     OR CONVERT ISOTROPIC ATOM TO ANISOTROPIC                          B
10    IF(XL(I,4))20,40,20                                               B
20    XXB=XL(I,4)                                                       B
      IF(JTF(I).EQ.2) JTF(I)=1                                          B
      XL(I,4)=0.0                                                       B
      CALL CONVER(XXB,COMP,NP)                                          B
      DO 30 J=1,6                                                       B
      L=J+8                                                             B
30    XL(I,L)=XL(I,L)+COMP(J)                                           B
40    CONTINUE                                                          B
      RETURN                                                            B
      END                                                               B
      SUBROUTINE CONPNT                                                 C
      PARAMETER (ML=500,MP=99,MX=103)
C     ***************************************                           C
C     PRINTS STRICT CONSTRAINT FUNCTIONS                                C
C     ***************************************                           C
C                                                                       C
      DIMENSION ATOM(14),OVERS(15),OVER2(15),FMT(45),DUM(20)            C
C                                                                       C
C                                                                       C
      COMMON /REFLN/ARG,DELTA,DLABDA(2),DUMPA(500),EPS,FLOG,FLOG2,      C
     2 OMEGA,PI,SKEW,SLABDA,SLOG,W,XN,YCALC,YOBS                        C
C                                                                       C
      COMMON /PREF/IALFA,IOMEG,IORD1,IORD2,IPREV,ISTAP,ISS,KSTART,LIMO, C
     2 LLL,NCD(50),NKEY,NO,NTEXT,NUM,NTYP(50),MEQ(50),MTYP(50)          C
C                                                                       C
      COMMON /SYMCEL/ALL(3,3),AL(5,3,3),ATEXT(60),BB(8),CELL(6),        C
     2 CELLN(5,6),F(4,21),FF(4),PH(5,3),RM(5,24,2,9),S(4,21),SM(5,24,3, C
     3 3),T(5,24,3),TR(5,24),RCELL(6),RCELLN(5,6),VOLUME,VOL(5),        C
     4 PTEXT(5,4)                                                       C
C                                                                       C
      COMMON /MINIM/COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),HL(3,3),  C
     2 RSA(50,3),RSB(50,3),SA(50),SB(50),SINAR(50,24),SINARI(24),       C
     3 SNEX(50),TEMP(50),TEXT(20),XL(66,39)                             C
C                                                                       C
      COMMON /NUMB/N,N1,N2,N3,N4,N6,N7,NN,NN5,NA(5),NPAT,NREF,NSPAR
C                                                                       C
      COMMON /SCONTS/ICON(4,9),JCON(4,9),LCONI(4,9),LCONJ(4,9),LN(4),   C
     2 LCID(4,9),JCID(4,9),CI(4,9),CJ(4,9),CONA(4,9),DF(4)              C
C                                                                       C
      COMMON /INOUT/LCA,LPR,LPRE,LT4,LT8,LFOR,LPUN                      C
C                                                                       C
      COMMON /CONTRL/NCYCLE,MAXS,KXR(5),KKZ,LIM,LOUT,IPONS,LMAT,LCORD,  C
     2 LCOORD,ICELL,ICENT,ICENTT(5),IRL,IRLL(5),KL(5),KLL,KM(5),KMN,    C
     3 MN(5),MNN,ICYCLE,MAXCON,IFOUR,IPLOT,KLOOP,LINES,IWID,IPREF(5),   C
     4 IPREFF,NHALP(5),NUNIAX(5),NCUBIC(5),NGAUSS(5),NCAUCH(5),NPAC(5)  C
     5 ,CLOR(5),ELOR(5)
C                                                                       C
C                                                                       C
      DATA ATOM/" X"," Y"," Z"," BISO"," SITE"," MAGX"," MAGY"," MAGZ",
     1" B11"," B22"," B33"," B12"," B13"," B23"/
C
      DATA OVERS/" PREF"," ASYM"," SCAL"," SCAL"," OVER"," HFWD"," HFWD"
     1," HFWD"," ZERO"," CELL"," CELL"," CELL"," CELL"," CELL"," CELL"/
C
      DATA OVER2/" .OR."," MTRY"," E 1"," E 2"," ALLB"," TH U"," TH V",
     1" TH W"," PNT"," 1"," 2"," 3"," 4"," 5"," 6"/
C
      DATA ONEH/4H(1H /,FATE/4H,F8./,THR/4H3   /,STAR/4H,1H*/,
     1  AFOR/4H,A4 /,PLUS/4H,1H+/,B1/4H,1H(/,B2/4H,1H)/,END/4H)   /
C
C                                                                       C
C      THE OUTPUT FROM THIS SUBROUTINE IS ACHIEVED BY USE OF            C
C     DYNAMIC FORMAT STATEMENTS STORED IN THE ARRAY FMT. THE ARRAY      C
C     DUM CONTAINS THE INFORMATION TO BE PRINTED.                       C
C                                                                       C
C     THE HOLLERITH DATA IN ARRAYS ATOM, OVERS, AND OVER2               C
C     REPRESENTS KEYWORDS DENOTING THE PARAMETERS CONSTRAINED.          C
C     THE DATA IN LOCATIONS ONEH ETC. IS USED TO BUILD THE              C
C     FORMAT STATEMENT.                                                 C
C                                                                       C
C      ON A MACHINE WHICH DOES NOT ALLOW THE USE OF DYNAMIC FORMAT      C
C     THIS ROUTINE SHOULD BE REWRITTEN TO PRINT ONLY ONE                C
C     CONSTRAINT TERM PER LINE INSTEAD OF THE THREE OUTPUT HERE.        C
C     THIS COULD BE ACCOMPLISHED BY CHOOSING BETWEEN TWO FORMAT         C
C     STATEMENTS, ONE FOR A LINEAR TERM, AND A SECOND FOR A QUADRATIC.  C
C                                                                       C
C     FIRST PART OF ALL FORMATS                                         C
C                                                                       C
      FMT(1)=ONEH                                                       C
C                                                                       C
C     CYCLE THROUGH CONSTRAINTS                                         C
C                                                                       C
      DO 160 I=1,MAXCON                                                 C
C                                                                       C
C     COUNTER FOR ARRAY FMT                                             C
C                                                                       C
      LFM=1                                                             C
C                                                                       C
C     COUNTER FOR ARRAY DUM                                             C
C                                                                       C
      LDUM=0                                                            C
C                                                                       C
C     NUMBER OF TERMS IN CONSTRAINT (I)                                 C
C                                                                       C
      LNI=LN(I)                                                         C
      LIN=0                                                             C
C                                                                       C
C     PRINT THREE TERMS PER LINE                                        C
C                                                                       C
      DO 150 M=1,LNI,3                                                  C
      DO 130 K=1,3                                                      C
      LIN=LIN+1                                                         C
      IF(LNI-LIN)140,10,10                                              C
C                                                                       C
C     FORMAT FOR CONSTRAINT MULTIPLIER                                  C
C                                                                       C
10    CALL XCOPY(LFM,FATE,FMT)                                          C
      CALL XCOPY(LFM,THR,FMT)                                           C
      CALL XCOPY(LFM,STAR,FMT)                                          C
      B=CONA(I,LIN)                                                     C
      CALL XCOPY(LDUM,B,DUM)                                            C
C                                                                       C
C     FIRST PART OF CONSTRAINT TERM                                     C
C                                                                       C
      IF(LCONI(I,LIN))100,100,20                                        C
20    L1=LCONI(I,LIN)                                                   C
      L2=LCONJ(I,LIN)                                                   C
      KK=2                                                              C
30    IF(L1-N1)40,50,60                                                 C
40    B=ATOM(L2)                                                        C
C                                                                       C
C     SET UP HOLLERITH IDENTIFIERS FOR CONSTRAINED PARAMETER            C
C                                                                       C
      CALL XCOPY(LDUM,B,DUM)                                            C
      CALL XCOPY(LFM,AFOR,FMT)                                          C
      CALL XCOPY(LFM,B1,FMT)                                            C
      CALL XCOPY(LFM,AFOR,FMT)                                          C
      B=ATEXT(L1)                                                       C
      CALL XCOPY(LDUM,B,DUM)                                            C
      CALL XCOPY(LFM,B2,FMT)                                            C
      GOTO 70                                                           C
50    B=OVERS(L2)                                                       C
      CALL XCOPY(LDUM,B,DUM)                                            C
      CALL XCOPY(LFM,AFOR,FMT)                                          C
      CALL XCOPY(LFM,AFOR,FMT)                                          C
      B=OVER2(L2)                                                       C
      CALL XCOPY(LDUM,B,DUM)                                            C
      GOTO 70                                                           C
60    L2=L2+5                                                           C
      GOTO 50                                                           C
70    KK=KK-1                                                           C
      IF(KK)120,120,80                                                  C
80    IF(ICON(I,LIN))120,120,90                                         C
90    CALL XCOPY(LFM,STAR,FMT)                                          C
100   KK=1                                                              C
      IF(ICON(I,LIN))120,120,110                                        C
110   L1=ICON(I,LIN)                                                    C
      L2=JCON(I,LIN)                                                    C
      GOTO 30                                                           C
120   CALL XCOPY(LFM,PLUS,FMT)                                          C
130   CONTINUE                                                          C
C                                                                       C
C     CLOSE FORMAT ARRAY                                                C
C                                                                       C
140   CALL XCOPY(LFM,END,FMT)                                           C
C                                                                       C
C     PRINT LINE                                                        C
C                                                                       C
      WRITE(LPR,FMT)(DUM(K),K=1,LDUM)                                   C
      LFM=1                                                             C
      LDUM=0                                                            C
150   CONTINUE                                                          C
C                                                                       C
C     END OF LOOP THROUGH  A CONSTRAINT                                 C
C                                                                       C
C     PRINT CONSTANT TERM                                               C
C                                                                       C
      WRITE(LPR,180)DF(I)                                               C
160   CONTINUE                                                          C
C                                                                       C
C     END OF LOOP THROUGH ALL CONSTRAINTS                               C
C                                                                       C
      RETURN                                                            C
170   FORMAT(21H0CONSTRAINT FUNCTION ,I2)                               C
180   FORMAT(3H0= ,F10.5)                                               C
      END                                                               C
      SUBROUTINE DCELL(NP)                                              D
      PARAMETER (ML=500,MP=99,MX=103)
C     ****************************************************              D
C     THIS SUBROUTINE READS THE CELL DIMENSIONS FROM THE INPUT STREAM,  D
C     AND OBTAINS THE APPROPRIATE QUANTITIES IN THE LIST:               D
C                                                                       D
C     DIRECT CELL                           (CELL)                      D
C     RECIPROCAL CELL                       (RCELL)                     D
C     RECIPROCAL METRIC TENSOR              (AL)                        D
C     CONVERSION FACTORS FROM BISO TO BIJ   (PRCP)                      D
C     ****************************************************              D
C                                                                       D
      DIMENSION IREDIN(50),V(5)                                         D
C                                                                       D
C                                                                       D
      COMMON /REFLN/ARG,DELTA,DLABDA(2),DUMPA(500),EPS,FLOG,FLOG2,      D
     2 OMEGA,PI,SKEW,SLABDA,SLOG,W,XN,YCALC,YOBS                        D
C                                                                       D
      COMMON /PREF/IALFA,IOMEG,IORD1,IORD2,IPREV,ISTAP,ISS,KSTART,LIMO, D
     2 LLL,NCD(50),NKEY,NO,NTEXT,NUM,NTYP(50),MEQ(50),MTYP(50)          D
C                                                                       D
      COMMON /SYMCEL/ALL(3,3),AL(5,3,3),ATEXT(60),B(8),CELL(6),CELLN(5, D
     2 6),F(4,21),FF(4),PH(5,3),RM(5,24,2,9),S(4,21),SM(5,24,3,3),T(5,  D
     3 24,3),TR(5,24),RCELL(6),RCELLN(5,6),VOLUME,VOL(5),PTEXT(5,4)     D
C                                                                       D
      COMMON /MINIM/COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),HL(3,3),  D
     2 RSA(50,3),RSB(50,3),SA(50),SB(50),SINAR(50,24),SINARI(24),       D
     3 SNEX(50),TEMP(50),TEXT(20),XL(66,39)                             D
C                                                                       D
      COMMON /INOUT/LCA,LPR,LPRE,LT4,LT8,LFOR,LPUN                      D
C                                                                       D
      COMMON /BFAC/ITF,JTF(50),PRCP(5,6),ITF3,ITF4                      D
C                                                                       D
      COMMON /GEOM/DEGRAD,RADDEG                                        D
C                                                                       D
      COMMON /INPUT/NREAD,NPOINT,IRED,MODE,LINO,IDISML,ICHARS(14),      D
     2 ICARDIN(80),READIN(50)                                           D
C                                                                       D
C                                                                       D
      EQUIVALENCE (READIN(1),IREDIN(1)),(VOL,V)                         D
C                                                                       D
      DO 10 I=1,3                                                       D
      DO 10 J=1,3                                                       D
10    AL(NP,I,J)=0.0                                                    D
C     DIMENSION READIN(6)                                               D
C     EQUIVALENCE (RCELL(1),READIN(1))                                  D
C     READ(LCA,1) (READIN(I),I=1,6)                                     D
      NREAD=6                                                           D
      IBEG=1                                                            D
      IRED=1                                                            D
      CALL INITT(IBEG)                                                  D
      CALL FREEIN                                                       D
      IF(READIN(1)-1.0)20,60,60                                         D
20    ICELL=1                                                           D
      DO 50 I=1,3                                                       D
      AL(NP,I,I)=READIN(I)                                              D
      CALL PERM(I,J,K)                                                  D
      IF(J-K)30,40,40                                                   D
30    AL(NP,J,K)=READIN(I+3)                                            D
      GOTO 50                                                           D
40    AL(NP,K,J)=READIN(I+3)                                            D
50    CONTINUE                                                          D
      GOTO 140                                                          D
60    ICELL=0                                                           D
      DO 70 I=1,6                                                       D
      CELLN(NP,I)=READIN(I)                                             D
70    CONTINUE                                                          D
      DO 100 I=1,3                                                      D
      L=I+3                                                             D
      IF(CELLN(NP,L)-90.0)90,80,90                                      D
80    CELLN(NP,L)=0.0                                                   D
      GOTO 100                                                          D
90    CELLN(NP,L)=COS(DEGRAD*CELLN(NP,L))                               D
100   CONTINUE                                                          D
      CALL TRCL(CELLN,RCELLN,V,NP)                                      D
C     RCELL IS THE RECIPROCAL CELL CONSTANTS                            D
      DO 130 I=1,3                                                      D
      AL(NP,I,I)=RCELLN(NP,I)*RCELLN(NP,I)                              D
      CALL PERM(I,J,K)                                                  D
      IF(J-K)110,120,120                                                D
110   AL(NP,J,K)=2.0*RCELLN(NP,J)*RCELLN(NP,K)*RCELLN(NP,I+3)           D
      GOTO 130                                                          D
120   AL(NP,K,J)=2.0*RCELLN(NP,J)*RCELLN(NP,K)*RCELLN(NP,I+3)           D
130   CONTINUE                                                          D
      CALL BCONV(PRCP,AL,NP)                                            D
      RETURN                                                            D
140   CALL REPCEL(RCELLN,AL,NP)                                         D
      CALL TRCL(RCELLN,CELLN,V,NP)                                      D
      V(NP)=1.0/V(NP)                                                   D
      CALL BCONV(PRCP,AL,NP)                                            D
C     PRCP NOW CONTAINS THE COEFFICIENTS TO CONVERT BISO TO BIJ         D
C   1 FORMAT(6F10.4)                                                    D
      RETURN                                                            D
      END                                                               D
      SUBROUTINE CHEKUP                                                 C
      PARAMETER (ML=500,MP=99,MX=103)
C     ************************************************                  C
C     CHECKS THE VALIDITY OF, AND REORGANISES CODEWORDS IF NECESSARY    C
C     ************************************************                  C
C                                                                       C
      DIMENSION ICODE(MP),IAM(MP)                                       C
C                                                                       C
C                                                                       C
      COMMON /MATR/MATSIZ,MATORD,SIGD,V(MP),SMM(MP*(MP+1)/2)                   C
C                                                                       C
      COMMON /KEYS/LP(66,39),FLAG(66,39)                                C
C                                                                       C
      COMMON /NUMB/N,N1,N2,N3,N4,N6,N7,NN,NN5,NA(5),NPAT,NREF,NSPAR
C                                                                       C
      COMMON /MAXDIM/MNCONF,MNCONT,MMATOR,MNEUTS,MNXMAG,MNATOM,MNMAGR,  C
     2 MNEQPO,MNOVRF,MNMAGP,MNPATS                                      C
C                                                                       C
      COMMON /INOUT/LCA,LPR,LPRE,LT4,LT8,LFOR,LPUN                      C
C                                                                       C
      EQUIVALENCE (V(1),ICODE(1)),(SMM(1),IAM(1))                       C
C                                                                       C
      IN=0                                                              C
      IOUT=0                                                            C
      DO 10 I=1,MATORD                                                  C
      IAM(I)=0                                                          C
      ICODE(I)=1                                                        C
10    CONTINUE                                                          C
      DO 60 I=1,N4                                                      C
      CALL PARS(I,NPAR)                                                 C
      DO 50 J=1,NPAR                                                    C
      IF(LP(I,J))50,50,20                                               C
20    IF(LP(I,J)-MATORD)30,30,40                                        C
30    K=LP(I,J)                                                         C
      ICODE(K)=0                                                        C
      GOTO 50                                                           C
40    IOUT=IOUT+1                                                       C
      IAM(IOUT)=10000*LP(I,J)+100*I+J                                   C
50    CONTINUE                                                          C
60    CONTINUE                                                          C
C                                                                       C
C     ARE THERE ANY CODENUMBERS IN THE RANGE 1 TO MATORD                C
C                                                                       C
C     WHICH ARE NOT USED                                                C
C                                                                       C
      DO 80 I=1,MATORD                                                  C
      IF(ICODE(I))70,70,80                                              C
70    IN=IN+1                                                           C
80    CONTINUE                                                          C
      IF(IN-MATORD)110,90,90                                            C
C                                                                       C
C     NO                                                                C
C     ARE THERE ANY CODENUMBERS OUTSIDE THIS RANGE                      C
C                                                                       C
90    IF(IOUT)240,240,100                                               C
C                                                                       C
C     ERROR                                                             C
C                                                                       C
100   CALL ERR(24,0)                                                    C
C                                                                       C
C     NOT ALL CODENUMBERS IN THE RANGE 1 TO MATORD ARE USED             C
C                                                                       C
C     ARE THERE ANY GREATER THAN MATORD                                 C
C                                                                       C
110   IF(IOUT)100,100,120                                               C
120   KOUT=IOUT                                                         C
      DO 190 I=1,MATORD                                                 C
      IF(ICODE(I))190,190,130                                           C
C                                                                       C
C     CODENUMBER NOT USED                                               C
C                                                                       C
130   DO 140 K=1,IOUT                                                   C
      J=K                                                               C
      IF(IAM(J))140,140,150                                             C
140   CONTINUE                                                          C
      GOTO 200                                                          C
150   MC=IAM(J)/10000                                                   C
      L=(IAM(J)-10000*MC)/100                                           C
      K=IAM(J)-10000*MC-100*L                                           C
      WRITE(LPR,155) MC,I
  155 FORMAT(1X,'CODEWORD',I3,'CHANGED TO ',I3)
C                                                                       C
C     CODENUMBER MC WILL BE CHANGED TO I                                C
C                                                                       C
      LP(L,K)=I                                                         C
      ICODE(I)=0                                                        C
      KOUT=KOUT-1                                                       C
      IAM(J)=0                                                          C
C                                                                       C
C     ARE THER ANY OTHER CODENUMBERS WITH THE SAME VALUE                C
C                                                                       C
      J1=J+1                                                            C
      IF(J1-IOUT)160,160,190                                            C
160   DO 180 M=J1,IOUT                                                  C
      M2=IAM(M)/10000                                                   C
      IF(M2-MC)180,170,180                                              C
170   L=(IAM(M)-10000*M2)/100                                           C
      K=IAM(M)-10000*M2-100*L                                           C
      LP(L,K)=I                                                         C
      IAM(M)=0                                                          C
      KOUT=KOUT-1                                                       C
180   CONTINUE                                                          C
190   CONTINUE                                                          C
C                                                                       C
C     ARE THERE MORE VARIABLES THAN REQUIRED                            C
C                                                                       C
200   IF(KOUT)100,210,100                                               C
C                                                                       C
C     ARE THERE FEWER VARIABLES THAN REQUIRED                           C
C                                                                       C
210   IN=0                                                              C
      DO 230 I=1,MATORD                                                 C
      IF(ICODE(I))220,220,230                                           C
220   IN=IN+1                                                           C
230   CONTINUE                                                          C
      IF(IN-MATORD)100,240,240                                          C
240   RETURN                                                            C
      END                                                               C
      SUBROUTINE ITER                                                   I
      PARAMETER (ML=500,MP=99,MX=103)
C     ************************************************************      I
C     THIS ROUTINE BEGINS THE LEAST SQUARES ITERATION BY CALLING        I
C     THE ROUTINES TO CALCULATE THE MATRIX AND VECTOR                   I
C     ************************************************************      I
C                                                                       I
      LOGICAL FORCEL,FORCED,PUNCH,STAR,TYPE,VERT,YES,ANGCOU                    I
C                                                                       I
C                                                                       I
      DIMENSION IH(3),THET(1000)                                        I
C                                                                       I
C                                                                       I
      COMMON /REFLN/ARG,DELTA,DLABDA(2),DUMPA(500),EPS,FLOG,FLOG2,      I
     2 OMEGA,PI,SKEW,SLABDA,SLOG,W,XN,YCALC,YOBS                        I
C                                                                       I
      COMMON /PREF/IALFA,IOMEG,IORD1,IORD2,IPREV,ISTAP,ISS,KSTART,LIMO, I
     2 LLL,NCD(50),NKEY,NO,NTEXT,NUM,NTYP(50),MEQ(50),MTYP(50)          I
C                                                                       I
      COMMON /SYMCEL/ALL(3,3),AL(5,3,3),ATEXT(60),B(8),CELL(6),CELLN(5, I
     2 6),F(4,21),FF(4),PH(5,3),RM(5,24,2,9),S(4,21),SM(5,24,3,3),T(5,  I
     3 24,3),TR(5,24),RCELL(6),RCELLN(5,6),VOLUME,VOL(5),PTEXT(5,4)     I
C                                                                       I
      COMMON /MINIM/COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),HL(3,3),  I
     2 RSA(50,3),RSB(50,3),SA(50),SB(50),SINAR(50,24),SINARI(24),       I
     3 SNEX(50),TEMP(50),TEXT(20),XL(66,39)                             I
C                                                                       I
      COMMON /INOUT/LCA,LPR,LPRE,LT4,LT8,LFOR,LPUN                      I
C                                                                       I
      COMMON /NUMB/N,N1,N2,N3,N4,N6,N7,NN,NN5,NA(5),NPAT,NREF,NSPAR     I
C                                                                       I
      COMMON /CONTRL/NCYCLE,MAXS,KXR(5),KKZ,LIM,LOUT,IPONS,LMAT,LCORD,  I
     2 LCOORD,ICELL,ICENT,ICENTT(5),IRL,IRLL(5),KL(5),KLL,KM(5),KMN,    I
     3 MN(5),MNN,ICYCLE,MAXCON,IFOUR,IPLOT,KLOOP,LINES,IWID,IPREF(5),   I
     4 IPREFF,NHALP(5),NUNIAX(5),NCUBIC(5),NGAUSS(5),NCAUCH(5),NPAC(5)  I
     5 ,CLOR(5),ELOR(5)
C                                                                       I
      COMMON /LOGIC/FORCEL,FORCED,PUNCH,STAR,TYPE,VERT,YES,ANGCOU              I
C                                                                       I
      COMMON /REFS/HN(5,3),HH(3,ML),ICODE(ML),SJJ(ML),SOMEGA(ML),       I
     2 FOBS(ML),FESD(ML),FNUC(ML),FMAG(ML),THETA(ML),TANN(ML),HW(ML),   I
     3 DERSTO(ML,MX),AKEEP(ML),BKEEP(ML),PKEEP(ML),ICODD(ML)            I
C                                                                       I
      COMMON /BACKGR/IBACK,BAKGRD,WAVE1,WAVE3,VOLW3,VOLW(5),CROSS(8),   I
     2 BMAG(8),BAKG(5)                                                  I
C                                                                       I
      COMMON /OPTION/ISF,ISUB,IXTRA,ISW                                 I
C                                                                       I
C                                                                       I
      EQUIVALENCE (DERSTO(1,1),THET(1))                                 I
C                                                                       I
C      REWIND DATA FILE LT8                                             I
      REWIND(UNIT=LT8,IOSTAT=IOS,ERR=5)                                 I
      IF(IOS.EQ.0)GOTO6
5     CALL ERR(26,IOS)
6     IDO=1                                                             I
      LFOUR=0                                                           I
      MMM=0                                                             I
      NUM=0                                                             I
C     SELECT OUTPUT OPTIONS                                             I
10    YES=.FALSE.                                                       I
      IF(NCYCLE)30,20,30                                                I
C     PUNCH FILE FOR PLOTTING PROGRAM                                   I
20    YES=(IPONS.GE.1)                                                  I
C     PRINT OBSERVED AND CALCULATED PROFILE INTENSITIES                 I
C     AND STRUCTURE FACTORS                                             I
      TYPE=(LOUT.GE.1)                                                  I
C     PUNCH OBSERVED AND CALCULATED STRUCTURE FACTORS                   I
      PUNCH=(IPONS.EQ.1)                                                I
C     FOURIER OUTPUT REQUIRED?                                          I
      LFOUR=IFOUR                                                       I
C     SET INPUT FILE TO PROFILE DATA                                    I
30    REWIND LPRE                                                       I
C                                                                       I
C.....READ THE FIRST BLOCK OF DATA                                      I
C                                                                       I
      LLL=499                                                           I
      X=FLEES(0.)                                                       I
      NK=NN5                                                            I
40    IF(NK-500)60,50,50                                                I
50    LLL=499                                                           I
      X=FLEES(0.)                                                       I
      NK=NK-500                                                         I
      GOTO 40                                                           I
60    LLL=NK                                                            I
C                                                                       I
C.....READ PROFILE STARTING ANGLE, STEP, AND FINISHING ANGLE            I
C                                                                       I
      IALFA=FLEES(0.)+0.5                                               I
      ISTAP=FLEES(0.)+0.5                                               I
      IOMEG=FLEES(0.)+0.5                                               I
C     ANGLE.COUNT INPUT OF PROFILE DATA                                 I
      IF(IALFA-10)70,70,80                                              I
70    ANGCOU=.TRUE.                                                     I
      GOTO 90                                                           I
80    ANGCOU=.FALSE.                                                    I
      IDO=IALFA                                                         I
C     COUNTERS FOR "DO LOOP" FORMING THE MAJOR PART OF THIS ROUTINE     I
90    NSTAP=ISTAP                                                       I
      NOMEG=IOMEG                                                       I
C     GEOMETRICAL FACTOR FOR CALCULATION OF PROFILE INTENSITY FROM FCALCI
      ARG=SQRT(3.14159265359)/FLOAT(ISTAP)                              I
C     OPEN OUTPUT CHANNELS                                              I
C.....KJ IS INDEX FOR PATTERN                                           I
C                                                                       I
C     OUTPUT INITIAL DATA FOR FOURIER PROGRAM IF REQUESTED              I
      IF(LFOUR)120,120,100                                              I
100   REWIND LFOR                                                       I
C     REAL CELL CONSTANTS                                               I
      KJ=LFOUR                                                          I
C     WRITE(LFOR)CELLN(KJ,1),CELLN(KJ,2),CELLN(KJ,3),CELLN(KJ,4),       I
C    2 CELLN(KJ,5),CELLN(KJ,6)                                          I
C     NUMBER OF SYMMETRY OPERATORS                                      I
C     WRITE(LFOR)IRLL(KJ)                                               I
C     SYMMETRY OPERATORS                                                I
      DO 110 IR=1,IRLL(KJ)                                              I
C     WRITE(LFOR)((SM(KJ,IR,I,J),J=1,3),T(KJ,IR,I),I=1,3)               I
110   CONTINUE                                                          I
C                                                                       I
120   IF(.NOT.PUNCH)GOTO 140                                            I
      WRITE(LPUN,360)NN                                                 I
      DO 130 I=1,3                                                      I
      DO 130 KJ=1,NPAT
130   IH(I)=HN(KJ,I)                                                    I
      WRITE(LPUN,370)(IH(I),I=1,3)                                      I
140   IF(.NOT.ANGCOU)GOTO 150                                           I
C     ANGLE.COUNT DATA                                                  I
      NSTAP=1                                                           I
      NOMEG=9000                                                        I
150   IPREV=1                                                           I
      IOLD1=0                                                           I
      NO=0                                                              I
      IF(IBACK)160,170,160                                              I
160   CALL CALBAK                                                       I
C                                                                       I
C.....CYCLE THROUGH PROFILE POINTS CALCULATING DESIRED QUANTITIES       I
C                                                                       I
170   IORD2=0
      DO 270 IST=IDO,NOMEG,NSTAP                                        I
C     TWO THETA FOR CONSTANT STEP DATA                                  I
      ISS=IST                                                           I
C..   INPUT OBSERVED PROFILE INTENSITY                                  I
180   YOBS=FLEES(0.)                                                    I
      NO=NO+1                                                           I
      IF(ABS(YOBS).GE.0.001)GOTO 190                                    I
C     ZERO COUNT FOR THIS POINT                                         I
      YOBS=0.0                                                          I
      YCALC=0.0                                                         I
      IF(ANGCOU)ISS=FLEES(0.)+0.5                                       I
      GOTO 240                                                          I
190   NUM=NUM+1                                                         I
C     WEIGHT OF PROFILE INTENSITY                                       I
      W=FLEES(0.)                                                       I
CHEL  SW=SW+W                                                           I
C      ORDINAL NUMBERS OF FIRST AND LAST REFLECTIONS CONTRIBUTING       I
C     TO THIS POINT                                                     I
      IORD1=FLEES(0.)+0.5                                               I
      IORD2M=IORD2
      IORD2=FLEES(0.)+0.5                                               I
      IF(IORD2.LT.IORD2M) IORD2=IORD2M
C     TWO THETA FOR ANGLE.COUNT DATA                                    I
      IF(ANGCOU)ISS=FLEES(0.)+0.5                                       I
C     AVOID RECALCULATION FOR REFLECTIONS PREVIOUSLY ENCOUNTERED        I
      IF(IPREV.GT.IORD2)GOTO 230                                        I
C     DUMP UNWANTED REFLECTIONS BEFORE INPUTTING MORE                   I
      IFIN=IORD1-1                                                      I
      IF(IFIN)210,210,200                                               I
200   IF(IFIN.LT.IOLD1)GOTO 210                                         I
      CALL REFOUT(IOLD1,IFIN,LFOUR,ARG)                                 I
CAD1
      IFINL=IFIN+1
CAD1
210   IOLD1=IORD1                                                       I
C     CALCULATE DERIVATIVES AND STRUCTURE FACTORS FOR REFLECTIONS NOT   I
C     PREVIOUSLY ENCOUNTERED                                            I
      DO 220 J=IPREV,IORD2                                              I
220   CALL CALCUL(J)                                                    I
      NREF=IORD2
230   IPREV=IORD2+1                                                     I
C                                                                       I
C.....SUM DERIVATIVES TO OBTAIN ELEMENTS OF NORMAL MATRIX AND VECTOR    I
C                                                                       I
      CALL SUMMAT                                                       I
240   IF(NCYCLE)260,250,260                                             I
250   CALL PROFOT(MMM)                                                  I
C     ANY MORE PROFILE DATA?                                            I
260   IF(ISS.GE.(IOMEG-ISTAP/2))GOTO 280
270   CONTINUE                                                          I
C                                                                       I
C.....DUMP LAST BLOCK OF REFLECTIONS                                    I
C                                                                       I
CAD1   IORD1 CHANGED TO IFINL TO OUTPUT ALL(!) LAST REFLEXIONS
280   CALL REFOUT(IFINL,IORD2,LFOUR,ARG)                                I
C                                                                       I
C     FINISH OFF OUTPUT FILES                                           I
C                                                                       I
      IF(.NOT.YES)GOTO 300                                              I
      WRITE(LPUN,400)NN                                                 I
      IC=1                                                              I
      DO 290 I=1,NN                                                     I
      WRITE(LPUN,390)IC,THET(I)                                         I
290   CONTINUE                                                          I
      WRITE(LPUN,340)                                                   I
300   IF(PUNCH)WRITE(LPUN,340)                                          I
      IF(.NOT.TYPE)GOTO 310                                             I
      MMM=1                                                             I
      CALL PROFOT(MMM)                                                  I
310   IF(LFOUR)380,380,320                                              I
320   ENDFILE LFOR                                                      I
330   FORMAT(3I6/)                                                      I
340   FORMAT(/)                                                         I
350   FORMAT(1H1)                                                       I
360   FORMAT(/I8)                                                       I
370   FORMAT(3I8)                                                       I
380   RETURN                                                            I
390   FORMAT(I8,F8.1)                                                   I
400   FORMAT('   -1000',/,'  -10000',/,I8)
      END                                                               I
      SUBROUTINE REFOUT(IOLD1,IFIN,LFOUR,ARG)                           R
      PARAMETER (ML=500,MP=99,MX=103)
C     ********************************************************          R
C     DUMPS USED REFLECTIONS TO ENABLE ARRAY SPACE TO BE RE-USED        R
C     AND ACCUMULATES SUM FUNCTIONS OF F AND DELTAF                     R
C     ALSO OUTPUTS STRUCTURE FACTORS ON LAST CYCLE IF REQUESTED         R
C     ********************************************************          R
C                                                                       R
      LOGICAL FORCEL,FORCED,PUNCH,STAR,TYPE,VERT,YES,ANGCOU                    R
C                                                                       R
C                                                                       R
      DIMENSION IH(3),THET(1000)                                        R
C                                                                       R
C                                                                       R
      COMMON /REFS/HN(5,3),HH(3,ML),ICODE(ML),SJJ(ML),SOMEGA(ML),       R
     2 FOBS(ML),FESD(ML),FNUC(ML),FMAG(ML),THETA(ML),TANN(ML),HW(ML),   R
     3 DERSTO(ML,MX),AKEEP(ML),BKEEP(ML),PKEEP(ML),ICODD(ML)            R
C                                                                       R
      COMMON /INOUT/LCA,LPR,LPRE,LT4,LT8,LFOR,LPUN                      R
C                                                                       R
      COMMON /LOGIC/FORCEL,FORCED,PUNCH,STAR,TYPE,VERT,YES,ANGCOU              R
C                                                                       R
      COMMON /NUMB/N,N1,N2,N3,N4,N6,N7,NN,NN5,NA(5),NPAT,NREF,NSPAR
C                                                                       R
      COMMON /SUMS/SW,SUMAW,SUMW,SUMCAL,SUMOBS,SUMDIF,SUMWBS,SOM,       R
     2 SUMFOB,SUMDFF,FNUCDI,FNUCOB,FMAGDI,FMAGOB,SUWFOB,SUWDFF          C
C                                                                       R
      COMMON /MOVE/AH(5),BH(5),CH(5),B1(5),B2(5),ZERO(5),SCALE(5,2)     R
C                                                                       R
      COMMON /MAXDIM/MCONF,MNCONT,MMATOR,MNEUTS,MNXMAG,MNATOM,MNMAGR,   R
     2 MNEQPO,MNOVRF,MNMAGP,MNPATS                                      R
C                                                                       R
      COMMON /CONTRL/NCYCLE,MAXS,KXR(5),KKZ,LIM,LOUT,IPONS,LMAT,LCORD,  R
     2 LCOORD,ICELL,ICENT,ICENTT(5),IRL,IRLL(5),KL(5),KLL,KM(5),KMN,    R
     3 MN(5),MNN,ICYCLE,MAXCON,IFOUR,IPLOT,KLOOP,LINES,IWID,IPREF(5),   R
     4 IPREFF,NHALP(5),NUNIAX(5),NCUBIC(5),NGAUSS(5),NCAUCH(5),NPAC(5)  R
     5 ,CLOR(5),ELOR(5)
C                                                                       R
      COMMON /OPTION/ISF,ISUB,IXTRA,ISW                                 R
C                                                                       R
C                                                                       R
      EQUIVALENCE (DERSTO(1,1),THET(1))                                 R
C                                                                       R
C                                                                       R
      SAVE
      DATA ICOUNT/0/                                                    R
C                                                                       R
C     CYCLE THROUGH UNWANTED REFLECTIONS                                R
      DO 430 II=IOLD1,IFIN                                              R
C     I IS ARRAY SUBSCRIPT FOR THIS REFLECTION                          R
      I=MOD(II,MNOVRF)+1                                                R
      IC=1                                                              R
      NP=ICODD(I)                                                       R
      ICENT=ICENTT(NP)                                                  R
      IF(ICODE(I).EQ.5)IC=2                                             R
C     BB IS THE GEOMETRICAL FACTOR RELATING PROFILE INTENSITY TO        R
C     STRUCTURE FACTOR                                                  R
      BB=SJJ(I)*ARG/SCALE(NP,IC)                                        R
C     TEMPORARY STORAGE TO AVOID CONTINUAL ARRAY LOOK-UP                R
      SOMEGI=SOMEGA(I)                                                  R
      FOBSI=FOBS(I)                                                     R
      FESDI=FESD(I)                                                     R
      FNUCI=FNUC(I)                                                     R
      FMAGI=FMAG(I)                                                     R
C                                                                       R
      IF(SOMEGI)10,20,10                                                R
10    IF(FOBSI)20,20,30                                                 R
C     REJECTED REFLECTION                                               R
20    FOBSI=0.0                                                         R
      FESDI=0.0                                                         R
      GOTO 40                                                           R
C     ARE STRUCTURE FACTORS OR COUNTS REQUIRED?                         R
30    IF(ISF.EQ.1)BB=1.0                                                R
C     CORRECT 'FOBS' FOR LP, MULTIPLICITY, AND PREFERRED                R
C     ORIENTATION                                                       R
      FOBSI=FOBSI*BB/(SOMEGI*PKEEP(I))                                  R
      FESDI=SQRT(FESDI*250.0)/(SOMEGI*PKEEP(I))*BB                      R
40    FNUCI=FNUCI*BB/PKEEP(I)                                           R
      FMAGI=FMAGI*BB/PKEEP(I)                                           R
C     FNUCI,FMAGI, FOBSI ARE NOW (STRUCTURE FACTOR)**2                  R
C     ON ABSOLUTE SCALE OR COUNTS                                       R
C     ACCUMULATE SUMS                                                   R
C     SUM OVER DELTA(FOBS)                                              R
      X=FNUCI+FMAGI                                                     R
C     SUM OVER FOBS                                                     R
      SUMDFF=SUMDFF+ABS(FOBSI-ABS(X))                                   R
      SUMFOB=SUMFOB+ABS(FOBSI)                                          R
      IF (FESDI.NE.0.) THEN
       FESDQ=FESDI*FESDI
      FOBSQ=FOBSI*FOBSI
      SUWFOB=SUWFOB+FOBSQ/FESDQ
      SUWDFF=SUWDFF+(FOBSI-X)**2/FESDQ
      ENDIF
      IF(X)50,60,50                                                     R
50    X=FOBSI/X                                                         R
C     SUM OVER NUCLEAR PART OF FOBS                                     R
60    FNUCOB=FNUCOB+FNUCI*X                                             R
C     SUM OVER MAGNETIC PART OF FOBS                                    R
      FMAGOB=FMAGOB+FMAGI*X                                             R
      X=1.0-X                                                           R
C     SUM OVER DELTAS IN NUCLEAR AND MAGNETIC PARTS OF FOBS             R
      FNUCDI=FNUCDI+ABS(FNUCI*X)                                        R
      FMAGDI=FMAGDI+ABS(FMAGI*X)                                        R
      IF(.NOT.TYPE)GOTO 310                                             R
C     PRINT STRUCTURE FACTORS TO THE NEAREST INTEGER                    R
      IF(MOD(ICOUNT,LINES).EQ.0)GOTO 70                                 R
      GOTO 170                                                          R
70    WRITE(LPR,580)                                                    R
      GOTO(80,110,140,140),NPAT                                             R
80    IF(KMN)90,90,100                                                  R
90    WRITE(LPR,520)                                                    R
      GOTO 170                                                          R
100   WRITE(LPR,530)                                                    R
      GOTO 170                                                          R
110   IF(KMN)120,120,130                                                R
120   WRITE(LPR,540)                                                    R
      GOTO 170                                                          R
130   WRITE(LPR,550)                                                    R
      GOTO 170                                                          R
140   IF(KMN)150,150,160                                                R
150   WRITE(LPR,560)                                                    R
      GOTO 170                                                          R
160   WRITE(LPR,570)                                                    R
170   CONTINUE                                                          R
      ICOUNT=ICOUNT+1                                                   R
      LFTOT=FNUCI+FMAGI+0.5                                             R
      LDIF=FOBSI-FLOAT(LFTOT)+0.5                                       R
      LPOS=THETA(I)+ZERO(NP)+0.5                                        R
      DO 180 J=1,3                                                      R
      T=CIGN(0.5,HH(J,I))                                               R
180   IH(J)=HH(J,I)*HN(NP,J)+T                                          R
      LNUC=FNUCI+0.5                                                    R
      LMAG=FMAGI+0.5                                                    R
      LOBS=FOBSI+0.5                                                    R
      LESD=FESDI+0.5                                                    R
      WRITE(2,701) NP,IH(1),IH(2),IH(3),FOBSI/BB,FESDI/BB,LFTOT/BB
  701 FORMAT(4I3,3F14.3)
      WRITE(22,702) LPOS,LFTOT,NP
  702 FORMAT(2I12,I3)
C                                                                       R
      IF(NPAT.NE.1)GOTO 200                                             R
      IF(KMN.EQ.0)GOTO 190                                              R
      WRITE(LPR,440)(IH(J),J=1,3),LPOS,LNUC,LMAG,LFTOT,LOBS,LDIF,LESD   R
      GOTO 300                                                          R
190   WRITE(LPR,450)(IH(J),J=1,3),LPOS,LNUC,LOBS,LDIF,LESD              R
      GOTO 300                                                          R
200   CONTINUE                                                          R
      GOTO(270,210,240),NP                                              R
210   IF(KMN)220,220,230                                                R
220   WRITE(LPR,480)NP,(IH(J),J=1,3),LPOS,LNUC,LOBS,LDIF,LESD           R
      GOTO 300                                                          R
230   WRITE(LPR,490)NP,(IH(J),J=1,3),LPOS,LNUC,LMAG,LFTOT,LOBS,LDIF,    R
     2 LESD                                                             R
      GOTO 300                                                          R
240   IF(KMN)250,250,260                                                R
250   WRITE(LPR,500)NP,(IH(J),J=1,3),LPOS,LNUC,LOBS,LDIF,LESD           R
      GOTO 300                                                          R
260   WRITE(LPR,510)NP,(IH(J),J=1,3),LPOS,LNUC,LMAG,LFTOT,LOBS,LDIF,    R
     2 LESD                                                             R
      GOTO 300                                                          R
270   IF(KMN)280,280,290                                                R
280   WRITE(LPR,460)NP,(IH(J),J=1,3),LPOS,LNUC,LOBS,LDIF,LESD           R
      GOTO 300                                                          R
290   WRITE(LPR,470)NP,(IH(J),J=1,3),LPOS,LNUC,LMAG,LFTOT,LOBS,LDIF,    R
     2 LESD                                                             R
300   CONTINUE                                                          R
310   IF(.NOT.YES)GOTO 320                                              R
C     PUNCH CARDS CONTAINING THETA VALUES FOR REFLECTIONS               R
      THET(II)=THETA(I)+ZERO(NP)                                        R
320   IF(.NOT.PUNCH)GOTO 340                                            R
C     PUNCH OBSERVED AND CALCULATED STRUCTURE FACTORS                   R
      X=FOBSI/(FNUCI+FMAGI)                                             R
      X1=FNUCI                                                          R
      X2=X1*X                                                           R
      X3=FMAGI                                                          R
      X4=X3*X                                                           R
      DO 330 J=1,3                                                      R
      T=CIGN(0.5,HH(J,I))                                               R
330   IH(J)=HH(J,I)*HN(NP,J)+T                                          R
      WRITE(LPUN,600)(IH(J),J=1,3),X1,X2,X3,X4                          R
C     FOURIER OUTPUT                                                    R
340   IF(LFOUR.NE.NP)GOTO 430                                           R
350   IF(FOBSI)360,370,370                                              R
360   SOBS=0.0                                                          R
      GOTO 380                                                          R
370   SOBS=SQRT(FOBSI/BB)                                               R
380   SCALC=SQRT(FNUCI/BB)                                              R
      DEL=SOBS-SCALC                                                    R
      DO 390 J=1,3                                                      R
      IH(J)=HH(J,I)*HN(NP,J)+CIGN(0.5,HH(J,I))                          R
390   CONTINUE                                                          R
      IF(ICENT-1)400,410,400                                            R
400   SQAB=ABS(AKEEP(I))                                                R
      GOTO 420                                                          R
410   SQAB=SQRT(AKEEP(I)*AKEEP(I)+BKEEP(I)*BKEEP(I))                    R
420   AK=AKEEP(I)/SQAB                                                  R
      BK=BKEEP(I)/SQAB                                                  R
      IF(SOBS.LT.0.001) GOTO 430
      SESD=FESDI/(2*SOBS*BB)
      WRITE(LFOR,700)IH(1),IH(2),IH(3),SOBS,SESD,SCALC,AK,BK,DEL
700   FORMAT(13X,3I3,9X,F9.3,9X,F5.3,4F9.3)
430   CONTINUE                                                          R
440   FORMAT(4X,3I3,7I8)                                                R
450   FORMAT(4X,3I3,5I8)                                                R
460   FORMAT(1X,I3,18X,3I3,5I8)                                         R
470   FORMAT(1X,I3,18X,3I3,7I8)                                         R
480   FORMAT(1X,I3,9X,3I3,9X,5I8)                                       R
490   FORMAT(1X,I3,9X,3I3,9X,7I8)                                       R
500   FORMAT(1X,4I3,18X,5I8)                                            R
510   FORMAT(1X,4I3,18X,7I8)                                            R
520   FORMAT(6X,1HH,2X,1HK,2X,1HL,4X,4HPOSN,3X,5HICALC,4X,4HIOBS,3X,    R
     2 5HDELTA,5X,3HESD)                                                R
530   FORMAT(6X,1HH,2X,1HK,2X,1HL,4X,4HPOSN,4X,4HINUC,4X,4HIMAG,4X,     R
     2 4HITOT,4X,4HIOBS,3X,5HDELTA,5X,3HESD)                            R
540   FORMAT(3X,1HP,9X,2(2X,1HH,2X,1HK,2X,1HL),4X,4HPOSN,3X,5HICALC,4X, R
     2 4HIOBS,3X,5HDELTA,5X,3HESD)                                      R
550   FORMAT(3X,1HP,9X,2(2X,1HH,2X,1HK,2X,1HL),4X,4HPOSN,4X,4HINUC,4X,  R
     2 4HIMAG,4X,4HITOT,4X,4HIOBS,3X,5HDELTA,5X,3HESD)                  R
560   FORMAT(3X,1HP,3(2X,1HH,2X,1HK,2X,1HL),4X,4HPOSN,3X,5HICALC,4X,    R
     2 4HIOBS,3X,5HDELTA,5X,3HESD)                                      R
570   FORMAT(3X,1HP,3(2X,1HH,2X,1HK,2X,1HL),4X,4HPOSN,4X,4HINUC,4X,     R
     2 4HIMAG,4X,4HITOT,4X,4HIOBS,3X,5HDELTA,5X,3HESD)                  R
580   FORMAT(1H1)                                                       R
590   FORMAT(1X,4I3,7I8)                                                R
600   FORMAT(3I8,4F12.4)                                                R
      RETURN                                                            R
      END                                                               R
      SUBROUTINE SUMMAT                                                 S
      PARAMETER (ML=500,MP=99,MX=103)
C     ******************************************************************S
C     THIS SUBROUTINE FORMS THE ELEMENTS OF THE NORMAL MATRIX AND VECTORS
C     AND CALCULATES SEVERAL SUMS                                       S
C     ******************************************************************S
C                                                                       S
      LOGICAL FORCEL,FORCED,PUNCH,STAR,TYPE,VERT,YES,ANGCOU                    S
C                                                                       S
C                                                                       S
      DIMENSION DUMP(ML)                                                S
C                                                                       S
C                                                                       S
      COMMON /REFLN/ARG,DELTA,DLABDA(2),DUMPA(500),EPS,FLOG,FLOG2,      S
     2 OMEGA,PI,SKEW,SLABDA,SLOG,W,XN,YCALC,YOBS                        S
C                                                                       S
      COMMON /PREF/IALFA,IOMEG,IORD1,IORD2,IPREV,ISTAP,ISS,KSTART,LIMO, S
     2 LLL,NCD(50),NKEY,NO,NTEXT,NUM,NTYP(50),MEQ(50),MTYP(50)          S
C                                                                       S
      COMMON /SYMCEL/ALL(3,3),AL(5,3,3),ATEXT(60),B(8),CELL(6),CELLN(5, S
     2 6),F(4,21),FF(4),PH(5,3),RM(5,24,2,9),S(4,21),SM(5,24,3,3),T(5,  S
     3 24,3),TR(5,24),RCELL(6),RCELLN(5,6),VOLUME,VOL(5),PTEXT(5,4)     S
C                                                                       S
      COMMON /MINIM/COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),HL(3,3),  S
     2 RSA(50,3),RSB(50,3),SA(50),SB(50),SINAR(50,24),SINARI(24),       S
     3 SNEX(50),TEMP(50),TEXT(20),XL(66,39)                             S
C                                                                       S
      COMMON /CONTRL/NCYCLE,MAXS,KXR(5),KKZ,LIM,LOUT,IPONS,LMAT,LCORD,  S
     2 LCOORD,ICELL,ICENT,ICENTT(5),IRL,IRLL(5),KL(5),KLL,KM(5),KMN,    S
     3 MN(5),MNN,ICYCLE,MAXCON,IFOUR,IPLOT,KLOOP,LINES,IWID,IPREF(5),   S
     4 IPREFF,NHALP(5),NUNIAX(5),NCUBIC(5),NGAUSS(5),NCAUCH(5),NPAC(5)  S
     5 ,CLOR(5),ELOR(5)
C                                                                       S
      COMMON /MATR/MATSIZ,MATORD,SIGD,V(MP),SMM(MP*(MP+1)/2)                   S
C                                                                       S
      COMMON /MAXDIM/MNCONF,MNCONT,MMATOR,MNEUTS,MNXMAG,MNATOM,MNMAGR,  S
     2 MNEQPO,MNOVRF,MNMAGP,MNPATS                                      S
C                                                                       S
      COMMON /REFS/HN(5,3),HH(3,ML),ICODE(ML),SJJ(ML),SOMEGA(ML),       S
     2 FOBS(ML),FESD(ML),FNUC(ML),FMAG(ML),THETA(ML),TANN(ML),HW(ML),   S
     3 DERSTO(ML,MX),AKEEP(ML),BKEEP(ML),PKEEP(ML),ICODD(ML)            S
C                                                                       S
      COMMON /SUMS/SW,SUMAW,SUMW,SUMCAL,SUMOBS,SUMDIF,SUMWBS,SOM,       S
     2 SUMFOB,SUMDFF,FNUCDI,FNUCOB,FMAGDI,FMAGOB,SUWFOB,SUWDFF          C
C                                                                       S
      COMMON /LOGIC/FORCEL,FORCED,PUNCH,STAR,TYPE,VERT,YES,ANGCOU              S
C                                                                       S
      COMMON /NUMB/N,N1,N2,N3,N4,N6,N7,NN,NN5,NA(5),NPAT,NREF,NSPAR
C                                                                       S
      COMMON /KEYS/LP(66,39),FLAG(66,39)                                S
C                                                                       S
      COMMON /MOVE/AH(5),BH(5),CH(5),B1(5),B2(5),ZERO(5),SCALE(5,2)     S
C                                                                       S
      COMMON /BACKGR/IBACK,BAKGRD,WAVE1,WAVE3,VOLW3,VOLW(5),CROSS(8),   S
     2 BMAG(8),BAKG(5)                                                  S
C                                                                       S
      COMMON /BACKD/DB(50)                                              S
C                                                                       S
      COMMON /OPTION/ISF,ISUB,IXTRA,ISW                                 S
C                                                                       S
C                                                                       S
      EQUIVALENCE (COSAR(1,1),DUMP(1))                                  S
C                                                                       S
C     DUMP REUSES COSAR WHICH IS NOT NEEDED IN THIS ROUTINE             S
C                                                                       S
      IF(NCYCLE)30,30,10                                                S
C                                                                       S
C     INITIALISE WORKING SPACE                                          S
10    DO 20 J=1,MATORD                                                  S
20    DERIV(J)=0.                                                       S
30    YCALC=0.                                                          S
      BCALC=0.                                                          S
      IL=0                                                              S
C                                                                       S
C     CALCULATE THE CONTRIBUTION OF THE REFLECTIONS ORD1 TO ORD2 TO THE S
C.....DERIVATIVES OF PROFILE INTENSITY YOBS WITH RESPECT TO THE         S
C     PARAMETERS PJ. FROM CALCUL WE HAVE D(S**2)/DPJ AND WE             S
C     NOW MULTIPLY BY D(YOBS)/D(S**2) TO GET D(YOBS)/DPJ.               S
C                                                                       S
      SCAL=0.
      DO 190 I=IORD1,IORD2                                              S
C     ARRAY SUBSCRIPT FOR COMMON BLOCK /REFS/                           S
      J=MOD(I,MNOVRF)+1                                                 S
      IL=IL+1                                                           S
      IC=1                                                              S
      NP=ICODD(J)                                                       S
      N1=N+NP                                                           S
      N2=N7+NP                                                          S
      N3=N6+NP                                                          S
      IF(ICODE(J).GE.5)IC=2                                             S
C     VERT IS .TRUE. TO APPLY ASYMMETRY CORRECTION                      S
      VERT=THETA(J).LE.FLOAT(LIM)                                       S
C     DELTA IS 2THETA(ABSOLUTE PROFILE POSITION) -2THETA(REFLECTION)    S
      DELTA=FLOAT(ISS)-THETA(J)-ZERO(NP)                                S
      DELT=DELTA*DELTA                                                  S
C     BB IS (HALF WIDTH)**2                                             S
      BB=HW(J)*HW(J)                                                    S
      IF(.NOT.VERT)GOTO 40                                              S
C     CALCULATE ASYMMETRY CORRECTION                                    S
      Y=DELT*CIGN(1.,DELTA)                                             S
      Z=1.-B2(NP)*Y/TANN(J)                                             S
      IF(Z.LE.0.)Z=.0001                                                S
      GOTO 50                                                           S
40    Z=1.                                                              S
50    IF(KXR(NP).GE.5)GOTO 60                                           S
C.....GAUSSIAN HALFWIDTH FUNCTION                                       S
      IF((DELT/BB).GT.100.) THEN
      PRINT*,' YOU TRIED TO CALCULATE AT 2THETA=',ISS,
     1' INTENSITY OF REFLECTION NO ',J
      PRINT*,' WHICH IS AT',THETA(J),' AND HAS WIDTH',HW(J),
     1' ZERO AT',ZERO(NP)
      PRINT*,' INTENSITY SET TO ZERO'
      OMEGA=0.
      GOTO 70
      ENDIF
      OMEGA=SJJ(J)*EXP(-DELT/BB)/HW(J)*Z                                S
      GOTO 70                                                           S
C.....CAUCHY HALFWIDTH FUNCTION                                         S
60    OMEGA=SJJ(J)*Z/(1.+CLOR(NP)*DELT/BB/FLOG)**ELOR(NP)/HW(J)         S
C     TOTAL (STRUCTURE FACTOR)**2                                       S
70    FNN=FNUC(J)+FMAG(J)                                               S
C     SUM CALCULATED PROFILE INTENSITY ON ABSOLUTE (BCALC) AND          S
C     OBSERVED (YCALC) SCALES                                           S
      OMEF=OMEGA*FNN                                                    S
      BCALC=BCALC+OMEF                                                  S
      YCALC=YCALC+OMEF/SCALE(NP,IC)                                     S
      SCAL=SCAL+SCALE(NP,IC)
C     SUM FOBS AND FESD                                                 S
      DUMP(J)=OMEF                                                      S
      SOMEGA(J)=SOMEGA(J)+OMEGA                                         S
C     BY PASS IF NO REFINEMENT                                          S
      IF(NCYCLE.EQ.0)GOTO 191                                           S
C     X IS MULTIPLIER FOR GAUSSIAN HALFWIDTH DERIVATIVES AND            S
C     LEADS TO COMBINATION OF DERIVATIVES OF LINEAR AND                 S
C     EXPONENTIAL TERMS IN THE HALFWIDTH FUNCTION                       S
C     CYCLE THROUGH REFINED PARAMETERS                                  S
      X=(2.0*DELT/BB)-1.0                                               S
      XZC=1.
      IF (KXR(NP).GE.5) THEN
       CLO=CLOR(NP)
       X=2*CLO*DELT/FLOG/BB/(1+CLO*DELT/BB/FLOG)-1.0
       XZC=CLO/FLOG/(1+CLO*DELT/BB/FLOG)
      ENDIF
      DO 180 K=1,MATORD                                                 S
      DER=1.                                                            S
C     CHECK WHETHER PARAMETER K IS A 'SPECIAL' PARAMETER                S
C     HALFWIDTH PARAMETER                                               S
      DO 90 M=1,3                                                       S
      IF(LP(N3,M)-K)90,80,90                                            S
80    DER=X                                                             S
      GOTO 170                                                          S
90    CONTINUE                                                          S
C     CORRECT FOR ASYMMETRY                                             S
      IF(.NOT.VERT)GOTO 110                                             S
      X1=B2(NP)*CIGN(1.,DELTA)*BB/Z                                     S
C     ASYMMETRY PARAMETER?                                              S
      IF(LP(N1,2)-K)120,100,120                                         S
100   DER=Y/Z                                                           S
      GOTO 170                                                          S
110   X1=0.0                                                            S
C     ZEROPOINT?                                                        S
120   IF(LP(N6,NP)-K)140,130,140                                        S
130   DER=DELTA*(1.0+X1)*XZC                                            S
      GOTO 170                                                          S
C     CELL PARAMETER?                                                   S
140   DO 160 M=1,6                                                      S
      IF(LP(N2,M)-K)160,150,160                                         S
150   DER=DELTA*XZC                                                     S
      GOTO 170                                                          S
160   CONTINUE                                                          S
170   CONTINUE                                                          S
      IF(LP(N1,3).NE.LP(N1,4)) THEN
      IF(LP(N1,4).EQ.K.AND.IC.NE.2) GOTO 180
      IF(LP(N1,3).EQ.K.AND.IC.NE.1) GOTO 180
      ENDIF
C     SUM D(YCALC)/DPJ OVER CONTRIBUTING REFLECTIONS                    S
      DERIV(K)=DERSTO(J,K)*DER*OMEGA/SCALE(NP,IC)+DERIV(K)
180   CONTINUE                                                          S
191   CONTINUE
C      PRINT*,ISS,I,IORD1,IORD2
C      PRINT*,YOBS,YCALC,FNN
C      PRINT*,OMEGA,SCALE(NP,IC)
C      PRINT*,J,IC
C      PRINT*,SJJ(J),HW(J),THETA(J)
190   CONTINUE                                                          S
      SCAL=SCAL/(IORD2-IORD1+1)
      IF(IBACK)200,260,200                                              S
200   CALL MAGBAK                                                       S
C     ADD IN DERIVATIVES OF NUCLEAR BACKGROUND WITH RESPECT             S
C     TO SITE OCCUPATION FACTOR                                         S
      IF(NCYCLE)230,230,210                                             S
210   DO 220 I=1,MATORD                                                 S
      DERIV(I)=DERIV(I)+DB(I)                                           S
220   CONTINUE                                                          S
230   YCALC=YCALC+BAKGRD                                                S
C     ADJUST DERIVATIVE OF SCALE FACTOR TO ACCOUNT FOR BACKGROUND       S
      IF(LP(N1,3))260,260,240                                           S
240   IF(LP(N1,3)-MATORD)250,250,260                                    S
250   K=LP(N1,3)                                                        S
      DERIV(K)=YCALC*SCALE(NP,1)*SCALE(NP,1)                            S
C     RESIDUAL PROFILE INTENSITY                                        S
260   DELTA=YOBS-YCALC                                                  S
C                                                                       S
C     CALCULATE 'FOBS' AND 'FESD' BY PARTITIONING AMONG REFLECTIONS     S
C                                                                       S
C     IF(YCALC.LT.1.0E-6)YCALC=0.0                                      S
C     IF(YCALC)270,290,270                                              S
C270   WYC=1.0/(W*YCALC**2)                                              S
C     YOY=YOBS/YCALC                                                    S
      DO 280 I=IORD1,IORD2                                              S
      J=MOD(I,MNOVRF)+1                                                 S
      XB=SCAL
      IF(YCALC.NE.0.) XB=DUMP(J)/YCALC
      FESD(J)=XB*XB/W+FESD(J)
      FOBS(J)=FOBS(J)+XB*YOBS                                            S
280   CONTINUE                                                          S
C                                                                       S
C     CALCULATE SUMS OVER PROFILE POINTS                                S
C                                                                       S
C     SUM W*DELTA**2                                                    S
290   SUMW=SUMW+W*DELTA*DELTA                                           S
C     SUM W*YOBS**2                                                     S
      SUMWBS=SUMWBS+W*YOBS*YOBS                                         S
C     SUM YCALC                                                         S
      SUMCAL=SUMCAL+YCALC                                               S
C     SUM YOBS                                                          S
      SUMOBS=SUMOBS+ABS(YOBS)                                           S
C     SUM DELTA Y                                                       S
      SUMDIF=SUMDIF+ABS(DELTA)                                          S
C.....NAECHSTER BEFEHL WEGEN UMSTELLUNG DER NORMAL MATRIX UND
C.....VEKTOR AUF MESSEINHEITEN (NICHT ABSOLUTE SCALE!)
C.....VGL. AENDERUNG FUER DERIV (VOR LABEL 180)
      GOTO 307
C     IF(YCALC)300,310,300     ERSETZT DURCH DIE NAECHSTEN 4 ZEILEN
C     WEGEN XX NICHT VORBESETZT! SCHRADER
      IF(YCALC.EQ.0.)THEN
        XX=DELTA*SCAL
        GOTO 310
      ENDIF
300   XX=DELTA*BCALC/YCALC                                              S
C     SUM W*DELTA**2  ON ABSOLUTE SCALE                                 S
C     OR SCALE OF Y(OBS)                                                S
C     THIS USES A WEIGHTED SCALE FACTOR FOR X-RAY REFINEMENT            S
      IF(ISW.NE.0)XX=DELTA                                              S
  307 XX=DELTA
      SUMAW=SUMAW+W*XX*XX                                               S
C     BYPASS IF NO REFINEMENT                                           S
310   IF(NCYCLE.EQ.0)GOTO 330                                           S
C                                                                       S
C     ACCUMULATE NORMAL MATRIX AND VECTOR                               S
C                                                                       S
      IF(ISW.EQ.0)DELTA=XX                                              S
      L=0                                                               S
      DO 320 I=1,MATORD                                                 S
      X=W*DERIV(I)                                                      S
C     VECTOR                                                            S
      V(I)=V(I)+DELTA*X                                                 S
      DO 320 J=I,MATORD                                                 S
      L=L+1                                                             S
      SMM(L)=SMM(L)+X*DERIV(J)                                          S
320   CONTINUE                                                          S
330   RETURN                                                            S
      END                                                               S
      SUBROUTINE PROFOT(MMM)                                            P
      PARAMETER (ML=500,MP=99,MX=103)
C     *************************************************************     P
C     OUTPUT OF PROFILE INTENSITY DATA                                  P
C     *************************************************************     P
C                                                                       P
      LOGICAL FORCEL,FORCED,PUNCH,STAR,TYPE,VERT,YES,ANGCOU                    P
C                                                                       P
C                                                                       P
      DIMENSION KK(500),OBS(10),CALC(10),IOBS(10),ICALC(10)             P
C                                                                       P
C                                                                       P
      COMMON /REFLN/ARG,DELTA,DLABDA(2),DUMPA(500),EPS,FLOG,FLOG2,      P
     2 OMEGA,PI,SKEW,SLABDA,SLOG,W,XN,YCALC,YOBS                        P
C                                                                       P
      COMMON /PREF/IALFA,IOMEG,IORD1,IORD2,IPREV,ISTAP,ISS,KSTART,LIMO, P
     2 LLL,NCD(50),NKEY,NO,NTEXT,NUM,NTYP(50),MEQ(50),MTYP(50)          P
C                                                                       P
      COMMON /SYMCEL/ALL(3,3),AL(5,3,3),ATEXT(60),B(8),CELL(6),CELLN(5, P
     2 6),F(4,21),FF(4),PH(5,3),RM(5,24,2,9),S(4,21),SM(5,24,3,3),T(5,  P
     3 24,3),TR(5,24),RCELL(6),RCELLN(5,6),VOLUME,VOL(5),PTEXT(5,4)     P
C                                                                       P
      COMMON /MINIM/COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),HL(3,3),  P
     2 RSA(50,3),RSB(50,3),SA(50),SB(50),SINAR(50,24),SINARI(24),       P
     3 SNEX(50),TEMP(50),TEXT(20),XL(66,39)                             P
C                                                                       P
      COMMON /CONTRL/NCYCLE,MAXS,KXR(5),KKZ,LIM,LOUT,IPONS,LMAT,LCORD,  P
     2 LCOORD,ICELL,ICENT,ICENTT(5),IRL,IRLL(5),KL(5),KLL,KM(5),KMN,    P
     3 MN(5),MNN,ICYCLE,MAXCON,IFOUR,IPLOT,KLOOP,LINES,IWID,IPREF(5),   P
     4 IPREFF,NHALP(5),NUNIAX(5),NCUBIC(5),NGAUSS(5),NCAUCH(5),NPAC(5)  P
     5 ,CLOR(5),ELOR(5)
C                                                                       P
      COMMON /LOGIC/FORCEL,FORCED,PUNCH,STAR,TYPE,VERT,YES,ANGCOU              P
C                                                                       P
      COMMON /INOUT/LCA,LPR,LPRE,LT4,LT8,LFOR,LPUN                      P
C                                                                       P
      COMMON /MATR/MATSIZ,MATORD,SIGD,V(MP),SMM(MP*(MP+1)/2)                   P
C                                                                       P
      COMMON /NUMB/N,N1,N2,N3,N4,N6,N7,NN,NN5,NA(5),NPAT,NREF,NSPAR
C                                                                       P
      COMMON /MOVE/AH(5),BH(5),CH(5),B1(5),B2(5),ZERO(5),SCALE(5,2)     P
C                                                                       P
C                                                                       P
      EQUIVALENCE (SMM(1),KK(1))                                        P
C                                                                       P
C     KK STORES THETA VALUES FOR PRINTED OUTPUT OF PROFILE,AT           P
C     WHICH POINT SMM IS NO LONGER USEFUL                               P
C                                                                       P
C                                                                       P
      SAVE
      DATA NPP/0/,NT/0/                                                 P
C                                                                       P
      IF(MMM)190,10,190                                                 P
C     CHECK OUTPUT OPTIONS REQUIRED                                     P
10    IF(YES)GOTO 30                                                    P
      IF(TYPE)GOTO 30                                                   P
      IF(IPLOT)20,20,30                                                 P
C     NO OUTPUT REQUIRED                                                P
20    RETURN                                                            P
C     INDEX IN ARRAYS (I)OBS AND (I)CALC                                P
30    MO=MOD(NO,10)                                                     P
      IF(MO)40,40,50                                                    P
40    MO=10                                                             P
50    NPP=NPP+1                                                         P
      IF(NPP-1)60,60,70                                                 P
60    IALFA=ISS                                                         P
      IF(.NOT.YES)GOTO 70                                               P
      BALFA=FLOAT(IALFA)/100.
      BSTAP=FLOAT(ISTAP)/100.
      WRITE(LPUN,170)TEXT,BSTAP,BALFA                                   P
70    IF(MO-1)90,80,90                                                  P
80    NT=NT+1                                                           P
      KK(NT)=ISS                                                        P
90    IOBS(MO)=YOBS+CIGN(0.5,YOBS)                                      P
      OBS(MO)=YOBS                                                      P
      ICALC(MO)=YCALC+CIGN(0.5,YCALC)                                   P
      CALC(MO)=YCALC                                                    P
      WRITE(18,280)ISS,ICALC(MO),IOBS(MO)
C     CHECK WHETHER 10 REFLECTIONS HAVE BEEN STORED OR WHETHER          P
C     END OF DATA HAS BEEN REACHED                                      P
      IF(MO-10)100,120,120                                              P
100   IF(ISS-IOMEG)180,110,110                                          P
110   OBS(MO+1)=-100001.0                                               P
C     STORE DATA FOR SUBSEQUENT USE (IN PROFPL) OR FOR PRINTING         P
120   IF(TYPE)GOTO 130                                                  P
      IF(IPLOT)140,140,130                                              P
130   WRITE(LT4)(OBS(J),J=1,10),(CALC(J),J=1,10)                        P
C     PUNCHED OUTPUT                                                    P
140   IF(YES)WRITE(LPUN,280)(IOBS(J),J=1,MO),(ICALC(J),J=1,MO)          P
C     CHECK WHETHER THERE ARE 10*N POINTS                               P
      IF(TYPE)GOTO 150                                                  P
      IF(IPLOT)180,180,150                                              P
150   IF(ISS-IOMEG)180,160,160                                          P
160   OBS(1)=-100001.0                                                  P
      WRITE(LT4)(OBS(J),J=1,10),(CALC(J),J=1,10)                        P
170   FORMAT(20A4,/,'      10       0',F8.2,'       0       1',/,F8.2,/)P
      ENDFILE LT4                                                       P
      REWIND LT4                                                        P
180   RETURN                                                            P
C                                                                       P
C     PRINT PROFILE INTENSITIES                                         P
C                                                                       P
190   REWIND LT4
      NT=1                                                              P
      NL=LINES                                                          P
200   K=MOD(NT,NL)-1                                                    P
      IF(K)220,210,220                                                  P
210   WRITE(LPR,290)                                                    P
220   IF(NPP-10)230,240,240                                             P
230   K=NPP                                                             P
      IF(K)180,180,250                                                  P
240   K=10                                                              P
250   READ(LT4,END=180)(OBS(J),J=1,10),(CALC(J),J=1,10)
      DO 260 J=1,K                                                      P
      IOBS(J)=OBS(J)+CIGN(0.5,OBS(J))                                   P
      ICALC(J)=CALC(J)+CIGN(0.5,CALC(J))                                P
260   CONTINUE                                                          P
      WRITE(LPR,270)KK(NT),(IOBS(J),ICALC(J),J=1,K)                     P
      NT=NT+1                                                           P
      NPP=NPP-10                                                        P
      GOTO 200                                                          P
270   FORMAT(1X,I6,2X,20I6)
280   FORMAT(10I8)                                                      P
290   FORMAT(9H1POSITION,10(12H  YOBS YCALC))                            P
      END                                                               P
      SUBROUTINE MAGBAK                                                 M
      PARAMETER (ML=500,MP=99,MX=103)
C     ************************************************************      M
C     CALCULATES THE CONTRIBUTION TO THE BACKGROUND                     M
C     BY RANDOMLY ORIENTED PARAMAGNETS.                                 M
C     ************************************************************      M
C                                                                       M
      COMMON /KEYS/LP(66,39),FLAG(66,39)                                M
C                                                                       M
      COMMON /REFLN/ARG,DELTA,DLABDA(2),DUMPA(500),EPS,FLOG,FLOG2,      M
     2 OMEGA,PI,SKEW,SLABDA,SLOG,W,XN,YCALC,YOBS                        M
C                                                                       M
      COMMON /PREF/IALFA,IOMEG,IORD1,IORD2,IPREV,ISTAP,ISS,KSTART,LIMO, M
     2 LLL,NCD(50),NKEY,NO,NTEXT,NUM,NTYP(50),MEQ(50),MTYP(50)          M
C                                                                       M
      COMMON /SYMCEL/ALL(3,3),AL(5,3,3),ATEXT(60),B(8),CELL(6),CELLN(5, M
     2 6),F(4,21),FF(4),PH(5,3),RM(5,24,2,9),S(4,21),SM(5,24,3,3),T(5,  M
     3 24,3),TR(5,24),RCELL(6),RCELLN(5,6),VOLUME,VOL(5),PTEXT(5,4)     M
C                                                                       M
      COMMON /MINIM/COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),HL(3,3),  M
     2 RSA(50,3),RSB(50,3),SA(50),SB(50),SINAR(50,24),SINARI(24),       M
     3 SNEX(50),TEMP(50),TEXT(20),XL(66,39)                             M
C                                                                       M
      COMMON /NUMB/N,N1,N2,N3,N4,N6,N7,NN,NN5,NA(5),NPAT,NREF,NSPAR
C                                                                       M
      COMMON /CONTRL/NCYCLE,MAXS,KXR(5),KKZ,LIM,LOUT,IPONS,LMAT,LCORD,  M
     2 LCOORD,ICELL,ICENT,ICENTT(5),IRL,IRLL(5),KL(5),KLL,KM(5),KMN,    M
     3 MN(5),MNN,ICYCLE,MAXCON,IFOUR,IPLOT,KLOOP,LINES,IWID,IPREF(5),   M
     4 IPREFF,NHALP(5),NUNIAX(5),NCUBIC(5),NGAUSS(5),NCAUCH(5),NPAC(5)  M
     5 ,CLOR(5),ELOR(5)
C                                                                       M
      COMMON /MOVE/AH(5),BH(5),CH(5),B1(5),B2(5),ZERO(5),SCALE(5,2)     M
C                                                                       M
      COMMON /MAXDIM/MCONF,MNCONT,MMATOR,MNEUTS,MNXMAG,MNATOM,MNMAGR,   M
     2 MNEQPO,MNOVRF,MNMAGP,MNPATS                                      M
C                                                                       M
      COMMON /MATR/MATSIZ,MATORD,SIGD,V(MP),SMM(MP*(MP+1)/2)                   M
C                                                                       M
      COMMON /BACKGR/IBACK,BAKGRD,WAVE1,WAVE3,VOLW3,VOLW(5),CROSS(8),   M
     2 BMAG(8),BAKG(5)                                                  M
C                                                                       M
C     ARE THERE MAGNETIC SCATTERERS?                                    M
      IF(KMN)130,130,10                                                 M
10    SS=FLOAT(ISS)-ZERO(1)                                             M
      SS=SIN(SS/PI)/WAVE1                                               M
C                                                                       M
C     SS IS NOW SINE THETA / LAMBDA FOR THE PROFILE POINT               M
C                                                                       M
      DO 120 I=1,N                                                      M
      MT=MTYP(I)                                                        M
      NP=NCD(I)                                                         M
      ICENT=ICENTT(NP)                                                  M
      IRL=IRLL(NP)                                                      M
C                                                                       M
C     JUMP FOR NON MAGNETIC ATOMS                                       M
C                                                                       M
      IF(MT)120,120,20                                                  M
C                                                                       M
C     INTERPOLATE THE SCATTERING FACTOR                                 M
C                                                                       M
20    IF(BMAG(MT))30,120,30                                             M
30    SQRF=0.0                                                          M
      DO 70 J=1,MNMAGP                                                  M
      J1=J-1                                                            M
      IF(J1)40,40,50                                                    M
40    J1=1                                                              M
50    IF(SS-S(MT,J))60,60,70                                            M
60    SQRF=(S(MT,J1)-SS)/(S(MT,J1)-S(MT,J))*(F(MT,J)-F(MT,J1))+F(MT,J1) M
      GOTO 80                                                           M
70    CONTINUE                                                          M
      SQRF=F(MT,MNMAGP)                                                 M
80    SQRF=SQRF*SQRF*VOLW(NP)                                           M
      SQRF=SQRF*0.066229*0.01*FLOAT(IRL*ICENT*ISTAP)                    M
      SQRF=SQRF*2.4429*BMAG(MT)*(BMAG(MT)+1.0)                          M
C                                                                       M
C     SQRF IS NOW                                                       M
C     8*PI*(SPIN FACTOR)*OCCUPATION*(ATOMS PER CELL)*(FORMFACTOR**2)    M
C                                                                       M
      YCALC=YCALC+SQRF*XL(I,5)/SCALE(NP,1)                              M
      IF(NCYCLE)120,120,90                                              M
90    IF(LP(I,5))120,120,100                                            M
100   J1=LP(I,5)                                                        M
      IF(J1-MATORD)110,110,120                                          M
110   DERIV(J1)=DERIV(J1)+SQRF*FLAG(I,5)                                M
120   CONTINUE                                                          M
130   RETURN                                                            M
      END                                                               M
      SUBROUTINE CALBAK                                                 C
      PARAMETER (ML=500,MP=99,MX=103)
C     *************************************************************     C
C      CALCULATES THE BACKGROUND DUE TO NUCLEAR SCATTERING              C
C     *************************************************************     C
C                                                                       C
      DIMENSION C(5)                                                    C
C                                                                       C
      COMMON /REFLN/ARG,DELTA,DLABDA(2),DUMPA(500),EPS,FLOG,FLOG2,      C
     2 OMEGA,PI,SKEW,SLABDA,SLOG,W,XN,YCALC,YOBS                        C
C                                                                       C
      COMMON /PREF/IALFA,IOMEG,IORD1,IORD2,IPREV,ISTAP,ISS,KSTART,LIMO, C
     2 LLL,NCD(50),NKEY,NO,NTEXT,NUM,NTYP(50),MEQ(50),MTYP(50)          C
C                                                                       C
      COMMON /SYMCEL/ALL(3,3),AL(5,3,3),ATEXT(60),B(8),CELL(6),CELLN(5, C
     2 6),F(4,21),FF(4),PH(5,3),RM(5,24,2,9),S(4,21),SM(5,24,3,3),T(5,  C
     3 24,3),TR(5,24),RCELL(6),RCELLN(5,6),VOLUME,VOL(5),PTEXT(5,4)     C
C                                                                       C
      COMMON /MINIM/COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),HL(3,3),  C
     2 RSA(50,3),RSB(50,3),SA(50),SB(50),SINAR(50,24),SINARI(24),       C
     3 SNEX(50),TEMP(50),TEXT(20),XL(66,39)                             C
C                                                                       C
      COMMON /INOUT/LCA,LPR,LPRE,LT4,LT8,LFOR,LPUN                      C
C                                                                       C
      COMMON /NUMB/N,N1,N2,N3,N4,N6,N7,NN,NN5,NA(5),NPAT,NREF,NSPAR
C                                                                       C
      COMMON /CONTRL/NCYCLE,MAXS,KXR(5),KKZ,LIM,LOUT,IPONS,LMAT,LCORD,  C
     2 LCOORD,ICELL,ICENT,ICENTT(5),IRL,IRLL(5),KL(5),KLL,KM(5),KMN,    C
     3 MN(5),MNN,ICYCLE,MAXCON,IFOUR,IPLOT,KLOOP,LINES,IWID,IPREF(5),   C
     4 IPREFF,NHALP(5),NUNIAX(5),NCUBIC(5),NGAUSS(5),NCAUCH(5),NPAC(5)  C
     5 ,CLOR(5),ELOR(5)
C                                                                       C
      COMMON /MOVE/AH(5),BH(5),CH(5),B1(5),B2(5),ZERO(5),SCALE(5,2)     C
C                                                                       C
      COMMON /BACKGR/IBACK,BAKGRD,WAVE1,WAVE3,VOLW3,VOLW(5),CROSS(8),   C
     2 BMAG(8),BAKG(5)                                                  C
C                                                                       C
      COMMON /MATR/MATSIZ,MATORD,SIGD,V(MP),SMM(MP*(MP+1)/2)                   C
C                                                                       C
      COMMON /KEYS/LP(66,39),FLAG(66,39)                                C
C                                                                       C
      COMMON /BACKD/DB(50)                                              C
C                                                                       C
      IF(NCYCLE)30,30,10                                                C
10    DO 20 I=1,MATORD                                                  C
      DB(I)=0.0                                                         C
20    CONTINUE                                                          C
30    DO 40 NP=1,NPAT                                                   C
      VOLW(NP)=VOL(NP)/WAVE3                                            C
      BAKG(NP)=0.0                                                      C
40    CONTINUE                                                          C
      DO 120 I=1,N                                                      C
      NP=NCD(I)                                                         C
      J=NTYP(I)                                                         C
      IF(CROSS(J))50,120,50                                             C
50    BAKG(NP)=BAKG(NP)+XL(I,5)*CROSS(J)                                C
C                                                                       C
C     CALCULATE DERIVATIVES OF BACKGROUND WITH RESPECT TO SITE OCCUPATIOC
C                                                                       C
      IF(NCYCLE)60,60,90                                                C
60    J1=0                                                              C
      IF(LP(I,5))90,90,70                                               C
70    IF(LP(I,5)-MATORD)80,80,90                                        C
80    J1=LP(I,5)                                                        C
      DB(J1)=DB(J1)+CROSS(I)                                            C
C                                                                       C
C     BAKGRD IS SUM OCCUPATION*(CROSS SECTION)                          C
C                                                                       C
C     ADD IN CORRECTION DUE TO PARTIAL SITE OCCUPATION                  C
C     SUM OCCUPATION OVER UNIT CELL TO TEST FOR DEFICIENCY              C
C                                                                       C
90    SITE=XL(I,5)*FLOAT(ICENTT(NP)*IRLL(NP))                           C
      ISITE=SITE+0.1                                                    C
      IF(SITE-FLOAT(ISITE))120,120,100                                  C
C                                                                       C
C     CORRECT FOR PARTIAL SITE OCCUPATION                               C
C                                                                       C
C     ADD IN B**2*OCCUPATION*(1.0 -OCCUPATION)*4*PI                     C
C                                                                       C
100   C(NP)=B(J)*B(J)*12.56637                                          C
      BAKG(NP)=BAKG(NP)+C(NP)*XL(I,5)*(1.0-XL(I,5))                     C
      IF(J1)120,120,110                                                 C
110   DB(J1)=DB(J1)+C(NP)*(1.0-2.0*XL(I,5))                             C
120   CONTINUE                                                          C
C                                                                       C
C     FACTOR ISTEP*V*(NUMBER OF ATOMS)*4*PI/                            C
C     (SQRT(36000)*100*LAMBDA**3)                                       C
C                                                                       C
      DO 140 NP=1,NPAT                                                  C
      C(NP)=VOLW(NP)*0.00066229*FLOAT(IRLL(NP)*ICENTT(NP)*ISTAP)        C
C                                                                       C
C     NUCLEAR BACKGROUND ON ABSOLUTE SCALE                              C
C                                                                       C
      BAKG(NP)=BAKG(NP)*C(NP)/SCALE(NP,1)                               C
      BAKGRD=BAKGRD+BAKG(NP)                                            C
140   CONTINUE                                                          C
      DO 170 I=1,N                                                      C
      NP=NCD(I)                                                         C
      IF(LP(I,5))160,160,150                                            C
150   IF(LP(I,5)-MATORD)160,160,170                                     C
160   J1=LP(I,5)                                                        C
      DB(J1)=DB(J1)*C(NP)                                               C
170   CONTINUE                                                          C
      WRITE(LPR,180)BAKGRD                                              C
180   FORMAT(                                                           C
     2 61H0THE UNIVERSAL NUCLEAR SCATTERING CONTRIBUTION TO THE BACKGRO,C
     3 6HUND IS,F10.4)                                                  C
      DO 200 NP=1,NPAT                                                  C
      WRITE(LPR,190)NP,(PTEXT(NP,J),J=1,4),BAKG(NP)                     C
190   FORMAT(6X,7HPATTERN,I2,2X,4A4,1H:,6X,F10.4)                       C
200   CONTINUE                                                          C
      RETURN                                                            C
      END                                                               C
      SUBROUTINE CALCUL(JJ)                                             C
      PARAMETER (ML=500,MP=99,MX=103)
C     **********************************************************        C
C     CALCULATE STRUCTURE FACTOR AND DERIVATIVES FOR REFLECTION         C
C     NUMBER JJ IN THE INPUT LIST                                       C
C     THIS ROUTINE IS CALLED ONCE PER REFLECTION PER CYCLE              C
C     **********************************************************        C
C                                                                       C
      LOGICAL FORCEL,FORCED,PUNCH,STAR,TYPE,VERT,YES,ANGCOU                    C
C                                                                       C
C                                                                       C
      DIMENSION HNN(5),AX(5),BX(5),RSAI(3,3),RSBI(3,3),AXI(5),BXI(5),   C
     2 DSA(5),DSB(5),DAX(5),DBX(5),HOVD(5),COSA(50,24),SINA(50,24)      C
C                                                                       C
C                                                                       C
      COMMON /REFLN/PARG,DELTA,DLABDA(2),DUMPA(500),EPS,FLOG,FLOG2,     C
     2 OMEGA,PI,SKEW,SLABDA,SLOG,W,XN,YCALC,YOBS                        C
C                                                                       C
      COMMON /PREF/IALFA,IOMEG,IORD1,IORD2,IPREV,ISTAP,ISS,KSTART,LIMO, C
     2 LLL,NCD(50),NKEY,NO,NTEXT,NUM,NTYP(50),MEQ(50),MTYP(50)          C
C                                                                       C
      COMMON /SYMCEL/ALL(3,3),AL(5,3,3),ATEXT(60),B(8),CELL(6),CELLN(5, C
     2 6),F(4,21),FF(4),PH(5,3),RM(5,24,2,9),S(4,21),SM(5,24,3,3),T(5,  C
     3 24,3),TR(5,24),RCELL(6),RCELLN(5,6),VOLUME,VOL(5),PTEXT(5,4)     C
C                                                                       C
      COMMON /MINIM/COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),HL(3,3),  C
     2 RSA(50,3),RSB(50,3),SA(50),SB(50),SINAR(50,24),SINARI(24),       C
     3 SNEX(50),TEMP(50),TEXT(20),XL(66,39)                             C
C                                                                       C
      COMMON /REFS/HN(5,3),HH(3,ML),ICODE(ML),SJJ(ML),SOMEGA(ML),       C
     2 FOBS(ML),FESD(ML),FNUC(ML),FMAG(ML),THETA(ML),TANN(ML),HW(ML),   C
     3 DERSTO(ML,MX),AKEEP(ML),BKEEP(ML),PKEEP(ML),ICODD(ML)            C
C                                                                       C
      COMMON /NUMB/N,N1,N2,N3,N4,N6,N7,NN,NN5,NA(5),NPAT,NREF,NSPAR
C                                                                       C
      COMMON /BFAC/ITF,JTF(50),PRCP(5,6),ITF3,ITF4                      C
C                                                                       C
      COMMON /KEYS/LP(66,39),FLAG(66,39)                                C
C                                                                       C
      COMMON /CONTRL/NCYCLE,MAXS,KXR(5),KKZ,LIM,LOUT,IPONS,LMAT,LCORD,  C
     2 LCOORD,ICELL,ICENT,ICENTT(5),IRL,IRLL(5),KL(5),KLL,KM(5),KMN,    C
     3 MN(5),MNN,ICYCLE,MAXCON,IFOUR,IPLOT,KLOOP,LINES,IWID,IPREF(5),   C
     4 IPREFF,NHALP(5),NUNIAX(5),NCUBIC(5),NGAUSS(5),NCAUCH(5),NPAC(5)  C
     5 ,CLOR(5),ELOR(5)
C                                                                       C
      COMMON /LOGIC/FORCEL,FORCED,PUNCH,STAR,TYPE,VERT,YES,ANGCOU              C
C                                                                       C
      COMMON /SCONTS/ICON(4,9),JCON(4,9),LCONI(4,9),LCONJ(4,9),LN(4),   C
     2 LCID(4,9),JCID(4,9),CI(4,9),CJ(4,9),CONA(4,9),DF(4)              C
C                                                                       C
      COMMON /MATR/MATSIZ,MATORD,SIGD,V(MP),SMM(MP*(MP+1)/2)                   C
C                                                                       C
      COMMON /MOVE/AH(5),BH(5),CH(5),B1(5),B2(5),ZERO(5),SCALE(5,2)     C
C                                                                       C
      COMMON /BACKGR/IBACK,BAKGRD,WAVE1,WAVE3,VOLW3,VOLW(5),CROSS(8),   C
     2 BMAG(8),BAKG(5)                                                  C
C                                                                       C
      COMMON /OPTION/ISF,ISUB,IXTRA,ISW                                 C
C
      DIMENSION SNX(50)                                                 C
C                                                                       C
      DATA FACCUB/0.666666666667/                                       C
C                                                                       C
C.....ZEROIZE COSA,SINA,T3,T4
C
      T3=0.
      T4=0.
      DO 5 I=1,50
      DO 5 J=1,24
      COSA(I,J)=0.
    5 SINA(I,J)=0.
C.....PARAMS USED FOR ANHARMONIC REFINEMENT
C
C.....SET UP INITIAL VALUES                                             C
C                                                                       C
      PAKNN=0.0                                                         C
      CALL SETUP(JJ,NM,FAC,PAK,IC,HNN)                                  C
      DO 10 I=1,MATORD                                                  C
10    DERIV(I)=0.0                                                      C
C                                                                       C
C.....PREFERRED ORIENTATION CORRECTION                                  C
C                                                                       C
      NP=ICODD(NM)                                                      C
      IF(NPAC(NP).EQ.1)PAKNN=PAK                                        C
C     SET UP AND STORE SYMMETRY TRANSFORMED INDICES                     C
      IRL=IRLL(NP)                                                      C
      DO 40 IR=1,IRL                                                    C
      TR(NP,IR)=0.0                                                     C
      DO 30 I=1,3                                                       C
      H(IR,I)=0.0                                                       C
C     TRANSLATIONAL COMPONENT                                           C
      TR(NP,IR)=TR(NP,IR)+T(NP,IR,I)*HNN(I)                             C
      DO 20 J=1,3                                                       C
C     ROTATIONAL COMPONENT                                              C
20    H(IR,I)= SM(NP,IR,J,I)*HNN(J)+H(IR,I)                             C
30    CONTINUE                                                          C
40    CONTINUE                                                          C
      NP=ICODD(NM)                                                      C
      ICENT=ICENTT(NP)                                                  C
      N1=N+NP                                                           C
      N3=N6+NP                                                          C
      N2=N7+NP                                                          C
C                                                                       C
C.....CALCULATE OVERALL TEMPERATURE FACTOR, TWO THETA,                  C
C     AND HALFWIDTH                                                     C
C                                                                       C
      SS=0.                                                             C
      DO 50 I=1,3                                                       C
      DO 50 J=I,3                                                       C
50    SS=AL(NP,I,J)*HNN(I)*HNN(J)+SS                                    C
C     SSNN IS (SINE THETA/LAMDA)**2                                     C
      SSNN=0.25*SS                                                      C
      PKEEP(NM)=EXP(B1(NP)*PAKNN)                                       C
      BFA=EXP(-2.0*XL(N1,5)*SSNN)                                       C
      TAV=BFA*PKEEP(NM)                                                 C
C     TAV CORRECTS F**2 FOR PREFERRED ORIENTATION AND                   C
C     OVERALL TEMPERATURE FACTOR                                        C
      SINTH=SLABDA*SS                                                   C
      COSTH=1.-SINTH                                                    C
      TANTH=SQRT(SINTH/COSTH)                                           C
      TANN(NM)=TANTH                                                    C
      THETA(NM)=ATAN(TANTH)*PI                                          C
      HWQQ=AH(NP)*TANTH*TANTH+BH(NP)*TANTH+CH(NP)                       C
      IF(HWQQ.LE.0.) THEN
       PRINT*,'HALFWIDTH**2 NEGATIVE!!!!!!!!!'
       STOP
      ENDIF
      HW(NM)=SQRT(HWQQ)*SLOG                                            C
C     HW IS HALFWIDTH FOR NEUTRONS                                      C
      BB=HW(NM)**2                                                      C
C     VERT=.TRUE. IF ASYMMETRY CORRECTION IS TO BE CALCULATED           C
      VERT=.FALSE.                                                      C
      IF(THETA(NM).LE.FLOAT(LIM))VERT=.TRUE.                            C
C                                                                       C
C.....CALCULATE VECTOR PRODUCT (H.X) AND TEMPERATURE FACTOR             C
C     FOR EACH ATOM                                                     C
C                                                                       C
C     CYCLE THROUGH ATOM LIST FOR PATTERN CONCERNED                     C
      DO 130 I=1,N                                                      C
      T3=0.
      T4=0.
      IF(NCD(I).NE.NP)GOTO 130                                          C
C     CYCLE THROUGH SYMMETRY POSITIONS                                  C
      DO 120 IR=1,IRL                                                   C
      ARG=TR(NP,IR)                                                     C
      DO 70 J=1,3                                                       C
C     ACCUMULATE (H.X)                                                  C
70    ARG=ARG+H(IR,J)*XL(I,J)                                           C
      ARG=6.28318530718*ARG                                             C
C     CALCULATE ANISOTROPIC TEMPERATURE FACTOR FOR EACH ATOM            C
      IF(JTF(I))80,80,90                                                C
80    EXPARG=1.0                                                        C
      GOTO 110                                                          C
90    ARG2=0.0                                                          C
      DO 100 K=1,3                                                      C
      L=K+8                                                             C
      ARG2=XL(I,L)*H(IR,K)**2+ARG2                                      C
      L=15-K                                                            C
      CALL PERM(K,L1,L2)                                                C
100   ARG2=ARG2+2.0*H(IR,L1)*H(IR,L2)*XL(I,L)                           C
      EXPARG=EXP(-ARG2)                                                 C
110   COSA(I,IR)=COS(ARG)*EXPARG                                         C
      SINA(I,IR)=SIN(ARG)*EXPARG                                         C
      IF(JTF(I).LT.3) GOTO 119
      CALL ANHA3(I,T3,H(IR,1),H(IR,2),H(IR,3))
      IF(JTF(I).LT.4) GOTO 119
      CALL ANHA4(I,T4,H(IR,1),H(IR,2),H(IR,3))
  119 COSAR(I,IR)=COSA(I,IR)*(1.+T4)+SINA(I,IR)*T3
      SINAR(I,IR)=SINA(I,IR)*(1.+T4)-COSA(I,IR)*T3
  120 CONTINUE
      TEMP(I)=EXP(-XL(I,4)*SSNN)                                        C
130   CONTINUE                                                          C
C     JUMP FORWARD IF MAGNETIC REFLECTION ONLY                          C
      IF(ICODE(NM).EQ.2)GOTO 360                                        C
C                                                                       C
C.....CALCULATE STRUCTURE FACTOR AND DERIVATIVES FOR NUCLEAR CONTRIBUTIOC
C                                                                       C
      ICENT=ICENTT(NP)                                                  C
      AV=0.                                                             C
      BV=0.                                                             C
      CV=0.
      DV=0.
C     CYCLE THROUGH ATOM LIST FOR PATTERN CONCERNED                     C
      DO 170 I=1,N                                                      C
C     REJECT DUMMY ATOMS                                                C
      IF(NTYP(I).EQ.0)GOTO 170                                          C
      IF(NCD(I).NE.NP)GOTO 170                                          C
      SNXI=0.                                                           C
      SAI=0.                                                            C
      SBI=0.                                                            C
      DO 140 IR=1,IRL                                                   C
C     SUM COS(H.X) AND SIN(H.X) CORRECTED FOR ANISOTROPIC               C
C     VIBRATION                                                         C
      SAI=SAI+COSAR(I,IR)                                               C
      IF(ICENT.EQ.1)SBI=SINAR(I,IR)+SBI                                 C
140   CONTINUE                                                          C
      SAI=FLOAT(ICENT)*SAI                                              C
      SA(I)=SAI                                                         C
      SB(I)=SBI                                                         C
C     GET ATOMIC SCATTERING FACTOR                                      C
      NI=NTYP(I)                                                        C
      BNI=B(NI)                                                         C
C     JUMP IF NEUTRON REFINEMENT                                        C
      IF(KXR(1).LT.4)GOTO 150                                           C
C     (REAL X-RAY FORMFACTOR)*(ISOTROPIC B-FACTOR)                      C
      SNEXI=FF(NI)*TEMP(I)*XL(I,5)                                      C
C     (ANOMALOUS X-RAY FORMFACTOR)*(OCCUPATION)*(ISO. B-FACTOR)         C
      SNXI=BNI*XL(I,5)*TEMP(I)                                          C
      GOTO 160                                                          C
C     (NEUTRON NUCLEAR FORMFACTOR)*(ISOTROPIC B-FACTOR)                 C
150   SNEXI=BNI*TEMP(I)*XL(I,5)                                         C
C     AV AND BV ACCUMULATE THE REAL PARTS OF THE STRUCTURE FACTOR       C
160   AV=SNEXI*SAI+AV                                                   C
      BV=SNEXI*SBI+BV                                                   C
C     STORE 2*(REAL FORMFACTOR)*(ISOTROPIC B-FACTOR)                    C
C     *(OVERALL B-FACTOR)*(PREFERRED ORIENTATION CORRECTION)            C
      SNEX(I)=2.*SNEXI*TAV                                              C
      SNX(I)=2.*SNXI*TAV
C     JUMP IF NEUTRON REFINEMENT                                        C
      IF(KXR(1).LT.4)GOTO 170                                           C
C     CONTRIBUTORS TO X-RAY F**2 ACCOUNTING FOR ANOMALOUS               C
C     DISPERSION                                                        C
      CV=CV+SNXI*SAI                                                    C
      DV=DV+SNXI*SBI                                                    C
170   CONTINUE                                                          C
      AVI=AV-DV
      BVI=CV+BV
      IF(IFOUR)190,190,180                                              C
180   AKEEP(NM)=AV                                                      C
      BKEEP(NM)=BV                                                      C
C     JUMP IF NEUTRON REFINEMENT                                        C
190   IF(KXR(1).LT.4)GOTO 200                                           C
C     F**2 FOR X-RAYS CORRECTED FOR OVERALL B AND                       C
C     PREFERRED ORIENTATION                                             C
C?????FNN=(AVI*AVI+AVJ*AVJ+BVI*BVI+BVJ*BVJ)*TAV/2.                      C
      FNN=(AVI*AVI+BVI*BVI)*TAV                                         C
      GOTO 210                                                          C
C     F**2 FOR NEUTRONS CORRECTED FOR OVERALL B AND                     C
C     PREFERRED ORIENTATION                                             C
200   FNN=(AV*AV+BV*BV)*TAV                                             C
C     STORE FCALC, AND REAL PARTS OF A AND B                            C
210   FNUC(NM)=FNN                                                      C
      X=SQRT(BFA)                                                       C
      AKEEP(NM)=AV*X                                                    C
      BKEEP(NM)=BV*X                                                    C
C     BY-PASS DERIVATIVE CALCULATION IF NO REFINEMENT                   C
      IF(NCYCLE.EQ.0)GOTO 350                                           C
C                                                                       C
C.....CALCULATE DERIVATIVES OF NUCLEAR STRUCTURE FACTOR                 C
C     WITH RESPECT TO ATOMIC PARAMETERS                                 C
C                                                                       C
      DO 340 I=1,N                                                      C
C     REJECT DUMMY ATOMS                                                C
      IF(NTYP(I).EQ.0)GOTO 340                                          C
      IF(NCD(I).NE.NP)GOTO 340                                          C
      SNEXI=SNEX(I)                                                     C
      SNXI=SNX(I)
C     SA(I), SB(I) ARE SUMS OF COS(H.X) AND SIN(H.X) OVER EQUIVALENT    C
C     POSITIONS FOR ATOM I                                              C
      SAI=SA(I)                                                         C
      SBI=SB(I)                                                         C
      DO 220 IR=1,IRL                                                   C
      SINARI(IR)=SINAR(I,IR)                                            C
220   COSARI(IR)=COSAR(I,IR)                                            C
      CALL PARS(I,NPAR)                                                 C
      DO 330 J=1,NPAR                                                   C
C     JUMP IF MAGNETIC PARAMETERS                                       C
      IF(J.LT.9.AND.J.GT.5)GOTO 330                                     C
C     LEAST SQUARES PARAMETER NUMBER                                    C
      K=LP(I,J)                                                         C
      IF(K.EQ.0)GOTO 330                                                C
C     ANISOTROPIC TEMPERATURE FACTOR?                                   C
      IF(J-9)230,280,280                                                C
230   GOTO(240,240,240,260,270),J                                       C
C     COORDINATES                                                       C
240   SUMA=0.                                                           C
      SUMB=0.                                                           C
      DO 250 IR=1,IRL                                                   C
      SUMA=H(IR,J)*SINARI(IR)+SUMA                                      C
      IF(ICENT.EQ.1)SUMB=H(IR,J)*COSARI(IR)+SUMB                        C
250   CONTINUE                                                          C
      SUMAA=SNEXI*SUMA+SNXI*SUMB
      SUMBB=-SNXI*SUMA+SNEXI*SUMB
      DER=-(AVI*SUMAA-BVI*SUMBB)*FLOAT(ICENT)*6.28318530718             C
      GOTO 320                                                          C
C     ISOTROPIC B-FACTOR                                                C
260   SAAI=SNEXI*SAI-SNXI*SBI
      SBBI=SNXI*SAI+SNEXI*SBI
      DER=-(SAAI*AVI+SBBI*BVI)*SSNN                                     C
      GOTO 320                                                          C
C     SITE OCCUPATION                                                   C
270   SAAI=SNEXI*SAI-SNXI*SBI
      SBBI=SNXI*SAI+SNEXI*SBI
      DER=(SAAI*AVI+SBBI*BVI)/XL(I,5)                                   C
      GOTO 320                                                          C
C     ANISOTROPIC TEMPERATURE FACTORS                                   C
  280 IF(J.GT.14) GOTO 315
      IF(J.GT.11)GOTO 290                                               C
C     B(I,I)                                                            C
      K1=J-8                                                            C
      K2=K1                                                             C
      GOTO 300                                                          C
C     B(I,J)                                                            C
290   L=15-J                                                            C
      CALL PERM(L,K1,K2)                                                C
300   SUMA=0.                                                           C
      SUMB=0.                                                           C
      DO 310 IR=1,IRL                                                   C
      SUMA=SUMA+H(IR,K1)*H(IR,K2)*COSARI(IR)                            C
      IF(ICENT.EQ.1)SUMB=SUMB+H(IR,K1)*H(IR,K2)*SINARI(IR)              C
310   CONTINUE                                                          C
      SUMAA=SNEXI*SUMA-SNXI*SUMB
      SUMBB=SNXI*SUMA+SNEXI*SUMB
      DER=-(AVI*SUMAA+BVI*SUMBB)*FLOAT(ICENT)                           C
      IF(K1.NE.K2)DER=2.*DER                                            C
      GOTO 320
C     3RD ORDER ANHARMONIC
  315 IF(J.GT.24) GOTO 316
      CALL PERM3(J,K1,K2,K3,FAC3)
      SUMA=0.
      SUMB=0.
      DO 313 IR=1,IRL
      SUMA=SUMA+H(IR,K1)*H(IR,K2)*H(IR,K3)*FAC3*SINA(I,IR)
      IF(ICENT.EQ.2) GOTO 313
      SUMB=SUMB+H(IR,K1)*H(IR,K2)*H(IR,K3)*FAC3*COSA(I,IR)
  313 CONTINUE
      SUMAA=SNEXI*SUMA-SNXI*SUMB
      SUMBB=SNXI*SUMA+SNEXI*SUMB
      DER=(AVI*SUMAA-BVI*SUMBB)*FLOAT(ICENT)
      GOTO 320
C     4RTH ORDER ANHARMONIC
  316 CALL PERM4(J,K1,K2,K3,K4,FAC4)
      SUMA=0.
      SUMB=0.
      DO 314 IR=1,IRL
      SUMA=SUMA+H(IR,K1)*H(IR,K2)*H(IR,K3)*H(IR,K4)*FAC4*COSA(I,IR)
      IF(ICENT.EQ.2) GOTO 314
      SUMB=SUMB+H(IR,K1)*H(IR,K2)*H(IR,K3)*H(IR,K4)*FAC4*SINA(I,IR)
  314 CONTINUE
      SUMAA=SNEXI*SUMA-SNXI*SUMB
      SUMBB=SNXI*SUMA+SNEXI*SUMB
      DER=(AVI*SUMAA+BVI*SUMBB)*FLOAT(ICENT)
C     STORE DERIVATIVES                                                 C
320   DERIV(K)=FLAG(I,J)*DER+DERIV(K)                                   C
330   CONTINUE                                                          C
340   CONTINUE                                                          C
C     DERIVATIVES FOR OVERALL PARAMETERS                                C
      CALL DOVER(PAKNN,FNN,SSNN,NP,IC)                                  C
350   IF(ICODE(NM).EQ.1)GOTO 1000                                       C
      IF(ICODE(NM).GE.4)GOTO 1000                                       C
C                                                                       C
C                                                                       C
C.....CALCULATE MAGNETIC CONTRIBUTION TO STRUCTURE FACTOR               C
C     AND DERIVATIVES                                                   C
C                                                                       C
C                                                                       C
C     CYCLE THROUGH ATOM LIST                                           C
360   DO 370 I=1,3                                                      C
      AX(I)=0.0                                                         C
      BX(I)=0.0                                                         C
370   CONTINUE                                                          C
      DO 420 I=1,N                                                      C
C     REJECT NON-MAGNETIC ATOMS                                         C
      IF(MTYP(I).EQ.0)GOTO 420                                          C
      IF(NCD(I).NE.NP)GOTO 420                                          C
      MEQI=MEQ(I)                                                       C
      MTYPI=MTYP(I)                                                     C
C     (MAGNETIC FORMFACTOR)*(ISOTROPIC B-FACTOR)                        C
      SNEXI=FF(MTYPI)*TEMP(I)*0.2695                                    C
      SNEX(I)=SNEXI                                                     C
      SNEXI=SNEXI*XL(I,5)                                               C
      DO 380 IR=1,IRL                                                   C
      SINARI(IR)=SINAR(I,IR)                                            C
380   COSARI(IR)=COSAR(I,IR)                                            C
C     SUM (MAGNETIC VECTOR COMPONENT)* COS(H.X) OR SIN(H.X)             C
C     OVER EQUIVALENT POSITIONS                                         C
      LOOP1=1                                                           C
      LOOP2=IRL                                                         C
      CALL MAGSYM(LOOP1,LOOP2,MEQI,RSAI,RSBI,NP)                        C
      DO 400 J=1,3                                                      C
      AXI(J)=0.                                                         C
      BXI(J)=0.                                                         C
C.....MULTIPLY MATRICES RSAI AND RSBI BY COMPONENTS OF  MAGNETIC        C
C     VECTOR TO GET CONTRIBUTIONS A AND B TO MAGNETIC                   C
C     STRUCTURE FACTOR P                                                C
      DO 390 K=1,3                                                      C
      AXI(J)=XL(I,K+5)*RSAI(J,K)+AXI(J)                                 C
      IF(ICENT.EQ.1)BXI(J)=XL(I,K+5)*RSBI(J,K)+BXI(J)                   C
390   CONTINUE                                                          C
      RSA(I,J)=AXI(J)                                                   C
      RSB(I,J)=BXI(J)                                                   C
C     RSA AND RSB COMTAIN COMPONENTS OF A AND B MULTIPLIED BY           C
C     MAGNETIC VECTOR                                                   C
400   CONTINUE                                                          C
      DO 410 J=1,3                                                      C
C.....MULTIPLY COMPONENTS OF A AND B BY SCATTERING AND B-               C
C     FACTORS AND SITE OCCUPATION                                       C
C     (BUT NOT PREFERRED ORIENTATION OR OVERALL B-FACTOR)               C
      AX(J)=AXI(J)*SNEXI+AX(J)                                          C
      IF(ICENT.EQ.1)BX(J)=BXI(J)*SNEXI+BX(J)                            C
410   CONTINUE                                                          C
      SNEX(I)=SNEX(I)*TAV                                               C
420   CONTINUE                                                          C
C     MAGNETIC COHERENT SCATTERING CROSS SECTION ACCORDING              C
C     TO HALPERN AND JOHNSON(1939)                                      C
      IF(NHALP(NP).EQ.0)GOTO 440                                        C
      X=0.                                                              C
      Y=0.                                                              C
      DO 430 I=1,3                                                      C
      DO 430 J=I,3                                                      C
      K=9-I-J                                                           C
      X1=1.                                                             C
      IF(I.NE.J)X1=2.*CELLN(NP,K)                                       C
C     X ACCUMULATES MAGNETIC STRUCTURE FACTOR P**2                      C
      X=X1*AX(I)*AX(J)+X                                                C
      X1=2.                                                             C
      IF(I.EQ.J)X1=1.                                                   C
C     Y ACCUMULATES THE SCALAR PRODUCT (E.P)**2                         C
      Y=X1*HNN(I)*HNN(J)*AX(I)*AX(J)/(CELLN(NP,I)*CELLN(NP,J))+Y        C
      IF(ICENT.NE.1)GOTO 430                                            C
      X1=1.                                                             C
      IF(I.NE.J)X1=2.*CELLN(NP,K)                                       C
      X=X1*BX(I)*BX(J)+X                                                C
      X1=2.                                                             C
      IF(I.EQ.J)X1=1.                                                   C
      Y=X1*HNN(I)*HNN(J)*BX(I)*BX(J)/(CELLN(NP,I)*CELLN(NP,J))+Y        C
430   CONTINUE                                                          C
C     FNN IS THE MAGNETIC COHERENT SCATTERING CROSS SECTION             C
C     SS NORMALISES VECTOR E TO UNIT VECTOR                             C
      FNN=X-Y/SS                                                        C
      GOTO 480                                                          C
C                                                                       C
C.....CALCULATE MAGNETIC COHERENT SCATTERING CROSS SECTION              C
C     FOR UNIAXIAL SPIN SYMMETRY                                        C
C                                                                       C
440   IF(NUNIAX(NP).EQ.0)GOTO 460                                       C
C     FAC IS COS**2 FOR THE ANGLE (ETA) BETWEEN THE VECTOR              C
C     D* AND THE UNIQUE AXIS                                            C
      FACCOS=0.5*(1.0+FAC)                                              C
C     FACCOS= (1 +COS**2(ETA))/2                                        C
      FACSIN=1-FAC                                                      C
C     FACSIN= SIN**2(ETA)                                               C
      X=0.                                                              C
      Y=2.0*CELLN(NP,6)                                                 C
      DO 450 I=1,2                                                      C
      DO 450 J=I,2                                                      C
      X1=Y                                                              C
      IF(I.EQ.J)X1=1.                                                   C
C     X ACCUMULATES (AX +AY)**2 +(BX +BY)**2                            C
      X=X1*AX(I)*AX(J)+X                                                C
      IF(ICENT.EQ.1)X=X1*BX(I)*BX(J)+X                                  C
450   CONTINUE                                                          C
      X1=0.                                                             C
      IF(ICENT.EQ.1)X1=BX(3)*BX(3)                                      C
C     MAGNETIC COHERENT SCATTERING CROSS SECTION ACCORDING              C
C     TO RIETVELD'S FORMULA. IS THIS CORRECT WHEN UNIQUE AXIS           C
C     IS NOT (001)?                                                     C
      FNN=(X1+AX(3)*AX(3))*FACSIN+FACCOS*X                              C
      GOTO 480                                                          C
C..... MAGNETIC COHERENT SCATTERING CROSS SECTION FOR CUBIC             C
C     SPIN CONFIGURATION                                                C
460   IF(NCUBIC(NP).EQ.0)GOTO 480                                       C
      X=0.                                                              C
      DO 470 I=1,3                                                      C
C     X ACCUMULATES AX**2 +BX**2 +AY**2 +BY**2 +AZ**2 +BZ**2            C
      X=AX(I)*AX(I)+X                                                   C
      IF(ICENT.EQ.1)X=BX(I)*BX(I)+X                                     C
470   CONTINUE                                                          C
C     MULTIPLY BY 2/3 TO GET MAGNETIC CROSS SECTION                     C
      FNN=FACCUB*X                                                      C
C                                                                       C
C.....CORRECT FINAL MAGNETIC COHERENT SCATTERING CROSS                  C
C     SECTION FOR PREFERRED ORIENTATION AND OVERALL B-FACTOR            C
C                                                                       C
480   FNN=FNN*TAV                                                       C
C     STORE MAGNETIC COHERENT SCATTERING CROSS SECTION                  C
      FMAG(NM)=FNN                                                      C
C     SKIP DERIVATIVE CALCULATION IF NO REFINEMENT                      C
      IF(NCYCLE.EQ.0)GOTO 1160                                          C
C     CALCULATE DERIVATIVES OF COHERENT CROSS SECTION(J**2)             C
C     WITH RESPECT TO AX,AY,AZ,BX,BY,BZ AND STORE IN DSA, DSB           C
      DO 490 I=1,3                                                      C
      DSA(I)=0.                                                         C
490   DSB(I)=0.                                                         C
      IF(NHALP(NP).EQ.0)GOTO 550                                        C
C     DERIVATIVES ACCORDING TO HALPERN AND JOHNSON                      C
      FACTA=0.0                                                         C
      FACTB=0.0                                                         C
      DO 500 I=1,3                                                      C
500   HOVD(I)=HNN(I)/CELLN(NP,I)                                        C
      DO 520 I=1,3                                                      C
      FACTA=FACTA+HOVD(I)*AX(I)                                         C
      IF(ICENT-1)520,510,520                                            C
510   FACTB=FACTB+HOVD(I)*BX(I)                                         C
520   CONTINUE                                                          C
      FACTA=FACTA/SS                                                    C
      FACTB=FACTB/SS                                                    C
      DO 540 I=1,3                                                      C
      CALL PERM(I,J,K)                                                  C
      DSA(I)=2.0*(AX(I)+AX(J)*CELLN(NP,K+3)+AX(K)*CELLN(NP,J+3)-HOVD(I) C
     2 *FACTA)                                                          C
      IF(ICENT-1)540,530,540                                            C
530   DSB(I)=2.0*(BX(I)+BX(J)*CELLN(NP,K+3)+BX(K)*CELLN(NP,J+3)-HOVD(I) C
     2 *FACTB)                                                          C
540   CONTINUE                                                          C
      GOTO 590                                                          C
550   IF(NUNIAX(NP).EQ.0)GOTO 570                                       C
C     DERIVATIVES OF J**2 WITH RESPECT TO AX,BX FOR UNIAXIAL CASE       C
      DO 560 I=1,2                                                      C
C     CALL PERM(I,J,K)                                                  C
      J=I+1
      IF(J.GT.2) J=J-2
      DSA(I)=(AX(J)*CELLN(NP,6)+AX(I))*2.*FACCOS                        C
      IF(ICENT.EQ.1)DSB(I)=(BX(J)*CELLN(NP,6)+BX(I))*2.*FACCOS          C
560   CONTINUE                                                          C
      DSA(3)=AX(3)*2.*FACSIN                                            C
      IF(ICENT.EQ.1)DSB(3)=BX(3)*2.*FACSIN                              C
      GOTO 590                                                          C
C     DERIVATIVES OF J**2 WITH RESPECT TO AX, BX FOR                    C
C     CUBIC SPIN CONFIGURATION                                          C
570   IF(NCUBIC(NP).EQ.0)GOTO 590                                       C
      DO 580 I=1,3                                                      C
      DSA(I)=AX(I)*2.*FACCUB                                            C
      IF(ICENT.EQ.1)DSB(I)=BX(I)*2.*FACCUB                              C
580   CONTINUE                                                          C
C                                                                       C
C.....CALCULATE DERIVATIVES OF COMPONENTS AX,BX ETC. WITH RESPECT       C
C     TO PARAMETERS                                                     C
C                                                                       C
590   DO 990 I=1,N                                                      C
C     AVOID DERIVATIVE CALCULATION FOR NON-MAGNETIC ATOM                C
600   IF(MTYP(I).EQ.0)GOTO 990                                          C
      IF(NCD(I).NE.NP)GOTO 990                                          C
C     RETRIEVE (SCATTERING FACTOR)*(ISOTROPIC B-FACTOR)                 C
C     *(OCCUPATION)* (E**2)*GAMMA/2MC**2                                C
      SNEXI=SNEX(I)*XL(I,5)                                             C
C     MAGNETIC VECTOR ROTATION MATRIX INDEX                             C
      MEQI=MEQ(I)                                                       C
C     RETRIEVE COSINE AND SINE TERMS                                    C
      DO 610 IR=1,IRL                                                   C
      SINARI(IR)=SINAR(I,IR)                                            C
610   COSARI(IR)=COSAR(I,IR)                                            C
      CALL PARS(I,NPAR)                                                 C
620   DO 980 J=1,NPAR                                                   C
C     AVOID IF PARAMETER NOT REFINED                                    C
      IF(LP(I,J).EQ.0)GOTO 980                                          C
C     CLEAR ARRAYS TO ACCUMULATE COMPONENTS OF DA/DP AND DB/DP          C
630   DO 640 K=1,3                                                      C
      DAX(K)=0.                                                         C
640   DBX(K)=0.                                                         C
C     ANISOTROPIC TEMPERATURE FACTOR?                                   C
      IF(J-9)650,810,810                                                C
650   GOTO(660,660,660,700,700,780,780,780),J                           C
C                                                                       C
C.....DERIVATIVES WITH RESPECT TO COORDINATES                           C
C                                                                       C
660   DO 690 L=1,3                                                      C
      DO 680 M=1,3                                                      C
      X=0.                                                              C
      Y=0.                                                              C
      LM=(L-1)*3+M                                                      C
      DO 670 IR=1,IRL                                                   C
      X=H(IR,J)*SINARI(IR)*RM(NP,IR,MEQI,LM)+X                          C
      IF(ICENT.EQ.1)Y=H(IR,J)*COSARI(IR)*RM(NP,IR,MEQI,LM)+Y            C
670   CONTINUE                                                          C
C     MULTIPLY BY MAGNETIC VECTOR COMPONENT                             C
      DAX(L)=XL(I,M+5)*X+DAX(L)                                         C
      IF(ICENT.EQ.1)DBX(L)=XL(I,M+5)*Y+DBX(L)                           C
680   CONTINUE                                                          C
C     MULTIPLY BY 2PI AND SCATTERING FACTOR ETC.                        C
      DAX(L)=-DAX(L)*SNEXI*FLOAT(ICENT)*6.28318530718                   C
      IF(ICENT.EQ.1)DBX(L)=DBX(L)*SNEXI*6.28318530718                   C
690   CONTINUE                                                          C
      GOTO 920                                                          C
C                                                                       C
C.....CALCULATE DERIVATIVES OF COMPONENTS OF A AND B                    C
C     WITH RESPECT TO ISOTROPIC B-FACTOR AND SITE OCCUPATION            C
C                                                                       C
700   DO 770 K=1,3                                                      C
      DAX(K)=RSA(I,K)+DAX(K)                                            C
      IF(ICENT-1)710,710,720                                            C
710   DBX(K)=RSB(I,K)+DBX(K)                                            C
720   IF(J-4)730,730,740                                                C
C.....ISOTROPIC B-FACTOR                                                C
730   X1=-SSNN*SNEXI                                                    C
      GOTO 750                                                          C
C.....SITE OCCUPATION                                                   C
740   X1=SNEX(I)                                                        C
750   DAX(K)=DAX(K)*X1                                                  C
      IF(ICENT-1)770,770,760                                            C
760   DBX(K)=DBX(K)*X1                                                  C
770   CONTINUE                                                          C
      GOTO 920                                                          C
C                                                                       C
C.....DERIVATIVES OF COMPONENTS OF A AND B WITH RESPECT TO              C
C     MAGNETIC VECTOR COMPONENTS                                        C
C                                                                       C
780   LOOP1=1                                                           C
      LOOP2=IRL                                                         C
      CALL MAGSYM(LOOP1,LOOP2,MEQI,RSAI,RSBI,NP)                        C
      DO 800 K=1,3                                                      C
      DAX(K)=RSAI(K,J-5)*SNEXI                                          C
      IF(ICENT-1)790,790,800                                            C
790   DBX(K)=RSBI(K,J-5)*SNEXI                                          C
800   CONTINUE                                                          C
      GOTO 920                                                          C
C                                                                       C
C.....ANISOTROPIC TEMPERATURE FACTORS                                   C
C                                                                       C
C     THIS SECTION INVOLVES A CERTAIN AMOUNT OF DUPLICATION OF PREVIOUS C
C     CALCULATION AND SAVES CORE AT THE EXPENSE OF TIME.                C
C     IN ANY CASE, ANISOTROPIC MAGNETIC ATOMS ARE NOT VERY COMMON       C
C                                                                       C
C     CYCLE THROUGH EQUIVALENT POSITIONS                                C
810   DO 890 IR=1,IRL                                                   C
      VOLW3=VOLW(NP)                                                    C
      LOOP1=IR                                                          C
      LOOP2=IR                                                          C
C     OBTAIN (MAGNETIC VECTOR ROTATION MATRIX)*(COS(H.X) AND SIN(H.X))  C
      CALL MAGSYM(LOOP1,LOOP2,MEQI,RSAI,RSBI,NP)                        C
C     MULTIPLY BY MAGNETIC VECTOR                                       C
      DO 830 M=1,3                                                      C
      AXI(M)=0.0                                                        C
      BXI(M)=0.0                                                        C
      DO 830 K=1,3                                                      C
      AXI(M)=AXI(M)+XL(I,K+5)*RSAI(M,K)                                 C
      IF(ICENT-1)820,820,830                                            C
820   BXI(M)=BXI(M)+XL(I,K+5)*RSBI(M,K)                                 C
830   CONTINUE                                                          C
      IF(J-12)840,850,850                                               C
C     B(I,I)                                                            C
840   M=J-8                                                             C
      K=M                                                               C
      FMU=1.0                                                           C
      GOTO 860                                                          C
C     B(I,J)                                                            C
850   L=15-J                                                            C
      CALL PERM(L,M,K)                                                  C
      FMU=2.0                                                           C
860   FMU=FMU*H(IR,M)*H(IR,K)                                           C
      DO 880 L=1,3                                                      C
      DAX(L)=DAX(L)+AXI(L)*FMU                                          C
      IF(ICENT-1)870,870,880                                            C
870   DBX(L)=DBX(L)+BXI(L)*FMU                                          C
880   CONTINUE                                                          C
890   CONTINUE                                                          C
C     MULTIPLY BY SCATTERING FACTOR ETC.                                C
      DO 910 K=1,3                                                      C
      DAX(K)=DAX(K)*SNEXI                                               C
      IF(ICENT-1)900,900,910                                            C
900   DBX(K)=DBX(K)*SNEXI                                               C
910   CONTINUE                                                          C
920   DER=0.                                                            C
      DO 930 K=1,3                                                      C
C     MULTIPLY DA/DP AND D(J**2)/DA TO GET D(J**2)/DP,                  C
C     THE DERIVATIVE OF THE MAGNETIC COHERENT SCATTERING                C
C     CROSS SECTION WITH RESPECT TO PARAMETER P                         C
      X1=0.                                                             C
      IF(ICENT.EQ.1)X1=DSB(K)*DBX(K)                                    C
930   DER=X1+DSA(K)*DAX(K)+DER                                          C
940   LPIJ=LP(I,J)                                                      C
C     DERIVATIVE OF MAGNETIC BACKGROUND WITH RESPECT TO                 C
C     SITE OCCUPATION                                                   C
      IF(IBACK)950,970,950                                              C
950   IF(J-5)970,960,970                                                C
960   MTYPI=MTYP(I)                                                     C
      DER=DER+VOLW3*0.066229*FF(MTYPI)*FF(MTYPI)*BMAG(MTYPI)            C
     2 *(BMAG(MTYPI)+1.0)*FLOAT(IRL*ICENT*ISTAP)*0.24429                C
C     STORE DERIVATIVE                                                  C
970   DERIV(LPIJ)=FLAG(I,J)*DER+DERIV(LPIJ)                             C
980   CONTINUE                                                          C
990   CONTINUE                                                          C
C     CALCULATE DERIVATIVES WITH RESPECT TO OVERALL PARAMETERS          C
      CALL DOVER(PAKNN,FNN,SSNN,NP,IC)                                  C
C     CALCULATE DERIVATIVES WITH RESPECT TO CELL AND                    C
C     HALFWIDTH PARAMETERS                                              C
C     FNN IS TOTAL (STRUCTURE FACTOR)**2  S**2                          C
1000  FNN=FNUC(NM)+FMAG(NM)                                             C
      IF(NCYCLE.EQ.0)GOTO 1160                                          C
C     SINTH IS MULTIPLIER FOR DERIVATIVES WITH RESPECT TO               C
C     CELL PARAMETERS                                                   C
      SINTH=FNN*PI*SLABDA/(SQRT(SINTH*COSTH)*BB)                        C
C     SS IS MULTIPLIER FOR DERIVATIVES WITH RESPECT TO                  C
C     HALFWIDTH PARAMETERS (GAUSSIAN)                                   C
      SS=FNN/(FLOG2*BB)                                                 C
C     X CARRYS D(HW**2)/D(U, V, OR W)                                   C
      X=TANTH*TANTH                                                     C
C     CALCULATE (GAUSSIAN) CONTRIBUTORS TO DY/D(U, V, OR W)             C
C     = (DY/DHW)*(DHW/D(HW**2))*(D(HW**2)/D(U, V, OR W))                C
      DO 1010 J=1,3                                                     C
      K=LP(N3,J)                                                        C
C     SKIP IF PARAMETER NOT REFINED                                     C
      IF(K.EQ.0)GOTO 1010                                               C
      DERIV(K)=X*FLAG(N3,J)*SS+DERIV(K)                                 C
1010  X=X/TANTH                                                         C
C                                                                       C
C     DERIVATIVES WITH RESPECT TO ZERO POINT                            C
      K=LP(N6,NP)                                                       C
      IF(K.NE.0)DERIV(K)=FLAG(N6,NP)*2.*FNN/BB+DERIV(K)                 C
C     DERIVATIVES WITH RESPECT TO CELL PARAMETERS                       C
C     THESE APPEAR TO IGNORE THE DEPENDENCE OF HALFWIDTH                C
C     ON CELL PARAMETER                                                 C
      DO 1050 J=1,6                                                     C
      K=LP(N2,J)                                                        C
C     SKIP IF PARAMETER NOT REFINED                                     C
      IF(K.EQ.0)GOTO 1050                                               C
      IF(J-3)1020,1020,1030                                             C
1020  K1=J                                                              C
      K2=J                                                              C
      GOTO 1040                                                         C
1030  M=J-3                                                             C
      CALL PERM(M,K1,K2)                                                C
1040  DERIV(K)=DERIV(K)+HNN(K1)*HNN(K2)*SINTH*FLAG(N2,J)                C
1050  CONTINUE                                                          C
C     CONTRIBUTION TO DERIVATIVE OF Y WITH RESPECT TO                   C
C     ASYMMETRY PARAMETER                                               C
      K=LP(N1,2)                                                        C
      IF((K.NE.0).AND.VERT)DERIV(K)=-FLAG(N1,2)*FNN/TANTH+DERIV(K)      C
C     STORE DERIVATIVES FOR THIS REFLECTION                             C
      DO 1060 I=1,MATORD                                                C
      IF(ISW.NE.0)DERIV(I)=DERIV(I)/SCALE(NP,1)                         C
1060  DERSTO(NM,I)=DERIV(I)                                             C
C                                                                       C
C.....ACCOUNT FOR STRICT CONSTRAINTS                                    C
C     DUE TO HEAVILY NESTED LOOPS THIS SECTION IS PROBABLY FASTER       C
C     IF CONSTRAINED PARAMETERS HAVE HIGH PARAMETER NUMBERS             C
C                                                                       C
      IF(MAXCON)1160,1160,1070                                          C
C     CYCLE THROUGH CONSTRAINED PARAMETER SET PJ                        C
1070  DO 1150 J=1,MATORD                                                C
      K=J+1                                                             C
      IF(K-MAXS)1080,1080,1150                                          C
C     CYCLE THROUGH CONSTRAINTS                                         C
1080  DO 1140 M=1,MAXCON                                                C
C     IS PARAMETER J A MEMBER OF CONSTRAINT M?                          C
      CALL DTDP(J,DTJ,M)                                                C
      IF(DTJ)1090,1140,1090                                             C
C     YES                                                               C
C     CYCLE THROUGH ORIGINAL PARAMETER SET PI                           C
1090  DO 1130 I=1,MAXS                                                  C
      IF(I-J)1100,1130,1100                                             C
C     IS PARAMETER I A MEMBER OF CONSTRAINT M?                          C
1100  CALL DTDP(I,DTI,M)                                                C
      IF(DTI)1110,1130,1110                                             C
C     BOTH PARAMETERS APPEAR IN THIS CONSTRAINT                         C
1110  DPIDPJ=-DTJ/DTI                                                   C
C     ADD IN DF/DPI * DPI/DPJ                                           C
      DERSTO(NM,J)=DERSTO(NM,J)+DERIV(I)*DPIDPJ                         C
C     REVERSE PARAMETERS                                                C
      IF(I-MATORD)1120,1120,1130                                        C
1120  DERSTO(NM,I)=DERSTO(NM,I)+DERIV(J)/DPIDPJ                         C
1130  CONTINUE                                                          C
1140  CONTINUE                                                          C
1150  CONTINUE                                                          C
1160  RETURN                                                            C
      END                                                               C
      SUBROUTINE DOVER(PAKNN,FNN,SSNN,NP,IC)                            D
      PARAMETER (ML=500 , MP=99 , MX=103)
C     ************************************                              D
C     CALCULATES DERIVATIVES OF F**2 WITH RESPECT TO                    D
C     'OVERALL' PARAMETERS                                              D
C     ************************************                              D
C                                                                       D
      COMMON /REFLN/PARG,DELTA,DLABDA(2),DUMPA(500),EPS,FLOG,FLOG2,     D
     2 OMEGA,PI,SKEW,SLABDA,SLOG,W,XN,YCALC,YOBS                        D
C                                                                       D
      COMMON /PREF/IALFA,IOMEG,IORD1,IORD2,IPREV,ISTAP,ISS,KSTART,LIMO, D
     2 LLL,NCD(50),NKEY,NO,NTEXT,NUM,NTYP(50),MEQ(50),MTYP(50)          D
C                                                                       D
      COMMON /SYMCEL/ALL(3,3),AL(5,3,3),ATEXT(60),B(8),CELL(6),CELLN(5, D
     2 6),F(4,21),FF(4),PH(5,3),RM(5,24,2,9),S(4,21),SM(5,24,3,3),T(5,  D
     3 24,3),TR(5,24),RCELL(6),RCELLN(5,6),VOLUME,VOL(5),PTEXT(5,4)     D
C                                                                       D
      COMMON /MINIM/COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),HL(3,3),  D
     2 RSA(50,3),RSB(50,3),SA(50),SB(50),SINAR(50,24),SINARI(24),       D
     3 SNEX(50),TEMP(50),TEXT(20),XL(66,39)                             D
C                                                                       D
      COMMON /KEYS/LP(66,39),FLAG(66,39)                                D
C                                                                       D
      COMMON /NUMB/N,N1,N2,N3,N4,N6,N7,NN,NN5,NA(5),NPAT,NREF,NSPAR
C                                                                       D
      COMMON /MATR/MATSIZ,MATORD,SIGD,V(MP),SMM(MP*(MP+1)/2)                   D
C                                                                       D
      COMMON /MOVE/AH(5),BH(5),CH(5),B1(5),B2(5),ZERO(5),SCALE(5,2)     D
C                                                                       D
      COMMON /OPTION/ISF,ISUB,IXTRA,ISW                                 D
C                                                                       S
      N1=N+NP                                                           D
      DO 70 J=1,5                                                       D
      K=LP(N1,J)                                                        D
C                                                                       D
C     SKIP IF PARAMETER NOT REFINED                                     D
C                                                                       D
      IF(K.EQ.0)GOTO 70                                                 D
      IF(K-MATORD)10,10,70                                              D
10    GOTO(20,70,30,40,50),J                                            D
C                                                                       D
C     PREFERRED ORIENTATION PARAMETER                                   D
C                                                                       D
20    DER=PAKNN*FNN                                                     D
      GOTO 60                                                           D
C                                                                       D
C     SCALE FACTOR                                                      D
C                                                                       D
30    DER=0.                                                                        D
      IF(IC.EQ.1) DER=SCALE(NP,1)*FNN                                               D
      GOTO 60                                                           D
40    DER=0.                                                                        D
      IF(IC.EQ.2) DER=SCALE(NP,2)*FNN                                               D
      GOTO 60                                                           D
C                                                                       D
C     OVERALL TEMPERATURE FACTOR                                        D
C                                                                       D
50    DER=-2.0*SSNN*FNN                                                 D
60    DERIV(K)=DERIV(K)+FLAG(N1,J)*DER                                  D
70    CONTINUE                                                          D
      RETURN                                                            D
      END                                                               D
      SUBROUTINE SETUP(JJ,NM,FAC,PAK,IC,HNN)                            S
      PARAMETER (ML=500 , MP=99 , MX=103)
C     **************************************                            S
C     INITIALISES STORAGE FOR REFLECTION NUMBER JJ                      S
C     **************************************                            S
C                                                                       S
      DIMENSION HNN(3)                                                  S
C                                                                       S
C                                                                       S
      COMMON /REFLN/ARG,DELTA,DLABDA(2),DUMPA(500),EPS,FLOG,FLOG2,      S
     2 OMEGA,PI,SKEW,SLABDA,SLOG,W,XN,YCALC,YOBS                        S
C                                                                       S
      COMMON /PREF/IALFA,IOMEG,IORD1,IORD2,IPREV,ISTAP,ISS,KSTART,LIMO, S
     2 LLL,NCD(50),NKEY,NO,NTEXT,NUM,NTYP(50),MEQ(50),MTYP(50)          S
C                                                                       S
      COMMON /SYMCEL/ALL(3,3),AL(5,3,3),ATEXT(60),B(8),CELL(6),CELLN(5, S
     2 6),F(4,21),FF(4),PH(5,3),RM(5,24,2,9),S(4,21),SM(5,24,3,3),T(5,  S
     3 24,3),TR(5,24),RCELL(6),RCELLN(5,6),VOLUME,VOL(5),PTEXT(5,4)     S
C                                                                       S
      COMMON /MINIM/COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),HL(3,3),  S
     2 RSA(50,3),RSB(50,3),SA(50),SB(50),SINAR(50,24),SINARI(24),       S
     3 SNEX(50),TEMP(50),TEXT(20),XL(66,39)                             S
C                                                                       S
      COMMON /REFS/HN(5,3),HH(3,ML),ICODE(ML),SJJ(ML),SOMEGA(ML),       S
     2 FOBS(ML),FESD(ML),FNUC(ML),FMAG(ML),THETA(ML),TANN(ML),HW(ML),   S
     3 DERSTO(ML,MX),AKEEP(ML),BKEEP(ML),PKEEP(ML),ICODD(ML)            S
C                                                                       S
      COMMON /INOUT/LCA,LPR,LPRE,LT4,LT8,LFOR,LPUN                      S
C                                                                       S
      COMMON /MAXDIM/MNCONF,MNCONT,MMATOR,MNEUTS,MNXMAG,MNATOM,MNMAGR,  S
     2 MNEQPO,MNOVRF,MNMAGP,MNPATS                                      S
C                                                                       S
      COMMON /CONTRL/NCYCLE,MAXS,KXR(5),KKZ,LIM,LOUT,IPONS,LMAT,LCORD,  S
     2 LCOORD,ICELL,ICENT,ICENTT(5),IRL,IRLL(5),KL(5),KLL,KM(5),KMN,    S
     3 MN(5),MNN,ICYCLE,MAXCON,IFOUR,IPLOT,KLOOP,LINES,IWID,IPREF(5),   S
     4 IPREFF,NHALP(5),NUNIAX(5),NCUBIC(5),NGAUSS(5),NCAUCH(5),NPAC(5)  S
     5 ,CLOR(5),ELOR(5)
C                                                                       S
      COMMON /NUMB/N,N1,N2,N3,N4,N6,N7,NN,NN5,NA(5),NPAT,NREF,NSPAR
C                                                                       S
C                                                                       S
C     NM IS ARRAY SUBSCRIPT IN COMMON BLOCK /REFS/                      S
      NM=MOD(JJ,MNOVRF)+1                                               S
      IF(JJ.EQ.1)REWIND LT8
      READ(LT8,*)(HH(I,NM),I=1,3),ICODE(NM),ICODD(NM),SJJ(NM),FAC,PAK,    S
     2 (FF(I),I=1,KLOOP)                                                S
CH    WRITE(16,*)JJ,(HH(I,NM),I=1,3),ICODE(NM),ICODD(NM),SJJ(NM),FAC,
CH   2 PAK,(FF(I),I=1,KLOOP)                                                S
      IC=1                                                              S
10    CONTINUE                                                          S
C                                                                       S
C     IS THIS AN X-RAY ALPHA2 REFLECTION?                               S
C                                                                       S
      IF(ICODE(NM).EQ.5)IC=2                                            S
      SLABDA=DLABDA(IC)                                                 S
C                                                                       S
C     INITIALISE ARRAYS                                                 S
C                                                                       S
      SOMEGA(NM)=0.0                                                    S
      FOBS(NM)=0.0                                                      S
      FESD(NM)=0.0                                                      S
      FNUC(NM)=0.0                                                      S
      FMAG(NM)=0.0                                                      S
C                                                                       S
      IF(NCYCLE)40,40,20                                                S
20    DO 30 I=1,MAXS                                                    S
30    DERIV(I)=0.0                                                      S
40    DO 50 I=1,N                                                       S
50    TEMP(I)=1.0                                                       S
      DO 60 I=1,3                                                       S
60    HNN(I)=HH(I,NM)                                                   S
      RETURN                                                            S
      END                                                               S
      SUBROUTINE MAGSYM(LOOP1,LOOP2,MEQI,RSAI,RSBI,NP)                  M
      PARAMETER (ML=500 , MP=99 , MX=103)
C     *************************************************                 M
C     SETS UP COMBINED MATRIX FOR MULTIPLICATION OF MAGNETIC            M
C     VECTOR COMPONENTS TO OBTAIN COMPONENTS OF A AND B FOR ATOM I      M
C     *************************************************                 M
C                                                                       M
      DIMENSION RSAI(3,3),RSBI(3,3)                                     M
C                                                                       M
C                                                                       M
      COMMON /REFLN/ARG,DELTA,DLABDA(2),DUMPA(500),EPS,FLOG,FLOG2,      M
     2 OMEGA,PI,SKEW,SLABDA,SLOG,W,XN,YCALC,YOBS                        M
C                                                                       M
      COMMON /PREF/IALFA,IOMEG,IORD1,IORD2,IPREV,ISTAP,ISS,KSTART,LIMO, M
     2 LLL,NCD(50),NKEY,NO,NTEXT,NUM,NTYP(50),MEQ(50),MTYP(50)          M
C                                                                       M
      COMMON /SYMCEL/ALL(3,3),AL(5,3,3),ATEXT(60),B(8),CELL(6),CELLN(5, M
     2 6),F(4,21),FF(4),PH(5,3),RM(5,24,2,9),S(4,21),SM(5,24,3,3),T(5,  M
     3 24,3),TR(5,24),RCELL(6),RCELLN(5,6),VOLUME,VOL(5),PTEXT(5,4)     M
C                                                                       M
      COMMON /MINIM/COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),HL(3,3),  M
     2 RSA(50,3),RSB(50,3),SA(50),SB(50),SINAR(50,24),SINARI(24),       M
     3 SNEX(50),TEMP(50),TEXT(20),XL(66,39)                             M
C                                                                       M
      COMMON /CONTRL/NCYCLE,MAXS,KXR(5),KKZ,LIM,LOUT,IPONS,LMAT,LCORD,  M
     2 LCOORD,ICELL,ICENT,ICENTT(5),IRL,IRLL(5),KL(5),KLL,KM(5),KMN,    M
     3 MN(5),MNN,ICYCLE,MAXCON,IFOUR,IPLOT,KLOOP,LINES,IWID,IPREF(5),   M
     4 IPREFF,NHALP(5),NUNIAX(5),NCUBIC(5),NGAUSS(5),NCAUCH(5),NPAC(5)  M
     5 ,CLOR(5),ELOR(5)
C                                                                       M
      DO 20 J=1,3                                                       M
      DO 20 K=1,3                                                       M
      JK=(J-1)*3+K                                                      M
      RSAI(J,K)=0.                                                      M
      RSBI(J,K)=0.                                                      M
C     ACCOUNT FOR MAGNETIC VECTOR SYMMETRY                              M
C     MATRIX RSAI IS THE SUM OVER SYMMETRY POSITIONS OF                 M
C      COS(H.X)*(MAGNETIC VECTOR ROTATION MATRIX)                       M
      DO 10 IR=LOOP1,LOOP2                                              M
      RSAI(J,K)=COSARI(IR)*RM(NP,IR,MEQI,JK)+RSAI(J,K)                  M
10    IF(ICENT.EQ.1)RSBI(J,K)=SINARI(IR)*RM(NP,IR,MEQI,JK)+RSBI(J,K)    M
20    CONTINUE                                                          M
      IF(ICENT-1)50,50,30                                               M
30    DO 40 J=1,3                                                       M
      DO 40 K=1,3                                                       M
40    RSAI(J,K)=RSAI(J,K)*2.0                                           M
50    RETURN                                                            M
      END                                                               M
      SUBROUTINE DTDP(J,DTJ,L)                                          D
      PARAMETER (ML=500 , MP=99 , MX=103)
C     ***********************************************                   D
C     SEARCHES FOR PARAMETER J IN CONSTRAINT L AND SUMS THE DIFFERENTIALD
C     OF CONSTRAINT TERMS WITH RESPECT TO PARAMETER J                   D
C     ***********************************************                   D
C                                                                       D
      COMMON /SCONTS/ICON(4,9),JCON(4,9),LCONI(4,9),LCONJ(4,9),LN(4),   D
     2 LCID(4,9),JCID(4,9),CI(4,9),CJ(4,9),CONA(4,9),DF(4)              D
C                                                                       D
      DTJ=0.0                                                           D
      LNI=LN(L)                                                         D
      DO 20 I=1,LNI                                                     D
      IF(LCID(L,I).NE.J)GOTO 10                                         D
C                                                                       D
C      PARAMETER J APPEARS AS FIRST PART OF TERM                        D
C                                                                       D
      DTJ=DTJ+CI(L,I)                                                   D
10    IF(JCID(L,I).NE.J)GOTO 20                                         D
C                                                                       D
C     PARAMETER J APPEARS AS SECOND PART OF TERM                        D
C                                                                       D
      DTJ=DTJ+CJ(L,I)                                                   D
20    CONTINUE                                                          D
C                                                                       D
C     DTJ IS SUM D(TERM)/DPJ                                            D
C                                                                       D
      RETURN                                                            D
      END                                                               D
      SUBROUTINE CONPRE                                                 C
      PARAMETER (ML=500 , MP=99 , MX=103)
C     *************************************************************     C
C     THIS SUBROUTINE PREPARES VALUES FOR USE IN SUBROUTINE  CALCUL     C
C     AND CALCULATES THE DIFFERENTIAL OF EACH TERM IN THE CONSTRAINT    C
C     FUNCTIONS WITH RESPECT TO THE PARAMETERS IN EACH TERM (DT/DPI)    C
C     THIS ALLOWS APPLICATION OF THE PAWLEY METHOD OF STRICT CONSTRAINT C
C     AND THUS THE USE OF THE CHOLESKI MATRIX INVERSION ALGORITHM       C
C     FOR DEALING WITH THE NORMAL MATRIX                                C
C     *************************************************************     C
C                                                                       C
      COMMON /REFLN/ARG,DELTA,DLABDA(2),DUMPA(500),EPS,FLOG,FLOG2,      C
     2 OMEGA,PI,SKEW,SLABDA,SLOG,W,XN,YCALC,YOBS                        C
C                                                                       C
      COMMON /PREF/IALFA,IOMEG,IORD1,IORD2,IPREV,ISTAP,ISS,KSTART,LIMO, C
     2 LLL,NCD(50),NKEY,NO,NTEXT,NUM,NTYP(50),MEQ(50),MTYP(50)          C
C                                                                       C
      COMMON /SYMCEL/ALL(3,3),AL(5,3,3),ATEXT(60),B(8),CELL(6),CELLN(5, C
     2 6),F(4,21),FF(4),PH(5,3),RM(5,24,2,9),S(4,21),SM(5,24,3,3),T(5,  C
     3 24,3),TR(5,24),RCELL(6),RCELLN(5,6),VOLUME,VOL(5),PTEXT(5,4)     C
C                                                                       C
      COMMON /MINIM/COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),HL(3,3),  C
     2 RSA(50,3),RSB(50,3),SA(50),SB(50),SINAR(50,24),SINARI(24),       C
     3 SNEX(50),TEMP(50),TEXT(20),XL(66,39)                             C
C                                                                       C
      COMMON /SCONTS/ICON(4,9),JCON(4,9),LCONI(4,9),LCONJ(4,9),LN(4),   C
     2 LCID(4,9),JCID(4,9),CI(4,9),CJ(4,9),CONA(4,9),DF(4)              C
C                                                                       C
      COMMON /MATR/MATSIZ,MATORD,SIGD,V(MP),SMM(MP*(MP+1)/2)                   C
C                                                                       C
      COMMON /MAXDIM/MNCONF,MNCONT,MMATOR,MNEUTS,MNXMAG,MNATOM,MNMAGR,  C
     2 MNEQPO,MNOVRF,MNMAGP,MNPATS                                      C
C                                                                       C
      COMMON /CONTRL/NCYCLE,MAXS,KXR(5),KKZ,LIM,LOUT,IPONS,LMAT,LCORD,  C
     2 LCOORD,ICELL,ICENT,ICENTT(5),IRL,IRLL(5),KL(5),KLL,KM(5),KMN,    C
     3 MN(5),MNN,ICYCLE,MAXCON,IFOUR,IPLOT,KLOOP,LINES,IWID,IPREF(5),   C
     4 IPREFF,NHALP(5),NUNIAX(5),NCUBIC(5),NGAUSS(5),NCAUCH(5),NPAC(5)  C
     5 ,CLOR(5),ELOR(5)
C                                                                       C
      COMMON /KEYS/LP(66,39),FLAG(66,39)                                C
C                                                                       C
C                                                                       C
C     CLEAR ARRAYS FOR DERIVATIVES OF TERMS WITH RESPECT TO THE         C
C     ORIGINAL PARAMETERS PI                                            C
C                                                                       C
      MAXS=MATORD                                                       C
      DO 10 I=1,MAXCON                                                  C
      DO 10 J=1,MNCONT                                                  C
      LCID(I,J)=0                                                       C
      JCID(I,J)=0                                                       C
      CI(I,J)=0.0                                                       C
10    CJ(I,J)=0.0                                                       C
C                                                                       C
C     CYCLE THROUGH CONSTRAINTS                                         C
C                                                                       C
      DO 160 I=1,MAXCON                                                 C
C     COUNT FIXED PARAMETERS IN EACH CONSTRAINT                         C
      NFIX=0                                                            C
      IOLP=-1                                                           C
C                                                                       C
C     LN IS THE NUMBER OF VARIABLE TERMS                                C
C                                                                       C
      LNI=LN(I)                                                         C
C                                                                       C
C     CYCLE THROUGH THE TERMS FOR EACH CONSTRAINT                       C
C                                                                       C
      DO 140 J=1,LNI                                                    C
C                                                                       C
C     AA IS THE MULTIPLIER OF CONSTRAINT TERM                           C
C                                                                       C
      AA=CONA(I,J)                                                      C
C                                                                       C
C     IS THIS A LINEAR OR QUADRATIC CONSTRAINT?                         C
C                                                                       C
      IL=LCONI(I,J)                                                     C
      IF(IL)50,50,20                                                    C
20    JL=LCONJ(I,J)                                                     C
      LCID(I,J)=LP(IL,JL)                                               C
      IF(LCID(I,J))40,40,30                                             C
30    IF(LCID(I,J)-MATORD)50,50,40                                      C
C                                                                       C
C     PARAMETER NOT REFINED                                             C
C                                                                       C
40    CALL FIXDP(I,J,IL,JL,NFIX,LCID,IOLP)                              C
50    II=ICON(I,J)                                                      C
      IF(II)90,90,60                                                    C
60    JJ=JCON(I,J)                                                      C
      JCID(I,J)=LP(II,JJ)                                               C
      IF(JCID(I,J))80,80,70                                             C
70    IF(JCID(I,J)-MATORD)90,90,80                                      C
80    CALL FIXDP(I,J,II,JJ,NFIX,JCID,IOLP)                              C
C                                                                       C
C     SORT OUT DERIVATIVES OF TERMS W.R.T. PARAMETERS                   C
C                                                                       C
90    IF(II)100,100,110                                                 C
C                                                                       C
C     LINEAR TERMS FIRST                                                C
C                                                                       C
100   CI(I,J)=AA*FLAG(IL,JL)                                            C
      GOTO 140                                                          C
110   IF(IL)120,120,130                                                 C
120   CJ(I,J)=AA*FLAG(II,JJ)                                            C
      GOTO 140                                                          C
C                                                                       C
C     QUADRATIC TERMS                                                   C
C                                                                       C
130   CI(I,J)=AA*FLAG(IL,JL)*XL(II,JJ)                                  C
      CJ(I,J)=AA*FLAG(II,JJ)*XL(IL,JL)                                  C
140   CONTINUE                                                          C
C     CHECK WHETHER THERE IS A FIXED PARAMETER                          C
      IF(NFIX-1)150,160,150                                             C
150   CALL ERR(13,I)                                                    C
160   CONTINUE                                                          C
      RETURN                                                            C
      END                                                               C
      SUBROUTINE FIXDP(I,J,IL,JL,NFIX,LCID,IOLP)                        F
      PARAMETER (ML=500 , MP=99 , MX=103)
C     *********************************************************         F
C      RESETS STORAGE ON ENCOUNTERING A NON-REFINED PARAMETER IN A      F
C     CONSTRAINT FUNCTION. THERE SHOULD BE ONE SUCH PARAMETER PER       F
C     CONSTRAINT                                                        F
C     *********************************************************         F
C                                                                       F
      DIMENSION LCID(4,9)                                               F
C                                                                       F
C                                                                       F
      COMMON /KEYS/LP(66,39),FLAG(66,39)                                F
C                                                                       F
      COMMON /CONTRL/NCYCLE,MAXS,KXR(5),KKZ,LIM,LOUT,IPONS,LMAT,LCORD,  F
     2 LCOORD,ICELL,ICENT,ICENTT(5),IRL,IRLL(5),KL(5),KLL,KM(5),KMN,    F
     3 MN(5),MNN,ICYCLE,MAXCON,IFOUR,IPLOT,KLOOP,LINES,IWID,IPREF(5),   F
     4 IPREFF,NHALP(5),NUNIAX(5),NCUBIC(5),NGAUSS(5),NCAUCH(5),NPAC(5)  F
     5 ,CLOR(5),ELOR(5)
C                                                                       F
      K=100*IL+JL                                                       F
      IF(IOLP-K)10,20,10                                                F
10    NFIX=NFIX+1                                                       F
C                                                                       F
C     CHECK WHETHER MORE THAN ONE NON-REFINED PARAMETER PER CONSTRAINT  F
C                                                                       F
      IF(NFIX.GT.1)CALL ERR(12,I)                                       F
      MAXS=MAXS+1                                                       F
      LCID(I,J)=MAXS                                                    F
      LP(IL,JL)=MAXS                                                    F
      FLAG(IL,JL)=1.0                                                   F
      IOLP=100*IL+JL                                                    F
20    RETURN                                                            F
      END                                                               F
      SUBROUTINE OUTPUT                                                 O
      PARAMETER (ML=500 , MP=99 , MX=103)
C     ***************************************************************   O
C      THIS SUBROUTINE CALLS ROUTINES TO OUTPUT SHIFTS,NEW              O
C     PARAMETERS, AND STANDARD DEVIATIONS AFTER EACH CYCLE.             O
C     THE OUTPUT IS DIVIDED INTO THREE PARTS:                           O
C     1) ATOM PARAMETERS OTHER THAN ANISOTROPIC B AND MAGNETIC          O
C     2) ANISOTROPIC B AND MAGNETIC PARAMETERS                          O
C     3) OVERALL PARAMETERS, INCLUDING SCALE B-FACTOR, HALFWIDTH,CELL,  O
C        ASYMMETRY, AND PREFERRED ORIENTATION.                          O
C     ***************************************************************   O
C                                                                       O
      COMMON /REFLN/ARG,DELTA,DLABDA(2),DUMPA(500),EPS,FLOG,FLOG2,      O
     2 OMEGA,PI,SKEW,SLABDA,SLOG,W,XN,YCALC,YOBS                        O
C                                                                       O
      COMMON /PREF/IALFA,IOMEG,IORD1,IORD2,IPREV,ISTAP,ISS,KSTART,LIMO, O
     2 LLL,NCD(50),NKEY,NO,NTEXT,NUM,NTYP(50),MEQ(50),MTYP(50)          O
C                                                                       O
      COMMON /SYMCEL/ALL(3,3),AL(5,3,3),ATEXT(60),B(8),CELL(6),CELLN(5, O
     2 6),F(4,21),FF(4),PH(5,3),RM(5,24,2,9),S(4,21),SM(5,24,3,3),T(5,  O
     3 24,3),TR(5,24),RCELL(6),RCELLN(5,6),VOLUME,VOL(5),PTEXT(5,4)     O
C                                                                       O
      COMMON /MINIM/COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),HL(3,3),  O
     2 RSA(50,3),RSB(50,3),SA(50),SB(50),SINAR(50,24),SINARI(24),       O
     3 SNEX(50),TEMP(50),TEXT(20),XL(66,39)                             O
C                                                                       O
      COMMON /CONTRL/NCYCLE,MAXS,KXR(5),KKZ,LIM,LOUT,IPONS,LMAT,LCORD,  O
     2 LCOORD,ICELL,ICENT,ICENTT(5),IRL,IRLL(5),KL(5),KLL,KM(5),KMN,    O
     3 MN(5),MNN,ICYCLE,MAXCON,IFOUR,IPLOT,KLOOP,LINES,IWID,IPREF(5),   O
     4 IPREFF,NHALP(5),NUNIAX(5),NCUBIC(5),NGAUSS(5),NCAUCH(5),NPAC(5)  O
     5 ,CLOR(5),ELOR(5)
C                                                                       O
      COMMON /INOUT/LCA,LPR,LPRE,LT4,LT8,LFOR,LPUN                      O
C                                                                       O
      COMMON /KEYS/LP(66,39),FLAG(66,39)                                O
C                                                                       O
      COMMON /NUMB/N,N1,N2,N3,N4,N6,N7,NN,NN5,NA(5),NPAT,NREF,NSPAR
C                                                                       O
C                                                                       O
C     ANY REFINEMENT?                                                   O
      IF(NCYCLE)90,90,10                                                O
C                                                                       O
C     UPDATE PARAMETERS AFTER REFINEMENT                                O
C                                                                       O
10    CALL UPDATE                                                       O
C                                                                       O
C     ENTER PRINTING SEQUENCE                                           O
C                                                                       O
      CALL SHOUT                                                        O
C                                                                       O
C      OUTPUT REAL AND RECIPROCAL CELLS WITH STANDARD DEVIATIONS        O
C                                                                       O
      DO 20 NP=1,NPAT                                                   O
      CALL CELLSD(NP)                                                   O
20    CONTINUE                                                          O
C                                                                       O
C     CHECK THAT TEMPERATURE FACTORS ARE POSITIVE DEFINITE              O
C                                                                       O
      CALL CHECKB                                                       O
      IF(NCYCLE-ICYCLE)90,30,90                                         O
30    IF(LCOORD)40,50,40                                                O
C                                                                       O
C     PUNCH FINAL PARAMETERS                                            O
C                                                                       O
40    CALL PAROUT                                                       O
50    IF(LMAT)70,70,60                                                  O
C                                                                       O
C     PUNCH VARIANCE COVARIANCE MATRIX                                  O
C                                                                       O
60    CALL VCOUT                                                        O
70    IF(LCORD)80,90,80                                                 O
C                                                                       O
C     OUTPUT CORRELATION MATRIX                                         O
C                                                                       O
80    CALL CORREL                                                       O
C                                                                       O
C     OUTPUT PROFILE AGREEMENT IN BLOCKS                                O
C                                                                       O
C                                                                       O
C     OUTPUT AGREEMENT FACTORS                                          O
C                                                                       O
90    CALL ROUT(NUM)                                                    O
C     FURTHER OUTPUT OPTIONS AFTER LAST REFINEMENT CYCLE                O
C                                                                       O
      IF(NCYCLE)120,100,120                                             O
100   IF(IPLOT)110,120,110                                              O
110   CALL DELPRF                                                       O
C                                                                       O
C     PLOT PROFILE PATTERN ON LINE PRINTER                              O
C                                                                       O
      CALL PROFPL                                                       O
C                                                                       O
C     END OF OUTPUT                                                     O
C                                                                       O
120   RETURN                                                            O
      END                                                               O
      SUBROUTINE CHECKB                                                 C
      PARAMETER (ML=500 , MP=99 , MX=103)
C     ********************************************************          C
C     CHECKS POSITIVE DEFINITE NATURE OF TEMPERATURE FACTORS AND        C
C     REDISTRIBUTES OVERALL B-FACTOR                                    C
C     ********************************************************          C
C                                                                       C
      DIMENSION COMP(6)                                                 C
C                                                                       C
C     IS THE OVERALL B-FACTOR NON ZERO?                                 C
C                                                                       C
C                                                                       C
      COMMON /REFLN/ARG,DELTA,DLABDA(2),DUMPA(500),EPS,FLOG,FLOG2,      C
     2 OMEGA,PI,SKEW,SLABDA,SLOG,W,XN,YCALC,YOBS                        C
C                                                                       C
      COMMON /PREF/IALFA,IOMEG,IORD1,IORD2,IPREV,ISTAP,ISS,KSTART,LIMO, C
     2 LLL,NCD(50),NKEY,NO,NTEXT,NUM,NTYP(50),MEQ(50),MTYP(50)          C
C                                                                       C
      COMMON /SYMCEL/ALL(3,3),AL(5,3,3),ATEXT(60),B(8),CELL(6),CELLN(5, C
     2 6),F(4,21),FF(4),PH(5,3),RM(5,24,2,9),S(4,21),SM(5,24,3,3),T(5,  C
     3 24,3),TR(5,24),RCELL(6),RCELLN(5,6),VOLUME,VOL(5),PTEXT(5,4)     C
C                                                                       C
      COMMON /MINIM/COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),HL(3,3),  C
     2 RSA(50,3),RSB(50,3),SA(50),SB(50),SINAR(50,24),SINARI(24),       C
     3 SNEX(50),TEMP(50),TEXT(20),XL(66,39)                             C
C                                                                       C
      COMMON /BFAC/ITF,JTF(50),PRCP(5,6),ITF3,ITF4                      C
C                                                                       C
      COMMON /NUMB/N,N1,N2,N3,N4,N6,N7,NN,NN5,NA(5),NPAT,NREF,NSPAR
C                                                                       C
      COMMON /INOUT/LCA,LPR,LPRE,LT4,LT8,LFOR,LPUN                      C
C                                                                       C
      DO 180 NP=1,NPAT                                                  C
      N1=N+NP                                                           C
      IF(XL(N1,5))10,80,10                                              C
C                                                                       C
C     REDISTRIBUTE OVERALL B-FACTOR                                     C
C                                                                       C
10    XXB=XL(N1,5)                                                      C
      XL(N1,5)=0.0                                                      C
      IF(ITF)20,30,20                                                   C
C                                                                       C
C      GET ANISOTROPIC COMPONENTS OF OVERALL B-FACTOR                   C
C                                                                       C
20    CALL CONVER(XXB,COMP,NP)                                          C
C                                                                       C
C     CYCLE THROUGH ATOM LIST                                           C
C                                                                       C
30    DO 70 I=1,N                                                       C
      IF(JTF(I))40,40,50                                                C
C                                                                       C
C     CORRECT ISOTROPIC ATOM                                            C
C                                                                       C
40    XL(I,4)=XL(I,4)+XXB                                               C
      GOTO 70                                                           C
C                                                                       C
C     CORRECT ANISOTROPIC ATOM                                          C
C                                                                       C
50    DO 60 J=9,14                                                      C
      XL(I,J)=XL(I,J)+COMP(J-8)                                         C
60    CONTINUE                                                          C
70    CONTINUE                                                          C
C                                                                       C
C     CHECK POSITIVE DEFINITE CHARACTER                                 C
C                                                                       C
80    DO 170 I=1,N                                                      C
      IF(NCD(I).NE.NP)GOTO 170                                          C
      IF(JTF(I))90,90,100                                               C
C                                                                       C
C     ISOTROPIC ATOM                                                    C
C                                                                       C
90    IF(XL(I,4))140,170,170                                            C
C                                                                       C
C     ANISOTROPIC ATOM                                                  C
C                                                                       C
100   DO 110 J=9,11                                                     C
      IF(XL(I,J))140,110,110                                            C
110   CONTINUE                                                          C
      A=1.0                                                             C
      C=2.0                                                             C
      D=0.0                                                             C
      DO 130 J=1,3                                                      C
      CALL PERM(J,K,L)                                                  C
      M=J+11                                                            C
C                                                                       C
C     B(I,I)*B(J,J) -B(I,J)**2                                          C
C                                                                       C
      XXB=XL(I,K+8)*XL(I,L+8)-XL(I,M)*XL(I,M)                           C
C                                                                       C
C     PRODUCT B(I,J)                                                    C
C                                                                       C
120   C=C*XL(I,M)                                                       C
      M=J+8                                                             C
C                                                                       C
C     PRODUCT B(I,I)                                                    C
C                                                                       C
      A=A*XL(I,M)                                                       C
      MM=15-J                                                           C
C                                                                       C
C     SUM(I=1,3;K.NE.J.NE.I)  B(I,I)*B(K,J) **2                         C
C                                                                       C
      D=D+XL(I,MM)*XL(I,MM)*XL(I,M)*XL(I,M)                             C
130   CONTINUE                                                          C
      XXB=A+C-D                                                         C
      IF(XXB)140,150,150                                                C
C                                                                       C
C     CAPTION FOR NEGATIVE TEMPERATURE FACTOR                           C
C                                                                       C
140   WRITE(LPR,190)ATEXT(I)                                            C
C                                                                       C
C     CHECK SITE OCCUPATION FACTOR                                      C
C                                                                       C
150   IF(XL(I,5))160,170,170                                            C
C                                                                       C
C     CAPTION FOR NEGATIVE SITE OCCUPATION FACTOR                       C
C                                                                       C
160   WRITE(LPR,200)ATEXT(I)                                            C
170   CONTINUE                                                          C
180   CONTINUE                                                          C
      RETURN                                                            C
190   FORMAT(38H0WARNING: TEMPERATURE FACTOR FOR ATOM ,A4,              C
     2 25H IS NOT POSITIVE DEFINITE)                                    C
200   FORMAT(42H0WARNING: SITE OCCUPATION FACTOR FOR ATOM ,A4,          C
     2 12H IS NEGATIVE)                                                 C
      END                                                               C
      SUBROUTINE ROUT(NUM)                                              R
      PARAMETER (ML=500 , MP=99 , MX=103)
C     ********************************************************          R
C     PRINTS R-FACTORS AND OTHER AGREEMENT FACTORS                      R
C     ********************************************************          R
C                                                                       R
      LOGICAL FORCEL,FORCED,PUNCH,STAR,TYPE,VERT,YES,ANGCOU                    R
      COMMON /LOGIC/FORCEL,FORCED,PUNCH,STAR,TYPE,VERT,YES,ANGCOU              R
      COMMON /CONTRL/NCYCLE,MAXS,KXR(5),KKZ,LIM,LOUT,IPONS,LMAT,LCORD,  R
     2 LCOORD,ICELL,ICENT,ICENTT(5),IRL,IRLL(5),KL(5),KLL,KM(5),KMN,    R
     3 MN(5),MNN,ICYCLE,MAXCON,IFOUR,IPLOT,KLOOP,LINES,IWID,IPREF(5),   R
     4 IPREFF,NHALP(5),NUNIAX(5),NCUBIC(5),NGAUSS(5),NCAUCH(5),NPAC(5)  R
     5 ,CLOR(5),ELOR(5)
C                                                                       R
      COMMON /MATR/MATSIZ,MATORD,SIGD,V(MP),SMM(MP*(MP+1)/2)                   R
C                                                                       R
      COMMON /SUMS/SW,SUMAW,SUMW,SUMCAL,SUMOBS,SUMDIF,SUMWBS,SOM,       R
     2 SUMFOB,SUMDFF,FNUCDI,FNUCOB,FMAGDI,FMAGOB,SUWFOB,SUWDFF          C
C                                                                       R
      COMMON /INOUT/LCA,LPR,LPRE,LT4,LT8,LFOR,LPUN                      R
      COMMON /NUMB/N,N1,N2,N3,N4,N6,N7,NN,NN5,NA(5),NPAT,NREF,NSPAR
C                                                                       R
      WRITE(LPR,160)ICYCLE                                              R
C                                                                       R
C     PRINT R-FACTORS (TYPES ACCORDING TO CAPTION)                      R
C                                                                       R
      IF(SUMFOB)20,10,20                                                R
10    X=0.0                                                             R
      GOTO 30                                                           R
20    X=100.0*SUMDFF/SUMFOB                                             R
      XINT=X
      WRITE(LPR,170)X                                                   R
      IF (SUWFOB.EQ.0.) GOTO 30
      X=100.0*SQRT(SUWDFF/SUWFOB)                                       R
      XWINT=X
      WRITE(LPR,171)X                                                   R
      FNDFI=FLOAT(NREF-NSPAR)
      IF(FNDFI.GT.0.) X=100.0*SQRT(FNDFI/SUWFOB)                        R
      XEINT=X
      WRITE(LPR,172)X                                                   R
 30   X=100.0*SUMDIF/SUMOBS                                             R
      WRITE(LPR,180)X                                                   R
      X=100.0*SQRT(SUMW/SUMWBS)                                         R
      XWPR=X
      WRITE(LPR,190)X                                                   R
C                                                                       R
C     NUMBER OF DEGREES OF FREEDOM                                      R
C                                                                       R
      NDEG=NUM-MATORD+MAXCON                                            R
      X=100.0*SQRT(FLOAT(NDEG)*250.0/SUMWBS)                            R
      PRINT '(A,I3,A,F6.2,A,F6.2,a,f6.2)' ,' CYCLE= ',ICYCLE-1,
     1' RINT= ',XINT,' RWP= ',XWPR,' REXP= ',X
      IF(FORCEL) THEN
      WRITE(12,'(A)') '   '
      WRITE(12,'(A,I3,5(A,F6.2))') ' CYCLE= ',ICYCLE-1,
     1' RINT=',XINT,' RWP=',XWPR,' REXP=',X,' GOF=',XWPR/X,
     2' SIGMA=',1./(2.*SQRT(2.*NDEG))
      WRITE(12,'( 11X,4(A,F6.2))')
     1' RWINT=',XWINT,' RINT EXP=',XEINT,' GOF=',XWINT/XEINT,
     2' SIGMA=',1./(2.*SQRT(2.*FNDFI))
      ENDIF
      WRITE(LPR,200)X                                                   R
      IF(FNUCOB)50,40,50                                                R
40    X=0.0                                                             R
      GOTO 60                                                           R
50    X=100.0*FNUCDI/FNUCOB                                             R
60    WRITE(LPR,210)X                                                   R
      IF(KXR(1))120,120,70                                              R
70    IF(KXR(1)-3)80,80,120                                             R
80    IF(FMAGOB)100,90,100                                              R
90    X=0.0                                                             R
      GOTO 110                                                          R
100   X=100.0*FMAGDI/FMAGOB                                             R
110   WRITE(LPR,220)X                                                   R
120   WRITE(LPR,230)NDEG                                                R
      WRITE(LPR,'(3(A,I5,/))') 'NUMBER OF REFLEXIONS',NREF,             R
     1'NUMBER OF STRUCTURAL PARAMETERS',NSPAR,
     2'TOTAL NUMBER OF PARAMETERS',MATORD
      IF(ICYCLE.EQ.1)
     1WRITE( 12,'(3(A,I5,/))') 'NUMBER OF REFLEXIONS',NREF,             R
     1'NUMBER OF STRUCTURAL PARAMETERS',NSPAR,
     2'TOTAL NUMBER OF PARAMETERS',MATORD
C                                                                       R
C     PRINT VARIOUS SUMS (DERIVED ACCORDING TO CAPTIONS)                R
C                                                                       R
      WRITE(LPR,240)ICYCLE                                              R
      WRITE(LPR,250)SUMDIF,SUMOBS                                       R
      X=SUMWBS/250.0                                                    R
      WRITE(LPR,260)SUMCAL,X                                            R
      WRITE(LPR,270)SUMDFF,SUMFOB                                       R
      WRITE(LPR,271)SUWDFF,SUWFOB                                       R
      WRITE(LPR,280)FNUCDI,FNUCOB                                       R
      IF(KXR(1))150,150,130                                             R
130   IF(KXR(1)-3)140,140,150                                           R
140   WRITE(LPR,290)FMAGDI,FMAGOB                                       R
150   X=SUMW/(250.0*FLOAT(NDEG))                                        R
      WRITE(LPR,300)X                                                   R
      RETURN                                                            R
160   FORMAT(1H0,47X,23HR FACTORS BEFORE CYCLE ,I3)                     R
170   FORMAT(                                                           R
     2 61H0(STRUCTURE FACTOR)**2 R (100*SUM ABS(I<OBS>-I<CALC>*SCALE) / R
     3 ,17HSUM ABS(I<OBS>) ),10X,F6.2)                                  R
171   FORMAT(                                                           R
     2 61H0(WEIGHT INTENS R (100*SQRT(SUM W*(I<OBS>-I<CALC>*SCALE)**2 / R
     3 ,18HSUM W*(I<OBS>)**2),9X,F6.2)                                  R
172   FORMAT(                                                           R
     2 52H EXP INTS R (100*SQRT(NUMBER OF DEGREES OF FREEDOM)/,         R
     3 19HSUM W*(I<OBS>)**2 ),17X,F6.2)                                 R
180   FORMAT(                                                           R
     2 57H PROFILE INTENSITY R (100*SUM ABS(Y<OBS>-Y<CALC>*SCALE) /,    R
     3  17HSUM ABS(Y<OBS>) ),14X,F6.2)                                  R
190   FORMAT(                                                           R
     2 58H WEIGHTED PROFILE R (100*SQRT(SUM W*(Y<OBS>-Y<CALC>*SCALE),   R
     3 25H**2 /SUM W*(Y<OBS>)**2 )),5X,F6.2)                            R
200   FORMAT(                                                           R
     2 52H EXPECTED R (100*SQRT(NUMBER OF DEGREES OF FREEDOM)/,         R
     3 19HSUM W*(Y<OBS>)**2 ),17X,F6.2)                                 R
210   FORMAT(                                                           R
     2 53H NUCLEAR R (100*SUM ABS(INUC<OBS>-INUC<CALC>*SCALE) /,        R
     3 15HSUM INUC<OBS> ),20X,F6.2)                                     R
220   FORMAT(                                                           R
     2 54H MAGNETIC R (100*SUM ABS(IMAG<OBS>-IMAG<CALC>*SCALE) /,       R
     3 15HSUM IMAG<OBS> ),19X,F6.2)                                     R
230   FORMAT(30H0NUMBER OF DEGREES OF FREEDOM ,I4)                      R
240   FORMAT(1H0,42X,36HSUMMATIONS ACCUMULATED BEFORE CYCLE ,I3)        R
250   FORMAT(31H0SUM ABS(Y<OBS>-Y<CALC>*SCALE) ,9X,E11.4,               R
     2 18H; SUM ABS(Y<OBS>) ,5X,E11.4)                                  R
260   FORMAT(19H SUM Y<CALC>*SCALE ,21X,E11.4,20H; SUM W*(Y<OBS>)**2 ,  R
     2 3X,E11.4)                                                        R
270   FORMAT(31H SUM ABS(I<OBS>-I<CALC>*SCALE) ,9X,E11.4,               R
     2 18H; SUM ABS(I<OBS>) ,5X,E11.4)                                  R
271   FORMAT(32H SUM W*(I<OBS>-I<CALC>*SCALE)**2,8X,E11.4,               R
     2 20H; SUM W*(I<OBS>)**2 ,3X,E11.4)                                  R
280   FORMAT(37H SUM ABS(INUC<OBS>-INUC<CALC>*SCALE) ,3X,E11.4,         R
     2 16H; SUM INUC<OBS> ,7X,E11.4)                                    R
290   FORMAT(37H SUM ABS(IMAG<OBS>-IMAG<CALC>*SCALE) ,3X,E11.4,         R
     2 16H; SUM IMAG<OBS> ,7X,E11.4)                                    R
300   FORMAT(                                                           R
     2 34H SUM W*(Y<OBS>-Y<CALC>*SCALE)**2 /,                           R
     3 30H(NUMBER OF DEGREES OF FREEDOM),10X,E11.4)                     R
      END                                                               R
      SUBROUTINE CELLSD(NP)                                             C
      PARAMETER (ML=500 , MP=99 , MX=103)
C     **************************************************************    C
C      THIS ROUTINE TRANSFORMS THE VARIANCE - COVARIANCE (V.C.) MATRIX  C
C     FOR THE METRIC TENSOR COMPONENT INTO THE V.C. MATRICES FOR THE    C
C     REAL AND RECIPROCAL PARAMETERS, HENCE YIELDING ESTIMATED STANDARD C
C     DEVIATIONS FOR BOTH.                                              C
C                                                                       C
C     THE METHOD IS THAT OF: SANDS, D.E., 1966, ACTA CRYST., 21, 868-872C
C     **************************************************************    C
C                                                                       C
      DIMENSION COV(5,6,6),RCOV(5,6,6),D(5,6,6),P(5,6),DRSIN(5,3),      C
     2 DRTAN(5,3),TCOV(5,6,6),RCSIN(5,3),SCOV(5,6,6),DVDP(5,6),         C
     3 DEQ(ML*MX),dd(6,6)
C                                                                       C
C                                                                       C
      COMMON /REFLN/ARG,DELTA,DLABDA(2),DUMPA(500),EPS,FLOG,FLOG2,      C
     2 OMEGA,PI,SKEW,SLABDA,SLOG,W,XN,YCALC,YOBS                        C
C                                                                       C
      COMMON /PREF/IALFA,IOMEG,IORD1,IORD2,IPREV,ISTAP,ISS,KSTART,LIMO, C
     2 LLL,NCD(50),NKEY,NO,NTEXT,NUM,NTYP(50),MEQ(50),MTYP(50)          C
C                                                                       C
      COMMON /SYMCEL/ALL(3,3),AL(5,3,3),ATEXT(60),B(8),CELL(6),CELLN(5, C
     2 6),F(4,21),FF(4),PH(5,3),RM(5,24,2,9),S(4,21),SM(5,24,3,3),T(5,  C
     3 24,3),TR(5,24),RCELL(6),RCELLN(5,6),VOLUME,VOL(5),PTEXT(5,4)     C
C                                                                       C
      COMMON /MINIM/COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),HL(3,3),  C
     2 RSA(50,3),RSB(50,3),SA(50),SB(50),SINAR(50,24),SINARI(24),       C
     3 SNEX(50),TEMP(50),TEXT(20),XL(66,39)                             C
C                                                                       C
      COMMON /MATR/MATSIZ,MATORD,SIGD,V(MP),SMM(MP*(MP+1)/2)                   C
C                                                                       C
      COMMON /REFS/HN(5,3),HH(3,ML),ICODE(ML),SJJ(ML),SOMEGA(ML),       C
     2 FOBS(ML),FESD(ML),FNUC(ML),FMAG(ML),THETA(ML),TANN(ML),HW(ML),   C
     3 DERSTO(ML,MX),AKEEP(ML),BKEEP(ML),PKEEP(ML),ICODD(ML)            C
C                                                                       C
      COMMON /INOUT/LCA,LPR,LPRE,LT4,LT8,LFOR,LPUN                      C
C                                                                       C
      COMMON /NUMB/N,N1,N2,N3,N4,N6,N7,NN,NN5,NA(5),NPAT,NREF,NSPAR
C                                                                       C
      COMMON /KEYS/LP(66,39),FLAG(66,39)                                C
      COMMON /LOGIC/FORCEL,FORCED,PUNCH,STAR,TYPE,VERT,YES,ANGCOU              M
      LOGICAL FORCEL,FORCED,PUNCH,STAR,TYPE,VERT,YES,ANGCOU                    M
C                                                                       C
C     THE FOLLOWING EQIVALENCES ARE INTENDED TO SAVE STORAGE REQUIREMENTC
C     BY RE-USING THE ARRAY DERSTO. THE ARRAYS CONTAINED IN THE         C
C     DIMENSION STATEMENT BELOW ARE OF FIXED SIZE, AND HENCE THE        C
C     EQUIVALENCE STATEMENT REQUIRES ALTERATION ONLY WHEN THE  SIZE     C
C     OF MATRIX DERSTO IS REDUCED FROM THE PRESENT VALUE OF (40,50).    C
C                                                                       C
C     NOTE THAT A CALL TO THIS ROUTINE OVERWRITES THE VALUES OF THE     C
C     STANDARD DEVIATIONS OF THE REFINED PARAMETERS STORED IN ARRAY     C
C     STAND, WHICH WAS EQUIVALENCED TO THE FIRST PART OF DERSTO.        C
C      WITH THE PRESENT ORDER OF CALCULATION, THESE E.S.D. S ARE        C
C     OF NO FURTHER USE.                                                C
C                                                                       C
      EQUIVALENCE (DERSTO(1,1),DEQ(1))                                  C
      EQUIVALENCE (DEQ(1),COV(1,1,1)),(DEQ(181),RCOV(1,1,1)),
     2 (DEQ(361),D(1,1,1)),(DEQ(541),P(1,1)),(DEQ(571),
     3 TCOV(1,1,1)),(DEQ(586),DRSIN(1,1)),(DEQ(601),DRTAN(1,
     4 1)),(DEQ(781),RCSIN(1,1)),(DEQ(796),SCOV(1,1,1)),
     5 (DEQ(976),DVDP(1,1))
C                                                                       C
C    COV    OCCUPIES DERSTO(1)   TO  DERSTO(180)                        C
C    RCOV                  (181)           (360)                        C
C    D                     (361)           (540)                        C
C    P                     (541)           (570)                        C
C    TCOV                  (571)           (585)                        C
C    DRSIN                 (586)           (600)                        C
C    DRTAN                 (601)           (780)                        C
C    RCSIN                 (781)           (795)                        C
C    SCOV                  (796)           (975)                        C
C    DVDP                  (976)          (1005)                        C
C                                                                       C
C                                                                       C
C     WERE ANY CELL PARAMETERS REFINED?                                 C
C                                                                       C
      N2=N7+NP                                                          C
      DO 20 I=1,6                                                       C
      IF(LP(N2,I))20,20,10                                              C
10    IF(LP(N2,I)-MATORD)30,30,20                                       C
20    CONTINUE                                                          C
C                                                                       C
C     NO CELL REFINEMENT                                                C
C                                                                       C
C                                                                       C
C     REST TO REAL CELL VOLUME                                          C
C                                                                       C
      VOL(NP)=1.0/VOL(NP)                                               C
      RETURN                                                            C
C                                                                       C
C     COPY RECIPROCAL METRIC TENSOR COMPONENTS INTO ARRAY P             C
C     AND CLEAR V.C. AND DERIVATIVE MATRICES.                           C
C                                                                       C
30    DO 40 I=1,6                                                       C
      P(NP,I)=XL(N2,I)                                                  C
      DO 40 J=1,6                                                       C
      COV(NP,I,J)=0.0                                                   C
      D(NP,I,J)=0.0                                                     C
40    CONTINUE                                                          C
C                                                                       C
C                                                                       C
C     SET UP TRIGONOMETRIC FUNCTIONS FOR DERIVATIVE CALCULATION         C
C                                                                       C
      G=1.0                                                             C
      DO 80 I=1,3                                                       C
      J=I+3                                                             C
C                                                                       C
C     DIRECT CELL SINES                                                 C
C                                                                       C
      DRSIN(NP,I)=SQRT(1.0-CELLN(NP,J)*CELLN(NP,J))                     C
C                                                                       C
C     DIRECT CELL TANGENTS                                              C
C                                                                       C
      IF(CELLN(NP,J))60,50,60                                           C
50    DRTAN(NP,I)=1.0E22                                                C
      GOTO 70                                                           C
60    DRTAN(NP,I)=DRSIN(NP,I)/CELLN(NP,J)                               C
C                                                                       C
C     RECIPROCAL CELL SINES                                             C
C                                                                       C
70    RCSIN(NP,I)=SQRT(1.0-RCELLN(NP,J)*RCELLN(NP,J))                   C
C                                                                       C
C     FACTOR COMMON TO MANY DERIVATIVES                                 C
C                                                                       C
      G=G*CELLN(NP,I)*CELLN(NP,I)*DRSIN(NP,I)                           C
80    CONTINUE                                                          C
C                                                                       C
C     VOLUME IS PRESENTLY THE RECIPROCAL VOLUME                         C
C                                                                       C
      VS=2.0*VOL(NP)*VOL(NP)                                            C
      G=G*VS                                                            C
C                                                                       C
C     SELECT OUT THE VARIANCE-COVARIANCE MATRIX FOR THE CELL            C
C     PARAMETERS FROM THE COMPLETE V.C. MATRIX AFTER THE REFINEMENT     C
C                                                                       C
      SD=SIGD*SIGD                                                      C
      DO 140 I=1,6                                                      C
      II=I                                                              C
      L1=LP(N2,II)                                                      C
      IF(L1)140,140,90                                                  C
90    IF(L1-MATORD)100,100,140                                          C
100   DO 130 J=I,6                                                      C
      JJ=J                                                              C
      L2=LP(N2,JJ)                                                      C
      IF(L2)130,130,110                                                 C
110   IF(L2-MATORD)120,120,130                                          C
120   CALL LOC(L1,L2,NU,MATORD)                                         C
      COV(NP,I,J)=SMM(NU)*FLAG(N2,II)*FLAG(N2,JJ)*SD                    C
      COV(NP,J,I)=COV(NP,I,J)                                           C
130   CONTINUE                                                          C
140   CONTINUE                                                          C
C                                                                       C
C     THE ARRAY COV IS NOW THE V.C. MATRIX FOR THE TENSOR COMPONENTS    C
C                                                                       C
C     CALCULATE THE MATRIX OF DERIVATIVES OF THE DIRECT CELL            C
C     PARAMETERS WITH RESPECT TO THE RECIPROCAL METRIC TENSOR COMPONENTSC
C     THIS MATRIX IS ESTABLISHED AS THE COMBINATION OF THE FOUR PARTITIOC
C                                                                       C
C     1) D (LENGTH*)**2 /D LENGTH                                       C
C     2) D (LENGTH*)**2 /D ANGLE                                        C
C     3) D CROSS PRODUCT /D LENGTH                                      C
C     4) D CROSS PRODUCT /D ANGLE                                       C
C                                                                       C
C     LIMIT BELOW WHICH CELL PARAMETERS ARE ASSUMED TO BE ZERO          C
C                                                                       C
      CLIM=1.0E-7                                                       C
C                                                                       C
      DO 310 I=1,3                                                      C
C                                                                       C
C     PART 1                                                            C
C                                                                       C
      D(NP,I,I)=-0.5*CELLN(NP,I)/P(NP,I)                                C
      L1=I+3                                                            C
C                                                                       C
C     PART 2                                                            C
C                                                                       C
      DO 300 J=1,3                                                      C
      L2=J+3                                                            C
      CALL PERM(J,K,L)                                                  C
      IF(J-I)170,150,170                                                C
150   IF(CELLN(NP,L2))160,200,160                                       C
160   D(NP,L1,J)=2.0/DRTAN(NP,J)                                        C
170   IF(P(NP,J))180,200,180                                            C
180   IF(RCELLN(NP,L1))190,200,190                                      C
190   D(NP,L1,J)=D(NP,L1,J)+G*RCELLN(NP,L1)                             C
      if(d(np,l1,j).lt.clim) then
      d(np,l1,j)=0.
      goto 200
      endif
      D(NP,L1,J)=1.0/(D(NP,L1,J)*P(NP,J))                               C
200   IF(I-J)210,230,210                                                C
C                                                                       C
C     PART 3                                                            C
C                                                                       C
210   IF(P(NP,L2))220,230,220                                           C
220   D(NP,I,L2)=-CELLN(NP,I)/P(NP,L2)                                  C
C                                                                       C
C                                                                       C
C     PART 4                                                            C
C                                                                       C
230   IF(P(NP,L2))240,300,240                                           C
240   IF(RCELLN(NP,L1))260,250,260                                      C
C 173 IF(ABS(CELLN(NP,9-I-J))-CLIM) 174,175,175                         C
250   IF(I-J)300,260,300                                                C
260   D(NP,L1,L2)=VS*CELLN(NP,J)*CELLN(NP,J)*CELLN(NP,K)*CELLN(NP,L)    C
     2 *DRSIN(NP,I)                                                     C
      IF(I-J)270,280,270                                                C
270   D(NP,L1,L2)=-D(NP,L1,L2)*CELLN(NP,9-I-J)                          C
280   D(NP,L1,L2)=D(NP,L1,L2)+G*P(NP,L2)*RCELLN(NP,L1)                  C
290   D(NP,L1,L2)=1.0/D(NP,L1,L2)                                       C
300   CONTINUE                                                          C
310   CONTINUE                                                          C
C                                                                       C
c     use (real!) inverse of matrix D
      do 3001 i=1,6
      do 3001 j=1,6
      dd(i,j)=0.
      if(d(np,i,j).eq.0.) goto 3001
      dd(i,j)=1./d(np,i,j)
ccc      write(98,*) 'dd: ',i,j,dd(i,j)
 3001 continue
      call invmat(dd,6)
      do 3002 i=1,6
      do 3002 j=1,6
      d(np,i,j)=dd(i,j)    
 3002 continue
C     MULTIPLY MATRICES COV AND D TO YIELD V.C. MATRIX TCOV FOR         C
C     THE DIRECT CELL PARAMETERS                                        C
C                                                                       C
      CALL MATABT(RCOV,COV,D,NP)                                        C
      CALL MATAB(TCOV,D,RCOV,NP)                                        C
C                                                                       C
C     CALCULATE THE V.C. MATRIX FOR THE RECIPROCAL CELL                 C
C                                                                       C
C     CLEAR DERIVATIVE MATRIX                                           C
C                                                                       C
      DO 320 I=1,6                                                      C
      DO 320 J=1,6                                                      C
ccc      write(98,*) i,j,d(np,i,j),'....cov.',cov(np,i,j)
ccc      write(98,*) '----rcov-',rcov(np,i,j), '---tcov-',tcov(np,i,j)
      D(NP,I,J)=0.0                                                     C
320   CONTINUE                                                          C
C                                                                       C
C     OBTAIN DERIVATIVE MATRIX,AGAIN AS THE COMBINATION OF              C
C     FOUR PARTS                                                        C
C                                                                       C
      DO 380 I=1,3                                                      C
C                                                                       C
C     PART 1                                                            C
C                                                                       C
      D(NP,I,I)=0.5/RCELLN(NP,I)                                        C
      L1=I+3                                                            C
      DO 370 J=1,3                                                      C
      L2=J+3                                                            C
C                                                                       C
C     PART 2 IS COMPLETELY ZERO                                         C
C                                                                       C
      IF(J-I)330,350,330                                                C
C                                                                       C
C     PART 3                                                            C
C                                                                       C
330   IF(RCELLN(NP,L2))340,370,340                                      C
340   D(NP,I,L2)=2.0*(RCELLN(NP,6-I-J)*RCELLN(NP,L2))                   C
      if(d(np,i,l2).eq.0.) goto 370
      d(np,i,l2)=1./d(np,i,l2)
      GOTO 370                                                          C
350   CALL PERM(J,K,L)                                                  C
C                                                                       C
C     PART 4                                                            C
C                                                                       C
      D(NP,L1,L2)=-2.0*(RCELLN(NP,L)*RCELLN(NP,K)*RCSIN(NP,J))          C
      IF(D(NP,L1,L2))360,370,360                                        C
360   D(NP,L1,L2)=1.0/D(NP,L1,L2)                                       C
370   CONTINUE                                                          C
380   CONTINUE                                                          C
C                                                                       C
C     MULTIPLY DERIVATIVE AND V.C. MATRICES TO GET RECIPROCAL           C
C     CELL V.C. MATRIX SCOV                                             C
C                                                                       C
      do 3003 i=1,6
      do 3003 j=1,6
      dd(i,j)=0.
      if(d(np,i,j).eq.0.) goto 3003
      dd(i,j)=1./d(np,i,j)
ccc      write(99,*) 'dd: ',i,j,dd(i,j)
 3003 continue
      call invmat(dd,6)
      do 3004 i=1,6
      do 3004 j=1,6
      d(np,i,j)=dd(i,j)    
 3004 continue
      CALL MATABT(RCOV,COV,D,NP)                                        C
      CALL MATAB(SCOV,D,RCOV,NP)                                        C
C                                                                       C
ccc      DO 3320 I=1,6                                                      C
ccc      DO 3320 J=1,6                                                      C
ccc      write(99,*) i,j,d(np,i,j),'....cov.',cov(np,i,j)
ccc      write(99,*) '----rcov-',rcov(np,i,j), '---tcov-',tcov(np,i,j)
ccc3320  CONTINUE                                                          C
C     CALCULATE THE STANDARD DEVIATIONS OF THE CELL VOLUME              C
C                                                                       C
C     REAL CELL VOLUME                                                  C
C                                                                       C
C                                                                       C
      VOL(NP)=1.0/VOL(NP)                                               C
C                                                                       C
C     VOLUME IS NOW THE REAL CELL VOLUME                                C
C                                                                       C
      G=0.5*G*VOL(NP)                                                   C
      CALL VESD(E1,G,VOL,CELLN,RCELLN,TCOV,DVDP,NP)                     C
C                                                                       C
C     ESD OF RECIPROCAL VOLUME                                          C
C                                                                       C
      G=1.0                                                             C
      DO 390 I=1,3                                                      C
      G=G*RCELLN(NP,I)*RCELLN(NP,I)*RCSIN(NP,I)                         C
390   CONTINUE                                                          C
      G=G*VOL(NP)                                                       C
      VS=VOL(NP)                                                        C
C                                                                       C
      VOL(NP)=1.0/VOL(NP)                                               C
C                                                                       C
C                                                                       C
C     VOLUME IS NOW THE RECIPROCAL CELL VOLUME                          C
C                                                                       C
      CALL VESD(E2,G,VOL,RCELLN,CELLN,SCOV,DVDP,NP)                     C
C                                                                       C
C     CALCULATE THE ESD'S OF THE CELL PARAMETERS                        C
C                                                                       C
      CALL CESD(TCOV,P,CELLN,NP)                                        C
      CALL CESD(SCOV,DVDP,RCELLN,NP)                                    C
C                                                                       C
C     OUTPUT THE RESULTS                                                C
C                                                                       C
      WRITE(LPR,400)NP,(PTEXT(NP,J),J=1,4)                              C
      WRITE(LPR,410)                                                    C
      WRITE(LPR,420)(CELLN(NP,I),TCOV(NP,I,I),I=1,3),(P(NP,I),TCOV(NP,  C
     2 I,I),I=4,6)                                                      C
      IF (FORCEL) THEN
      WRITE(12,400)NP,(PTEXT(NP,J),J=1,4)                               C
      WRITE( 12,420)(CELLN(NP,I),TCOV(NP,I,I),I=1,3),(P(NP,I),TCOV(NP,  C
     2 I,I),I=4,6)                                                      C
      ENDIF
      WRITE(LPR,430)                                                    C
      WRITE(LPR,440)(RCELLN(NP,I),SCOV(NP,I,I),I=1,3),(DVDP(NP,I),      C
     2 SCOV(NP,I,I),I=4,6)                                              C
      WRITE(LPR,450)                                                    C
      WRITE(LPR,460)VS,E1,VOL(NP),E2                                    C
C                                                                       C
C     RESET TO REAL CELL VOLUME                                         C
C                                                                       C
      VOL(NP)=1.0/VOL(NP)                                               C
      RETURN                                                            C
400   FORMAT(1H0,////10X,                                               C
     2 50HCELL PARAMETERS WITH ESTIMATED STANDARD DEVIATIONS,1X,        C
     3 11HFOR PATTERN,I2,2X,4A4)                                        C
410   FORMAT(1H0,5X,1HA,19X,1HB,19X,1HC,18X,5HALPHA,13X,4HBETA,15X,     C
     2 5HGAMMA)                                                         C
420   FORMAT(1H ,3(F10.4,1H(,F8.4,1H)),3(F10.3,1H(,F6.3,1H)))           C
430   FORMAT(1H0,5X,2HA*,18X,2HB*,18X,2HC*,17X,6HALPHA*,12X,5HBETA*,    C
     2 14X,6HGAMMA*)                                                    C
440   FORMAT(1H ,3(F10.6,1H(,F8.6,1H)),3(F10.3,1H(,F6.3,1H)))           C
450   FORMAT(1H0,27X,4HREAL,16X,12HCELL VOLUMES,8X,10HRECIPROCAL)       C
460   FORMAT(1H ,22X,F10.3,1H(,F6.3,1H),26X,F10.7,1H(,F10.7,1H))        C
      END                                                               C
      SUBROUTINE CESD(TCOV,P,CELLN,NP)                                  C
      PARAMETER (ML=500 , MP=99 , MX=103)
C     ************************************************************      C
C      TAKES THE SQUARE ROOT OF TCOV(I,I), AND CONVERTS RADIANS         C
C     TO DEGREES                                                        C
C     ************************************************************      C
C                                                                       C
      DIMENSION TCOV(5,6,6),P(5,6),CELLN(5,6)                           C
C                                                                       C
C                                                                       C
      COMMON /GEOM/DEGRAD,RADDEG                                        C
C                                                                       C
      DO 30 I=1,6                                                       C
      IF(TCOV(NP,I,I))10,30,20                                          C
10    TCOV(NP,I,I)=0.0                                                  C
      GOTO 30                                                           C
20    TCOV(NP,I,I)=SQRT(TCOV(NP,I,I))                                   C
30    CONTINUE                                                          C
      DO 40 I=4,6                                                       C
      TCOV(NP,I,I)=TCOV(NP,I,I)*RADDEG                                  C
      P(NP,I)=ACOS(CELLN(NP,I))*RADDEG                                  C
40    CONTINUE                                                          C
      RETURN                                                            C
      END                                                               C
      SUBROUTINE VESD(E,G,VOL,CELLN,RCELLN,TCOV,DVDP,NP)                V
      PARAMETER (ML=500 , MP=99 , MX=103)
C     *********************************************************         V
C     CALCULATES THE STANDARD DEVIATION OF THE CELL VOLUME FROM         V
C     THE V.C. MATRIX OF THE REAL OR RECIPROCAL CELL PARAMETERS         V
C      AND THE APPROPRIATE DERIVATIVES                                  V
C     *********************************************************         V
C                                                                       V
      DIMENSION CELLN(5,6),RCELLN(5,6),TCOV(5,6,6),DVDP(5,6)            V
      DIMENSION VOL(5)                                                  V
C                                                                       V
C     OBTAIN THE DERIVATIVES OF THE VOLUME WITH RESPECT TO THE          V
C     CELL PARAMETERS                                                   V
C                                                                       V
      DO 10 I=1,3                                                       V
      II=I+3                                                            V
      DVDP(NP,I)=VOL(NP)/CELLN(NP,I)                                    V
      DVDP(NP,II)=-G*RCELLN(NP,II)                                      V
10    CONTINUE                                                          V
      E=0.0                                                             V
C     SUM THE PRODUCT OF DVDP AND TCOV                                  V
      DO 60 I=1,6                                                       V
      DO 50 J=I,6                                                       V
      IF(I-J)20,30,20                                                   V
20    TOO=2.0                                                           V
      GOTO 40                                                           V
30    TOO=1.0                                                           V
40    E=E+DVDP(NP,I)*DVDP(NP,J)*TCOV(NP,I,J)*TOO                        V
50    CONTINUE                                                          V
60    CONTINUE                                                          V
C                                                                       V
C     E IS THE VARIANCE OF THE VOLUME                                   V
C                                                                       V
      IF(E)70,90,80                                                     V
70    E=0.0                                                             V
      RETURN                                                            V
80    E=SQRT(E)                                                         V
90    RETURN                                                            V
      END                                                               V
      SUBROUTINE MATAB(RCOV,D,COV,NP)                                   M
      PARAMETER (ML=500 , MP=99 , MX=103)
C     *********************************************************         M
C      MULTIPLIES SQUARE MATRICES OF ORDER 6 ACCORDING TO:              M
C                                                                       M
C     RCOV= D*COV                                                       M
C     *********************************************************         M
C                                                                       M
      DIMENSION RCOV(5,6,6),COV(5,6,6),D(5,6,6)                         M
C                                                                       M
      DO 30 I=1,6                                                       M
      DO 20 J=1,6                                                       M
      RCOV(NP,I,J)=0.0                                                  M
      DO 10 M=1,6                                                       M
      RCOV(NP,I,J)=RCOV(NP,I,J)+D(NP,I,M)*COV(NP,M,J)                   M
10    CONTINUE                                                          M
20    CONTINUE                                                          M
30    CONTINUE                                                          M
      RETURN                                                            M
      END                                                               M
      SUBROUTINE MATABT(RCOV,D,COV,NP)                                  M
      PARAMETER (ML=500 , MP=99 , MX=103)
C     ***************************************************               M
C     MULTIPLIES MATRIX D AND THE TRANSPOSE OF MATRIX COV               M
C     ALL MATRICES OF ORDER 6                                           M
C                                                                       M
C     RCOV= D*COV(T)                                                    M
C     ***************************************************               M
C                                                                       M
      DIMENSION RCOV(5,6,6),D(5,6,6),COV(5,6,6)                         M
C                                                                       M
      DO 30 I=1,6                                                       M
      DO 20 J=1,6                                                       M
      RCOV(NP,I,J)=0.0                                                  M
      DO 10 M=1,6                                                       M
      RCOV(NP,I,J)=RCOV(NP,I,J)+D(NP,I,M)*COV(NP,J,M)                   M
10    CONTINUE                                                          M
20    CONTINUE                                                          M
30    CONTINUE                                                          M
      RETURN                                                            M
      END                                                               M
      SUBROUTINE SHOUT                                                  S
      PARAMETER (ML=500 , MP=99 , MX=103)
C     *********************************************************         S
C     THIS ROUTINE PRINTS THE REFINED PARAMETERS, SHIFTS,               S
C     AND STANDARD DEVIATIONS AT THE END OF EACH CYCLE                  S
C     *********************************************************         S
C                                                                       S
      LOGICAL FORCEL,FORCED,PUNCH,STAR,TYPE,VERT,YES,ANGCOU                    S
C                                                                       S
C                                                                       S
      DIMENSION STAND(MP),FMT(MP),FA(ML*MX)                             S
C                                                                       S
C     STAND OCCUPIES DERSTO(1) TO DERSTO(MP)                            S
C     STAND CONTAINS THE ESD'S FROM THE REFINEMENT                      S
C     FMT OCCUPIES DERSTO(MP+1)                                            C
C                                                                       S
C                                                                       S
      COMMON /REFLN/ARG,DELTA,DLABDA(2),DUMPA(500),EPS,FLOG,FLOG2,      S
     2 OMEGA,PI,SKEW,SLABDA,SLOG,W,XN,YCALC,YOBS                        S
C                                                                       S
      COMMON /PREF/IALFA,IOMEG,IORD1,IORD2,IPREV,ISTAP,ISS,KSTART,LIMO, S
     2 LLL,NCD(50),NKEY,NO,NTEXT,NUM,NTYP(50),MEQ(50),MTYP(50)          S
C                                                                       S
      COMMON /SYMCEL/ALL(3,3),AL(5,3,3),ATEXT(60),B(8),CELL(6),CELLN(5, S
     2 6),F(4,21),FF(4),PH(5,3),RM(5,24,2,9),S(4,21),SM(5,24,3,3),T(5,  S
     3 24,3),TR(5,24),RCELL(6),RCELLN(5,6),VOLUME,VOL(5),PTEXT(5,4)     S
C                                                                       S
      COMMON /MINIM/COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),HL(3,3),  S
     2 RSA(50,3),RSB(50,3),SA(50),SB(50),SINAR(50,24),SINARI(24),       S
     3 SNEX(50),TEMP(50),TEXT(20),XL(66,39)                             S
C                                                                       S
      COMMON /KEYS/LP(66,39),FLAG(66,39)                                S
C                                                                       S
      COMMON /CONTRL/NCYCLE,MAXS,KXR(5),KKZ,LIM,LOUT,IPONS,LMAT,LCORD,  S
     2 LCOORD,ICELL,ICENT,ICENTT(5),IRL,IRLL(5),KL(5),KLL,KM(5),KMN,    S
     3 MN(5),MNN,ICYCLE,MAXCON,IFOUR,IPLOT,KLOOP,LINES,IWID,IPREF(5),   S
     4 IPREFF,NHALP(5),NUNIAX(5),NCUBIC(5),NGAUSS(5),NCAUCH(5),NPAC(5)  S
     5 ,CLOR(5),ELOR(5)
C                                                                       S
      COMMON /BFAC/ITF,JTF(50),PRCP(5,6),ITF3,ITF4                      S
C                                                                       S
      COMMON /REFS/HN(5,3),HH(3,ML),ICODE(ML),SJJ(ML),SOMEGA(ML),       S
     2 FOBS(ML),FESD(ML),FNUC(ML),FMAG(ML),THETA(ML),TANN(ML),HW(ML),   S
     3 DERSTO(ML,MX),AKEEP(ML),BKEEP(ML),PKEEP(ML),ICODD(ML)            S
C                                                                       S
      COMMON /INOUT/LCA,LPR,LPRE,LT4,LT8,LFOR,LPUN                      S
C                                                                       S
      COMMON /NUMB/N,N1,N2,N3,N4,N6,N7,NN,NN5,NA(5),NPAT,NREF,NSPAR
C                                                                       S
      COMMON /LOGIC/FORCEL,FORCED,PUNCH,STAR,TYPE,VERT,YES,ANGCOU              S
C                                                                       S
C     THE FOLLOWING EQUIVALENCE STATEMENT ALLOWS ACCESS TO VARIOUS      S
C     QUANTITIES THROUGH COMMON BLOCK /REFS/ AND IS INTENDED TO CONSERVES
C     STORE. ARRAY STAND IS DIMENSIONED TO THE NUMBER OF                S
C     REFINABLE PARAMETERS (MMATOR IN COMMON BLOCK /MAXDIM/),           S
C     AND ARRAY DERSTO HAS DIMENSIONS (MNOVRF,MMATOR)  ALSO STORED      S
C     IN BLOCK /MAXDIM/ . HENCE IF THE VALUE OF MNOVRF IS               S
C     DECREASED, OR THE VALUE OF MMATOR IS INCREASED, THE               S
C     EQUIVALENCE STATEMENT REQUIRES MODIFICATION.                      S
C                                                                       S
C                                                                       S
      EQUIVALENCE (DERSTO(1,1),FA(1))
      EQUIVALENCE (DERSTO(1,1),STAND(1)),(FA(MP+1),FMT(1)),               S
     2 (ICODE(1),II),(ICODE(2),III),(ICODE(3),L),(ICODE(4),M),(FOBS(1), S
     3 AGM),(FOBS(2),ESD),(FOBS(3),LGG)                                 S
C                                                                       S
C                                                                       S
      DATA ONEH/4H(1H /,THRX/4H,3X /,AFOR/4H,A4 /,BLANK/4H    /         S
C                                                                       S
C     OUTPUT TITLE                                                      S
C                                                                       S
       LPRA=LPR
      WRITE(LPR,190)ICYCLE                                              S
C     IF(FORCEL) WRITE(12,190) ICYCLE
C                                                                       S
C     SET STANDARD PART OF ARRAY FMT                                    S
C                                                                       S
      FMT(1)=ONEH                                                       S
      FMT(2)=AFOR                                                       S
      FMT(3)=THRX                                                       S
C                                                                       S
C     SET LOOPING PARAMETERS TO COVER ATOM LIST                         S
C                                                                       S
      II=1                                                              S
      III=N                                                             S
C                                                                       S
C     OUTPUT PART 1 OF THE DATA: X,Y,Z,ISOTROPIC B                      S
C                                                                       S
      L=1                                                               S
      M=4                                                               S
      WRITE(LPR,200)                                                    S
      CALL PRNT                                                         S
      IF(FORCEL) THEN
      LPR=12
      WRITE(LPR,200)                                                    S
      CALL PRNT
      LPR=LPRA
      ENDIF
C                                                                       S
C     OUTPUT SECOND PART OF DATA: OCCUPATION NUMBER                     S
C                                                                       S
      LGG=0                                                             S
      DO 40 I=1,N                                                       S
      IF(LP(I,5))40,40,10                                               S
10    IF(LGG)20,20,30                                                   S
20    WRITE(LPR,210)                                                    S
      IF(FORCEL) WRITE(12,210)
      LGG=1                                                             S
30    II=I                                                              S
      III=I                                                             S
      L=5                                                               S
      M=5                                                               S
      CALL PRNT                                                         S
      IF(FORCEL) THEN
      LPR=12
      CALL PRNT
      LPR=LPRA
      ENDIF
40    CONTINUE                                                          S
C                                                                       S
C     ARE THERE ANY MAGNETIC ATOMS?                                     S
C                                                                       S
      IF(KXR(1).GE.4) GOTO 80                                           S
50    IF(KXR(1).EQ.0.AND.KXR(2).EQ.0.AND.KXR(3).EQ.0)GOTO 80            S
C                                                                       S
C     YES                                                               S
C                                                                       S
      WRITE(LPR,220)                                                    S
      IF(FORCEL) WRITE(12,220)
      L=6                                                               S
      M=8                                                               S
C                                                                       S
C     CYCLE THROUGH ATOM LIST PICKING OUT MAGNETIC ATOMS                S
C                                                                       S
      DO 70 I=1,N                                                       S
      IF(MTYP(I))70,70,60                                               S
60    II=I                                                              S
      III=I                                                             S
      CALL PRNT                                                         S
      IF(FORCEL) THEN
      LPR=12
      CALL PRNT
      LPR=LPRA
      ENDIF
C                                                                       S
C     ENTER ROUTINE TO CALCULATE MAGNETIC MOMENT (WITH ESD)             S
C                                                                       S
      CALL MOM(I,AGM,ESD)                                               S
      WRITE(LPR,230)AGM,ESD                                             S
      IF(FORCEL) WRITE(12,230) AGM,ESD
70    CONTINUE                                                          S
C                                                                       S
C     ARE THERE ANY ANISOTROPIC ATOMS?                                  S
C                                                                       S
80    IF(ITF)140,140,90                                                 S
C                                                                       S
C     PART 1: B(I,I)                                                    S
C                                                                       S
90    WRITE(LPR,240)                                                    S
      IF(FORCEL) WRITE(12,240)
      L=9                                                               S
      M=11                                                              S
C                                                                       S
C     CYCLE THROUGH ATOM LIST PICKING OUT ANISOTROPIC ATOMS             S
C                                                                       S
100   DO 120 I=1,N                                                      S
      IF(JTF(I))120,120,110                                             S
110   II=I                                                              S
      III=I                                                             S
      CALL PRNT                                                         S
      IF(FORCEL) THEN
      LPR=12
      CALL PRNT
      LPR=LPRA
      ENDIF
120   CONTINUE                                                          S
C                                                                       S
C     PART 2: B(I,J)                                                    S
C                                                                       S
      IF(L-9)130,130,145                                                S
130   WRITE(LPR,250)                                                    S
      IF(FORCEL) WRITE(12,250)
      L=12                                                              S
      M=14                                                              S
      GOTO 100                                                          S
C
C     OUTPUT ANHARMONIC ATOMS
C
  145 IF(ITF3.EQ.0) GOTO 140
      WRITE(LPR,400)
      WRITE(LPR,410)
      WRITE(LPR,420)
      IF(FORCEL) WRITE(12,420)
      L=15
      M=17
  500 DO 510 I=1,N
      IF(JTF(I).LT.3) GOTO 510
      II=I
      III=I
      CALL PRNT
      IF(FORCEL) THEN
      LPR=12
      CALL PRNT
      LPR=LPRA
      ENDIF
  510 CONTINUE
      L=L+3
      M=M+3
      IF(L.EQ.24) M=24
      IF(L.GT.24) GOTO 550
      GOTO 500
  550 IF(ITF4.EQ.0) GOTO 140
      WRITE(LPR,430)
      WRITE(LPR,440)
      IF(FORCEL) WRITE(12,440)
      L=25
      M=27
  570 DO 560 I=1,N
      IF(JTF(I).LT.4) GOTO 560
      II=I
      III=I
      CALL PRNT
      IF(FORCEL) THEN
      LPR=12
      CALL PRNT
      LPR=LPRA
      ENDIF
  560 CONTINUE
      L=L+3
      M=M+3
      IF(L.GE.40) GOTO 140
      GOTO 570
C
  400 FORMAT(1H0,10X,'ANHARMONIC ATOMS')
  410 FORMAT(1H0,10X,'THIRD ORDER*1000')
  430 FORMAT(1H0,10X,'FOURTH ORDER*10000')
  420 FORMAT(5H0ATOM,7X,4HC111,5X,5HSHIFT,4X,3HESD,8X,
     1                  4HC222,5X,5HSHIFT,4X,3HESD,8X,
     2                  4HC333,5X,5HSHIFT,4X,3HESD,8X,/,
     3              12X,4HC112,5X,5HSHIFT,4X,3HESD,8X,
     4                  4HC122,5X,5HSHIFT,4X,3HESD,8X,
     5                  4HC113,5X,5HSHIFT,4X,3HESD,8X,/,
     6              12X,4HC133,5X,5HSHIFT,4X,3HESD,8X,
     8                  4HC223,5X,5HSHIFT,4X,3HESD,8X,
     8                  4HC233,5X,5HSHIFT,4X,3HESD,8X,/,
     9              12X,4HC123,5X,5HSHIFT,4X,3HESD,8X)
  440 FORMAT(5H0ATOM,5X,5HD1111,4X,5HSHIFT,4X,3HESD,6X,
     1                  5HD2222,4X,5HSHIFT,4X,3HESD,6X,
     2                  5HD3333,4X,5HSHIFT,4X,3HESD,6X,/,
     3              10X,5HD1112,4X,5HSHIFT,4X,3HESD,6X,
     4                  5HD1222,4X,5HSHIFT,4X,3HESD,6X,
     5                  5HD1113,4X,5HSHIFT,4X,3HESD,6X,/,
     6              10X,5HD1333,4X,5HSHIFT,4X,3HESD,6X,
     7                  5HD2223,4X,5HSHIFT,4X,3HESD,6X,
     8                  5HD2333,4X,5HSHIFT,4X,3HESD,6X,/,
     9              10X,5HD1122,4X,5HSHIFT,4X,3HESD,6X,
     1                  5HD1133,4X,5HSHIFT,4X,3HESD,6X,
     2                  5HD2233,4X,5HSHIFT,4X,3HESD,6X,/,
     3              10X,5HD1123,4X,5HSHIFT,4X,3HESD,6X,
     4                  5HD1223,4X,5HSHIFT,4X,3HESD,6X,
     5                  5HD1233,4X,5HSHIFT,4X,3HESD,6X)
C
C
C     ZEROPOINT                                                         S
140   ATEXT(N6)=BLANK                                                   S
      II=N6                                                             S
      III=N6                                                            S
      L=1                                                               S
      IF(XL(N6,2).NE.0.0)GOTO 150                                       S
      WRITE(LPR,320)                                                    S
      IF(FORCEL) WRITE(12,320)
      M=1                                                               S
      GOTO 160                                                          S
150   WRITE(LPR,390)                                                    S
      IF(FORCEL) WRITE(12,390)
      M=NPAT                                                            S
160   CALL PRNT                                                         S
      IF(FORCEL) THEN
      LPR=12
      CALL PRNT
      LPR=LPRA
      ENDIF
C                                                                       S
C     OVERALL PARAMETERS                                                S
C                                                                       S
      DO 170 NP=1,NPAT                                                  S
      N3=N6+NP                                                          S
      N2=N7+NP                                                          S
      WRITE(LPR,260)NP,(PTEXT(NP,J),J=1,4)                              S
      IF(FORCEL) WRITE(12,261)NP,(PTEXT(NP,J),J=1,4)
  261 FORMAT(2X,7HPATTERN,1X,I2,3X,4A4)
      N1=N+NP                                                           S
      WRITE(LPR,270)                                                    S
C     IF(FORCEL) WRITE(12,270)
      WRITE(LPR,280)                                                    S
      IF(FORCEL) WRITE(12,280)
      ATEXT(N1)=BLANK                                                   S
      II=N1                                                             S
      III=N1                                                            S
      L=1                                                               S
      M=4                                                               S
      CALL PRNT                                                         S
      IF(FORCEL) THEN
      LPR=12
      CALL PRNT
      LPR=LPRA
      ENDIF
      WRITE(LPR,290)                                                    S
      IF(FORCEL) WRITE(12,290)
      L=5                                                               S
      M=5                                                               S
      CALL PRNT                                                         S
      IF(FORCEL) THEN
      LPR=12
      CALL PRNT
      LPR=LPRA
      ENDIF
C                                                                       S
C     HALFWIDTHS                                                        S
C                                                                       S
      ATEXT(N3)=BLANK                                                   S
      II=N3                                                             S
      III=N3                                                            S
      L=1                                                               S
      M=3                                                               S
      WRITE(LPR,300)                                                    S
      WRITE(LPR,310)                                                    S
      IF(FORCEL) WRITE(12,310)
      CALL PRNT                                                         S
      IF(FORCEL) THEN
      LPR=12
      CALL PRNT
      LPR=LPRA
      ENDIF
C                                                                       S
C     CELL PARAMETERS                                                   S
C                                                                       S
      ATEXT(N2)=BLANK                                                   S
      II=N2                                                             S
      III=N2                                                            S
      L=1                                                               S
      M=3                                                               S
      WRITE(LPR,340)                                                    S
      WRITE(LPR,350)                                                    S
      CALL PRNT                                                         S
C     IF(FORCEL) THEN
C     LPR=12
C     CALL PRNT
C     LPR=LPRA
C     ENDIF
      WRITE(LPR,360)                                                    S
      L=4                                                               S
      M=6                                                               S
      CALL PRNT                                                         S
C     IF(FORCEL) THEN
C     LPR=12
C     CALL PRNT
C     LPR=LPRA
C     ENDIF
170   CONTINUE                                                          S
C                                                                       S
C     PARAMETER OUTPUT FINISHED                                         S
C                                                                       S
      IF(.NOT.FORCED)GOTO 180                                           S
      WRITE(LPR,370)                                                    S
      IF(FORCEL) WRITE(12,370)
      WRITE(LPR,380)                                                    S
      WRITE(LPR,370)                                                    S
180   RETURN                                                            S
190   FORMAT(1H1,40X,37HSHIFTS AND NEW PARAMETERS AFTER CYCLE,I2)       S
200   FORMAT(5H0ATOM,6X,1HX,8X,5HSHIFT,4X,3HESD,7X,1HY,8X,5HSHIFT,4X,   S
     2 3HESD,7X,1HZ,8X,5HSHIFT,4X,3HESD,7X,5HB ISO,4X,5HSHIFT,4X,3HESD) S
210   FORMAT(5H0ATOM,2X,10HOCCUPATION,3X,5HSHIFT,4X,3HESD)              S
220   FORMAT(5H0ATOM,6X,2HKX,7X,5HSHIFT,4X,3HESD,7X,2HKY,7X,5HSHIFT,4X, S
     2 3HESD,7X,2HKZ,7X,5HSHIFT,4X,3HESD,7X,6HMOMENT,3X,3HESD)          S
230   FORMAT(1H+,91X,2F9.5)                                             S
240   FORMAT(5H0ATOM,6X,3HB11,6X,5HSHIFT,4X,3HESD,7X,3HB22,6X,5HSHIFT,  S
     2 4X,3HESD,7X,3HB33,6X,5HSHIFT,4X,3HESD)                           S
250   FORMAT(5H0ATOM,6X,3HB12,6X,5HSHIFT,4X,3HESD,7X,3HB13,6X,5HSHIFT,  S
     2 4X,3HESD,7X,3HB23,6X,5HSHIFT,4X,3HESD)                           S
260   FORMAT(////,40X,18(1H*),/,40X,1H*,3X,7HPATTERN,1X,I2,3X,1H*,/,    S
     2 40X,1H*,4A4,1H*,/,40X,18(1H*),/)                                 S
270   FORMAT(1H0,5X,9HPREFERRED)                                        S
280   FORMAT(1H ,5X,11HORIENTATION,3X,5HSHIFT,4X,3HESD,4X,9HASYMMETRY,  S
     2 3X,5HSHIFT,4X,3HESD,7X,6HSCALE1,3X,5HSHIFT,4X,3HESD,7X,6HSCALE2, S
     3 3X,5HSHIFT,4X,3HESD)                                             S
290   FORMAT(17H0OVERALL B-FACTOR,3X,5HSHIFT,4X,3HESD)                  S
300   FORMAT(1H0,53X,20HHALFWIDTH PARAMETERS)                           S
310   FORMAT(1H ,10X,1HU,9X,5HSHIFT,5X,3HESD,5X,1HV,9X,5HSHIFT,5X,      S
     2 3HESD,5X,1HW,9X,5HSHIFT,5X,3HESD,3X)                             S
320   FORMAT(/,9X,9HZEROPOINT,3X,5HSHIFT,4X,3HESD)                      S
330   FORMAT(2X,8H*PATTERN,I2,1H*)                                      S
340   FORMAT(1H0,53X,15HCELL PARAMETERS)                                S
350   FORMAT(1H ,10X,1HA,11X,5HSHIFT,7X,3HESD,10X,1HB,11X,5HSHIFT,7X,   S
     2 3HESD,10X,1HC,11X,5HSHIFT,7X,3HESD/)                             S
360   FORMAT(1H0,10X,1HD,11X,5HSHIFT,7X,3HESD,10X,1HE,11X,5HSHIFT,7X,   S
     2 3HESD,10X,1HF,11X,5HSHIFT,7X,3HESD/)                             S
370   FORMAT(1H0,10(12HCONVERGENCE ))                                   S
380   FORMAT(1H0,37X,46HREFINEMENT TERMINATED AT THE END OF THIS CYCLE) S
390   FORMAT(/,9X,3(9HZEROPOINT,3X,5HSHIFT,4X,3HESD))                   S
      END                                                               S
      FUNCTION FREL(I,J)                                                F
C    ***************************************************************    F
C     GETS RELAXATION FACTOR FOR PARAMETER (I,J) IN ARRAY XL            F
C    ***************************************************************    F
C                                                                       F
      COMMON /RELX/RELAXB,RELAXC,RELAXS,RELAXH                          F
C                                                                       F
      COMMON /NUMB/N,N1,N2,N3,N4,N6,N7,NN,NN5,NA(5),NPAT,NREF,NSPAR
C                                                                       F
      NX=N+1                                                            F
C                                                                       F
      IF(I-NX)10,20,20                                                  F
10    IF(J-4)60,70,30                                                   F
20    IF(I-N6)40,50,50                                                  F
30    IF(J-9)80,70,70                                                   F
40    GOTO(70,50,80,80,70),J                                            F
C                                                                       F
C     ZEROPOINT,HALFWIDTHS,CELL PARAMS                                  F
C                                                                       F
50    FREL=RELAXH                                                       F
      RETURN                                                            F
C                                                                       F
C     ATOMIC PARAMETERS                                                 F
C                                                                       F
60    FREL=RELAXC                                                       F
      RETURN                                                            F
C                                                                       F
C     TEMPERATURE FACTORS                                               F
C                                                                       F
70    FREL=RELAXB                                                       F
      RETURN                                                            F
C                                                                       F
C     SCALES,OCCUPATIONS,MAGNETIC PARAMS                                F
C                                                                       F
80    FREL=RELAXS                                                       F
      RETURN                                                            F
      END                                                               F
      SUBROUTINE MOM(I,AGM,ESD)                                         M
      PARAMETER (ML=500 , MP=99 , MX=103)
C     *******************************************************           M
C     CALCULATES MAGNETIC MOMENT WITH ESD FOR ATOM I                    M
C     *******************************************************           M
C                                                                       M
      DIMENSION SMOM(3)                                                 M
C                                                                       M
C     THE FOLLOWING EQUIVALENCE RE-USES THE ARRAY V WHICH AT THIS       M
C     STAGE IS NO LONGER IMPORTANT                                      M
C                                                                       M
C                                                                       M
      COMMON /REFLN/ARG,DELTA,DLABDA(2),DUMPA(500),EPS,FLOG,FLOG2,      M
     2 OMEGA,PI,SKEW,SLABDA,SLOG,W,XN,YCALC,YOBS                        M
C                                                                       M
      COMMON /PREF/IALFA,IOMEG,IORD1,IORD2,IPREV,ISTAP,ISS,KSTART,LIMO, M
     2 LLL,NCD(50),NKEY,NO,NTEXT,NUM,NTYP(50),MEQ(50),MTYP(50)          M
C                                                                       M
      COMMON /SYMCEL/ALL(3,3),AL(5,3,3),ATEXT(60),B(8),CELL(6),CELLN(5, M
     2 6),F(4,21),FF(4),PH(5,3),RM(5,24,2,9),S(4,21),SM(5,24,3,3),T(5,  M
     3 24,3),TR(5,24),RCELL(6),RCELLN(5,6),VOLUME,VOL(5),PTEXT(5,4)     M
C                                                                       M
      COMMON /MINIM/COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),HL(3,3),  M
     2 RSA(50,3),RSB(50,3),SA(50),SB(50),SINAR(50,24),SINARI(24),       M
     3 SNEX(50),TEMP(50),TEXT(20),XL(66,39)                             M
C                                                                       M
      COMMON /MATR/MATSIZ,MATORD,SIGD,V(MP),SMM(MP*(MP+1)/2)                   M
C                                                                       M
      COMMON /KEYS/LP(66,39),FLAG(66,39)                                M
C                                                                       M
C                                                                       M
      EQUIVALENCE (V(1),SMOM(1)),(V(4),COV),(V(5),COR),(V(6),CRO)       M
C                                                                       M
C                                                                       M
C     WERE ANY OF THE VECTOR COMPONENTS REFINED?                        M
C                                                                       M
      NP=NCD(I)                                                         M
      L=MAX0(LP(I,6),LP(I,7),LP(I,8))                                   M
      IF(L)30,30,10                                                     M
C                                                                       M
C     CALCULATE RESOLVED MOMENTS ALONG CELL AXES                        M
C                                                                       M
10    DO 20 M=1,3                                                       M
      CALL PERM(M,J,K)                                                  M
      SMOM(M)=XL(I,M+5)+XL(I,J+5)*CELLN(NP,K)+XL(I,K+5)*CELLN(NP,J)     M
20    CONTINUE                                                          M
30    AGM=0.0                                                           M
      ESD=0.0                                                           M
      DO 150 M=1,3                                                      M
      DO 150 J=M,3                                                      M
      K=M+5                                                             M
      N=J+5                                                             M
      IF(L)80,80,40                                                     M
40    LPM=LP(I,K)                                                       M
      IF(LPM)80,80,50                                                   M
50    IF(LPM-MATORD)60,60,80                                            M
60    LPJ=LP(I,N)                                                       M
      IF(LPJ)80,80,70                                                   M
70    IF(LPJ-MATORD)90,90,80                                            M
80    COV=0.0                                                           M
      GOTO 100                                                          M
C                                                                       M
C     COVARIANCE OF PARAMETERS LPM AND LPJ                              M
C                                                                       M
90    CALL LOC(LPM,LPJ,NU,MATORD)                                       M
      COV=SMM(NU)*FLAG(I,K)*FLAG(I,N)                                   M
C                                                                       M
C     CROSS TERM?                                                       M
C                                                                       M
100   IF(M-J)110,120,110                                                M
110   CRO=2.0                                                           M
      NU=9-M-J                                                          M
      COR=CELLN(NP,NU)                                                  M
      GOTO 130                                                          M
120   CRO=1.0                                                           M
      COR=1.0                                                           M
C                                                                       M
C     SQUARE OF MAGNETIC MOMENT                                         M
C                                                                       M
130   AGM=AGM+XL(I,K)*XL(I,N)*CRO*COR                                   M
      IF(L)150,150,140                                                  M
C                                                                       M
C     UNNORMALISED ESD                                                  M
C                                                                       M
140   ESD=ESD+SMOM(M)*SMOM(J)*COV*CRO                                   M
150   CONTINUE                                                          M
C                                                                       M
C     NORMALISE ESD                                                     M
C                                                                       M
      IF(AGM)160,160,170                                                M
160   AGM=0.0                                                           M
      ESD=0.0                                                           M
      RETURN                                                            M
170   ESD=SQRT(ESD/AGM)*SIGD                                            M
      AGM=SQRT(AGM)                                                     M
      RETURN                                                            M
      END                                                               M
      SUBROUTINE UPDATE                                                 U
      PARAMETER (ML=500 , MP=99 , MX=103)
C     ********************************************************          U
C     THIS ROUTINE APPLIES THE SHIFTS TO THE PARAMETERS, CALCULATES     U
C     NEW CELL DIMENSION VALUES AND METRIC TENSOR AFTER EACH            U
C     CYCLE OF REFINEMENT                                               U
C     ********************************************************          U
C                                                                       U
      DIMENSION STAND(MP)                                               U
C                                                                       U
C      THE FOLLOWING EQUIVALENCE STATEMENT ALLOWS ACCESS TO THE         U
C     STANDARD DEVIATIONS OF THE REFINED PARAMETERS STORED IN THE       U
C     ARRAY STAND. SINCE THERE ARE ONLY TWO EQUIVALNCED ITEMS, THIS     U
C     STATEMENT SHOULD BE VIRTUALLY INDEPENDENT OF ANY RE-DIMENSIONING. U
C                                                                       U
C                                                                       U
C                                                                       U
      COMMON /REFLN/ARG,DELTA,DLABDA(2),DUMPA(500),EPS,FLOG,FLOG2,      U
     2 OMEGA,PI,SKEW,SLABDA,SLOG,W,XN,YCALC,YOBS                        U
C                                                                       U
      COMMON /PREF/IALFA,IOMEG,IORD1,IORD2,IPREV,ISTAP,ISS,KSTART,LIMO, U
     2 LLL,NCD(50),NKEY,NO,NTEXT,NUM,NTYP(50),MEQ(50),MTYP(50)          U
C                                                                       U
      COMMON /SYMCEL/ALL(3,3),AL(5,3,3),ATEXT(60),B(8),CELL(6),CELLN(5, U
     2 6),F(4,21),FF(4),PH(5,3),RM(5,24,2,9),S(4,21),SM(5,24,3,3),T(5,  U
     3 24,3),TR(5,24),RCELL(6),RCELLN(5,6),VOLUME,VOL(5),PTEXT(5,4)     U
C                                                                       U
      COMMON /MINIM/COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),HL(3,3),  U
     2 RSA(50,3),RSB(50,3),SA(50),SB(50),SINAR(50,24),SINARI(24),       U
     3 SNEX(50),TEMP(50),TEXT(20),XL(66,39)                             U
C                                                                       U
      COMMON /CONTRL/NCYCLE,MAXS,KXR(5),KKZ,LIM,LOUT,IPONS,LMAT,LCORD,  U
     2 LCOORD,ICELL,ICENT,ICENTT(5),IRL,IRLL(5),KL(5),KLL,KM(5),KMN,    U
     3 MN(5),MNN,ICYCLE,MAXCON,IFOUR,IPLOT,KLOOP,LINES,IWID,IPREF(5),   U
     4 IPREFF,NHALP(5),NUNIAX(5),NCUBIC(5),NGAUSS(5),NCAUCH(5),NPAC(5)  U
     5 ,CLOR(5),ELOR(5)
C                                                                       U
      COMMON /MATR/MATSIZ,MATORD,SIGD,V(MP),SMM(MP*(MP+1)/2)                   U
C                                                                       U
      COMMON /BFAC/ITF,JTF(50),PRCP(5,6),ITF3,ITF4                      U
C                                                                       U
      COMMON /REFS/HN(5,3),HH(3,ML),ICODE(ML),SJJ(ML),SOMEGA(ML),       U
     2 FOBS(ML),FESD(ML),FNUC(ML),FMAG(ML),THETA(ML),TANN(ML),HW(ML),   U
     3 DERSTO(ML,MX),AKEEP(ML),BKEEP(ML),PKEEP(ML),ICODD(ML)            U
C                                                                       U
      COMMON /INOUT/LCA,LPR,LPRE,LT4,LT8,LFOR,LPUN                      U
C                                                                       U
      COMMON /NUMB/N,N1,N2,N3,N4,N6,N7,NN,NN5,NA(5),NPAT,NREF,NSPAR
C                                                                       U
      COMMON /KEYS/LP(66,39),FLAG(66,39)                                U
C                                                                       U
C                                                                       U
      EQUIVALENCE (DERSTO(1,1),STAND(1))                                U
C                                                                       U
C     APPLY SHIFTS TO PARAMETERS                                        U
C                                                                       U
C     CYCLE THROUGH PARAMETER LIST                                      U
C                                                                       U
      DO 40 I=1,N4                                                      U
C                                                                       U
C     HOW MANY PARAMETERS FOR THIS ATOM?                                U
C                                                                       U
      CALL PARS(I,NPAR)                                                 U
C                                                                       U
C     CYCLE THROUGH ATOM PARAMETERS                                     U
C                                                                       U
      DO 30 J=1,NPAR                                                    U
C                                                                       U
C     WAS THIS PARAMETER VARIED?                                        U
C                                                                       U
      IF(LP(I,J))10,30,10                                               U
C                                                                       U
C     IS THIS A NON-REFINED PARAMETER IN A STRICT CONSTRAINT?           U
C                                                                       U
10    IF(LP(I,J)-MATORD)20,20,30                                        U
C                                                                       U
C     REFINED PARAMETER                                                 U
C                                                                       U
20    K=LP(I,J)                                                         U
C                                                                       U
C     GET RELAXATION FACTOR                                             U
C                                                                       U
      REL=FREL(I,J)                                                     U
C                                                                       U
C     APPLY SHIFT STORED IN DERIV(K)                                    U
C                                                                       U
      XL(I,J)=XL(I,J)+FLAG(I,J)*DERIV(K)*REL                            U
30    CONTINUE                                                          U
40    CONTINUE                                                          U
C                                                                       U
C     WERE THERE ANY STRICT CONSTRAINTS?                                U
C                                                                       U
      IF(MAXCON)60,60,50                                                U
C                                                                       U
C     CORRECT NON-REFINED PARAMETER IN STRICT CONSTRAINTS               U
C                                                                       U
50    CALL CONCOR                                                       U
C                                                                       U
C     ALL PARAMETERS NOW CORRECTED                                      U
C                                                                       U
C     COPY NEW PARAMETERS INTO SUBSIDIARY STORAGE                       U
C                                                                       U
60    CALL SHIFT                                                        U
C                                                                       U
C     GET NEW RECIPROCAL CELL                                           U
C                                                                       U
      DO 70 NP=1,NPAT                                                   U
      CALL REPCEL(RCELLN,AL,NP)                                         U
C                                                                       U
C     GET NEW DIRECT CELL                                               U
C                                                                       U
      CALL TRCL(RCELLN,CELLN,VOL,NP)                                    U
C                                                                       U
C     VOLUME CONTAINS THE RECIPROCAL CELL VOLUME                        U
C                                                                       U
C     ALL STORED QUANTITIES DEPENDENT ON PARAMETERS NOW UPDATED         U
C                                                                       U
70    CONTINUE                                                          U
      RETURN                                                            U
      END                                                               U
      SUBROUTINE CONCOR                                                 C
      PARAMETER (ML=500 , MP=99 , MX=103)
C     **************************************************************    C
C     THIS ROUTINE SOLVES THE CONSTRAINT FUNCTIONS TO DETERMINE THE NEW C
C     VALUES OF THE NON-REFINED PARAMETERS. IT WILL HANDLE LINEAR       C
C     AND QUADRATIC EQUATIONS IN THE NON-REFINED PARAMETER,BUT ARBITRAIRC
C     AS THAT NEAREST TO THE ORIGINAL VALUE OF THE NON-REFINED PARAMETERC
C     **************************************************************    C
C                                                                       C
      COMMON /REFLN/ARG,DELTA,DLABDA(2),DUMPA(500),EPS,FLOG,FLOG2,      C
     2 OMEGA,PI,SKEW,SLABDA,SLOG,W,XN,YCALC,YOBS                        C
C                                                                       C
      COMMON /PREF/IALFA,IOMEG,IORD1,IORD2,IPREV,ISTAP,ISS,KSTART,LIMO, C
     2 LLL,NCD(50),NKEY,NO,NTEXT,NUM,NTYP(50),MEQ(50),MTYP(50)          C
C                                                                       C
      COMMON /SYMCEL/ALL(3,3),AL(5,3,3),ATEXT(60),B(8),CELL(6),CELLN(5, C
     2 6),F(4,21),FF(4),PH(5,3),RM(5,24,2,9),S(4,21),SM(5,24,3,3),T(5,  C
     3 24,3),TR(5,24),RCELL(6),RCELLN(5,6),VOLUME,VOL(5),PTEXT(5,4)     C
C                                                                       C
      COMMON /MINIM/COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),HL(3,3),  C
     2 RSA(50,3),RSB(50,3),SA(50),SB(50),SINAR(50,24),SINARI(24),       C
     3 SNEX(50),TEMP(50),TEXT(20),XL(66,39)                             C
C                                                                       C
      COMMON /CONTRL/NCYCLE,MAXS,KXR(5),KKZ,LIM,LOUT,IPONS,LMAT,LCORD,  C
     2 LCOORD,ICELL,ICENT,ICENTT(5),IRL,IRLL(5),KL(5),KLL,KM(5),KMN,    C
     3 MN(5),MNN,ICYCLE,MAXCON,IFOUR,IPLOT,KLOOP,LINES,IWID,IPREF(5),   C
     4 IPREFF,NHALP(5),NUNIAX(5),NCUBIC(5),NGAUSS(5),NCAUCH(5),NPAC(5)  C
     5 ,CLOR(5),ELOR(5)
C                                                                       C
      COMMON /MATR/MATSIZ,MATORD,SIGD,V(MP),SMM(MP*(MP+1)/2)                   C
C                                                                       C
      COMMON /SCONTS/ICON(4,9),JCON(4,9),LCONI(4,9),LCONJ(4,9),LN(4),   C
     2 LCID(4,9),JCID(4,9),CI(4,9),CJ(4,9),CONA(4,9),DF(4)              C
C                                                                       C
C                                                                       C
C                                                                       C
C                                                                       C
C     CYCLE THROUGH CONSTRAINTS                                         C
C                                                                       C
      DO 300 I=1,MAXCON                                                 C
      FXQUO=0.0                                                         C
      FXLQUO=0.0                                                        C
      SUMCON=0.0                                                        C
C                                                                       C
C     NUMBER OF TERMS IN THIS CONSTRAINT                                C
C                                                                       C
      LNI=LN(I)                                                         C
C                                                                       C
C     CYCLE THROUGH NUMBER OF TERMS                                     C
C                                                                       C
      DO 160 J=1,LNI                                                    C
      QMUL=1.0                                                          C
C                                                                       C
C      IS THIS A QUADRATIC TERM                                         C
C                                                                       C
      IF(LCONI(I,J))50,50,10                                            C
10    IF(ICON(I,J))20,20,70                                             C
C                                                                       C
C     LINEAR TERM IN FIRST PARAMETER                                    C
C                                                                       C
C      IS THIS A NON-REFINED PARAMETER                                  C
C                                                                       C
20    IL=LCONI(I,J)                                                     C
      JL=LCONJ(I,J)                                                     C
      IF(LCID(I,J)-MATORD)30,30,40                                      C
C                                                                       C
C     NO                                                                C
C                                                                       C
30    QMUL=QMUL*XL(IL,JL)                                               C
      GOTO 150                                                          C
C                                                                       C
C     YES                                                               C
C                                                                       C
40    FXLQUO=FXLQUO+CONA(I,J)                                           C
      ILF=IL                                                            C
      JLF=JL                                                            C
      GOTO 160                                                          C
C                                                                       C
C      LINEAR TERM IN SECOND PARAMETER                                  C
C                                                                       C
C     IS THIS A NON-REFINED PARAMETER                                   C
50    IL=ICON(I,J)                                                      C
      JL=JCON(I,J)                                                      C
      IF(JCID(I,J)-MATORD)60,60,40                                      C
C                                                                       C
C     NO                                                                C
C                                                                       C
60    QMUL=QMUL*XL(IL,JL)                                               C
      GOTO 150                                                          C
C                                                                       C
C     QUADRATIC TERM                                                    C
C                                                                       C
C     IS FIRST PARAMETER NON-REFINED                                    C
C                                                                       C
70    IL=LCONI(I,J)                                                     C
      JL=LCONJ(I,J)                                                     C
      IF(LCID(I,J)-MATORD)80,80,90                                      C
C                                                                       C
C     NO                                                                C
C                                                                       C
80    QMUL=QMUL*XL(IL,JL)                                               C
      GOTO 120                                                          C
C                                                                       C
C     YES                                                               C
C                                                                       C
C     IS THIS A QUADRATIC TERM IN THE NON-REFINED PARAMETER             C
C                                                                       C
90    IF(JCID(I,J)-MATORD)100,100,110                                   C
C                                                                       C
C     NO                                                                C
C                                                                       C
100   IL=ICON(I,J)                                                      C
      JL=JCON(I,J)                                                      C
      ILF=LCONI(I,J)                                                    C
      JLF=LCONJ(I,J)                                                    C
      FXLQUO=FXLQUO+CONA(I,J)*XL(IL,JL)                                 C
      GOTO 160                                                          C
C                                                                       C
C     YES                                                               C
C                                                                       C
110   FXQUO=FXQUO+CONA(I,J)                                             C
      ILF=IL                                                            C
      JLF=JL                                                            C
      GOTO 160                                                          C
C                                                                       C
C      IS SECOND PARAMETER NON-REFINED                                  C
C                                                                       C
120   IF(JCID(I,J)-MATORD)130,130,140                                   C
C                                                                       C
C     NO                                                                C
C                                                                       C
130   IL=ICON(I,J)                                                      C
      JL=JCON(I,J)                                                      C
      QMUL=QMUL*XL(IL,JL)                                               C
      GOTO 150                                                          C
C                                                                       C
C     YES                                                               C
C                                                                       C
140   IL=LCONI(I,J)                                                     C
      JL=LCONJ(I,J)                                                     C
      ILF=ICON(I,J)                                                     C
      JLF=JCON(I,J)                                                     C
      FXLQUO=FXLQUO+CONA(I,J)*XL(IL,JL)                                 C
      GOTO 160                                                          C
C                                                                       C
C     SUM VARIABLE CONSTRAINT TERMS                                     C
C                                                                       C
150   SUMCON=SUMCON+CONA(I,J)*QMUL                                      C
C                                                                       C
160   CONTINUE                                                          C
C                                                                       C
C      SOLVE CONSTRAINT EQUATION FOR NON-REFINED PARAMETER              C
C                                                                       C
      IF(FXQUO)200,170,200                                              C
C                                                                       C
C     LINEAR TERM ONLY                                                  C
C                                                                       C
170   IF(FXLQUO)190,180,190                                             C
C                                                                       C
C     ERROR TRAP                                                        C
C                                                                       C
180   CALL ERR(16,I)                                                    C
190   XL(ILF,JLF)=(DF(I)-SUMCON)/FXLQUO                                 C
      GOTO 300                                                          C
200   IF(FXLQUO)240,210,240                                             C
C                                                                       C
C      SOLUTION BY TAKING THE SQUARE ROOT                               C
C                                                                       C
210   XX=(DF(I)-SUMCON)/FXQUO                                           C
      IF(XX)180,230,220                                                 C
220   XX=SQRT(XX)                                                       C
230   S1=XX                                                             C
      S2=-XX                                                            C
      GOTO 270                                                          C
C                                                                       C
C      QUADRATIC EQUATION SOLUTION                                      C
240   XX=SUMCON-DF(I)                                                   C
      SS=FXLQUO*FXLQUO-4.0*FXQUO*XX                                     C
      IF(SS)180,260,250                                                 C
250   SS=SQRT(SS)                                                       C
260   S1=(-FXLQUO+SS)                                                   C
      S2=-FXLQUO-SS                                                     C
      SS=0.5/FXQUO                                                      C
      S1=S1*SS                                                          C
      S2=S2*SS                                                          C
C                                                                       C
C     THE SOLUTION ARE S1 AND S2. PICK THAT CLOSEST TO THE ORIGINAL ONE C
C                                                                       C
270   D1=ABS(XL(ILF,JLF)-S1)                                            C
      D2=ABS(XL(ILF,JLF)-S2)                                            C
      IF(D1-D2)280,280,290                                              C
280   XL(ILF,JLF)=S1                                                    C
      GOTO 300                                                          C
290   XL(ILF,JLF)=S2                                                    C
300   CONTINUE                                                          C
      RETURN                                                            C
      END                                                               C
      SUBROUTINE PRNT                                                   P
      PARAMETER (ML=500 , MP=99 , MX=103)
C     ************************************************************      P
C     ARRANGES PRINTING OF PARAMETERS L TO M FOR EACH ATOM              P
C     ************************************************************      P
C                                                                       P
      DIMENSION STAND(MP),FMT(50),DUM(50),FA(ML*MX)                     P
C                                                                       P
C     THE FOLLOWING EQUIVALENCE IS INTENDED TO SAVE STORE AS WELL AS TO P
C     ALLOW ACCESS TO VARIOUS STORED QUANTITIES. THE ARRAY STAND CONTAINP
C     THE STANDARD DEVIATIONS FROM THE REFINEMENT, AND IS THE ONLY      P
C     VARIABLE DIMENSION. THE EQUIVALENCE NEEDS ALTERATION IF THE VALUE P
C     MNATOM IS INCREASED IN COMMON BLOCK /MAXDIM/. THE CURRENT VALUE OFP
C     MNATOM IS 50.                                                     P
C                                                                       P
C                                                                       P
      COMMON /REFLN/ARG,DELTA,DLABDA(2),DUMPA(500),EPS,FLOG,FLOG2,      P
     2 OMEGA,PI,SKEW,SLABDA,SLOG,W,XN,YCALC,YOBS                        P
C                                                                       P
      COMMON /PREF/IALFA,IOMEG,IORD1,IORD2,IPREV,ISTAP,ISS,KSTART,LIMO, P
     2 LLL,NCD(50),NKEY,NO,NTEXT,NUM,NTYP(50),MEQ(50),MTYP(50)          P
C                                                                       P
      COMMON /SYMCEL/ALL(3,3),AL(5,3,3),ATEXT(60),B(8),CELL(6),CELLN(5, P
     2 6),F(4,21),FF(4),PH(5,3),RM(5,24,2,9),S(4,21),SM(5,24,3,3),T(5,  P
     3 24,3),TR(5,24),RCELL(6),RCELLN(5,6),VOLUME,VOL(5),PTEXT(5,4)     P
C                                                                       P
      COMMON /MINIM/COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),HL(3,3),  P
     2 RSA(50,3),RSB(50,3),SA(50),SB(50),SINAR(50,24),SINARI(24),       P
     3 SNEX(50),TEMP(50),TEXT(20),XL(66,39)                             P
C                                                                       P
      COMMON /REFS/HN(5,3),HH(3,ML),ICODE(ML),SJJ(ML),SOMEGA(ML),       P
     2 FOBS(ML),FESD(ML),FNUC(ML),FMAG(ML),THETA(ML),TANN(ML),HW(ML),   P
     3 DERSTO(ML,MX),AKEEP(ML),BKEEP(ML),PKEEP(ML),ICODD(ML)            P
C                                                                       P
      COMMON /MATR/MATSIZ,MATORD,SIGD,V(MP),SMM(MP*(MP+1)/2)                   P
C                                                                       P
      COMMON /INOUT/LCA,LPR,LPRE,LT4,LT8,LFOR,LPUN                      P
C                                                                       P
      COMMON /NUMB/N,N1,N2,N3,N4,N6,N7,NN,NN5,NA(5),NPAT,NREF,NSPAR
C                                                                       P
      COMMON /KEYS/LP(66,39),FLAG(66,39)                                P
C                                                                       P
      EQUIVALENCE (DERSTO(1,1),FA(1))
C                                                                       P
      EQUIVALENCE (DERSTO(1,1),STAND(1)),(FA(MP+1),FMT(1)),
     2 (FA(MP+52),DUM(1)),(ICODE(1),II),(ICODE(2),III),(ICODE(3),L),
     3 (ICODE(4),M),(ICODE(5),INDEX),(ICODE(6),KK),(ICODE(7),K)         P
C                                                                       P
C     STAND OCCUPIES DERSTO(1) TO DERSTO(MP)                            P
C     FMT OCCUPIES DERSTO(MP+1) TO DERSTO(MP+51)                          P
C     DUM OCCUPIES DERSTO(MP+52) TO DERSTO(MP+102)                         P
C                                                                       P
C                                                                       P
C     THE OUTPUT FORMAT IS CREATED ACCORDING TO THE REQUIREMENTS OF     P
C     THE QUANTITIES TO BE PRINTED IN THE SO-CALLED DYNAMIC FORMAT MODE P
C     FOR OUTPUT. THIS IS ALSO USED IN SUBROUTINE CONPR. IF YOUR MACHINEP
C     DOES NOT ALLOW THE OUTPUT TO BE GENERATED IN THIS WAY, THEN       P
C     THIS ROUTINE SHOULD BE MODIFIED TO PRINT :                        P
C     ATOM NAME; 4(PARAMETER VALUE; SHIFT; ESD)                         P
C     PERHAPS WITH SHIFT AND ESD SET TO ZERO IN THE CASE OF NON-VARIED  P
C     PARAMETERS.                                                       P
C                                                                       P
C     THE FOLLOWING DATA IS USED TO CONSTRUCT THE FORMAT STATEMENT      P
C                                                                       P
C                                                                       P
      DATA ONEX/4H,1X /,THR/4H,3  /,FONE/4H,F9 /,FONEB/                 P
     2 4HF9  /,FTWO/4H,F12/,FTWOB/4HF12 /,PONE/4H.5  /,PTWO/4H.8  /,    P
     3 PTHR/4H.2  /,XONE/4H,18X/,XTWO/4H,24X/,                          P
     4 END/8H)       /,BLANK/8H        /
C                                                                       P
C     STANDARD FORMAT FOR ATOM PARAMETERS                               P
C                                                                       P
      FAC=1.
      IF(L.GT.14) FAC=1000.
      IF(L.GT.24) FAC=10000.
      DO 5 IBL=4,50
    5 FMT(IBL)=BLANK
      FTEN=FONE                                                         P
      FTENB=FONEB                                                       P
      PT5=PONE                                                          P
      ATEENX=XONE                                                       P
      IF(III-N3)40,10,30                                                P
10    IF(M-3)20,20,40                                                   P
C                                                                       P
C     HALFWIDTH PARAMETERS                                              P
C                                                                       P
20    PT5=PTHR                                                          P
      GOTO 40                                                           P
C                                                                       P
C     CELL PARAMETERS                                                   P
30    FTEN=FTWO                                                         P
      FTENB=FTWOB                                                       P
      PT5=PTWO                                                          P
      ATEENX=XTWO                                                       P
C     CYCLE THROUGH ATOM LIST                                           P
C                                                                       P
40    DO 100 I=II,III                                                   P
C                                                                       P
C     SUBSCRIPT FOR ARRAY FMT IN WHICH THE FORMAT IS STORED             P
C                                                                       P
      INDEX=3                                                           P
C                                                                       P
C     SUBSCRIPT FOR ARRAY DUM IN WHICH THE NUMBERS TO BE PRINTED ARE STOP
C                                                                       P
      KK=0                                                              P
C                                                                       P
C     ATOM NAME                                                         P
C                                                                       P
      CHAR=ATEXT(I)                                                     P
      CALL XCOPY(KK,CHAR,DUM)                                           P
C                                                                       P
C     CYCLE THROUGH REQUIRED PARAMETERS                                 P
C                                                                       P
      DO 90 J=L,M                                                       P
C                                                                       P
C     PARAMETER VALUE                                                   P
C                                                                       P
      CHAR=XL(I,J)*FAC                                                  P
      CALL XCOPY(KK,CHAR,DUM)                                           P
C                                                                       P
C     WAS IT REFINED?                                                   P
C                                                                       P
      K=LP(I,J)                                                         P
      IF(K)70,70,50                                                     P
50    IF(K-MATORD)60,60,70                                              P
C                                                                       P
C     YES                                                               P
C                                                                       P
C                                                                       P
C     GET RELAXATION FACTOR                                             P
C                                                                       P
60    REL=FREL(I,J)                                                     P
      CHAR=DERIV(K)*FLAG(I,J)*REL*FAC                                   P
      CALL XCOPY(KK,CHAR,DUM)                                           P
      CHAR=STAND(K)*ABS(FLAG(I,J))*FAC                                  P
      CALL XCOPY(KK,CHAR,DUM)                                           P
C                                                                       P
C     FORMAT FOR SHIFT AND ESD                                          P
C                                                                       P
      CALL XCOPY(INDEX,THR,FMT)                                         P
      CALL XCOPY(INDEX,FTENB,FMT)                                       P
      CALL XCOPY(INDEX,PT5,FMT)                                         P
      GOTO 80                                                           P
C                                                                       P
C     NOT REFINED                                                       P
C                                                                       P
70    CALL XCOPY(INDEX,FTEN,FMT)                                        P
      CALL XCOPY(INDEX,PT5,FMT)                                         P
      CALL XCOPY(INDEX,ATEENX,FMT)                                      P
80    CALL XCOPY(INDEX,ONEX,FMT)                                        P
90    CONTINUE                                                          P
C                                                                       P
C     LINE LOADED                                                       P
C                                                                       P
C     TERMINATE FORMAT                                                  P
C                                                                       P
      CALL XCOPY(INDEX,END,FMT)                                         P
C                                                                       P
C     PRINT LINE                                                        P
C                                                                       P
      WRITE(LPR,FMT)(DUM(K),K=1,KK)                                     P
100   CONTINUE                                                          P
      RETURN                                                            P
      END                                                               P
      SUBROUTINE PROFPL                                                 P
      PARAMETER (ML=500 , MP=99 , MX=103)
C     *************************************************************     P
C     PLOTS PROFILE AND DIFFERENCE CURVE ON THE LINE PRINTER            P
C                                                                       P
C     THE WIDTH OF THE LINE PRINTER IN VISIBLE CHARACTERS IS SPECIFIED  P
C     BY THE PARAMETER IWID                                             P
C     *************************************************************     P
C                                                                       P
      DIMENSION IOUT(200),COUNT(10,3),LABEL(20),IREDIN(50)              P
C                                                                       P
      DIMENSION FMT1(5),FMT2(5),FMT3(5),FMT4(4),W120(3),W130(3)         P
C                                                                       P
C                                                                       P
      COMMON /REFLN/ARG,DELTA,DLABDA(2),DUMPA(500),EPS,FLOG,FLOG2,      P
     2 OMEGA,PI,SKEW,SLABDA,SLOG,W,XN,YCALC,YOBS                        P
C                                                                       P
      COMMON /PREF/IALFA,IOMEG,IORD1,IORD2,IPREV,ISTAP,ISS,KSTART,LIMO, P
     2 LLL,NCD(50),NKEY,NO,NTEXT,NUM,NTYP(50),MEQ(50),MTYP(50)          P
C                                                                       P
      COMMON /SYMCEL/ALL(3,3),AL(5,3,3),ATEXT(60),B(8),CELL(6),CELLN(5, P
     2 6),F(4,21),FF(4),PH(5,3),RM(5,24,2,9),S(4,21),SM(5,24,3,3),T(5,  P
     3 24,3),TR(5,24),RCELL(6),RCELLN(5,6),VOLUME,VOL(5),PTEXT(5,4)     P
C                                                                       P
      COMMON /MINIM/COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),HL(3,3),  P
     2 RSA(50,3),RSB(50,3),SA(50),SB(50),SINAR(50,24),SINARI(24),       P
     3 SNEX(50),TEMP(50),TEXT(20),XL(66,39)                             P
C                                                                       P
      COMMON /CONTRL/NCYCLE,MAXS,KXR(5),KKZ,LIM,LOUT,IPONS,LMAT,LCORD,  P
     2 LCOORD,ICELL,ICENT,ICENTT(5),IRL,IRLL(5),KL(5),KLL,KM(5),KMN,    P
     3 MN(5),MNN,ICYCLE,MAXCON,IFOUR,IPLOT,KLOOP,LINES,IWID,IPREF(5),   P
     4 IPREFF,NHALP(5),NUNIAX(5),NCUBIC(5),NGAUSS(5),NCAUCH(5),NPAC(5)  P
     5 ,CLOR(5),ELOR(5)
C                                                                       P
      COMMON /REFS/HN(5,3),HH(3,ML),ICODE(ML),SJJ(ML),SOMEGA(ML),       P
     2 FOBS(ML),FESD(ML),FNUC(ML),FMAG(ML),THETA(ML),TANN(ML),HW(ML),   P
     3 DERSTO(ML,MX),AKEEP(ML),BKEEP(ML),PKEEP(ML),ICODD(ML)            P
C                                                                       P
      COMMON /INOUT/LCA,LPR,LPRE,LT4,LT8,LFOR,LPUN                      P
C                                                                       P
      COMMON /INPUT/NREAD,NPOINT,IRED,MODE,LINO,IDISML,ICHARS(14),      P
     2 ICARD(80),READIN(50)                                             P
C                                                                       P
C                                                                       P
      EQUIVALENCE (READIN(1),IREDIN(1))                                 P
C                                                                       P
C     THE FOLLOWING EQUIVALENCE RE-USES THE STORAGE IN ARRAY            P
C     DERSTO. IT IS NOT INTENDED TO PASS ANY OF THESE VALUES TO         P
C     OTHER ROUTINES THROUGH THIS ARRAY, AND HENCE REDIMENSIONING       P
C     IS ONLY OF CONSEQUENCE IN THAT NOT MORE THAN ONE VARIABLE SHOULD  P
C     OCCUPY ONE STORAGE LOCATION.                                      P
C                                                                       P
C     THE ARRAYS THAT MIGHT REQUIRE CHANGE ARE DERSTO ITSELF,           P
C     IN WHICH CASE THE WHOLE EQUIVALENCE MAY NEED REVIEW, AND          P
C     THE ARRAYS IOUT AND LABEL WHICH ARE FORMALLY RELATED TO           P
C     THE WIDTH OF THE LINE PRINTER IN CHARACTERS. AT PRESENT           P
C     THESE ARRAYS ARE SET TO COPE WITH A LINE OF 200 CHARACTERS,       P
C     AND IT UNLIKELY THAT THIS WILL BE EXCEEDED.                       P
C                                                                       P
      EQUIVALENCE (DERSTO(1,1),COUNT(1,1)),(DERSTO(31,1),IOUT(1)),      P
     2 (DERSTO(5,31),LABEL(1))                                          P
C                                                                       P
C     COUNT OCCUPIES DERSTO(1) TO DERSTO(30)                            P
C     IOUT OCCUPIES DERSTO(31) TO DERSTO(230)                           P
C     LABEL OCCUPIES DERSTO(231) TO DERSTO(250)                         P
C                                                                       P
C     THE FOLLOWING EQUIVALENCE SHOULD NOT REQUIRE ALTERATION UNDER     P
C     ANY FORSEEABLE CIRCUMSTANCE.                                      P
C                                                                       P
      EQUIVALENCE (ICODE(2),L),(ICODE(3),LL),(ICODE(4),IW1),(ICODE(5),  P
     2 IW2),(ICODE(6),IW6),(ICODE(7),IDIF),(ICODE(8),IOBS),(ICODE(9),   P
     3 ICB),(ICODE(10),IPN),(ICODE(11),I),(ICODE(12),K),(ICODE(13),     P
     4 MOUT),(ICODE(14),IW3)                                            P
C                                                                       P
      EQUIVALENCE (FOBS(1),IBC),(FOBS(2),ALF),(FOBS(3),ITHETA),(FOBS(4) P
     2 ,CMIN),(FOBS(5),FOB),(FOBS(6),FDIF),(FOBS(7),CDMIN),(FOBS(8),    P
     3 XFF)                                                             P
C                                                                       P
C     THE FOLLOWING DATA IS USED FOR THE GRAPHICAL REPRESENTATION       P
C     OF THE OUTPUT PLOT.                                               P
C                                                                       P
C                                                                       P
      DATA ISP/4H    /,IPT/4H.   /,ID/4HD   /,IO/4HO   /,IMIN/4H-   /,  P
     2 IC/4H*   /,IUP/4H>   /,IDOWN/4H<   /,IDSH/4H-   /                P
C                                                                       P
      DATA FMT1/"(2H" ,      "I10","I9)"," "," "/
C
      DATA FMT2/"(2H" ,      "I10","I9)"," "," "/
C
      DATA FMT3/"(1H+",      "1X","F6 .2)"," ", " "/
C
      DATA FMT4/"(1H" ,        "A1)"," "," "/
C
      DATA W120/"11","114","120"/
C
      DATA W130/"12","124","130"/
C                                                                       P
C     THE FINAL VALUES OF FMT1(3),FMT2(3),FMT3(3),FMT4(3)               P
C     DEPEND ON IWID                                                    P
C                                                                       P
      IF(IWID-120)10,20,30                                              P
10    IWID=120                                                          P
      WRITE(LPR,470)IWID                                                P
20    FMT1(3)=W120(1)                                                   P
      FMT2(3)=W120(1)                                                   P
      FMT3(3)=W120(2)                                                   P
      FMT4(3)=W120(3)                                                   P
      GOTO 60                                                           P
30    IF(IWID-130)10,50,40                                              P
40    IWID=130                                                          P
      WRITE(LPR,470)IWID                                                P
50    FMT1(3)=W130(1)                                                   P
      FMT2(3)=W130(1)                                                   P
      FMT3(3)=W130(2)                                                   P
      FMT4(3)=W130(3)                                                   P
60    L=IWID/10                                                         P
      LL=L+1                                                            P
      IW2=IWID/2                                                        P
      IW3=IW2+1                                                         P
      IW1=IWID-1                                                        P
      IW6=IWID-6                                                        P
      BC=0.0                                                            P
C                                                                       P
C     START AND STEP IN 2THETA                                          P
C                                                                       P
      ALF=FLOAT(IALFA)/100.0                                            P
      ITHETA=IALFA                                                      P
C                                                                       P
C     PRINT TITLE                                                       P
C                                                                       P
      WRITE(LPR,440)TEXT                                                P
      WRITE(LPR,450)                                                    P
C                                                                       P
C     READ INPUT DATA                                                   P
C                                                                       P
C     READ(LCA,1020) IOBS,IDIF,ICB                                      P
C                                                                       P
C     IOBS IS THE COUNT INTERVAL CORRESPONDING TO ONE LINE              P
C     PRINTER COLUMN.                                                   P
C     IDIF IS THE SAME QUANTITY FOR THE DIFFERENCE PLOT.                P
C     CMIN IS THE MINIMUM NET COUNT ORDINATE ON THE PLOTTED CURVE       P
C                                                                       P
C     CHECK THAT THE INPUT DATA IS REASONABLE                           P
C                                                                       P
      IBEG=1                                                            P
      MODE=0                                                            P
      NREAD=3                                                           P
      IRED=1                                                            P
      IDISML=1                                                          P
      CALL INITT(IBEG)                                                  P
      CALL FREEIN                                                       P
      IOBS=IREDIN(1)                                                    P
      IDIF=IREDIN(2)                                                    P
      ICB=IREDIN(3)                                                     P
      IF(IOBS)70,70,80                                                  P
70    IOBS=50                                                           P
80    IF(IDIF)90,90,100                                                 P
90    IDIF=50                                                           P
C                                                                       P
C     REAL EQUIVALENTS OF INPUT DATA                                    P
C                                                                       P
100   FOB=FLOAT(IOBS)                                                   P
      FDIF=FLOAT(IDIF)                                                  P
      CMIN=FLOAT(ICB)                                                   P
C                                                                       P
C     PRINT ORDINATE LABELS FOR PROFILE PLOT                            P
C                                                                       P
      DO 110 I=1,L                                                      P
      LABEL(I)=10*I*IOBS+ICB                                            P
110   CONTINUE                                                          P
      WRITE(LPR,480)                                                    P
      WRITE(LPR,'(13I10)')(LABEL(I),I=1,L)                                   P
C                                                                       P
C     PRINT ORDINATE LABELS FOR DIFFERENCE PLOT                         P
C                                                                       P
      DO 120 I=1,L                                                      P
      LABEL(I)=-IW2*IDIF+10*I*IDIF                                      P
120   CONTINUE                                                          P
      WRITE(LPR,490)                                                    P
      WRITE(LPR,'(13I10)')(LABEL(I),I=1,L)                                   P
C                                                                       P
C     MINIMUM OUTPUT VALUE FOR DIFFERENCE PLOT                          P
C                                                                       P
      CDMIN=FLOAT(LABEL(1)-10*IDIF)                                     P
C                                                                       P
C     CYCLE THROUGH DATA IN BLOCKS OF 10 POINTS (READ FROM SCRATCH FILE)P
C                                                                       P
      REWIND 18                                                         P
C                                                                       P
C     COUNTER FOR NUMBER OF POINTS                                      P
C                                                                       P
      IPN=0                                                             P
      ISTART=0                                                          P
C                                                                       P
C     READ DATA FROM SCRATCH FILE                                       P
C                                                                       P
130   DO 135 I=1,10
135   READ(18,*,END=430)IRED,COUNT(I,2),COUNT(I,1)
C                                                                       P
C     LOOP THROUGH DATA PRINTING POINTS                                 P
C                                                                       P
      DO 420 I=1,10                                                     P
C                                                                       P
C      AVOID PRINTING LEADING ZEROS OF PROFILE                          P
C                                                                       P
      IF(ISTART)160,140,160                                             P
140   IF(COUNT(I,1))410,410,150                                         P
150   ISTART=1                                                          P
C                                                                       P
C      CHECK FOR DATA TERMINATOR                                        P
C                                                                       P
160   IF(COUNT(I,1)+100001.0)430,430,170                                P
170   COUNT(I,1)=COUNT(I,1)-CMIN                                        P
      COUNT(I,2)=COUNT(I,2)-CMIN                                        P
      COUNT(I,3)=COUNT(I,1)-COUNT(I,2)-CDMIN                            P
      XFF=FOB                                                           P
C                                                                       P
C     PREPARE LINE TO RECEIVE COUNT DATA                                P
C                                                                       P
      DO 180 K=1,IWID                                                   P
      IOUT(K)=ISP                                                       P
180   CONTINUE                                                          P
      IF(IPN)190,190,210                                                P
190   DO 200 K=1,LL                                                     P
      IOUT(10*K-9)=IPT                                                  P
200   CONTINUE                                                          P
      GOTO 220                                                          P
210   IOUT(1)=IPT                                                       P
      IOUT(IW3)=IPT                                                     P
      IOUT(IWID)=IPT                                                    P
C                                                                       P
C     IS 2 THETA FOR THIS POINT  A WHOLE NUMBER OF DEGREES?             P
C                                                                       P
      IBC=MOD(ITHETA,100)                                               P
      IF(IBC)220,190,220                                                P
220   IPN=IPN+1                                                         P
C                                                                       P
C     CYCLE THROUGH COUNTS FOR ONE POINT                                P
C                                                                       P
      DO 360 K=1,3                                                      P
      IF(K-3)240,230,240                                                P
230   XFF=FDIF                                                          P
240   MOUT=INT((COUNT(I,K)/XFF)+1.5)                                    P
      IF(MOUT)250,250,260                                               P
250   IOUT(1)=IDOWN                                                     P
      GOTO 360                                                          P
260   IF(MOUT-IWID)280,280,270                                          P
270   IOUT(IWID)=IUP                                                    P
      GOTO 360                                                          P
280   IF(K-2)290,340,350                                                P
290   IOUT(MOUT)=IO                                                     P
C                                                                       P
      IF(MOUT-1)310,310,300                                             P
300   IOUT(MOUT-1)=IDSH                                                 P
310   IF(MOUT-IWID+1)320,330,330                                        P
320   IOUT(MOUT+1)=IDSH                                                 P
330   GOTO 360                                                          P
C                                                                       P
340   IOUT(MOUT)=IC                                                     P
      GOTO 360                                                          P
350   IOUT(MOUT)=ID                                                     P
360   CONTINUE                                                          P
C                                                                       P
C     LINE FILLED AT THIS POINT APART FROM POSSIBLE ANGLE VALUE         P
C                                                                       P
      IF(IBC)390,370,390                                                P
370   DO 380 K=IW6,IWID                                                 P
      IOUT(K)=ISP                                                       P
380   CONTINUE                                                          P
390   WRITE(LPR,'(A130)')(IOUT(K),K=1,IWID)                                 P
C                                                                       P
C     PRINT ANGLE VALUE IF REQUIRED                                     P
      IF(IBC)410,400,410                                                P
400   WRITE(LPR,'(1H+,1X,F6.2)')ALF
410   ITHETA=ITHETA+ISTAP                                               P
      ALF=FLOAT(ITHETA)/100.0                                           P
420   CONTINUE                                                          P
C                                                                       P
C     GO BACK FOR ANOTHER DATA BLOCK                                    P
C                                                                       P
      GOTO 130                                                          P
C                                                                       P
C     END OF DATA                                                       P
C                                                                       P
430   RETURN                                                            P
440   FORMAT(1H1,20X,20A4)                                              P
450   FORMAT(1H0,                                                       P
     2 53HPLOT OF OBSERVED(O), CALCULATED(*), AND DIFFERENCE(D),        P
     3 10H INTENSITY//)                                                 P
460   FORMAT(3I8)                                                       P
470   FORMAT(25H0A LINE PRINTER WIDTH OF ,I3,                           P
     2 49H CHARACTERS HAS BEEN ASSUMED BY SUBROUTINE PROFPL)            P
480   FORMAT(18H0PROFILE ORDINATE:)                                     P
490   FORMAT(21H0DIFFERENCE ORDINATE:)                                  P
      END                                                               P
      SUBROUTINE DELPRF                                                 D
      PARAMETER (ML=500 , MP=99 , MX=103)
C     ********************************************************          D
C     PRINTS AVERAGE INTENSITY DIFFERENCE FOR BLOCKS OF "IBLOCK"        D
C     PROFILE POINTS.                                                   D
C     ********************************************************          D
C                                                                       D
      DIMENSION N(10),CALC(10),OBS(10),DIF(10)                          D
C                                                                       D
C     THE FOLLOWING EQUIVALENCE IS USED TO SAVE STORAGE AND NOT TO      D
C     TRANSFER DATA TO OTHER ROUTINES                                   D
C                                                                       D
C     ALTERATIONS SHOULD ONLY BE NECESSARY IF THE DIMENSIONS OF DERSTO  D
C     ARE CHANGED.                                                      D
C                                                                       D
C                                                                       D
      COMMON /REFS/HN(5,3),HH(3,ML),ICODE(ML),SJJ(ML),SOMEGA(ML),       D
     2 FOBS(ML),FESD(ML),FNUC(ML),FMAG(ML),THETA(ML),TANN(ML),HW(ML),   D
     3 DERSTO(ML,MX),AKEEP(ML),BKEEP(ML),PKEEP(ML),ICODD(ML)            D
C                                                                       D
      COMMON /INOUT/LCA,LPR,LPRE,LT4,LT8,LFOR,LPUN                      D
C                                                                       D
C                                                                       D
      EQUIVALENCE (DERSTO(1,1),OBS(1)),(DERSTO(11,1),CALC(1)),          D
     2 (DERSTO(21,1),DIF(1)),(ICODE(1),N(1)),(ICODE(11),IBLOCK),        D
     3 (ICODE(12),L),(ICODE(13),IN),(ICODE(14),I),(ICODE(15),IEND),     D
     4 (ICODE(16),J),(ICODE(17),NR),(ICODE(18),K)                       D
C                                                                       D
C     OBS OCCUPIES DERSTO(1) TO DERSTO(10)                              D
C     CALC OCCUPIES DERSTO(11) TO DERSTO(20)                            D
C     DIF OCCUPIES DERSTO(21) TO DERSTO(30)                             D
C     N OCCUPIES ICODE(1) TO ICODE(10)                                  D
C                                                                       D
C     CONTINUATION LINES 4 ONWARD IN THE ABOVE EQUIVALENCE              D
C     REQUIRE CHANGE ONLY WHEN THE DIMENSION OF ICODE                   D
C     IS LESS THAN 18, AN APPARANTLY UNLIKELY SITUATION.                D
C                                                                       D
C     NUMBER OF PROFILE POINTS PER BLOCK                                D
C     (MUST BE 10*N)                                                    D
C                                                                       D
      REWIND LT4                                                        D
      IBLOCK=20                                                         D
      WRITE(LPR,140)IBLOCK                                              D
      WRITE(LPR,150)                                                    D
      L=IBLOCK/10                                                       D
C                                                                       D
C      COUNTER FOR NUMBER OF BLOCKS PREVIOUSLY PRINTED                  D
C                                                                       D
      IN=-10                                                            D
10    IN=IN+10                                                          D
C                                                                       D
C      PRINT 10 BLOCKS PER LINE                                         D
      DO 100 I=1,10                                                     D
      ASSIGN 100 TO LABEL                                               D
      N(I)=I+IN                                                         D
      DIF(I)=0.0                                                        D
      IEND=I                                                            D
C                                                                       D
C     COUNTER FOR NUMBER OF NON-ZERO POINTS PER BLOCK                   D
C                                                                       D
      NR=0                                                              D
C                                                                       D
C     ACCUMULATE DATA FOR ONE BLOCK                                     D
C                                                                       D
      DO 50 J=1,L                                                       D
      READ(LT4,END=130)(OBS(K),K=1,10),(CALC(K),K=1,10)
      DO 40 K=1,10                                                      D
C                                                                       D
C     DATA TERMINATOR?                                                  D
C                                                                       D
      IF(OBS(K)+100001.0)60,60,20                                       D
C                                                                       D
C     ZERO INTENSITY?                                                   D
C                                                                       D
20    IF(OBS(K))30,40,30                                                D
30    NR=NR+1                                                           D
      DIF(I)=DIF(I)+ABS(OBS(K)-CALC(K))                                 D
40    CONTINUE                                                          D
50    CONTINUE                                                          D
      GOTO 70                                                           D
60    ASSIGN 110 TO LABEL                                               D
70    IF(NR)90,90,80                                                    D
80    DIF(I)=DIF(I)/FLOAT(NR)                                           D
90    GOTO LABEL                                                        D
100   CONTINUE                                                          D
      ASSIGN 10 TO LABEL                                                D
      GOTO 120                                                          D
110   ASSIGN 130 TO LABEL                                               D
C                                                                       D
C     PRINT RESULTS                                                     D
C                                                                       D
120   WRITE(LPR,160)(N(I),DIF(I),I=1,IEND)                              D
      GOTO LABEL                                                        D
130   REWIND LT4                                                        D
      RETURN                                                            D
140   FORMAT(1H0,30X,42HAVERAGE INTENSITY DIFFERENCE FOR BLOCKS OF,I3,  D
     2 15H PROFILE POINTS)                                              D
150   FORMAT(1H0,10(11HBLOCK DELTA,1X))                                 D
160   FORMAT(1H0,10(1X,I4,1X,F5.1,1X))                                  D
      END                                                               D
      SUBROUTINE CORREL                                                 C
      PARAMETER (ML=500 , MP=99 , MX=103)
C     ++++++++++++++++++++++++++++++++++++++++++++++++++++++++          C
C     PRINTS UPPER TRIANGLE OF CORRELATION MATRIX                       C
C                                                                       C
C     NOTE THAT THE MATRIX SMM IS REPLACED BY THE CORRELATION COEFFICIENC
C     ++++++++++++++++++++++++++++++++++++++++++++++++++++++++          C
C                                                                       C
      DIMENSION FMT(MP),L1(4),L2(4),CO(4)                               C
C                                                                       C
C                                                                       C
      COMMON /CONTRL/NCYCLE,MAXS,KXR(5),KKZ,LIM,LOUT,IPONS,LMAT,LCORD,  C
     2 LCOORD,ICELL,ICENT,ICENTT(5),IRL,IRLL(5),KL(5),KLL,KM(5),KMN,    C
     3 MN(5),MNN,ICYCLE,MAXCON,IFOUR,IPLOT,KLOOP,LINES,IWID,IPREF(5),   C
     4 IPREFF,NHALP(5),NUNIAX(5),NCUBIC(5),NGAUSS(5),NCAUCH(5),NPAC(5)  C
     5 ,CLOR(5),ELOR(5)
C                                                                       C
      COMMON /MATR/MATSIZ,MATORD,SIGD,V(MP),SMM(MP*(MP+1)/2)                   C
C                                                                       C
      COMMON /INOUT/LCA,LPR,LPRE,LT4,LT8,LFOR,LPUN                      C
C                                                                       C
C     THE FOLLOWING DATA IS USED TO CONSTRUCT A DYNAMIC FORMAT STATEMENTC
C     FOR OUTPUT OF THE TRIANGULAR MATRIX.                              C
C     IN THE EVENT OF IMPLEMENTATION OF THIS PROGRAM ON A MACHINE WITHOUC
C     DYNAMIC FORMAT CAPABILITY,REPLACE THIS ROUTINE WITH ONE THAT WRITEC
C     THE MATRIX A WHOLE LINE OF THE HORIZONTAL SUBSCRIPT AT A TIME IN  C
C     FOR EXAMPLE A FORMAT OF (5(1H ,20F5.2/))                          C
C                                                                       C
C                                                                       C
      EQUIVALENCE (V(1),FMT(1))                                         C
C                                                                       C
      DATA ONEH/4H(1H0/,FORX/4H,4X /,EY3/4H,I3 /,F6PT/4H,F6./,          C
     2 TOO/4H2   /,END/4H)   /,SIX/4H,6X /,BLANK/4H    /                C
C                                                                       C
C     THE FOLLOWING EQUIVALENCE IS INTENDED TO SAVE STORAGE, AND WILL NEC
C      ALTERATION IF THE ARRAY V IS DIMENSIONED LESS THAN 50            C
C                                                                       C
      VCORR=0.0                                                         C
      K=0                                                               C
C                                                                       C
C     OUTPUT HEADING                                                    C
C                                                                       C
      WRITE(LPR,260)                                                    C
      IF(LCORD)20,20,10                                                 C
10    WRITE(LPR,270)                                                    C
      VCORR=FLOAT(LCORD)/100.0                                          C
C                                                                       C
C     CALCULATE CORRELATION COEFFICIENTS                                C
C                                                                       C
20    II=1                                                              C
      IID=MATORD                                                        C
      DO 90 I=1,MATORD                                                  C
      JJ=II                                                             C
      JJD=MATORD-I+1                                                    C
      S=SMM(II)                                                         C
      DO 80 J=I,MATORD                                                  C
      Z=SQRT(S*SMM(JJ))                                                 C
      L=II+J-I                                                          C
      SMM(L)=SMM(L)/Z                                                   C
      IF(LCORD)70,70,30                                                 C
C                                                                       C
C     PRINT CORRELATION COEFFICIENTS GREATER THAN VCORR                 C
C                                                                       C
30    IF(ABS(SMM(L))-VCORR)70,40,40                                     C
40    IF(I-J)50,70,50                                                   C
50    K=K+1                                                             C
      L1(K)=I                                                           C
      L2(K)=J                                                           C
      CO(K)=SMM(L)                                                      C
      IF(K-4)70,60,60                                                   C
60    WRITE(LPR,280)(L1(K),L2(K),CO(K),K=1,4)                           C
      K=0                                                               C
70    JJ=JJ+JJD                                                         C
      JJD=JJD-1                                                         C
80    CONTINUE                                                          C
      II=II+IID                                                         C
      IID=IID-1                                                         C
90    CONTINUE                                                          C
C                                                                       C
C      PRINT COMPLETE CORRELATION MATRIX IF REQUIRED                    C
C                                                                       C
      IF(LCORD)120,100,100                                              C
100   IF(K)250,250,110                                                  C
110   WRITE(LPR,280)(L1(I),L2(I),CO(I),I=1,K)                           C
      GOTO 250                                                          C
C                                                                       C
C      MAXIMUM NUMBER OF MATRIX COLUMNS ACROSS PAGE                     C
C                                                                       C
120   JJD=(IWID-10)/6                                                   C
C                                                                       C
C     NUMBER OF MATRIX COLUMNS STILL TO BE PRINTED                      C
C                                                                       C
      IID=MATORD                                                        C
C                                                                       C
C      FIXED PART OF FORMAT ARRAY                                       C
C                                                                       C
      DO 125 IBL=1,50
  125  FMT(IBL)=BLANK
C
      FMT(1)=ONEH                                                       C
      FMT(2)=FORX                                                       C
C                                                                       C
C     LOOP THROUGH MATRIX                                               C
C                                                                       C
      II=1                                                              C
C                                                                       C
C                                                                       C
C     FORMAT COUNTER                                                    C
C                                                                       C
130   IFM=2                                                             C
      IF(IID-JJD)150,140,140                                            C
140   JJ=II+JJD-1                                                       C
      GOTO 160                                                          C
150   JJ=MATORD                                                         C
160   IID=IID-JJD                                                       C
C                                                                       C
C     PRINT HEADING                                                     C
C                                                                       C
      WRITE(LPR,290)(I,I=II,JJ)                                         C
C                                                                       C
C     FILL UP FORMAT ARRAY FOR MAXIMUM LINE WIDTH                       C
      CALL XCOPY(IFM,EY3,FMT)                                           C
      CALL XCOPY(IFM,FORX,FMT)                                          C
      DO 170 I=1,JJD                                                    C
      CALL XCOPY(IFM,F6PT,FMT)                                          C
      CALL XCOPY(IFM,TOO,FMT)                                           C
170   CONTINUE                                                          C
      CALL XCOPY(IFM,END,FMT)                                           C
C                                                                       C
C     RESET FORMAT COUNTER TO OVERWRITE I FORMAT FIELD                  C
C                                                                       C
      IFM=2                                                             C
C                                                                       C
C     CYCLE THROUGH MATRIX ELEMENTS                                     C
      DO 230 J=1,MATORD                                                 C
180   IF(II-J)190,220,220                                               C
190   II=II+1                                                           C
      IF(II-JJ)200,200,240                                              C
C                                                                       C
C     UPDATE FORMAT                                                     C
C                                                                       C
200   CALL XCOPY(IFM,SIX,FMT)                                           C
      I=IFM                                                             C
      CALL XCOPY(I,EY3,FMT)                                             C
      CALL XCOPY(I,FORX,FMT)                                            C
      KK=J-1                                                            C
      DO 210 MMM=1,KK                                                   C
      CALL XCOPY(I,BLANK,FMT)                                           C
210   CONTINUE                                                          C
      GOTO 180                                                          C
C                                                                       C
C     START PRINT AT MATRIX ELEMENT (J,II)                              C
C                                                                       C
220   CALL LOC(J,II,I,MATORD)                                           C
      CALL LOC(J,JJ,M,MATORD)                                           C
      WRITE(LPR,FMT)J,(SMM(K),K=I,M)                                    C
230   CONTINUE                                                          C
240   IF(IID)250,250,130                                                C
250   RETURN                                                            C
260   FORMAT(1H0,50X,18HCORRELATION MATRIX)                             C
270   FORMAT(1H0,4(10HPARAMETERS,5X,11HCOEFFICIENT,2X))                 C
280   FORMAT(1H ,4(2I5,8X,F6.2,2X))                                     C
290   FORMAT(11H0PARAMETER ,30I6)                                       C
      END                                                               C
      SUBROUTINE VCOUT                                                  V
      PARAMETER (ML=500 , MP=99 , MX=103)
C     ++++++++++++++++++++++++++++++++++++++++++++++++++++++++          V
C     PUNCHES THE UPPER TRIANGLE OF THE VARIANCE COVARIANCE MATRIX      V
C     ++++++++++++++++++++++++++++++++++++++++++++++++++++++++          V
C     NOTE THAT THE INVERSE MATRIX SMM IS REPLACED BY THE STATISTICALLY V
C     ONE WHERE SUM W*(DELTA)**2 /(NUMBER OF DEGREES OF FREEDOM) = 1.0  V
C                                                                       V
C                                                                       V
      COMMON /MATR/MATSIZ,MATORD,SIGD,V(MP),SMM(MP*(MP+1)/2)                   V
C                                                                       V
      COMMON /INOUT/LCA,LPR,LPRE,LT4,LT8,LFOR,LPUN                      V
      LPUN=27
      OPEN(UNIT=27,FILE='neupar')
C                                                                       V
      SD=SIGD*SIGD                                                      V
      DO 10 I=1,MATSIZ                                                  V
      SMM(I)=SMM(I)*SD                                                  V
10    CONTINUE                                                          V
C                                                                       V
C     THE FOLLOWING WRITE STATEMENT IS RATHER EXPERIMENTAL AND SHOULD BEV
C     ALTERED TO SUIT SPECIFIC NEEDS                                    V
C                                                                       V
      WRITE(LPUN,20)(SMM(I),I=1,MATSIZ)                                 V
20    FORMAT(4E20.10)                                                   V
      LPUN=7
      RETURN                                                            V
      END                                                               V
      SUBROUTINE PAROUT                                                 P
      PARAMETER (ML=500 , MP=99 , MX=103)
C     ++++++++++++++++++++++++++++++++++++++++++++++++++++++            P
C     PUNCHES FINAL PARAMETERS AT THE END OF THE REFINEMENT IN A FORM   P
C     SUITABLE FOR RE-INPUT                                             P
C     ++++++++++++++++++++++++++++++++++++++++++++++++++++++            P
C                                                                       P
      DIMENSION ANG(3)                                                  P
C                                                                       P
      COMMON /REFLN/ARG,DELTA,DLABDA(2),DUMPA(500),EPS,FLOG,FLOG2,      P
     2 OMEGA,PI,SKEW,SLABDA,SLOG,W,XN,YCALC,YOBS                        P
C                                                                       P
      COMMON /PREF/IALFA,IOMEG,IORD1,IORD2,IPREV,ISTAP,ISS,KSTART,LIMO, P
     2 LLL,NCD(50),NKEY,NO,NTEXT,NUM,NTYP(50),MEQ(50),MTYP(50)          P
C                                                                       P
      COMMON /SYMCEL/ALL(3,3),AL(5,3,3),ATEXT(60),B(8),CELL(6),CELLN(5, P
     2 6),F(4,21),FF(4),PH(5,3),RM(5,24,2,9),S(4,21),SM(5,24,3,3),T(5,  P
     3 24,3),TR(5,24),RCELL(6),RCELLN(5,6),VOLUME,VOL(5),PTEXT(5,4)     P
C                                                                       P
      COMMON /MINIM/COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),HL(3,3),  P
     2 RSA(50,3),RSB(50,3),SA(50),SB(50),SINAR(50,24),SINARI(24),       P
     3 SNEX(50),TEMP(50),TEXT(20),XL(66,39)                             P
C                                                                       P
      COMMON /BFAC/ITF,JTF(50),PRCP(5,6),ITF3,ITF4                      P
C                                                                       P
      COMMON /GEOM/DEGRAD,RADDEG                                        P
C                                                                       P
      COMMON /CONTRL/NCYCLE,MAXS,KXR(5),KKZ,LIM,LOUT,IPONS,LMAT,LCORD,  P
     2 LCOORD,ICELL,ICENT,ICENTT(5),IRL,IRLL(5),KL(5),KLL,KM(5),KMN,    P
     3 MN(5),MNN,ICYCLE,MAXCON,IFOUR,IPLOT,KLOOP,LINES,IWID,IPREF(5),   P
     4 IPREFF,NHALP(5),NUNIAX(5),NCUBIC(5),NGAUSS(5),NCAUCH(5),NPAC(5)  P
     5 ,CLOR(5),ELOR(5)
C                                                                       P
      COMMON /NUMB/N,N1,N2,N3,N4,N6,N7,NN,NN5,NA(5),NPAT,NREF,NSPAR
C                                                                       P
      COMMON /INOUT/LCA,LPR,LPRE,LT4,LT8,LFOR,LPUN                      P
C                                                                       P
C                                                                       P
      LPUN=27
C     ATOM PARAMETERS                                                   P
C                                                                       P
10    DO 90 I=1,N                                                       P
      NP=NCD(I)                                                         P
      IF(KXR(NP)-1)30,40,20                                             P
20    IF(KXR(NP)-3)40,40,50                                             P
30    WRITE(LPUN,280)ATEXT(I),NCD(I),NTYP(I),(XL(I,J),J=1,5),JTF(I)     P
      GOTO 70                                                           P
40    IF(MTYP(I))50,50,60                                               P
50    WRITE(LPUN,280)ATEXT(I),NCD(I),NTYP(I),(XL(I,J),J=1,5),MTYP(I),   P
     2 JTF(I)                                                           P
      GOTO 70                                                           P
60    WRITE(LPUN,280)ATEXT(I),NCD(I),NTYP(I),(XL(I,J),J=1,5),MTYP(I),   P
     2 MEQ(I)                                                           P
      WRITE(LPUN,340)(XL(I,J),J=6,8),JTF(I)                             P
70    IF(JTF(I)-1)90,80,80                                              P
80    WRITE(LPUN,320)(XL(I,J),J=9,14)                                   P
      IF(JTF(I).LT.3) GOTO 90
      WRITE(LPUN,323) (XL(I,J),J=15,24)
      IF(JTF(I).LT.4) GOTO 90
      WRITE(LPUN,324) (XL(I,J),J=25,39)
  323 FORMAT(10(E10.4,1X))
  324 FORMAT(10(E10.4,1X),/,5(E10.4,1X))
90    CONTINUE                                                          P
C                                                                       P
C     SCALE FACTORS AND OVERALL B                                       P
C                                                                       P
      DO 120 NP=1,NPAT                                                  P
      N1=N+NP                                                           P
      N2=N7+NP                                                          P
      IF(KXR(1)-4)100,110,110                                           P
100   WRITE(LPUN,270)XL(N1,3),XL(N1,5)                                  P
      GOTO 120                                                          P
110   WRITE(LPUN,270)(XL(N1,I),I=3,5)                                   P
120   CONTINUE                                                          P
C                                                                       P
C     ZEROPOINT                                                         P
C                                                                       P
      WRITE(LPUN,300)(XL(N6,NP),NP=1,NPAT)                               P
C                                                                       P
C     HALFWIDTHS                                                        P
C                                                                       P
130   DO 140 NP=1,NPAT                                                  P
      N3=N6+NP                                                          P
140   WRITE(LPUN,290)(XL(N3,I),I=1,3)                                   P
C                                                                       P
C     UNIT CELL PARAMETERS IN THE SAME FORM AS INPUT                    P
C                                                                       P
      IF(ICELL)180,150,180                                              P
C                                                                       P
C     OUTPUT DIRECT CELL                                                P
C                                                                       P
150   DO 170 NP=1,NPAT                                                  P
      N1=N+NP                                                           P
      DO 160 I=1,3                                                      P
      ANG(I)=RADDEG*ACOS(CELLN(NP,I+3))                                 P
160   CONTINUE                                                          P
      WRITE(LPUN,310)(CELLN(NP,I),I=1,3),(ANG(I),I=1,3)                 P
170   CONTINUE                                                          P
      GOTO 200                                                          P
C                                                                       P
C     OUTPUT METRIC TENSOR COMPONENTS                                   P
C                                                                       P
180   DO 190 NP=1,NPAT                                                  P
      N2=N7+NP                                                          P
      WRITE(LPUN,320)(XL(N2,I),I=1,6)                                   P
190   CONTINUE                                                          P
C                                                                       P
C     PREFFERED ORIENTATION AND ASYMMETRY PARAMETERS                    P
C                                                                       P
200   DO 260 NP=1,NPAT                                                  P
      N1=N+NP                                                           P
      I=LIM+IABS(IPREF(NP))                                             P
      IF(I)260,260,210                                                  P
210   J=2                                                               P
      IF(IPREF(NP))220,230,220                                          P
220   J=1                                                               P
      K=1                                                               P
230   IF(LIM)250,250,240                                                P
240   K=2                                                               P
250   WRITE(LPUN,330)(XL(N1,I),I=J,K)                                   P
260   CONTINUE                                                          P
      LPUN=7
      RETURN                                                            P
270   FORMAT(5F15.5)                                                    P
280   FORMAT(A4,1X,2(I2,1X),5(F10.6,1X),2(I2,1X))                       P
290   FORMAT(5F15.3)                                                    P
300   FORMAT(5F15.3)                                                     P
310   FORMAT(5F10.4,3F10.3)                                             P
320   FORMAT(6F13.7)                                                    P
330   FORMAT(2F20.7)                                                    P
340   FORMAT(3F15.5,I5)                                                 P
      END                                                               P
      SUBROUTINE ANHA3(I,T3,M,K,L)
C.....CALCULATES 3RD ORDER ANHARMONIC CONTRIBUTION
C     FOR ONE ATOM,I.E. WITHOUT EQUIVALENT ONES
C.................................................
      PARAMETER (ML=500,MP=99,MX=103)
      REAL M,K,L
      COMMON /MINIM/COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),HL(3,3),  M
     2 RSA(50,3),RSB(50,3),SA(50),SB(50),SINAR(50,24),SINARI(24),       M
     3 SNEX(50),TEMP(50),TEXT(20),XL(66,39)                             M
      T3=0.
      T3=T3+1*M*M*M*XL(I,15)
      T3=T3+1*K*K*K*XL(I,16)
      T3=T3+1*L*L*L*XL(I,17)
      T3=T3+3*M*M*K*XL(I,18)
      T3=T3+3*M*K*K*XL(I,19)
      T3=T3+3*M*M*L*XL(I,20)
      T3=T3+3*M*L*L*XL(I,21)
      T3=T3+3*K*K*L*XL(I,22)
      T3=T3+3*K*L*L*XL(I,23)
      T3=T3+6*M*K*L*XL(I,24)
      T3=T3*41.34170226
C     4/3*PI**3
      RETURN
      END
      SUBROUTINE ANHA4(I,T4,M,K,L)
C.....CALCULATES 4TM ORDER ANMARMONIC CONTRIBUTION
C     PER NONEQIVALENT ATOM
C..................................................
      PARAMETER (ML=500,MP=99,MX=103)
      REAL M,K,L
      COMMON /MINIM/COSAR(50,24),COSARI(24),DERIV(MX),H(24,3),HL(3,3),  M
     2 RSA(50,3),RSB(50,3),SA(50),SB(50),SINAR(50,24),SINARI(24),       M
     3 SNEX(50),TEMP(50),TEXT(20),XL(66,39)                             M
      T4=0.
      T4=T4+1*M*M*M*M*XL(I,25)
      T4=T4+1*K*K*K*K*XL(I,26)
      T4=T4+1*L*L*L*L*XL(I,27)
      T4=T4+4*M*M*M*K*XL(I,28)
      T4=T4+4*M*K*K*K*XL(I,29)
      T4=T4+4*M*M*M*L*XL(I,30)
      T4=T4+4*M*L*L*L*XL(I,31)
      T4=T4+4*K*K*K*L*XL(I,32)
      T4=T4+4*K*L*L*L*XL(I,33)
      T4=T4+6*M*M*K*K*XL(I,34)
      T4=T4+6*M*M*L*L*XL(I,35)
      T4=T4+6*K*K*L*L*XL(I,36)
      T4=T4+12*M*M*K*L*XL(I,37)
      T4=T4+12*M*K*K*L*XL(I,38)
      T4=T4+12*M*K*L*L*XL(I,39)
      T4=T4*64.93939406
C     2/3*PI**4
      RETURN
      END
      SUBROUTINE PERM3(J,K1,K2,K3,F)
C.....PERFORMS USEFUL PERMUTATIONS FOR
C     3RD ORDER ANHARMONIC DERIVATIVES
C......................................
      I=J-14
      FF=41.34170226
C     4/3*PI**3
      GOTO(1,2,3,4,5,6,7,8,9,10),I
    1 K1=1
      K2=1
      K3=1
      F=1*FF
      RETURN
    2 K1=2
      K2=2
      K3=2
      F=1*FF
      RETURN
    3 K1=3
      K2=3
      K3=3
      F=1*FF
      RETURN
    4 K1=1
      K2=1
      K3=2
      F=3*FF
      RETURN
    5 K1=1
      K2=2
      K3=2
      F=3*FF
      RETURN
    6 K1=1
      K2=1
      K3=3
      F=3*FF
      RETURN
    7 K1=1
      K2=3
      K3=3
      F=3*FF
      RETURN
    8 K1=2
      K2=2
      K3=3
      F=3*FF
      RETURN
    9 K1=2
      K2=3
      K3=3
      F=3*FF
      RETURN
   10 K1=1
      K2=2
      K3=3
      F=6*FF
      RETURN
      END
      SUBROUTINE PERM4(J,K1,K2,K3,K4,F)
C.....PERFORMS USEFUL PERMUTATIONS FOR
C     4TH ORDER ANHARMONIC DERIVATIVES
C.....................................
      I=J-24
      FF=64.93939406
C     2/3*PI**4
      GOTO (1,2,3,4,5,6,7,8,9,10,11,12,13,14,15),I
    1 K1=1
      K2=1
      K3=1
      K4=1
      F=1*FF
      RETURN
    2 K1=2
      K2=2
      K3=2
      K4=2
      F=1*FF
      RETURN
    3 K1=3
      K2=3
      K3=3
      K4=3
      F=1*FF
      RETURN
    4 K1=1
      K2=1
      K3=1
      K4=2
      F=4*FF
      RETURN
    5 K1=1
      K2=2
      K3=2
      K4=2
      F=4*FF
      RETURN
    6 K1=1
      K2=1
      K3=1
      K4=3
      F=4*FF
      RETURN
    7 K1=1
      K2=3
      K3=3
      K4=3
      F=4*FF
      RETURN
    8 K1=2
      K2=2
      K3=2
      K4=3
      F=4*FF
      RETURN
    9 K1=2
      K2=3
      K3=3
      K4=3
      F=4*FF
   10 K1=1
      K2=1
      K3=2
      K4=2
      F=6*FF
      RETURN
   11 K1=1
      K2=1
      K3=3
      K4=3
      F=6*FF
      RETURN
   12 K1=2
      K2=2
      K3=3
      K4=3
      F=6*FF
      RETURN
   13 K1=1
      K2=1
      K3=2
      K4=3
      F=12*FF
      RETURN
   14 K1=1
      K2=2
      K3=2
      K4=3
      F=12*FF
      RETURN
   15 K1=1
      K2=2
      K3=3
      K4=3
      F=12*FF
      RETURN
      END
      SUBROUTINE TENS(A,B)                                              DUMMY
      PARAMETER (ML=500 , MP=99 , MX=103)
      RETURN                                                            DUMMY
      END                                                               DUMMY
      subroutine invmat(a,n)
      dimension a(n,n),b(6,6),indx(6),l(6),aa(6,6)
c
      ll=0
      do 1 i=1,n
      as=0.
      l(i)=0
      do 2 j=1,n
      if(a(i,j).ne.0.) as=1.
    2 continue
      if(as.eq.1.) then
      ll=ll+1
      l(i)=ll
      endif
    1 continue
      do 3 i=1,n
      if(l(i).eq.0) goto 3
      do 4 j=1,n
      if(l(j).eq.0) goto 4
      aa(l(i),l(j))=a(i,j)
ccc      write(98,*) i,j,l(i),l(j),a(i,j)
    4 continue
    3 continue
c
ccc      write(98,*) 'll: ',ll
      do 12 i=1,ll
       do 11,j=1,ll
        b(i,j)=0.
   11  continue
       b(i,i)=1.
   12 continue
      call ludcmp(aa,ll,n,indx,d)
      do 13 j=1,ll
       call lubksb(aa,ll,n,indx,b(1,j))
   13 continue
      do 14 i=1,n
      if(l(i).eq.0) goto 14
      do 15 j=1,n
      if(l(j).eq.0) goto 15
      a(i,j)=b(l(i),l(j))
ccc      write(98,*) i,j,l(i),l(j),'a: ',a(i,j)
15    continue
14    continue
      return
      end
      SUBROUTINE LUDCMP(A,N,NP,INDX,D)
      PARAMETER (NMAX=100,TINY=1.0E-20)
      DIMENSION A(NP,NP),INDX(N),VV(NMAX)
      D=1.
      DO 12 I=1,N
        AAMAX=0.
        DO 11 J=1,N
          IF (ABS(A(I,J)).GT.AAMAX) AAMAX=ABS(A(I,J))
11      CONTINUE
        IF (AAMAX.EQ.0.) PAUSE 'Singular matrix in ludcmp.'
        VV(I)=1./AAMAX
12    CONTINUE
      DO 19 J=1,N
        IF (J.GT.1) THEN
          DO 14 I=1,J-1
            SUM=A(I,J)
            IF (I.GT.1)THEN
              DO 13 K=1,I-1
                SUM=SUM-A(I,K)*A(K,J)
13            CONTINUE
              A(I,J)=SUM
            ENDIF
14        CONTINUE
        ENDIF
        AAMAX=0.
        DO 16 I=J,N
          SUM=A(I,J)
          IF (J.GT.1)THEN
            DO 15 K=1,J-1
              SUM=SUM-A(I,K)*A(K,J)
15          CONTINUE
            A(I,J)=SUM
          ENDIF
          DUM=VV(I)*ABS(SUM)
          IF (DUM.GE.AAMAX) THEN
            IMAX=I
            AAMAX=DUM
          ENDIF
16      CONTINUE
        IF (J.NE.IMAX)THEN
          DO 17 K=1,N
            DUM=A(IMAX,K)
            A(IMAX,K)=A(J,K)
            A(J,K)=DUM
17        CONTINUE
          D=-D
          VV(IMAX)=VV(J)
        ENDIF
        INDX(J)=IMAX
        IF(J.NE.N)THEN
          IF(A(J,J).EQ.0.)A(J,J)=TINY
          DUM=1./A(J,J)
          DO 18 I=J+1,N
            A(I,J)=A(I,J)*DUM
18        CONTINUE
        ENDIF
19    CONTINUE
      IF(A(N,N).EQ.0.)A(N,N)=TINY
      RETURN
      END
      SUBROUTINE LUBKSB(A,N,NP,INDX,B)
      DIMENSION A(NP,NP),INDX(N),B(N)
      II=0
      DO 12 I=1,N
        LL=INDX(I)
        SUM=B(LL)
        B(LL)=B(I)
        IF (II.NE.0)THEN
          DO 11 J=II,I-1
            SUM=SUM-A(I,J)*B(J)
11        CONTINUE
        ELSE IF (SUM.NE.0.) THEN
          II=I
        ENDIF
        B(I)=SUM
12    CONTINUE
      DO 14 I=N,1,-1
        SUM=B(I)
        IF(I.LT.N)THEN
          DO 13 J=I+1,N
            SUM=SUM-A(I,J)*B(J)
13        CONTINUE
        ENDIF
        B(I)=SUM/A(I,I)
14    CONTINUE
      RETURN
      END
